import{bL as e}from"./index-CKOVumwO.js";const t={},r=Object.freeze(Object.defineProperty({__proto__:null,default:t},Symbol.toStringTag,{value:"Module"})),a=e(r);export{a as r};
//# sourceMappingURL=___vite-browser-external_commonjs-proxy-DnAlu-kN.js.map
