function __vite__mapDeps(indexes) {
  if (!__vite__mapDeps.viteFileDeps) {
    __vite__mapDeps.viteFileDeps = ["./Selection-GYCMl90M.js","./three.module-BOGuGDOF.js","./VList-KWaULYAY.js","./draco_decoder-Ce3rGr-Y.js","./___vite-browser-external_commonjs-proxy-DnAlu-kN.js","./draco_encoder-D4x7gQUW.js","./LineSegments2-Iva-HSlT.js","./LineSegmentsGeometry-dwUEb1B-.js","./LineMaterial-BqKi9HCI.js","./ModelViewerWrapper-CUAxQuq7.js"]
  }
  return indexes.map((i) => __vite__mapDeps.viteFileDeps[i])
}
var vm=Object.defineProperty;var pm=(t,e,n)=>e in t?vm(t,e,{enumerable:!0,configurable:!0,writable:!0,value:n}):t[e]=n;var Dt=(t,e,n)=>(pm(t,typeof e!="symbol"?e+"":e,n),n);(function(){const e=document.createElement("link").relList;if(e&&e.supports&&e.supports("modulepreload"))return;for(const r of document.querySelectorAll('link[rel="modulepreload"]'))s(r);new MutationObserver(r=>{for(const i of r)if(i.type==="childList")for(const o of i.addedNodes)o.tagName==="LINK"&&o.rel==="modulepreload"&&s(o)}).observe(document,{childList:!0,subtree:!0});function n(r){const i={};return r.integrity&&(i.integrity=r.integrity),r.referrerPolicy&&(i.referrerPolicy=r.referrerPolicy),r.crossOrigin==="use-credentials"?i.credentials="include":r.crossOrigin==="anonymous"?i.credentials="omit":i.credentials="same-origin",i}function s(r){if(r.ep)return;r.ep=!0;const i=n(r);fetch(r.href,i)}})();/**
* @vue/shared v3.4.21
* (c) 2018-present Yuxi (Evan) You and Vue contributors
* @license MIT
**/function za(t,e){const n=new Set(t.split(","));return e?s=>n.has(s.toLowerCase()):s=>n.has(s)}const Re={},Ts=[],Ct=()=>{},ym=()=>!1,Xi=t=>t.charCodeAt(0)===111&&t.charCodeAt(1)===110&&(t.charCodeAt(2)>122||t.charCodeAt(2)<97),Ha=t=>t.startsWith("onUpdate:"),$e=Object.assign,$a=(t,e)=>{const n=t.indexOf(e);n>-1&&t.splice(n,1)},bm=Object.prototype.hasOwnProperty,_e=(t,e)=>bm.call(t,e),ae=Array.isArray,As=t=>Zi(t)==="[object Map]",ed=t=>Zi(t)==="[object Set]",he=t=>typeof t=="function",De=t=>typeof t=="string",Ns=t=>typeof t=="symbol",ke=t=>t!==null&&typeof t=="object",td=t=>(ke(t)||he(t))&&he(t.then)&&he(t.catch),nd=Object.prototype.toString,Zi=t=>nd.call(t),xm=t=>Zi(t).slice(8,-1),sd=t=>Zi(t)==="[object Object]",Ua=t=>De(t)&&t!=="NaN"&&t[0]!=="-"&&""+parseInt(t,10)===t,tr=za(",key,ref,ref_for,ref_key,onVnodeBeforeMount,onVnodeMounted,onVnodeBeforeUpdate,onVnodeUpdated,onVnodeBeforeUnmount,onVnodeUnmounted"),Ki=t=>{const e=Object.create(null);return n=>e[n]||(e[n]=t(n))},wm=/-(\w)/g,xt=Ki(t=>t.replace(wm,(e,n)=>n?n.toUpperCase():"")),_m=/\B([A-Z])/g,is=Ki(t=>t.replace(_m,"-$1").toLowerCase()),dn=Ki(t=>t.charAt(0).toUpperCase()+t.slice(1)),Eo=Ki(t=>t?`on${dn(t)}`:""),Gt=(t,e)=>!Object.is(t,e),Mo=(t,e)=>{for(let n=0;n<t.length;n++)t[n](e)},bi=(t,e,n)=>{Object.defineProperty(t,e,{configurable:!0,enumerable:!1,value:n})},Sm=t=>{const e=parseFloat(t);return isNaN(e)?t:e},Cm=t=>{const e=De(t)?Number(t):NaN;return isNaN(e)?t:e};let Bl;const rd=()=>Bl||(Bl=typeof globalThis<"u"?globalThis:typeof self<"u"?self:typeof window<"u"?window:typeof global<"u"?global:{});function Ji(t){if(ae(t)){const e={};for(let n=0;n<t.length;n++){const s=t[n],r=De(s)?Am(s):Ji(s);if(r)for(const i in r)e[i]=r[i]}return e}else if(De(t)||ke(t))return t}const Em=/;(?![^(]*\))/g,Mm=/:([^]+)/,Tm=/\/\*[^]*?\*\//g;function Am(t){const e={};return t.replace(Tm,"").split(Em).forEach(n=>{if(n){const s=n.split(Mm);s.length>1&&(e[s[0].trim()]=s[1].trim())}}),e}function Qi(t){let e="";if(De(t))e=t;else if(ae(t))for(let n=0;n<t.length;n++){const s=Qi(t[n]);s&&(e+=s+" ")}else if(ke(t))for(const n in t)t[n]&&(e+=n+" ");return e.trim()}const Rm="itemscope,allowfullscreen,formnovalidate,ismap,nomodule,novalidate,readonly",Im=za(Rm);function id(t){return!!t||t===""}const rn=t=>De(t)?t:t==null?"":ae(t)||ke(t)&&(t.toString===nd||!he(t.toString))?JSON.stringify(t,od,2):String(t),od=(t,e)=>e&&e.__v_isRef?od(t,e.value):As(e)?{[`Map(${e.size})`]:[...e.entries()].reduce((n,[s,r],i)=>(n[To(s,i)+" =>"]=r,n),{})}:ed(e)?{[`Set(${e.size})`]:[...e.values()].map(n=>To(n))}:Ns(e)?To(e):ke(e)&&!ae(e)&&!sd(e)?String(e):e,To=(t,e="")=>{var n;return Ns(t)?`Symbol(${(n=t.description)!=null?n:e})`:t};/**
* @vue/reactivity v3.4.21
* (c) 2018-present Yuxi (Evan) You and Vue contributors
* @license MIT
**/let mt;class ad{constructor(e=!1){this.detached=e,this._active=!0,this.effects=[],this.cleanups=[],this.parent=mt,!e&&mt&&(this.index=(mt.scopes||(mt.scopes=[])).push(this)-1)}get active(){return this._active}run(e){if(this._active){const n=mt;try{return mt=this,e()}finally{mt=n}}}on(){mt=this}off(){mt=this.parent}stop(e){if(this._active){let n,s;for(n=0,s=this.effects.length;n<s;n++)this.effects[n].stop();for(n=0,s=this.cleanups.length;n<s;n++)this.cleanups[n]();if(this.scopes)for(n=0,s=this.scopes.length;n<s;n++)this.scopes[n].stop(!0);if(!this.detached&&this.parent&&!e){const r=this.parent.scopes.pop();r&&r!==this&&(this.parent.scopes[this.index]=r,r.index=this.index)}this.parent=void 0,this._active=!1}}}function ja(t){return new ad(t)}function km(t,e=mt){e&&e.active&&e.effects.push(t)}function Fm(){return mt}function Rt(t){mt&&mt.cleanups.push(t)}let Kn;class Wa{constructor(e,n,s,r){this.fn=e,this.trigger=n,this.scheduler=s,this.active=!0,this.deps=[],this._dirtyLevel=4,this._trackId=0,this._runnings=0,this._shouldSchedule=!1,this._depsLength=0,km(this,r)}get dirty(){if(this._dirtyLevel===2||this._dirtyLevel===3){this._dirtyLevel=1,os();for(let e=0;e<this._depsLength;e++){const n=this.deps[e];if(n.computed&&(Lm(n.computed),this._dirtyLevel>=4))break}this._dirtyLevel===1&&(this._dirtyLevel=0),as()}return this._dirtyLevel>=4}set dirty(e){this._dirtyLevel=e?4:0}run(){if(this._dirtyLevel=0,!this.active)return this.fn();let e=In,n=Kn;try{return In=!0,Kn=this,this._runnings++,Dl(this),this.fn()}finally{zl(this),this._runnings--,Kn=n,In=e}}stop(){var e;this.active&&(Dl(this),zl(this),(e=this.onStop)==null||e.call(this),this.active=!1)}}function Lm(t){return t.value}function Dl(t){t._trackId++,t._depsLength=0}function zl(t){if(t.deps.length>t._depsLength){for(let e=t._depsLength;e<t.deps.length;e++)ld(t.deps[e],t);t.deps.length=t._depsLength}}function ld(t,e){const n=t.get(e);n!==void 0&&e._trackId!==n&&(t.delete(e),t.size===0&&t.cleanup())}let In=!0,ia=0;const ud=[];function os(){ud.push(In),In=!1}function as(){const t=ud.pop();In=t===void 0?!0:t}function Ga(){ia++}function qa(){for(ia--;!ia&&oa.length;)oa.shift()()}function cd(t,e,n){if(e.get(t)!==t._trackId){e.set(t,t._trackId);const s=t.deps[t._depsLength];s!==e?(s&&ld(s,t),t.deps[t._depsLength++]=e):t._depsLength++}}const oa=[];function dd(t,e,n){Ga();for(const s of t.keys()){let r;s._dirtyLevel<e&&(r??(r=t.get(s)===s._trackId))&&(s._shouldSchedule||(s._shouldSchedule=s._dirtyLevel===0),s._dirtyLevel=e),s._shouldSchedule&&(r??(r=t.get(s)===s._trackId))&&(s.trigger(),(!s._runnings||s.allowRecurse)&&s._dirtyLevel!==2&&(s._shouldSchedule=!1,s.scheduler&&oa.push(s.scheduler)))}qa()}const fd=(t,e)=>{const n=new Map;return n.cleanup=t,n.computed=e,n},xi=new WeakMap,Jn=Symbol(""),aa=Symbol("");function lt(t,e,n){if(In&&Kn){let s=xi.get(t);s||xi.set(t,s=new Map);let r=s.get(n);r||s.set(n,r=fd(()=>s.delete(n))),cd(Kn,r)}}function on(t,e,n,s,r,i){const o=xi.get(t);if(!o)return;let a=[];if(e==="clear")a=[...o.values()];else if(n==="length"&&ae(t)){const l=Number(s);o.forEach((u,c)=>{(c==="length"||!Ns(c)&&c>=l)&&a.push(u)})}else switch(n!==void 0&&a.push(o.get(n)),e){case"add":ae(t)?Ua(n)&&a.push(o.get("length")):(a.push(o.get(Jn)),As(t)&&a.push(o.get(aa)));break;case"delete":ae(t)||(a.push(o.get(Jn)),As(t)&&a.push(o.get(aa)));break;case"set":As(t)&&a.push(o.get(Jn));break}Ga();for(const l of a)l&&dd(l,4);qa()}function Vm(t,e){var n;return(n=xi.get(t))==null?void 0:n.get(e)}const Pm=za("__proto__,__v_isRef,__isVue"),hd=new Set(Object.getOwnPropertyNames(Symbol).filter(t=>t!=="arguments"&&t!=="caller").map(t=>Symbol[t]).filter(Ns)),Hl=Om();function Om(){const t={};return["includes","indexOf","lastIndexOf"].forEach(e=>{t[e]=function(...n){const s=pe(this);for(let i=0,o=this.length;i<o;i++)lt(s,"get",i+"");const r=s[e](...n);return r===-1||r===!1?s[e](...n.map(pe)):r}}),["push","pop","shift","unshift","splice"].forEach(e=>{t[e]=function(...n){os(),Ga();const s=pe(this)[e].apply(this,n);return qa(),as(),s}}),t}function Nm(t){const e=pe(this);return lt(e,"has",t),e.hasOwnProperty(t)}class md{constructor(e=!1,n=!1){this._isReadonly=e,this._isShallow=n}get(e,n,s){const r=this._isReadonly,i=this._isShallow;if(n==="__v_isReactive")return!r;if(n==="__v_isReadonly")return r;if(n==="__v_isShallow")return i;if(n==="__v_raw")return s===(r?i?Zm:yd:i?pd:vd).get(e)||Object.getPrototypeOf(e)===Object.getPrototypeOf(s)?e:void 0;const o=ae(e);if(!r){if(o&&_e(Hl,n))return Reflect.get(Hl,n,s);if(n==="hasOwnProperty")return Nm}const a=Reflect.get(e,n,s);return(Ns(n)?hd.has(n):Pm(n))||(r||lt(e,"get",n),i)?a:Me(a)?o&&Ua(n)?a:a.value:ke(a)?r?Cr(a):tt(a):a}}class gd extends md{constructor(e=!1){super(!1,e)}set(e,n,s,r){let i=e[n];if(!this._isShallow){const l=Fs(i);if(!wi(s)&&!Fs(s)&&(i=pe(i),s=pe(s)),!ae(e)&&Me(i)&&!Me(s))return l?!1:(i.value=s,!0)}const o=ae(e)&&Ua(n)?Number(n)<e.length:_e(e,n),a=Reflect.set(e,n,s,r);return e===pe(r)&&(o?Gt(s,i)&&on(e,"set",n,s):on(e,"add",n,s)),a}deleteProperty(e,n){const s=_e(e,n);e[n];const r=Reflect.deleteProperty(e,n);return r&&s&&on(e,"delete",n,void 0),r}has(e,n){const s=Reflect.has(e,n);return(!Ns(n)||!hd.has(n))&&lt(e,"has",n),s}ownKeys(e){return lt(e,"iterate",ae(e)?"length":Jn),Reflect.ownKeys(e)}}class Bm extends md{constructor(e=!1){super(!0,e)}set(e,n){return!0}deleteProperty(e,n){return!0}}const Dm=new gd,zm=new Bm,Hm=new gd(!0),Ya=t=>t,eo=t=>Reflect.getPrototypeOf(t);function Nr(t,e,n=!1,s=!1){t=t.__v_raw;const r=pe(t),i=pe(e);n||(Gt(e,i)&&lt(r,"get",e),lt(r,"get",i));const{has:o}=eo(r),a=s?Ya:n?Ka:dr;if(o.call(r,e))return a(t.get(e));if(o.call(r,i))return a(t.get(i));t!==r&&t.get(e)}function Br(t,e=!1){const n=this.__v_raw,s=pe(n),r=pe(t);return e||(Gt(t,r)&&lt(s,"has",t),lt(s,"has",r)),t===r?n.has(t):n.has(t)||n.has(r)}function Dr(t,e=!1){return t=t.__v_raw,!e&&lt(pe(t),"iterate",Jn),Reflect.get(t,"size",t)}function $l(t){t=pe(t);const e=pe(this);return eo(e).has.call(e,t)||(e.add(t),on(e,"add",t,t)),this}function Ul(t,e){e=pe(e);const n=pe(this),{has:s,get:r}=eo(n);let i=s.call(n,t);i||(t=pe(t),i=s.call(n,t));const o=r.call(n,t);return n.set(t,e),i?Gt(e,o)&&on(n,"set",t,e):on(n,"add",t,e),this}function jl(t){const e=pe(this),{has:n,get:s}=eo(e);let r=n.call(e,t);r||(t=pe(t),r=n.call(e,t)),s&&s.call(e,t);const i=e.delete(t);return r&&on(e,"delete",t,void 0),i}function Wl(){const t=pe(this),e=t.size!==0,n=t.clear();return e&&on(t,"clear",void 0,void 0),n}function zr(t,e){return function(s,r){const i=this,o=i.__v_raw,a=pe(o),l=e?Ya:t?Ka:dr;return!t&&lt(a,"iterate",Jn),o.forEach((u,c)=>s.call(r,l(u),l(c),i))}}function Hr(t,e,n){return function(...s){const r=this.__v_raw,i=pe(r),o=As(i),a=t==="entries"||t===Symbol.iterator&&o,l=t==="keys"&&o,u=r[t](...s),c=n?Ya:e?Ka:dr;return!e&&lt(i,"iterate",l?aa:Jn),{next(){const{value:d,done:f}=u.next();return f?{value:d,done:f}:{value:a?[c(d[0]),c(d[1])]:c(d),done:f}},[Symbol.iterator](){return this}}}}function pn(t){return function(...e){return t==="delete"?!1:t==="clear"?void 0:this}}function $m(){const t={get(i){return Nr(this,i)},get size(){return Dr(this)},has:Br,add:$l,set:Ul,delete:jl,clear:Wl,forEach:zr(!1,!1)},e={get(i){return Nr(this,i,!1,!0)},get size(){return Dr(this)},has:Br,add:$l,set:Ul,delete:jl,clear:Wl,forEach:zr(!1,!0)},n={get(i){return Nr(this,i,!0)},get size(){return Dr(this,!0)},has(i){return Br.call(this,i,!0)},add:pn("add"),set:pn("set"),delete:pn("delete"),clear:pn("clear"),forEach:zr(!0,!1)},s={get(i){return Nr(this,i,!0,!0)},get size(){return Dr(this,!0)},has(i){return Br.call(this,i,!0)},add:pn("add"),set:pn("set"),delete:pn("delete"),clear:pn("clear"),forEach:zr(!0,!0)};return["keys","values","entries",Symbol.iterator].forEach(i=>{t[i]=Hr(i,!1,!1),n[i]=Hr(i,!0,!1),e[i]=Hr(i,!1,!0),s[i]=Hr(i,!0,!0)}),[t,n,e,s]}const[Um,jm,Wm,Gm]=$m();function Xa(t,e){const n=e?t?Gm:Wm:t?jm:Um;return(s,r,i)=>r==="__v_isReactive"?!t:r==="__v_isReadonly"?t:r==="__v_raw"?s:Reflect.get(_e(n,r)&&r in s?n:s,r,i)}const qm={get:Xa(!1,!1)},Ym={get:Xa(!1,!0)},Xm={get:Xa(!0,!1)},vd=new WeakMap,pd=new WeakMap,yd=new WeakMap,Zm=new WeakMap;function Km(t){switch(t){case"Object":case"Array":return 1;case"Map":case"Set":case"WeakMap":case"WeakSet":return 2;default:return 0}}function Jm(t){return t.__v_skip||!Object.isExtensible(t)?0:Km(xm(t))}function tt(t){return Fs(t)?t:Za(t,!1,Dm,qm,vd)}function Qm(t){return Za(t,!1,Hm,Ym,pd)}function Cr(t){return Za(t,!0,zm,Xm,yd)}function Za(t,e,n,s,r){if(!ke(t)||t.__v_raw&&!(e&&t.__v_isReactive))return t;const i=r.get(t);if(i)return i;const o=Jm(t);if(o===0)return t;const a=new Proxy(t,o===2?s:n);return r.set(t,a),a}function Rs(t){return Fs(t)?Rs(t.__v_raw):!!(t&&t.__v_isReactive)}function Fs(t){return!!(t&&t.__v_isReadonly)}function wi(t){return!!(t&&t.__v_isShallow)}function bd(t){return Rs(t)||Fs(t)}function pe(t){const e=t&&t.__v_raw;return e?pe(e):t}function xd(t){return Object.isExtensible(t)&&bi(t,"__v_skip",!0),t}const dr=t=>ke(t)?tt(t):t,Ka=t=>ke(t)?Cr(t):t;class wd{constructor(e,n,s,r){this.getter=e,this._setter=n,this.dep=void 0,this.__v_isRef=!0,this.__v_isReadonly=!1,this.effect=new Wa(()=>e(this._value),()=>Is(this,this.effect._dirtyLevel===2?2:3)),this.effect.computed=this,this.effect.active=this._cacheable=!r,this.__v_isReadonly=s}get value(){const e=pe(this);return(!e._cacheable||e.effect.dirty)&&Gt(e._value,e._value=e.effect.run())&&Is(e,4),Ja(e),e.effect._dirtyLevel>=2&&Is(e,2),e._value}set value(e){this._setter(e)}get _dirty(){return this.effect.dirty}set _dirty(e){this.effect.dirty=e}}function eg(t,e,n=!1){let s,r;const i=he(t);return i?(s=t,r=Ct):(s=t.get,r=t.set),new wd(s,r,i||!r,n)}function Ja(t){var e;In&&Kn&&(t=pe(t),cd(Kn,(e=t.dep)!=null?e:t.dep=fd(()=>t.dep=void 0,t instanceof wd?t:void 0)))}function Is(t,e=4,n){t=pe(t);const s=t.dep;s&&dd(s,e)}function Me(t){return!!(t&&t.__v_isRef===!0)}function ee(t){return _d(t,!1)}function be(t){return _d(t,!0)}function _d(t,e){return Me(t)?t:new tg(t,e)}class tg{constructor(e,n){this.__v_isShallow=n,this.dep=void 0,this.__v_isRef=!0,this._rawValue=n?e:pe(e),this._value=n?e:dr(e)}get value(){return Ja(this),this._value}set value(e){const n=this.__v_isShallow||wi(e)||Fs(e);e=n?e:pe(e),Gt(e,this._rawValue)&&(this._rawValue=e,this._value=n?e:dr(e),Is(this,4))}}function ng(t){Is(t,4)}function P(t){return Me(t)?t.value:t}const sg={get:(t,e,n)=>P(Reflect.get(t,e,n)),set:(t,e,n,s)=>{const r=t[e];return Me(r)&&!Me(n)?(r.value=n,!0):Reflect.set(t,e,n,s)}};function Sd(t){return Rs(t)?t:new Proxy(t,sg)}class rg{constructor(e){this.dep=void 0,this.__v_isRef=!0;const{get:n,set:s}=e(()=>Ja(this),()=>Is(this));this._get=n,this._set=s}get value(){return this._get()}set value(e){this._set(e)}}function ig(t){return new rg(t)}function Qa(t){const e=ae(t)?new Array(t.length):{};for(const n in t)e[n]=Cd(t,n);return e}class og{constructor(e,n,s){this._object=e,this._key=n,this._defaultValue=s,this.__v_isRef=!0}get value(){const e=this._object[this._key];return e===void 0?this._defaultValue:e}set value(e){this._object[this._key]=e}get dep(){return Vm(pe(this._object),this._key)}}class ag{constructor(e){this._getter=e,this.__v_isRef=!0,this.__v_isReadonly=!0}get value(){return this._getter()}}function fe(t,e,n){return Me(t)?t:he(t)?new ag(t):ke(t)&&arguments.length>1?Cd(t,e,n):ee(t)}function Cd(t,e,n){const s=t[e];return Me(s)?s:new og(t,e,n)}/**
* @vue/runtime-core v3.4.21
* (c) 2018-present Yuxi (Evan) You and Vue contributors
* @license MIT
**/function kn(t,e,n,s){try{return s?t(...s):t()}catch(r){Er(r,e,n)}}function Mt(t,e,n,s){if(he(t)){const i=kn(t,e,n,s);return i&&td(i)&&i.catch(o=>{Er(o,e,n)}),i}const r=[];for(let i=0;i<t.length;i++)r.push(Mt(t[i],e,n,s));return r}function Er(t,e,n,s=!0){const r=e?e.vnode:null;if(e){let i=e.parent;const o=e.proxy,a=`https://vuejs.org/error-reference/#runtime-${n}`;for(;i;){const u=i.ec;if(u){for(let c=0;c<u.length;c++)if(u[c](t,o,a)===!1)return}i=i.parent}const l=e.appContext.config.errorHandler;if(l){kn(l,null,10,[t,o,a]);return}}lg(t,n,r,s)}function lg(t,e,n,s=!0){console.error(t)}let fr=!1,la=!1;const Ze=[];let $t=0;const ks=[];let En=null,qn=0;const Ed=Promise.resolve();let el=null;function It(t){const e=el||Ed;return t?e.then(this?t.bind(this):t):e}function ug(t){let e=$t+1,n=Ze.length;for(;e<n;){const s=e+n>>>1,r=Ze[s],i=hr(r);i<t||i===t&&r.pre?e=s+1:n=s}return e}function to(t){(!Ze.length||!Ze.includes(t,fr&&t.allowRecurse?$t+1:$t))&&(t.id==null?Ze.push(t):Ze.splice(ug(t.id),0,t),Md())}function Md(){!fr&&!la&&(la=!0,el=Ed.then(Ad))}function cg(t){const e=Ze.indexOf(t);e>$t&&Ze.splice(e,1)}function dg(t){ae(t)?ks.push(...t):(!En||!En.includes(t,t.allowRecurse?qn+1:qn))&&ks.push(t),Md()}function Gl(t,e,n=fr?$t+1:0){for(;n<Ze.length;n++){const s=Ze[n];if(s&&s.pre){if(t&&s.id!==t.uid)continue;Ze.splice(n,1),n--,s()}}}function Td(t){if(ks.length){const e=[...new Set(ks)].sort((n,s)=>hr(n)-hr(s));if(ks.length=0,En){En.push(...e);return}for(En=e,qn=0;qn<En.length;qn++)En[qn]();En=null,qn=0}}const hr=t=>t.id==null?1/0:t.id,fg=(t,e)=>{const n=hr(t)-hr(e);if(n===0){if(t.pre&&!e.pre)return-1;if(e.pre&&!t.pre)return 1}return n};function Ad(t){la=!1,fr=!0,Ze.sort(fg);try{for($t=0;$t<Ze.length;$t++){const e=Ze[$t];e&&e.active!==!1&&kn(e,null,14)}}finally{$t=0,Ze.length=0,Td(),fr=!1,el=null,(Ze.length||ks.length)&&Ad()}}function hg(t,e,...n){if(t.isUnmounted)return;const s=t.vnode.props||Re;let r=n;const i=e.startsWith("update:"),o=i&&e.slice(7);if(o&&o in s){const c=`${o==="modelValue"?"model":o}Modifiers`,{number:d,trim:f}=s[c]||Re;f&&(r=n.map(h=>De(h)?h.trim():h)),d&&(r=n.map(Sm))}let a,l=s[a=Eo(e)]||s[a=Eo(xt(e))];!l&&i&&(l=s[a=Eo(is(e))]),l&&Mt(l,t,6,r);const u=s[a+"Once"];if(u){if(!t.emitted)t.emitted={};else if(t.emitted[a])return;t.emitted[a]=!0,Mt(u,t,6,r)}}function Rd(t,e,n=!1){const s=e.emitsCache,r=s.get(t);if(r!==void 0)return r;const i=t.emits;let o={},a=!1;if(!he(t)){const l=u=>{const c=Rd(u,e,!0);c&&(a=!0,$e(o,c))};!n&&e.mixins.length&&e.mixins.forEach(l),t.extends&&l(t.extends),t.mixins&&t.mixins.forEach(l)}return!i&&!a?(ke(t)&&s.set(t,null),null):(ae(i)?i.forEach(l=>o[l]=null):$e(o,i),ke(t)&&s.set(t,o),o)}function no(t,e){return!t||!Xi(e)?!1:(e=e.slice(2).replace(/Once$/,""),_e(t,e[0].toLowerCase()+e.slice(1))||_e(t,is(e))||_e(t,e))}let Ue=null,Id=null;function _i(t){const e=Ue;return Ue=t,Id=t&&t.type.__scopeId||null,e}function q(t,e=Ue,n){if(!e||t._n)return t;const s=(...r)=>{s._d&&ou(-1);const i=_i(e);let o;try{o=t(...r)}finally{_i(i),s._d&&ou(1)}return o};return s._n=!0,s._c=!0,s._d=!0,s}function Ao(t){const{type:e,vnode:n,proxy:s,withProxy:r,props:i,propsOptions:[o],slots:a,attrs:l,emit:u,render:c,renderCache:d,data:f,setupState:h,ctx:m,inheritAttrs:p}=t;let v,S;const g=_i(t);try{if(n.shapeFlag&4){const b=r||s,x=b;v=Ht(c.call(x,b,d,i,h,f,m)),S=l}else{const b=e;v=Ht(b.length>1?b(i,{attrs:l,slots:a,emit:u}):b(i,null)),S=e.props?l:mg(l)}}catch(b){or.length=0,Er(b,t,1),v=w(yt)}let y=v;if(S&&p!==!1){const b=Object.keys(S),{shapeFlag:x}=y;b.length&&x&7&&(o&&b.some(Ha)&&(S=gg(S,o)),y=Pn(y,S))}return n.dirs&&(y=Pn(y),y.dirs=y.dirs?y.dirs.concat(n.dirs):n.dirs),n.transition&&(y.transition=n.transition),v=y,_i(g),v}const mg=t=>{let e;for(const n in t)(n==="class"||n==="style"||Xi(n))&&((e||(e={}))[n]=t[n]);return e},gg=(t,e)=>{const n={};for(const s in t)(!Ha(s)||!(s.slice(9)in e))&&(n[s]=t[s]);return n};function vg(t,e,n){const{props:s,children:r,component:i}=t,{props:o,children:a,patchFlag:l}=e,u=i.emitsOptions;if(e.dirs||e.transition)return!0;if(n&&l>=0){if(l&1024)return!0;if(l&16)return s?ql(s,o,u):!!o;if(l&8){const c=e.dynamicProps;for(let d=0;d<c.length;d++){const f=c[d];if(o[f]!==s[f]&&!no(u,f))return!0}}}else return(r||a)&&(!a||!a.$stable)?!0:s===o?!1:s?o?ql(s,o,u):!0:!!o;return!1}function ql(t,e,n){const s=Object.keys(e);if(s.length!==Object.keys(t).length)return!0;for(let r=0;r<s.length;r++){const i=s[r];if(e[i]!==t[i]&&!no(n,i))return!0}return!1}function pg({vnode:t,parent:e},n){for(;e;){const s=e.subTree;if(s.suspense&&s.suspense.activeBranch===t&&(s.el=t.el),s===t)(t=e.vnode).el=n,e=e.parent;else break}}const kd="components",yg="directives",Fd=Symbol.for("v-ndc");function bg(t){return De(t)?Ld(kd,t,!1)||t:t||Fd}function ls(t){return Ld(yg,t)}function Ld(t,e,n=!0,s=!1){const r=Ue||Ge;if(r){const i=r.type;if(t===kd){const a=fv(i,!1);if(a&&(a===e||a===xt(e)||a===dn(xt(e))))return i}const o=Yl(r[t]||i[t],e)||Yl(r.appContext[t],e);return!o&&s?i:o}}function Yl(t,e){return t&&(t[e]||t[xt(e)]||t[dn(xt(e))])}const xg=t=>t.__isSuspense;function wg(t,e){e&&e.pendingBranch?ae(t)?e.effects.push(...t):e.effects.push(t):dg(t)}const _g=Symbol.for("v-scx"),Sg=()=>Ne(_g);function Nn(t,e){return so(t,null,e)}function Cg(t,e){return so(t,null,{flush:"sync"})}const $r={};function le(t,e,n){return so(t,e,n)}function so(t,e,{immediate:n,deep:s,flush:r,once:i,onTrack:o,onTrigger:a}=Re){if(e&&i){const _=e;e=(...C)=>{_(...C),x()}}const l=Ge,u=_=>s===!0?_:Xn(_,s===!1?1:void 0);let c,d=!1,f=!1;if(Me(t)?(c=()=>t.value,d=wi(t)):Rs(t)?(c=()=>u(t),d=!0):ae(t)?(f=!0,d=t.some(_=>Rs(_)||wi(_)),c=()=>t.map(_=>{if(Me(_))return _.value;if(Rs(_))return u(_);if(he(_))return kn(_,l,2)})):he(t)?e?c=()=>kn(t,l,2):c=()=>(h&&h(),Mt(t,l,3,[m])):c=Ct,e&&s){const _=c;c=()=>Xn(_())}let h,m=_=>{h=y.onStop=()=>{kn(_,l,4),h=y.onStop=void 0}},p;if(Ir)if(m=Ct,e?n&&Mt(e,l,3,[c(),f?[]:void 0,m]):c(),r==="sync"){const _=Sg();p=_.__watcherHandles||(_.__watcherHandles=[])}else return Ct;let v=f?new Array(t.length).fill($r):$r;const S=()=>{if(!(!y.active||!y.dirty))if(e){const _=y.run();(s||d||(f?_.some((C,M)=>Gt(C,v[M])):Gt(_,v)))&&(h&&h(),Mt(e,l,3,[_,v===$r?void 0:f&&v[0]===$r?[]:v,m]),v=_)}else y.run()};S.allowRecurse=!!e;let g;r==="sync"?g=S:r==="post"?g=()=>it(S,l&&l.suspense):(S.pre=!0,l&&(S.id=l.uid),g=()=>to(S));const y=new Wa(c,Ct,g),b=Fm(),x=()=>{y.stop(),b&&$a(b.effects,y)};return e?n?S():v=y.run():r==="post"?it(y.run.bind(y),l&&l.suspense):y.run(),p&&p.push(x),x}function Eg(t,e,n){const s=this.proxy,r=De(t)?t.includes(".")?Vd(s,t):()=>s[t]:t.bind(s,s);let i;he(e)?i=e:(i=e.handler,n=e);const o=Rr(this),a=so(r,i.bind(s),n);return o(),a}function Vd(t,e){const n=e.split(".");return()=>{let s=t;for(let r=0;r<n.length&&s;r++)s=s[n[r]];return s}}function Xn(t,e,n=0,s){if(!ke(t)||t.__v_skip)return t;if(e&&e>0){if(n>=e)return t;n++}if(s=s||new Set,s.has(t))return t;if(s.add(t),Me(t))Xn(t.value,e,n,s);else if(ae(t))for(let r=0;r<t.length;r++)Xn(t[r],e,n,s);else if(ed(t)||As(t))t.forEach(r=>{Xn(r,e,n,s)});else if(sd(t))for(const r in t)Xn(t[r],e,n,s);return t}function qt(t,e){if(Ue===null)return t;const n=lo(Ue)||Ue.proxy,s=t.dirs||(t.dirs=[]);for(let r=0;r<e.length;r++){let[i,o,a,l=Re]=e[r];i&&(he(i)&&(i={mounted:i,updated:i}),i.deep&&Xn(o),s.push({dir:i,instance:n,value:o,oldValue:void 0,arg:a,modifiers:l}))}return t}function zn(t,e,n,s){const r=t.dirs,i=e&&e.dirs;for(let o=0;o<r.length;o++){const a=r[o];i&&(a.oldValue=i[o].value);let l=a.dir[s];l&&(os(),Mt(l,n,8,[t.el,a,t,e]),as())}}const Mn=Symbol("_leaveCb"),Ur=Symbol("_enterCb");function Pd(){const t={isMounted:!1,isLeaving:!1,isUnmounting:!1,leavingVNodes:new Map};return Zt(()=>{t.isMounted=!0}),kt(()=>{t.isUnmounting=!0}),t}const St=[Function,Array],Od={mode:String,appear:Boolean,persisted:Boolean,onBeforeEnter:St,onEnter:St,onAfterEnter:St,onEnterCancelled:St,onBeforeLeave:St,onLeave:St,onAfterLeave:St,onLeaveCancelled:St,onBeforeAppear:St,onAppear:St,onAfterAppear:St,onAppearCancelled:St},Mg={name:"BaseTransition",props:Od,setup(t,{slots:e}){const n=ao(),s=Pd();return()=>{const r=e.default&&tl(e.default(),!0);if(!r||!r.length)return;let i=r[0];if(r.length>1){for(const f of r)if(f.type!==yt){i=f;break}}const o=pe(t),{mode:a}=o;if(s.isLeaving)return Ro(i);const l=Xl(i);if(!l)return Ro(i);const u=mr(l,o,s,n);gr(l,u);const c=n.subTree,d=c&&Xl(c);if(d&&d.type!==yt&&!Yn(l,d)){const f=mr(d,o,s,n);if(gr(d,f),a==="out-in")return s.isLeaving=!0,f.afterLeave=()=>{s.isLeaving=!1,n.update.active!==!1&&(n.effect.dirty=!0,n.update())},Ro(i);a==="in-out"&&l.type!==yt&&(f.delayLeave=(h,m,p)=>{const v=Nd(s,d);v[String(d.key)]=d,h[Mn]=()=>{m(),h[Mn]=void 0,delete u.delayedLeave},u.delayedLeave=p})}return i}}},Tg=Mg;function Nd(t,e){const{leavingVNodes:n}=t;let s=n.get(e.type);return s||(s=Object.create(null),n.set(e.type,s)),s}function mr(t,e,n,s){const{appear:r,mode:i,persisted:o=!1,onBeforeEnter:a,onEnter:l,onAfterEnter:u,onEnterCancelled:c,onBeforeLeave:d,onLeave:f,onAfterLeave:h,onLeaveCancelled:m,onBeforeAppear:p,onAppear:v,onAfterAppear:S,onAppearCancelled:g}=e,y=String(t.key),b=Nd(n,t),x=(M,R)=>{M&&Mt(M,s,9,R)},_=(M,R)=>{const F=R[1];x(M,R),ae(M)?M.every(V=>V.length<=1)&&F():M.length<=1&&F()},C={mode:i,persisted:o,beforeEnter(M){let R=a;if(!n.isMounted)if(r)R=p||a;else return;M[Mn]&&M[Mn](!0);const F=b[y];F&&Yn(t,F)&&F.el[Mn]&&F.el[Mn](),x(R,[M])},enter(M){let R=l,F=u,V=c;if(!n.isMounted)if(r)R=v||l,F=S||u,V=g||c;else return;let T=!1;const L=M[Ur]=A=>{T||(T=!0,A?x(V,[M]):x(F,[M]),C.delayedLeave&&C.delayedLeave(),M[Ur]=void 0)};R?_(R,[M,L]):L()},leave(M,R){const F=String(t.key);if(M[Ur]&&M[Ur](!0),n.isUnmounting)return R();x(d,[M]);let V=!1;const T=M[Mn]=L=>{V||(V=!0,R(),L?x(m,[M]):x(h,[M]),M[Mn]=void 0,b[F]===t&&delete b[F])};b[F]=t,f?_(f,[M,T]):T()},clone(M){return mr(M,e,n,s)}};return C}function Ro(t){if(Mr(t))return t=Pn(t),t.children=null,t}function Xl(t){return Mr(t)?t.children?t.children[0]:void 0:t}function gr(t,e){t.shapeFlag&6&&t.component?gr(t.component.subTree,e):t.shapeFlag&128?(t.ssContent.transition=e.clone(t.ssContent),t.ssFallback.transition=e.clone(t.ssFallback)):t.transition=e}function tl(t,e=!1,n){let s=[],r=0;for(let i=0;i<t.length;i++){let o=t[i];const a=n==null?o.key:String(n)+String(o.key!=null?o.key:i);o.type===Ie?(o.patchFlag&128&&r++,s=s.concat(tl(o.children,e,a))):(e||o.type!==yt)&&s.push(a!=null?Pn(o,{key:a}):o)}if(r>1)for(let i=0;i<s.length;i++)s[i].patchFlag=-2;return s}/*! #__NO_SIDE_EFFECTS__ */function fn(t,e){return he(t)?$e({name:t.name},e,{setup:t}):t}const nr=t=>!!t.type.__asyncLoader;/*! #__NO_SIDE_EFFECTS__ */function ua(t){he(t)&&(t={loader:t});const{loader:e,loadingComponent:n,errorComponent:s,delay:r=200,timeout:i,suspensible:o=!0,onError:a}=t;let l=null,u,c=0;const d=()=>(c++,l=null,f()),f=()=>{let h;return l||(h=l=e().catch(m=>{if(m=m instanceof Error?m:new Error(String(m)),a)return new Promise((p,v)=>{a(m,()=>p(d()),()=>v(m),c+1)});throw m}).then(m=>h!==l&&l?l:(m&&(m.__esModule||m[Symbol.toStringTag]==="Module")&&(m=m.default),u=m,m)))};return fn({name:"AsyncComponentWrapper",__asyncLoader:f,get __asyncResolved(){return u},setup(){const h=Ge;if(u)return()=>Io(u,h);const m=g=>{l=null,Er(g,h,13,!s)};if(o&&h.suspense||Ir)return f().then(g=>()=>Io(g,h)).catch(g=>(m(g),()=>s?w(s,{error:g}):null));const p=ee(!1),v=ee(),S=ee(!!r);return r&&setTimeout(()=>{S.value=!1},r),i!=null&&setTimeout(()=>{if(!p.value&&!v.value){const g=new Error(`Async component timed out after ${i}ms.`);m(g),v.value=g}},i),f().then(()=>{p.value=!0,h.parent&&Mr(h.parent.vnode)&&(h.parent.effect.dirty=!0,to(h.parent.update))}).catch(g=>{m(g),v.value=g}),()=>{if(p.value&&u)return Io(u,h);if(v.value&&s)return w(s,{error:v.value});if(n&&!S.value)return w(n)}}})}function Io(t,e){const{ref:n,props:s,children:r,ce:i}=e.vnode,o=w(t,s,r);return o.ref=n,o.ce=i,delete e.vnode.ce,o}const Mr=t=>t.type.__isKeepAlive;function Bd(t,e){zd(t,"a",e)}function Dd(t,e){zd(t,"da",e)}function zd(t,e,n=Ge){const s=t.__wdc||(t.__wdc=()=>{let r=n;for(;r;){if(r.isDeactivated)return;r=r.parent}return t()});if(ro(e,s,n),n){let r=n.parent;for(;r&&r.parent;)Mr(r.parent.vnode)&&Ag(s,e,n,r),r=r.parent}}function Ag(t,e,n,s){const r=ro(e,t,s,!0);Hd(()=>{$a(s[e],r)},n)}function ro(t,e,n=Ge,s=!1){if(n){const r=n[t]||(n[t]=[]),i=e.__weh||(e.__weh=(...o)=>{if(n.isUnmounted)return;os();const a=Rr(n),l=Mt(e,n,t,o);return a(),as(),l});return s?r.unshift(i):r.push(i),i}}const hn=t=>(e,n=Ge)=>(!Ir||t==="sp")&&ro(t,(...s)=>e(...s),n),io=hn("bm"),Zt=hn("m"),Rg=hn("bu"),nl=hn("u"),kt=hn("bum"),Hd=hn("um"),Ig=hn("sp"),kg=hn("rtg"),Fg=hn("rtc");function Lg(t,e=Ge){ro("ec",t,e)}function Vg(t,e,n,s){let r;const i=n&&n[s];if(ae(t)||De(t)){r=new Array(t.length);for(let o=0,a=t.length;o<a;o++)r[o]=e(t[o],o,void 0,i&&i[o])}else if(typeof t=="number"){r=new Array(t);for(let o=0;o<t;o++)r[o]=e(o+1,o,void 0,i&&i[o])}else if(ke(t))if(t[Symbol.iterator])r=Array.from(t,(o,a)=>e(o,a,void 0,i&&i[a]));else{const o=Object.keys(t);r=new Array(o.length);for(let a=0,l=o.length;a<l;a++){const u=o[a];r[a]=e(t[u],u,a,i&&i[a])}}else r=[];return n&&(n[s]=r),r}function Ys(t,e,n={},s,r){if(Ue.isCE||Ue.parent&&nr(Ue.parent)&&Ue.parent.isCE)return e!=="default"&&(n.name=e),w("slot",n,s&&s());let i=t[e];i&&i._c&&(i._d=!1),st();const o=i&&$d(i(n)),a=Yt(Ie,{key:n.key||o&&o.key||`_${e}`},o||(s?s():[]),o&&t._===1?64:-2);return!r&&a.scopeId&&(a.slotScopeIds=[a.scopeId+"-s"]),i&&i._c&&(i._d=!0),a}function $d(t){return t.some(e=>pr(e)?!(e.type===yt||e.type===Ie&&!$d(e.children)):!0)?t:null}const ca=t=>t?ef(t)?lo(t)||t.proxy:ca(t.parent):null,sr=$e(Object.create(null),{$:t=>t,$el:t=>t.vnode.el,$data:t=>t.data,$props:t=>t.props,$attrs:t=>t.attrs,$slots:t=>t.slots,$refs:t=>t.refs,$parent:t=>ca(t.parent),$root:t=>ca(t.root),$emit:t=>t.emit,$options:t=>sl(t),$forceUpdate:t=>t.f||(t.f=()=>{t.effect.dirty=!0,to(t.update)}),$nextTick:t=>t.n||(t.n=It.bind(t.proxy)),$watch:t=>Eg.bind(t)}),ko=(t,e)=>t!==Re&&!t.__isScriptSetup&&_e(t,e),Pg={get({_:t},e){const{ctx:n,setupState:s,data:r,props:i,accessCache:o,type:a,appContext:l}=t;let u;if(e[0]!=="$"){const h=o[e];if(h!==void 0)switch(h){case 1:return s[e];case 2:return r[e];case 4:return n[e];case 3:return i[e]}else{if(ko(s,e))return o[e]=1,s[e];if(r!==Re&&_e(r,e))return o[e]=2,r[e];if((u=t.propsOptions[0])&&_e(u,e))return o[e]=3,i[e];if(n!==Re&&_e(n,e))return o[e]=4,n[e];da&&(o[e]=0)}}const c=sr[e];let d,f;if(c)return e==="$attrs"&&lt(t,"get",e),c(t);if((d=a.__cssModules)&&(d=d[e]))return d;if(n!==Re&&_e(n,e))return o[e]=4,n[e];if(f=l.config.globalProperties,_e(f,e))return f[e]},set({_:t},e,n){const{data:s,setupState:r,ctx:i}=t;return ko(r,e)?(r[e]=n,!0):s!==Re&&_e(s,e)?(s[e]=n,!0):_e(t.props,e)||e[0]==="$"&&e.slice(1)in t?!1:(i[e]=n,!0)},has({_:{data:t,setupState:e,accessCache:n,ctx:s,appContext:r,propsOptions:i}},o){let a;return!!n[o]||t!==Re&&_e(t,o)||ko(e,o)||(a=i[0])&&_e(a,o)||_e(s,o)||_e(sr,o)||_e(r.config.globalProperties,o)},defineProperty(t,e,n){return n.get!=null?t._.accessCache[e]=0:_e(n,"value")&&this.set(t,e,n.value,null),Reflect.defineProperty(t,e,n)}};function Si(t){return ae(t)?t.reduce((e,n)=>(e[n]=null,e),{}):t}function Zl(t,e){return!t||!e?t||e:ae(t)&&ae(e)?t.concat(e):$e({},Si(t),Si(e))}let da=!0;function Og(t){const e=sl(t),n=t.proxy,s=t.ctx;da=!1,e.beforeCreate&&Kl(e.beforeCreate,t,"bc");const{data:r,computed:i,methods:o,watch:a,provide:l,inject:u,created:c,beforeMount:d,mounted:f,beforeUpdate:h,updated:m,activated:p,deactivated:v,beforeDestroy:S,beforeUnmount:g,destroyed:y,unmounted:b,render:x,renderTracked:_,renderTriggered:C,errorCaptured:M,serverPrefetch:R,expose:F,inheritAttrs:V,components:T,directives:L,filters:A}=e;if(u&&Ng(u,s,null),o)for(const z in o){const X=o[z];he(X)&&(s[z]=X.bind(n))}if(r){const z=r.call(n,n);ke(z)&&(t.data=tt(z))}if(da=!0,i)for(const z in i){const X=i[z],re=he(X)?X.bind(n,n):he(X.get)?X.get.bind(n,n):Ct,te=!he(X)&&he(X.set)?X.set.bind(n):Ct,Q=k({get:re,set:te});Object.defineProperty(s,z,{enumerable:!0,configurable:!0,get:()=>Q.value,set:ce=>Q.value=ce})}if(a)for(const z in a)Ud(a[z],s,n,z);if(l){const z=he(l)?l.call(n):l;Reflect.ownKeys(z).forEach(X=>{ut(X,z[X])})}c&&Kl(c,t,"c");function $(z,X){ae(X)?X.forEach(re=>z(re.bind(n))):X&&z(X.bind(n))}if($(io,d),$(Zt,f),$(Rg,h),$(nl,m),$(Bd,p),$(Dd,v),$(Lg,M),$(Fg,_),$(kg,C),$(kt,g),$(Hd,b),$(Ig,R),ae(F))if(F.length){const z=t.exposed||(t.exposed={});F.forEach(X=>{Object.defineProperty(z,X,{get:()=>n[X],set:re=>n[X]=re})})}else t.exposed||(t.exposed={});x&&t.render===Ct&&(t.render=x),V!=null&&(t.inheritAttrs=V),T&&(t.components=T),L&&(t.directives=L)}function Ng(t,e,n=Ct){ae(t)&&(t=fa(t));for(const s in t){const r=t[s];let i;ke(r)?"default"in r?i=Ne(r.from||s,r.default,!0):i=Ne(r.from||s):i=Ne(r),Me(i)?Object.defineProperty(e,s,{enumerable:!0,configurable:!0,get:()=>i.value,set:o=>i.value=o}):e[s]=i}}function Kl(t,e,n){Mt(ae(t)?t.map(s=>s.bind(e.proxy)):t.bind(e.proxy),e,n)}function Ud(t,e,n,s){const r=s.includes(".")?Vd(n,s):()=>n[s];if(De(t)){const i=e[t];he(i)&&le(r,i)}else if(he(t))le(r,t.bind(n));else if(ke(t))if(ae(t))t.forEach(i=>Ud(i,e,n,s));else{const i=he(t.handler)?t.handler.bind(n):e[t.handler];he(i)&&le(r,i,t)}}function sl(t){const e=t.type,{mixins:n,extends:s}=e,{mixins:r,optionsCache:i,config:{optionMergeStrategies:o}}=t.appContext,a=i.get(e);let l;return a?l=a:!r.length&&!n&&!s?l=e:(l={},r.length&&r.forEach(u=>Ci(l,u,o,!0)),Ci(l,e,o)),ke(e)&&i.set(e,l),l}function Ci(t,e,n,s=!1){const{mixins:r,extends:i}=e;i&&Ci(t,i,n,!0),r&&r.forEach(o=>Ci(t,o,n,!0));for(const o in e)if(!(s&&o==="expose")){const a=Bg[o]||n&&n[o];t[o]=a?a(t[o],e[o]):e[o]}return t}const Bg={data:Jl,props:Ql,emits:Ql,methods:Xs,computed:Xs,beforeCreate:et,created:et,beforeMount:et,mounted:et,beforeUpdate:et,updated:et,beforeDestroy:et,beforeUnmount:et,destroyed:et,unmounted:et,activated:et,deactivated:et,errorCaptured:et,serverPrefetch:et,components:Xs,directives:Xs,watch:zg,provide:Jl,inject:Dg};function Jl(t,e){return e?t?function(){return $e(he(t)?t.call(this,this):t,he(e)?e.call(this,this):e)}:e:t}function Dg(t,e){return Xs(fa(t),fa(e))}function fa(t){if(ae(t)){const e={};for(let n=0;n<t.length;n++)e[t[n]]=t[n];return e}return t}function et(t,e){return t?[...new Set([].concat(t,e))]:e}function Xs(t,e){return t?$e(Object.create(null),t,e):e}function Ql(t,e){return t?ae(t)&&ae(e)?[...new Set([...t,...e])]:$e(Object.create(null),Si(t),Si(e??{})):e}function zg(t,e){if(!t)return e;if(!e)return t;const n=$e(Object.create(null),t);for(const s in e)n[s]=et(t[s],e[s]);return n}function jd(){return{app:null,config:{isNativeTag:ym,performance:!1,globalProperties:{},optionMergeStrategies:{},errorHandler:void 0,warnHandler:void 0,compilerOptions:{}},mixins:[],components:{},directives:{},provides:Object.create(null),optionsCache:new WeakMap,propsCache:new WeakMap,emitsCache:new WeakMap}}let Hg=0;function $g(t,e){return function(s,r=null){he(s)||(s=$e({},s)),r!=null&&!ke(r)&&(r=null);const i=jd(),o=new WeakSet;let a=!1;const l=i.app={_uid:Hg++,_component:s,_props:r,_container:null,_context:i,_instance:null,version:mv,get config(){return i.config},set config(u){},use(u,...c){return o.has(u)||(u&&he(u.install)?(o.add(u),u.install(l,...c)):he(u)&&(o.add(u),u(l,...c))),l},mixin(u){return i.mixins.includes(u)||i.mixins.push(u),l},component(u,c){return c?(i.components[u]=c,l):i.components[u]},directive(u,c){return c?(i.directives[u]=c,l):i.directives[u]},mount(u,c,d){if(!a){const f=w(s,r);return f.appContext=i,d===!0?d="svg":d===!1&&(d=void 0),c&&e?e(f,u):t(f,u,d),a=!0,l._container=u,u.__vue_app__=l,lo(f.component)||f.component.proxy}},unmount(){a&&(t(null,l._container),delete l._container.__vue_app__)},provide(u,c){return i.provides[u]=c,l},runWithContext(u){const c=rr;rr=l;try{return u()}finally{rr=c}}};return l}}let rr=null;function ut(t,e){if(Ge){let n=Ge.provides;const s=Ge.parent&&Ge.parent.provides;s===n&&(n=Ge.provides=Object.create(s)),n[t]=e}}function Ne(t,e,n=!1){const s=Ge||Ue;if(s||rr){const r=s?s.parent==null?s.vnode.appContext&&s.vnode.appContext.provides:s.parent.provides:rr._context.provides;if(r&&t in r)return r[t];if(arguments.length>1)return n&&he(e)?e.call(s&&s.proxy):e}}function Ug(t,e,n,s=!1){const r={},i={};bi(i,oo,1),t.propsDefaults=Object.create(null),Wd(t,e,r,i);for(const o in t.propsOptions[0])o in r||(r[o]=void 0);n?t.props=s?r:Qm(r):t.type.props?t.props=r:t.props=i,t.attrs=i}function jg(t,e,n,s){const{props:r,attrs:i,vnode:{patchFlag:o}}=t,a=pe(r),[l]=t.propsOptions;let u=!1;if((s||o>0)&&!(o&16)){if(o&8){const c=t.vnode.dynamicProps;for(let d=0;d<c.length;d++){let f=c[d];if(no(t.emitsOptions,f))continue;const h=e[f];if(l)if(_e(i,f))h!==i[f]&&(i[f]=h,u=!0);else{const m=xt(f);r[m]=ha(l,a,m,h,t,!1)}else h!==i[f]&&(i[f]=h,u=!0)}}}else{Wd(t,e,r,i)&&(u=!0);let c;for(const d in a)(!e||!_e(e,d)&&((c=is(d))===d||!_e(e,c)))&&(l?n&&(n[d]!==void 0||n[c]!==void 0)&&(r[d]=ha(l,a,d,void 0,t,!0)):delete r[d]);if(i!==a)for(const d in i)(!e||!_e(e,d))&&(delete i[d],u=!0)}u&&on(t,"set","$attrs")}function Wd(t,e,n,s){const[r,i]=t.propsOptions;let o=!1,a;if(e)for(let l in e){if(tr(l))continue;const u=e[l];let c;r&&_e(r,c=xt(l))?!i||!i.includes(c)?n[c]=u:(a||(a={}))[c]=u:no(t.emitsOptions,l)||(!(l in s)||u!==s[l])&&(s[l]=u,o=!0)}if(i){const l=pe(n),u=a||Re;for(let c=0;c<i.length;c++){const d=i[c];n[d]=ha(r,l,d,u[d],t,!_e(u,d))}}return o}function ha(t,e,n,s,r,i){const o=t[n];if(o!=null){const a=_e(o,"default");if(a&&s===void 0){const l=o.default;if(o.type!==Function&&!o.skipFactory&&he(l)){const{propsDefaults:u}=r;if(n in u)s=u[n];else{const c=Rr(r);s=u[n]=l.call(null,e),c()}}else s=l}o[0]&&(i&&!a?s=!1:o[1]&&(s===""||s===is(n))&&(s=!0))}return s}function Gd(t,e,n=!1){const s=e.propsCache,r=s.get(t);if(r)return r;const i=t.props,o={},a=[];let l=!1;if(!he(t)){const c=d=>{l=!0;const[f,h]=Gd(d,e,!0);$e(o,f),h&&a.push(...h)};!n&&e.mixins.length&&e.mixins.forEach(c),t.extends&&c(t.extends),t.mixins&&t.mixins.forEach(c)}if(!i&&!l)return ke(t)&&s.set(t,Ts),Ts;if(ae(i))for(let c=0;c<i.length;c++){const d=xt(i[c]);eu(d)&&(o[d]=Re)}else if(i)for(const c in i){const d=xt(c);if(eu(d)){const f=i[c],h=o[d]=ae(f)||he(f)?{type:f}:$e({},f);if(h){const m=su(Boolean,h.type),p=su(String,h.type);h[0]=m>-1,h[1]=p<0||m<p,(m>-1||_e(h,"default"))&&a.push(d)}}}const u=[o,a];return ke(t)&&s.set(t,u),u}function eu(t){return t[0]!=="$"&&!tr(t)}function tu(t){return t===null?"null":typeof t=="function"?t.name||"":typeof t=="object"&&t.constructor&&t.constructor.name||""}function nu(t,e){return tu(t)===tu(e)}function su(t,e){return ae(e)?e.findIndex(n=>nu(n,t)):he(e)&&nu(e,t)?0:-1}const qd=t=>t[0]==="_"||t==="$stable",rl=t=>ae(t)?t.map(Ht):[Ht(t)],Wg=(t,e,n)=>{if(e._n)return e;const s=q((...r)=>rl(e(...r)),n);return s._c=!1,s},Yd=(t,e,n)=>{const s=t._ctx;for(const r in t){if(qd(r))continue;const i=t[r];if(he(i))e[r]=Wg(r,i,s);else if(i!=null){const o=rl(i);e[r]=()=>o}}},Xd=(t,e)=>{const n=rl(e);t.slots.default=()=>n},Gg=(t,e)=>{if(t.vnode.shapeFlag&32){const n=e._;n?(t.slots=pe(e),bi(e,"_",n)):Yd(e,t.slots={})}else t.slots={},e&&Xd(t,e);bi(t.slots,oo,1)},qg=(t,e,n)=>{const{vnode:s,slots:r}=t;let i=!0,o=Re;if(s.shapeFlag&32){const a=e._;a?n&&a===1?i=!1:($e(r,e),!n&&a===1&&delete r._):(i=!e.$stable,Yd(e,r)),o=e}else e&&(Xd(t,e),o={default:1});if(i)for(const a in r)!qd(a)&&o[a]==null&&delete r[a]};function ma(t,e,n,s,r=!1){if(ae(t)){t.forEach((f,h)=>ma(f,e&&(ae(e)?e[h]:e),n,s,r));return}if(nr(s)&&!r)return;const i=s.shapeFlag&4?lo(s.component)||s.component.proxy:s.el,o=r?null:i,{i:a,r:l}=t,u=e&&e.r,c=a.refs===Re?a.refs={}:a.refs,d=a.setupState;if(u!=null&&u!==l&&(De(u)?(c[u]=null,_e(d,u)&&(d[u]=null)):Me(u)&&(u.value=null)),he(l))kn(l,a,12,[o,c]);else{const f=De(l),h=Me(l);if(f||h){const m=()=>{if(t.f){const p=f?_e(d,l)?d[l]:c[l]:l.value;r?ae(p)&&$a(p,i):ae(p)?p.includes(i)||p.push(i):f?(c[l]=[i],_e(d,l)&&(d[l]=c[l])):(l.value=[i],t.k&&(c[t.k]=l.value))}else f?(c[l]=o,_e(d,l)&&(d[l]=o)):h&&(l.value=o,t.k&&(c[t.k]=o))};o?(m.id=-1,it(m,n)):m()}}}const it=wg;function Yg(t){return Xg(t)}function Xg(t,e){const n=rd();n.__VUE__=!0;const{insert:s,remove:r,patchProp:i,createElement:o,createText:a,createComment:l,setText:u,setElementText:c,parentNode:d,nextSibling:f,setScopeId:h=Ct,insertStaticContent:m}=t,p=(E,I,O,B=null,D=null,j=null,G=void 0,U=null,W=!!I.dynamicChildren)=>{if(E===I)return;E&&!Yn(E,I)&&(B=xe(E),ce(E,D,j,!0),E=null),I.patchFlag===-2&&(W=!1,I.dynamicChildren=null);const{type:H,ref:K,shapeFlag:ie}=I;switch(H){case Tr:v(E,I,O,B);break;case yt:S(E,I,O,B);break;case Lo:E==null&&g(I,O,B,G);break;case Ie:T(E,I,O,B,D,j,G,U,W);break;default:ie&1?x(E,I,O,B,D,j,G,U,W):ie&6?L(E,I,O,B,D,j,G,U,W):(ie&64||ie&128)&&H.process(E,I,O,B,D,j,G,U,W,_t)}K!=null&&D&&ma(K,E&&E.ref,j,I||E,!I)},v=(E,I,O,B)=>{if(E==null)s(I.el=a(I.children),O,B);else{const D=I.el=E.el;I.children!==E.children&&u(D,I.children)}},S=(E,I,O,B)=>{E==null?s(I.el=l(I.children||""),O,B):I.el=E.el},g=(E,I,O,B)=>{[E.el,E.anchor]=m(E.children,I,O,B,E.el,E.anchor)},y=({el:E,anchor:I},O,B)=>{let D;for(;E&&E!==I;)D=f(E),s(E,O,B),E=D;s(I,O,B)},b=({el:E,anchor:I})=>{let O;for(;E&&E!==I;)O=f(E),r(E),E=O;r(I)},x=(E,I,O,B,D,j,G,U,W)=>{I.type==="svg"?G="svg":I.type==="math"&&(G="mathml"),E==null?_(I,O,B,D,j,G,U,W):R(E,I,D,j,G,U,W)},_=(E,I,O,B,D,j,G,U)=>{let W,H;const{props:K,shapeFlag:ie,transition:ne,dirs:de}=E;if(W=E.el=o(E.type,j,K&&K.is,K),ie&8?c(W,E.children):ie&16&&M(E.children,W,null,B,D,Fo(E,j),G,U),de&&zn(E,null,B,"created"),C(W,E,E.scopeId,G,B),K){for(const Ee in K)Ee!=="value"&&!tr(Ee)&&i(W,Ee,null,K[Ee],j,E.children,B,D,Z);"value"in K&&i(W,"value",null,K.value,j),(H=K.onVnodeBeforeMount)&&zt(H,B,E)}de&&zn(E,null,B,"beforeMount");const ve=Zg(D,ne);ve&&ne.beforeEnter(W),s(W,I,O),((H=K&&K.onVnodeMounted)||ve||de)&&it(()=>{H&&zt(H,B,E),ve&&ne.enter(W),de&&zn(E,null,B,"mounted")},D)},C=(E,I,O,B,D)=>{if(O&&h(E,O),B)for(let j=0;j<B.length;j++)h(E,B[j]);if(D){let j=D.subTree;if(I===j){const G=D.vnode;C(E,G,G.scopeId,G.slotScopeIds,D.parent)}}},M=(E,I,O,B,D,j,G,U,W=0)=>{for(let H=W;H<E.length;H++){const K=E[H]=U?Tn(E[H]):Ht(E[H]);p(null,K,I,O,B,D,j,G,U)}},R=(E,I,O,B,D,j,G)=>{const U=I.el=E.el;let{patchFlag:W,dynamicChildren:H,dirs:K}=I;W|=E.patchFlag&16;const ie=E.props||Re,ne=I.props||Re;let de;if(O&&Hn(O,!1),(de=ne.onVnodeBeforeUpdate)&&zt(de,O,I,E),K&&zn(I,E,O,"beforeUpdate"),O&&Hn(O,!0),H?F(E.dynamicChildren,H,U,O,B,Fo(I,D),j):G||X(E,I,U,null,O,B,Fo(I,D),j,!1),W>0){if(W&16)V(U,I,ie,ne,O,B,D);else if(W&2&&ie.class!==ne.class&&i(U,"class",null,ne.class,D),W&4&&i(U,"style",ie.style,ne.style,D),W&8){const ve=I.dynamicProps;for(let Ee=0;Ee<ve.length;Ee++){const Ve=ve[Ee],je=ie[Ve],Lt=ne[Ve];(Lt!==je||Ve==="value")&&i(U,Ve,je,Lt,D,E.children,O,B,Z)}}W&1&&E.children!==I.children&&c(U,I.children)}else!G&&H==null&&V(U,I,ie,ne,O,B,D);((de=ne.onVnodeUpdated)||K)&&it(()=>{de&&zt(de,O,I,E),K&&zn(I,E,O,"updated")},B)},F=(E,I,O,B,D,j,G)=>{for(let U=0;U<I.length;U++){const W=E[U],H=I[U],K=W.el&&(W.type===Ie||!Yn(W,H)||W.shapeFlag&70)?d(W.el):O;p(W,H,K,null,B,D,j,G,!0)}},V=(E,I,O,B,D,j,G)=>{if(O!==B){if(O!==Re)for(const U in O)!tr(U)&&!(U in B)&&i(E,U,O[U],null,G,I.children,D,j,Z);for(const U in B){if(tr(U))continue;const W=B[U],H=O[U];W!==H&&U!=="value"&&i(E,U,H,W,G,I.children,D,j,Z)}"value"in B&&i(E,"value",O.value,B.value,G)}},T=(E,I,O,B,D,j,G,U,W)=>{const H=I.el=E?E.el:a(""),K=I.anchor=E?E.anchor:a("");let{patchFlag:ie,dynamicChildren:ne,slotScopeIds:de}=I;de&&(U=U?U.concat(de):de),E==null?(s(H,O,B),s(K,O,B),M(I.children||[],O,K,D,j,G,U,W)):ie>0&&ie&64&&ne&&E.dynamicChildren?(F(E.dynamicChildren,ne,O,D,j,G,U),(I.key!=null||D&&I===D.subTree)&&il(E,I,!0)):X(E,I,O,K,D,j,G,U,W)},L=(E,I,O,B,D,j,G,U,W)=>{I.slotScopeIds=U,E==null?I.shapeFlag&512?D.ctx.activate(I,O,B,G,W):A(I,O,B,D,j,G,W):N(E,I,W)},A=(E,I,O,B,D,j,G)=>{const U=E.component=av(E,B,D);if(Mr(E)&&(U.ctx.renderer=_t),lv(U),U.asyncDep){if(D&&D.registerDep(U,$),!E.el){const W=U.subTree=w(yt);S(null,W,I,O)}}else $(U,E,I,O,D,j,G)},N=(E,I,O)=>{const B=I.component=E.component;if(vg(E,I,O))if(B.asyncDep&&!B.asyncResolved){z(B,I,O);return}else B.next=I,cg(B.update),B.effect.dirty=!0,B.update();else I.el=E.el,B.vnode=I},$=(E,I,O,B,D,j,G)=>{const U=()=>{if(E.isMounted){let{next:K,bu:ie,u:ne,parent:de,vnode:ve}=E;{const ms=Zd(E);if(ms){K&&(K.el=ve.el,z(E,K,G)),ms.asyncDep.then(()=>{E.isUnmounted||U()});return}}let Ee=K,Ve;Hn(E,!1),K?(K.el=ve.el,z(E,K,G)):K=ve,ie&&Mo(ie),(Ve=K.props&&K.props.onVnodeBeforeUpdate)&&zt(Ve,de,K,ve),Hn(E,!0);const je=Ao(E),Lt=E.subTree;E.subTree=je,p(Lt,je,d(Lt.el),xe(Lt),E,D,j),K.el=je.el,Ee===null&&pg(E,je.el),ne&&it(ne,D),(Ve=K.props&&K.props.onVnodeUpdated)&&it(()=>zt(Ve,de,K,ve),D)}else{let K;const{el:ie,props:ne}=I,{bm:de,m:ve,parent:Ee}=E,Ve=nr(I);if(Hn(E,!1),de&&Mo(de),!Ve&&(K=ne&&ne.onVnodeBeforeMount)&&zt(K,Ee,I),Hn(E,!0),ie&&Dn){const je=()=>{E.subTree=Ao(E),Dn(ie,E.subTree,E,D,null)};Ve?I.type.__asyncLoader().then(()=>!E.isUnmounted&&je()):je()}else{const je=E.subTree=Ao(E);p(null,je,O,B,E,D,j),I.el=je.el}if(ve&&it(ve,D),!Ve&&(K=ne&&ne.onVnodeMounted)){const je=I;it(()=>zt(K,Ee,je),D)}(I.shapeFlag&256||Ee&&nr(Ee.vnode)&&Ee.vnode.shapeFlag&256)&&E.a&&it(E.a,D),E.isMounted=!0,I=O=B=null}},W=E.effect=new Wa(U,Ct,()=>to(H),E.scope),H=E.update=()=>{W.dirty&&W.run()};H.id=E.uid,Hn(E,!0),H()},z=(E,I,O)=>{I.component=E;const B=E.vnode.props;E.vnode=I,E.next=null,jg(E,I.props,B,O),qg(E,I.children,O),os(),Gl(E),as()},X=(E,I,O,B,D,j,G,U,W=!1)=>{const H=E&&E.children,K=E?E.shapeFlag:0,ie=I.children,{patchFlag:ne,shapeFlag:de}=I;if(ne>0){if(ne&128){te(H,ie,O,B,D,j,G,U,W);return}else if(ne&256){re(H,ie,O,B,D,j,G,U,W);return}}de&8?(K&16&&Z(H,D,j),ie!==H&&c(O,ie)):K&16?de&16?te(H,ie,O,B,D,j,G,U,W):Z(H,D,j,!0):(K&8&&c(O,""),de&16&&M(ie,O,B,D,j,G,U,W))},re=(E,I,O,B,D,j,G,U,W)=>{E=E||Ts,I=I||Ts;const H=E.length,K=I.length,ie=Math.min(H,K);let ne;for(ne=0;ne<ie;ne++){const de=I[ne]=W?Tn(I[ne]):Ht(I[ne]);p(E[ne],de,O,null,D,j,G,U,W)}H>K?Z(E,D,j,!0,!1,ie):M(I,O,B,D,j,G,U,W,ie)},te=(E,I,O,B,D,j,G,U,W)=>{let H=0;const K=I.length;let ie=E.length-1,ne=K-1;for(;H<=ie&&H<=ne;){const de=E[H],ve=I[H]=W?Tn(I[H]):Ht(I[H]);if(Yn(de,ve))p(de,ve,O,null,D,j,G,U,W);else break;H++}for(;H<=ie&&H<=ne;){const de=E[ie],ve=I[ne]=W?Tn(I[ne]):Ht(I[ne]);if(Yn(de,ve))p(de,ve,O,null,D,j,G,U,W);else break;ie--,ne--}if(H>ie){if(H<=ne){const de=ne+1,ve=de<K?I[de].el:B;for(;H<=ne;)p(null,I[H]=W?Tn(I[H]):Ht(I[H]),O,ve,D,j,G,U,W),H++}}else if(H>ne)for(;H<=ie;)ce(E[H],D,j,!0),H++;else{const de=H,ve=H,Ee=new Map;for(H=ve;H<=ne;H++){const ft=I[H]=W?Tn(I[H]):Ht(I[H]);ft.key!=null&&Ee.set(ft.key,H)}let Ve,je=0;const Lt=ne-ve+1;let ms=!1,Pl=0;const zs=new Array(Lt);for(H=0;H<Lt;H++)zs[H]=0;for(H=de;H<=ie;H++){const ft=E[H];if(je>=Lt){ce(ft,D,j,!0);continue}let Bt;if(ft.key!=null)Bt=Ee.get(ft.key);else for(Ve=ve;Ve<=ne;Ve++)if(zs[Ve-ve]===0&&Yn(ft,I[Ve])){Bt=Ve;break}Bt===void 0?ce(ft,D,j,!0):(zs[Bt-ve]=H+1,Bt>=Pl?Pl=Bt:ms=!0,p(ft,I[Bt],O,null,D,j,G,U,W),je++)}const Ol=ms?Kg(zs):Ts;for(Ve=Ol.length-1,H=Lt-1;H>=0;H--){const ft=ve+H,Bt=I[ft],Nl=ft+1<K?I[ft+1].el:B;zs[H]===0?p(null,Bt,O,Nl,D,j,G,U,W):ms&&(Ve<0||H!==Ol[Ve]?Q(Bt,O,Nl,2):Ve--)}}},Q=(E,I,O,B,D=null)=>{const{el:j,type:G,transition:U,children:W,shapeFlag:H}=E;if(H&6){Q(E.component.subTree,I,O,B);return}if(H&128){E.suspense.move(I,O,B);return}if(H&64){G.move(E,I,O,_t);return}if(G===Ie){s(j,I,O);for(let ie=0;ie<W.length;ie++)Q(W[ie],I,O,B);s(E.anchor,I,O);return}if(G===Lo){y(E,I,O);return}if(B!==2&&H&1&&U)if(B===0)U.beforeEnter(j),s(j,I,O),it(()=>U.enter(j),D);else{const{leave:ie,delayLeave:ne,afterLeave:de}=U,ve=()=>s(j,I,O),Ee=()=>{ie(j,()=>{ve(),de&&de()})};ne?ne(j,ve,Ee):Ee()}else s(j,I,O)},ce=(E,I,O,B=!1,D=!1)=>{const{type:j,props:G,ref:U,children:W,dynamicChildren:H,shapeFlag:K,patchFlag:ie,dirs:ne}=E;if(U!=null&&ma(U,null,O,E,!0),K&256){I.ctx.deactivate(E);return}const de=K&1&&ne,ve=!nr(E);let Ee;if(ve&&(Ee=G&&G.onVnodeBeforeUnmount)&&zt(Ee,I,E),K&6)Le(E.component,O,B);else{if(K&128){E.suspense.unmount(O,B);return}de&&zn(E,null,I,"beforeUnmount"),K&64?E.type.remove(E,I,O,D,_t,B):H&&(j!==Ie||ie>0&&ie&64)?Z(H,I,O,!1,!0):(j===Ie&&ie&384||!D&&K&16)&&Z(W,I,O),B&&Te(E)}(ve&&(Ee=G&&G.onVnodeUnmounted)||de)&&it(()=>{Ee&&zt(Ee,I,E),de&&zn(E,null,I,"unmounted")},O)},Te=E=>{const{type:I,el:O,anchor:B,transition:D}=E;if(I===Ie){ge(O,B);return}if(I===Lo){b(E);return}const j=()=>{r(O),D&&!D.persisted&&D.afterLeave&&D.afterLeave()};if(E.shapeFlag&1&&D&&!D.persisted){const{leave:G,delayLeave:U}=D,W=()=>G(O,j);U?U(E.el,j,W):W()}else j()},ge=(E,I)=>{let O;for(;E!==I;)O=f(E),r(E),E=O;r(I)},Le=(E,I,O)=>{const{bum:B,scope:D,update:j,subTree:G,um:U}=E;B&&Mo(B),D.stop(),j&&(j.active=!1,ce(G,E,I,O)),U&&it(U,I),it(()=>{E.isUnmounted=!0},I),I&&I.pendingBranch&&!I.isUnmounted&&E.asyncDep&&!E.asyncResolved&&E.suspenseId===I.pendingId&&(I.deps--,I.deps===0&&I.resolve())},Z=(E,I,O,B=!1,D=!1,j=0)=>{for(let G=j;G<E.length;G++)ce(E[G],I,O,B,D)},xe=E=>E.shapeFlag&6?xe(E.component.subTree):E.shapeFlag&128?E.suspense.next():f(E.anchor||E.el);let vn=!1;const Fe=(E,I,O)=>{E==null?I._vnode&&ce(I._vnode,null,null,!0):p(I._vnode||null,E,I,null,null,null,O),vn||(vn=!0,Gl(),Td(),vn=!1),I._vnode=E},_t={p,um:ce,m:Q,r:Te,mt:A,mc:M,pc:X,pbc:F,n:xe,o:t};let Qe,Dn;return e&&([Qe,Dn]=e(_t)),{render:Fe,hydrate:Qe,createApp:$g(Fe,Qe)}}function Fo({type:t,props:e},n){return n==="svg"&&t==="foreignObject"||n==="mathml"&&t==="annotation-xml"&&e&&e.encoding&&e.encoding.includes("html")?void 0:n}function Hn({effect:t,update:e},n){t.allowRecurse=e.allowRecurse=n}function Zg(t,e){return(!t||t&&!t.pendingBranch)&&e&&!e.persisted}function il(t,e,n=!1){const s=t.children,r=e.children;if(ae(s)&&ae(r))for(let i=0;i<s.length;i++){const o=s[i];let a=r[i];a.shapeFlag&1&&!a.dynamicChildren&&((a.patchFlag<=0||a.patchFlag===32)&&(a=r[i]=Tn(r[i]),a.el=o.el),n||il(o,a)),a.type===Tr&&(a.el=o.el)}}function Kg(t){const e=t.slice(),n=[0];let s,r,i,o,a;const l=t.length;for(s=0;s<l;s++){const u=t[s];if(u!==0){if(r=n[n.length-1],t[r]<u){e[s]=r,n.push(s);continue}for(i=0,o=n.length-1;i<o;)a=i+o>>1,t[n[a]]<u?i=a+1:o=a;u<t[n[i]]&&(i>0&&(e[s]=n[i-1]),n[i]=s)}}for(i=n.length,o=n[i-1];i-- >0;)n[i]=o,o=e[o];return n}function Zd(t){const e=t.subTree.component;if(e)return e.asyncDep&&!e.asyncResolved?e:Zd(e)}const Jg=t=>t.__isTeleport,ir=t=>t&&(t.disabled||t.disabled===""),ru=t=>typeof SVGElement<"u"&&t instanceof SVGElement,iu=t=>typeof MathMLElement=="function"&&t instanceof MathMLElement,ga=(t,e)=>{const n=t&&t.to;return De(n)?e?e(n):null:n},Qg={name:"Teleport",__isTeleport:!0,process(t,e,n,s,r,i,o,a,l,u){const{mc:c,pc:d,pbc:f,o:{insert:h,querySelector:m,createText:p,createComment:v}}=u,S=ir(e.props);let{shapeFlag:g,children:y,dynamicChildren:b}=e;if(t==null){const x=e.el=p(""),_=e.anchor=p("");h(x,n,s),h(_,n,s);const C=e.target=ga(e.props,m),M=e.targetAnchor=p("");C&&(h(M,C),o==="svg"||ru(C)?o="svg":(o==="mathml"||iu(C))&&(o="mathml"));const R=(F,V)=>{g&16&&c(y,F,V,r,i,o,a,l)};S?R(n,_):C&&R(C,M)}else{e.el=t.el;const x=e.anchor=t.anchor,_=e.target=t.target,C=e.targetAnchor=t.targetAnchor,M=ir(t.props),R=M?n:_,F=M?x:C;if(o==="svg"||ru(_)?o="svg":(o==="mathml"||iu(_))&&(o="mathml"),b?(f(t.dynamicChildren,b,R,r,i,o,a),il(t,e,!0)):l||d(t,e,R,F,r,i,o,a,!1),S)M?e.props&&t.props&&e.props.to!==t.props.to&&(e.props.to=t.props.to):jr(e,n,x,u,1);else if((e.props&&e.props.to)!==(t.props&&t.props.to)){const V=e.target=ga(e.props,m);V&&jr(e,V,null,u,0)}else M&&jr(e,_,C,u,1)}Kd(e)},remove(t,e,n,s,{um:r,o:{remove:i}},o){const{shapeFlag:a,children:l,anchor:u,targetAnchor:c,target:d,props:f}=t;if(d&&i(c),o&&i(u),a&16){const h=o||!ir(f);for(let m=0;m<l.length;m++){const p=l[m];r(p,e,n,h,!!p.dynamicChildren)}}},move:jr,hydrate:ev};function jr(t,e,n,{o:{insert:s},m:r},i=2){i===0&&s(t.targetAnchor,e,n);const{el:o,anchor:a,shapeFlag:l,children:u,props:c}=t,d=i===2;if(d&&s(o,e,n),(!d||ir(c))&&l&16)for(let f=0;f<u.length;f++)r(u[f],e,n,2);d&&s(a,e,n)}function ev(t,e,n,s,r,i,{o:{nextSibling:o,parentNode:a,querySelector:l}},u){const c=e.target=ga(e.props,l);if(c){const d=c._lpa||c.firstChild;if(e.shapeFlag&16)if(ir(e.props))e.anchor=u(o(t),e,a(t),n,s,r,i),e.targetAnchor=d;else{e.anchor=o(t);let f=d;for(;f;)if(f=o(f),f&&f.nodeType===8&&f.data==="teleport anchor"){e.targetAnchor=f,c._lpa=e.targetAnchor&&o(e.targetAnchor);break}u(d,e,c,n,s,r,i)}Kd(e)}return e.anchor&&o(e.anchor)}const tv=Qg;function Kd(t){const e=t.ctx;if(e&&e.ut){let n=t.children[0].el;for(;n&&n!==t.targetAnchor;)n.nodeType===1&&n.setAttribute("data-v-owner",e.uid),n=n.nextSibling;e.ut()}}const Ie=Symbol.for("v-fgt"),Tr=Symbol.for("v-txt"),yt=Symbol.for("v-cmt"),Lo=Symbol.for("v-stc"),or=[];let Nt=null;function st(t=!1){or.push(Nt=t?null:[])}function nv(){or.pop(),Nt=or[or.length-1]||null}let vr=1;function ou(t){vr+=t}function Jd(t){return t.dynamicChildren=vr>0?Nt||Ts:null,nv(),vr>0&&Nt&&Nt.push(t),t}function Ar(t,e,n,s,r,i){return Jd(Wt(t,e,n,s,r,i,!0))}function Yt(t,e,n,s,r){return Jd(w(t,e,n,s,r,!0))}function pr(t){return t?t.__v_isVNode===!0:!1}function Yn(t,e){return t.type===e.type&&t.key===e.key}const oo="__vInternal",Qd=({key:t})=>t??null,di=({ref:t,ref_key:e,ref_for:n})=>(typeof t=="number"&&(t=""+t),t!=null?De(t)||Me(t)||he(t)?{i:Ue,r:t,k:e,f:!!n}:t:null);function Wt(t,e=null,n=null,s=0,r=null,i=t===Ie?0:1,o=!1,a=!1){const l={__v_isVNode:!0,__v_skip:!0,type:t,props:e,key:e&&Qd(e),ref:e&&di(e),scopeId:Id,slotScopeIds:null,children:n,component:null,suspense:null,ssContent:null,ssFallback:null,dirs:null,transition:null,el:null,anchor:null,target:null,targetAnchor:null,staticCount:0,shapeFlag:i,patchFlag:s,dynamicProps:r,dynamicChildren:null,appContext:null,ctx:Ue};return a?(ol(l,n),i&128&&t.normalize(l)):n&&(l.shapeFlag|=De(n)?8:16),vr>0&&!o&&Nt&&(l.patchFlag>0||i&6)&&l.patchFlag!==32&&Nt.push(l),l}const w=sv;function sv(t,e=null,n=null,s=0,r=null,i=!1){if((!t||t===Fd)&&(t=yt),pr(t)){const a=Pn(t,e,!0);return n&&ol(a,n),vr>0&&!i&&Nt&&(a.shapeFlag&6?Nt[Nt.indexOf(t)]=a:Nt.push(a)),a.patchFlag|=-2,a}if(hv(t)&&(t=t.__vccOpts),e){e=rv(e);let{class:a,style:l}=e;a&&!De(a)&&(e.class=Qi(a)),ke(l)&&(bd(l)&&!ae(l)&&(l=$e({},l)),e.style=Ji(l))}const o=De(t)?1:xg(t)?128:Jg(t)?64:ke(t)?4:he(t)?2:0;return Wt(t,e,n,s,r,o,i,!0)}function rv(t){return t?bd(t)||oo in t?$e({},t):t:null}function Pn(t,e,n=!1){const{props:s,ref:r,patchFlag:i,children:o}=t,a=e?Pe(s||{},e):s;return{__v_isVNode:!0,__v_skip:!0,type:t.type,props:a,key:a&&Qd(a),ref:e&&e.ref?n&&r?ae(r)?r.concat(di(e)):[r,di(e)]:di(e):r,scopeId:t.scopeId,slotScopeIds:t.slotScopeIds,children:o,target:t.target,targetAnchor:t.targetAnchor,staticCount:t.staticCount,shapeFlag:t.shapeFlag,patchFlag:e&&t.type!==Ie?i===-1?16:i|16:i,dynamicProps:t.dynamicProps,dynamicChildren:t.dynamicChildren,appContext:t.appContext,dirs:t.dirs,transition:t.transition,component:t.component,suspense:t.suspense,ssContent:t.ssContent&&Pn(t.ssContent),ssFallback:t.ssFallback&&Pn(t.ssFallback),el:t.el,anchor:t.anchor,ctx:t.ctx,ce:t.ce}}function Ae(t=" ",e=0){return w(Tr,null,t,e)}function Ei(t="",e=!1){return e?(st(),Yt(yt,null,t)):w(yt,null,t)}function Ht(t){return t==null||typeof t=="boolean"?w(yt):ae(t)?w(Ie,null,t.slice()):typeof t=="object"?Tn(t):w(Tr,null,String(t))}function Tn(t){return t.el===null&&t.patchFlag!==-1||t.memo?t:Pn(t)}function ol(t,e){let n=0;const{shapeFlag:s}=t;if(e==null)e=null;else if(ae(e))n=16;else if(typeof e=="object")if(s&65){const r=e.default;r&&(r._c&&(r._d=!1),ol(t,r()),r._c&&(r._d=!0));return}else{n=32;const r=e._;!r&&!(oo in e)?e._ctx=Ue:r===3&&Ue&&(Ue.slots._===1?e._=1:(e._=2,t.patchFlag|=1024))}else he(e)?(e={default:e,_ctx:Ue},n=32):(e=String(e),s&64?(n=16,e=[Ae(e)]):n=8);t.children=e,t.shapeFlag|=n}function Pe(...t){const e={};for(let n=0;n<t.length;n++){const s=t[n];for(const r in s)if(r==="class")e.class!==s.class&&(e.class=Qi([e.class,s.class]));else if(r==="style")e.style=Ji([e.style,s.style]);else if(Xi(r)){const i=e[r],o=s[r];o&&i!==o&&!(ae(i)&&i.includes(o))&&(e[r]=i?[].concat(i,o):o)}else r!==""&&(e[r]=s[r])}return e}function zt(t,e,n,s=null){Mt(t,e,7,[n,s])}const iv=jd();let ov=0;function av(t,e,n){const s=t.type,r=(e?e.appContext:t.appContext)||iv,i={uid:ov++,vnode:t,type:s,parent:e,appContext:r,root:null,next:null,subTree:null,effect:null,update:null,scope:new ad(!0),render:null,proxy:null,exposed:null,exposeProxy:null,withProxy:null,provides:e?e.provides:Object.create(r.provides),accessCache:null,renderCache:[],components:null,directives:null,propsOptions:Gd(s,r),emitsOptions:Rd(s,r),emit:null,emitted:null,propsDefaults:Re,inheritAttrs:s.inheritAttrs,ctx:Re,data:Re,props:Re,attrs:Re,slots:Re,refs:Re,setupState:Re,setupContext:null,attrsProxy:null,slotsProxy:null,suspense:n,suspenseId:n?n.pendingId:0,asyncDep:null,asyncResolved:!1,isMounted:!1,isUnmounted:!1,isDeactivated:!1,bc:null,c:null,bm:null,m:null,bu:null,u:null,um:null,bum:null,da:null,a:null,rtg:null,rtc:null,ec:null,sp:null};return i.ctx={_:i},i.root=e?e.root:i,i.emit=hg.bind(null,i),t.ce&&t.ce(i),i}let Ge=null;const ao=()=>Ge||Ue;let Mi,va;{const t=rd(),e=(n,s)=>{let r;return(r=t[n])||(r=t[n]=[]),r.push(s),i=>{r.length>1?r.forEach(o=>o(i)):r[0](i)}};Mi=e("__VUE_INSTANCE_SETTERS__",n=>Ge=n),va=e("__VUE_SSR_SETTERS__",n=>Ir=n)}const Rr=t=>{const e=Ge;return Mi(t),t.scope.on(),()=>{t.scope.off(),Mi(e)}},au=()=>{Ge&&Ge.scope.off(),Mi(null)};function ef(t){return t.vnode.shapeFlag&4}let Ir=!1;function lv(t,e=!1){e&&va(e);const{props:n,children:s}=t.vnode,r=ef(t);Ug(t,n,r,e),Gg(t,s);const i=r?uv(t,e):void 0;return e&&va(!1),i}function uv(t,e){const n=t.type;t.accessCache=Object.create(null),t.proxy=xd(new Proxy(t.ctx,Pg));const{setup:s}=n;if(s){const r=t.setupContext=s.length>1?dv(t):null,i=Rr(t);os();const o=kn(s,t,0,[t.props,r]);if(as(),i(),td(o)){if(o.then(au,au),e)return o.then(a=>{lu(t,a,e)}).catch(a=>{Er(a,t,0)});t.asyncDep=o}else lu(t,o,e)}else tf(t,e)}function lu(t,e,n){he(e)?t.type.__ssrInlineRender?t.ssrRender=e:t.render=e:ke(e)&&(t.setupState=Sd(e)),tf(t,n)}let uu;function tf(t,e,n){const s=t.type;if(!t.render){if(!e&&uu&&!s.render){const r=s.template||sl(t).template;if(r){const{isCustomElement:i,compilerOptions:o}=t.appContext.config,{delimiters:a,compilerOptions:l}=s,u=$e($e({isCustomElement:i,delimiters:a},o),l);s.render=uu(r,u)}}t.render=s.render||Ct}{const r=Rr(t);os();try{Og(t)}finally{as(),r()}}}function cv(t){return t.attrsProxy||(t.attrsProxy=new Proxy(t.attrs,{get(e,n){return lt(t,"get","$attrs"),e[n]}}))}function dv(t){const e=n=>{t.exposed=n||{}};return{get attrs(){return cv(t)},slots:t.slots,emit:t.emit,expose:e}}function lo(t){if(t.exposed)return t.exposeProxy||(t.exposeProxy=new Proxy(Sd(xd(t.exposed)),{get(e,n){if(n in e)return e[n];if(n in sr)return sr[n](t)},has(e,n){return n in e||n in sr}}))}function fv(t,e=!0){return he(t)?t.displayName||t.name:t.name||e&&t.__name}function hv(t){return he(t)&&"__vccOpts"in t}const k=(t,e)=>eg(t,e,Ir);function cu(t,e,n=Re){const s=ao(),r=xt(e),i=is(e),o=ig((l,u)=>{let c;return Cg(()=>{const d=t[e];Gt(c,d)&&(c=d,u())}),{get(){return l(),n.get?n.get(c):c},set(d){const f=s.vnode.props;!(f&&(e in f||r in f||i in f)&&(`onUpdate:${e}`in f||`onUpdate:${r}`in f||`onUpdate:${i}`in f))&&Gt(d,c)&&(c=d,u()),s.emit(`update:${e}`,n.set?n.set(d):d)}}}),a=e==="modelValue"?"modelModifiers":`${e}Modifiers`;return o[Symbol.iterator]=()=>{let l=0;return{next(){return l<2?{value:l++?t[a]||{}:o,done:!1}:{done:!0}}}},o}function Bn(t,e,n){const s=arguments.length;return s===2?ke(e)&&!ae(e)?pr(e)?w(t,null,[e]):w(t,e):w(t,null,e):(s>3?n=Array.prototype.slice.call(arguments,2):s===3&&pr(n)&&(n=[n]),w(t,e,n))}const mv="3.4.21";/**
* @vue/runtime-dom v3.4.21
* (c) 2018-present Yuxi (Evan) You and Vue contributors
* @license MIT
**/const gv="http://www.w3.org/2000/svg",vv="http://www.w3.org/1998/Math/MathML",An=typeof document<"u"?document:null,du=An&&An.createElement("template"),pv={insert:(t,e,n)=>{e.insertBefore(t,n||null)},remove:t=>{const e=t.parentNode;e&&e.removeChild(t)},createElement:(t,e,n,s)=>{const r=e==="svg"?An.createElementNS(gv,t):e==="mathml"?An.createElementNS(vv,t):An.createElement(t,n?{is:n}:void 0);return t==="select"&&s&&s.multiple!=null&&r.setAttribute("multiple",s.multiple),r},createText:t=>An.createTextNode(t),createComment:t=>An.createComment(t),setText:(t,e)=>{t.nodeValue=e},setElementText:(t,e)=>{t.textContent=e},parentNode:t=>t.parentNode,nextSibling:t=>t.nextSibling,querySelector:t=>An.querySelector(t),setScopeId(t,e){t.setAttribute(e,"")},insertStaticContent(t,e,n,s,r,i){const o=n?n.previousSibling:e.lastChild;if(r&&(r===i||r.nextSibling))for(;e.insertBefore(r.cloneNode(!0),n),!(r===i||!(r=r.nextSibling)););else{du.innerHTML=s==="svg"?`<svg>${t}</svg>`:s==="mathml"?`<math>${t}</math>`:t;const a=du.content;if(s==="svg"||s==="mathml"){const l=a.firstChild;for(;l.firstChild;)a.appendChild(l.firstChild);a.removeChild(l)}e.insertBefore(a,n)}return[o?o.nextSibling:e.firstChild,n?n.previousSibling:e.lastChild]}},yn="transition",Hs="animation",Ls=Symbol("_vtc"),Xt=(t,{slots:e})=>Bn(Tg,sf(t),e);Xt.displayName="Transition";const nf={name:String,type:String,css:{type:Boolean,default:!0},duration:[String,Number,Object],enterFromClass:String,enterActiveClass:String,enterToClass:String,appearFromClass:String,appearActiveClass:String,appearToClass:String,leaveFromClass:String,leaveActiveClass:String,leaveToClass:String},yv=Xt.props=$e({},Od,nf),$n=(t,e=[])=>{ae(t)?t.forEach(n=>n(...e)):t&&t(...e)},fu=t=>t?ae(t)?t.some(e=>e.length>1):t.length>1:!1;function sf(t){const e={};for(const T in t)T in nf||(e[T]=t[T]);if(t.css===!1)return e;const{name:n="v",type:s,duration:r,enterFromClass:i=`${n}-enter-from`,enterActiveClass:o=`${n}-enter-active`,enterToClass:a=`${n}-enter-to`,appearFromClass:l=i,appearActiveClass:u=o,appearToClass:c=a,leaveFromClass:d=`${n}-leave-from`,leaveActiveClass:f=`${n}-leave-active`,leaveToClass:h=`${n}-leave-to`}=t,m=bv(r),p=m&&m[0],v=m&&m[1],{onBeforeEnter:S,onEnter:g,onEnterCancelled:y,onLeave:b,onLeaveCancelled:x,onBeforeAppear:_=S,onAppear:C=g,onAppearCancelled:M=y}=e,R=(T,L,A)=>{Sn(T,L?c:a),Sn(T,L?u:o),A&&A()},F=(T,L)=>{T._isLeaving=!1,Sn(T,d),Sn(T,h),Sn(T,f),L&&L()},V=T=>(L,A)=>{const N=T?C:g,$=()=>R(L,T,A);$n(N,[L,$]),hu(()=>{Sn(L,T?l:i),sn(L,T?c:a),fu(N)||mu(L,s,p,$)})};return $e(e,{onBeforeEnter(T){$n(S,[T]),sn(T,i),sn(T,o)},onBeforeAppear(T){$n(_,[T]),sn(T,l),sn(T,u)},onEnter:V(!1),onAppear:V(!0),onLeave(T,L){T._isLeaving=!0;const A=()=>F(T,L);sn(T,d),of(),sn(T,f),hu(()=>{T._isLeaving&&(Sn(T,d),sn(T,h),fu(b)||mu(T,s,v,A))}),$n(b,[T,A])},onEnterCancelled(T){R(T,!1),$n(y,[T])},onAppearCancelled(T){R(T,!0),$n(M,[T])},onLeaveCancelled(T){F(T),$n(x,[T])}})}function bv(t){if(t==null)return null;if(ke(t))return[Vo(t.enter),Vo(t.leave)];{const e=Vo(t);return[e,e]}}function Vo(t){return Cm(t)}function sn(t,e){e.split(/\s+/).forEach(n=>n&&t.classList.add(n)),(t[Ls]||(t[Ls]=new Set)).add(e)}function Sn(t,e){e.split(/\s+/).forEach(s=>s&&t.classList.remove(s));const n=t[Ls];n&&(n.delete(e),n.size||(t[Ls]=void 0))}function hu(t){requestAnimationFrame(()=>{requestAnimationFrame(t)})}let xv=0;function mu(t,e,n,s){const r=t._endId=++xv,i=()=>{r===t._endId&&s()};if(n)return setTimeout(i,n);const{type:o,timeout:a,propCount:l}=rf(t,e);if(!o)return s();const u=o+"end";let c=0;const d=()=>{t.removeEventListener(u,f),i()},f=h=>{h.target===t&&++c>=l&&d()};setTimeout(()=>{c<l&&d()},a+1),t.addEventListener(u,f)}function rf(t,e){const n=window.getComputedStyle(t),s=m=>(n[m]||"").split(", "),r=s(`${yn}Delay`),i=s(`${yn}Duration`),o=gu(r,i),a=s(`${Hs}Delay`),l=s(`${Hs}Duration`),u=gu(a,l);let c=null,d=0,f=0;e===yn?o>0&&(c=yn,d=o,f=i.length):e===Hs?u>0&&(c=Hs,d=u,f=l.length):(d=Math.max(o,u),c=d>0?o>u?yn:Hs:null,f=c?c===yn?i.length:l.length:0);const h=c===yn&&/\b(transform|all)(,|$)/.test(s(`${yn}Property`).toString());return{type:c,timeout:d,propCount:f,hasTransform:h}}function gu(t,e){for(;t.length<e.length;)t=t.concat(t);return Math.max(...e.map((n,s)=>vu(n)+vu(t[s])))}function vu(t){return t==="auto"?0:Number(t.slice(0,-1).replace(",","."))*1e3}function of(){return document.body.offsetHeight}function wv(t,e,n){const s=t[Ls];s&&(e=(e?[e,...s]:[...s]).join(" ")),e==null?t.removeAttribute("class"):n?t.setAttribute("class",e):t.className=e}const Ti=Symbol("_vod"),af=Symbol("_vsh"),uo={beforeMount(t,{value:e},{transition:n}){t[Ti]=t.style.display==="none"?"":t.style.display,n&&e?n.beforeEnter(t):$s(t,e)},mounted(t,{value:e},{transition:n}){n&&e&&n.enter(t)},updated(t,{value:e,oldValue:n},{transition:s}){!e!=!n&&(s?e?(s.beforeEnter(t),$s(t,!0),s.enter(t)):s.leave(t,()=>{$s(t,!1)}):$s(t,e))},beforeUnmount(t,{value:e}){$s(t,e)}};function $s(t,e){t.style.display=e?t[Ti]:"none",t[af]=!e}const _v=Symbol(""),Sv=/(^|;)\s*display\s*:/;function Cv(t,e,n){const s=t.style,r=De(n);let i=!1;if(n&&!r){if(e)if(De(e))for(const o of e.split(";")){const a=o.slice(0,o.indexOf(":")).trim();n[a]==null&&fi(s,a,"")}else for(const o in e)n[o]==null&&fi(s,o,"");for(const o in n)o==="display"&&(i=!0),fi(s,o,n[o])}else if(r){if(e!==n){const o=s[_v];o&&(n+=";"+o),s.cssText=n,i=Sv.test(n)}}else e&&t.removeAttribute("style");Ti in t&&(t[Ti]=i?s.display:"",t[af]&&(s.display="none"))}const pu=/\s*!important$/;function fi(t,e,n){if(ae(n))n.forEach(s=>fi(t,e,s));else if(n==null&&(n=""),e.startsWith("--"))t.setProperty(e,n);else{const s=Ev(t,e);pu.test(n)?t.setProperty(is(s),n.replace(pu,""),"important"):t[s]=n}}const yu=["Webkit","Moz","ms"],Po={};function Ev(t,e){const n=Po[e];if(n)return n;let s=xt(e);if(s!=="filter"&&s in t)return Po[e]=s;s=dn(s);for(let r=0;r<yu.length;r++){const i=yu[r]+s;if(i in t)return Po[e]=i}return e}const bu="http://www.w3.org/1999/xlink";function Mv(t,e,n,s,r){if(s&&e.startsWith("xlink:"))n==null?t.removeAttributeNS(bu,e.slice(6,e.length)):t.setAttributeNS(bu,e,n);else{const i=Im(e);n==null||i&&!id(n)?t.removeAttribute(e):t.setAttribute(e,i?"":n)}}function Tv(t,e,n,s,r,i,o){if(e==="innerHTML"||e==="textContent"){s&&o(s,r,i),t[e]=n??"";return}const a=t.tagName;if(e==="value"&&a!=="PROGRESS"&&!a.includes("-")){const u=a==="OPTION"?t.getAttribute("value")||"":t.value,c=n??"";(u!==c||!("_value"in t))&&(t.value=c),n==null&&t.removeAttribute(e),t._value=n;return}let l=!1;if(n===""||n==null){const u=typeof t[e];u==="boolean"?n=id(n):n==null&&u==="string"?(n="",l=!0):u==="number"&&(n=0,l=!0)}try{t[e]=n}catch{}l&&t.removeAttribute(e)}function Av(t,e,n,s){t.addEventListener(e,n,s)}function Rv(t,e,n,s){t.removeEventListener(e,n,s)}const xu=Symbol("_vei");function Iv(t,e,n,s,r=null){const i=t[xu]||(t[xu]={}),o=i[e];if(s&&o)o.value=s;else{const[a,l]=kv(e);if(s){const u=i[e]=Vv(s,r);Av(t,a,u,l)}else o&&(Rv(t,a,o,l),i[e]=void 0)}}const wu=/(?:Once|Passive|Capture)$/;function kv(t){let e;if(wu.test(t)){e={};let s;for(;s=t.match(wu);)t=t.slice(0,t.length-s[0].length),e[s[0].toLowerCase()]=!0}return[t[2]===":"?t.slice(3):is(t.slice(2)),e]}let Oo=0;const Fv=Promise.resolve(),Lv=()=>Oo||(Fv.then(()=>Oo=0),Oo=Date.now());function Vv(t,e){const n=s=>{if(!s._vts)s._vts=Date.now();else if(s._vts<=n.attached)return;Mt(Pv(s,n.value),e,5,[s])};return n.value=t,n.attached=Lv(),n}function Pv(t,e){if(ae(e)){const n=t.stopImmediatePropagation;return t.stopImmediatePropagation=()=>{n.call(t),t._stopped=!0},e.map(s=>r=>!r._stopped&&s&&s(r))}else return e}const _u=t=>t.charCodeAt(0)===111&&t.charCodeAt(1)===110&&t.charCodeAt(2)>96&&t.charCodeAt(2)<123,Ov=(t,e,n,s,r,i,o,a,l)=>{const u=r==="svg";e==="class"?wv(t,s,u):e==="style"?Cv(t,n,s):Xi(e)?Ha(e)||Iv(t,e,n,s,o):(e[0]==="."?(e=e.slice(1),!0):e[0]==="^"?(e=e.slice(1),!1):Nv(t,e,s,u))?Tv(t,e,s,i,o,a,l):(e==="true-value"?t._trueValue=s:e==="false-value"&&(t._falseValue=s),Mv(t,e,s,u))};function Nv(t,e,n,s){if(s)return!!(e==="innerHTML"||e==="textContent"||e in t&&_u(e)&&he(n));if(e==="spellcheck"||e==="draggable"||e==="translate"||e==="form"||e==="list"&&t.tagName==="INPUT"||e==="type"&&t.tagName==="TEXTAREA")return!1;if(e==="width"||e==="height"){const r=t.tagName;if(r==="IMG"||r==="VIDEO"||r==="CANVAS"||r==="SOURCE")return!1}return _u(e)&&De(n)?!1:e in t}const lf=new WeakMap,uf=new WeakMap,Ai=Symbol("_moveCb"),Su=Symbol("_enterCb"),cf={name:"TransitionGroup",props:$e({},yv,{tag:String,moveClass:String}),setup(t,{slots:e}){const n=ao(),s=Pd();let r,i;return nl(()=>{if(!r.length)return;const o=t.moveClass||`${t.name||"v"}-move`;if(!$v(r[0].el,n.vnode.el,o))return;r.forEach(Dv),r.forEach(zv);const a=r.filter(Hv);of(),a.forEach(l=>{const u=l.el,c=u.style;sn(u,o),c.transform=c.webkitTransform=c.transitionDuration="";const d=u[Ai]=f=>{f&&f.target!==u||(!f||/transform$/.test(f.propertyName))&&(u.removeEventListener("transitionend",d),u[Ai]=null,Sn(u,o))};u.addEventListener("transitionend",d)})}),()=>{const o=pe(t),a=sf(o);let l=o.tag||Ie;r=i,i=e.default?tl(e.default()):[];for(let u=0;u<i.length;u++){const c=i[u];c.key!=null&&gr(c,mr(c,a,s,n))}if(r)for(let u=0;u<r.length;u++){const c=r[u];gr(c,mr(c,a,s,n)),lf.set(c,c.el.getBoundingClientRect())}return w(l,null,i)}}},Bv=t=>delete t.mode;cf.props;const df=cf;function Dv(t){const e=t.el;e[Ai]&&e[Ai](),e[Su]&&e[Su]()}function zv(t){uf.set(t,t.el.getBoundingClientRect())}function Hv(t){const e=lf.get(t),n=uf.get(t),s=e.left-n.left,r=e.top-n.top;if(s||r){const i=t.el.style;return i.transform=i.webkitTransform=`translate(${s}px,${r}px)`,i.transitionDuration="0s",t}}function $v(t,e,n){const s=t.cloneNode(),r=t[Ls];r&&r.forEach(a=>{a.split(/\s+/).forEach(l=>l&&s.classList.remove(l))}),n.split(/\s+/).forEach(a=>a&&s.classList.add(a)),s.style.display="none";const i=e.nodeType===1?e:e.parentNode;i.appendChild(s);const{hasTransform:o}=rf(s);return i.removeChild(s),o}const Uv=["ctrl","shift","alt","meta"],jv={stop:t=>t.stopPropagation(),prevent:t=>t.preventDefault(),self:t=>t.target!==t.currentTarget,ctrl:t=>!t.ctrlKey,shift:t=>!t.shiftKey,alt:t=>!t.altKey,meta:t=>!t.metaKey,left:t=>"button"in t&&t.button!==0,middle:t=>"button"in t&&t.button!==1,right:t=>"button"in t&&t.button!==2,exact:(t,e)=>Uv.some(n=>t[`${n}Key`]&&!e.includes(n))},Cu=(t,e)=>{const n=t._withMods||(t._withMods={}),s=e.join(".");return n[s]||(n[s]=(r,...i)=>{for(let o=0;o<e.length;o++){const a=jv[e[o]];if(a&&a(r,e))return}return t(r,...i)})},Wv=$e({patchProp:Ov},pv);let Eu;function Gv(){return Eu||(Eu=Yg(Wv))}const qv=(...t)=>{const e=Gv().createApp(...t),{mount:n}=e;return e.mount=s=>{const r=Xv(s);if(!r)return;const i=e._component;!he(i)&&!i.render&&!i.template&&(i.template=r.innerHTML),r.innerHTML="";const o=n(r,!1,Yv(r));return r instanceof Element&&(r.removeAttribute("v-cloak"),r.setAttribute("data-v-app","")),o},e};function Yv(t){if(t instanceof SVGElement)return"svg";if(typeof MathMLElement=="function"&&t instanceof MathMLElement)return"mathml"}function Xv(t){return De(t)?document.querySelector(t):t}const Zv="modulepreload",Kv=function(t,e){return new URL(t,e).href},Mu={},Ut=function(e,n,s){let r=Promise.resolve();if(n&&n.length>0){const i=document.getElementsByTagName("link"),o=document.querySelector("meta[property=csp-nonce]"),a=(o==null?void 0:o.nonce)||(o==null?void 0:o.getAttribute("nonce"));r=Promise.all(n.map(l=>{if(l=Kv(l,s),l in Mu)return;Mu[l]=!0;const u=l.endsWith(".css"),c=u?'[rel="stylesheet"]':"";if(!!s)for(let h=i.length-1;h>=0;h--){const m=i[h];if(m.href===l&&(!u||m.rel==="stylesheet"))return}else if(document.querySelector(`link[href="${l}"]${c}`))return;const f=document.createElement("link");if(f.rel=u?"stylesheet":Zv,u||(f.as="script",f.crossOrigin=""),f.href=l,a&&f.setAttribute("nonce",a),document.head.appendChild(f),u)return new Promise((h,m)=>{f.addEventListener("load",h),f.addEventListener("error",()=>m(new Error(`Unable to preload CSS for ${l}`)))})}))}return r.then(()=>e()).catch(i=>{const o=new Event("vite:preloadError",{cancelable:!0});if(o.payload=i,window.dispatchEvent(o),!o.defaultPrevented)throw i})};function Y(t,e){return n=>Object.keys(t).reduce((s,r)=>{const o=typeof t[r]=="object"&&t[r]!=null&&!Array.isArray(t[r])?t[r]:{type:t[r]};return n&&r in n?s[r]={...o,default:n[r]}:s[r]=o,e&&!s[r].source&&(s[r].source=e),s},{})}const we=Y({class:[String,Array],style:{type:[String,Array,Object],default:null}},"component"),Oe=typeof window<"u",al=Oe&&"IntersectionObserver"in window,Jv=Oe&&("ontouchstart"in window||window.navigator.maxTouchPoints>0);function Tu(t,e,n){Qv(t,e),e.set(t,n)}function Qv(t,e){if(e.has(t))throw new TypeError("Cannot initialize the same private elements twice on an object")}function ep(t,e,n){var s=ff(t,e,"set");return tp(t,s,n),n}function tp(t,e,n){if(e.set)e.set.call(t,n);else{if(!e.writable)throw new TypeError("attempted to set read only private field");e.value=n}}function Un(t,e){var n=ff(t,e,"get");return np(t,n)}function ff(t,e,n){if(!e.has(t))throw new TypeError("attempted to "+n+" private field on non-instance");return e.get(t)}function np(t,e){return e.get?e.get.call(t):e.value}function hf(t,e,n){const s=e.length-1;if(s<0)return t===void 0?n:t;for(let r=0;r<s;r++){if(t==null)return n;t=t[e[r]]}return t==null||t[e[s]]===void 0?n:t[e[s]]}function co(t,e){if(t===e)return!0;if(t instanceof Date&&e instanceof Date&&t.getTime()!==e.getTime()||t!==Object(t)||e!==Object(e))return!1;const n=Object.keys(t);return n.length!==Object.keys(e).length?!1:n.every(s=>co(t[s],e[s]))}function pa(t,e,n){return t==null||!e||typeof e!="string"?n:t[e]!==void 0?t[e]:(e=e.replace(/\[(\w+)\]/g,".$1"),e=e.replace(/^\./,""),hf(t,e.split("."),n))}function Lw(t,e,n){if(e===!0)return t===void 0?n:t;if(e==null||typeof e=="boolean")return n;if(t!==Object(t)){if(typeof e!="function")return n;const r=e(t,n);return typeof r>"u"?n:r}if(typeof e=="string")return pa(t,e,n);if(Array.isArray(e))return hf(t,e,n);if(typeof e!="function")return n;const s=e(t,n);return typeof s>"u"?n:s}function ll(t){let e=arguments.length>1&&arguments[1]!==void 0?arguments[1]:0;return Array.from({length:t},(n,s)=>e+s)}function oe(t){let e=arguments.length>1&&arguments[1]!==void 0?arguments[1]:"px";if(!(t==null||t===""))return isNaN(+t)?String(t):isFinite(+t)?`${Number(t)}${e}`:void 0}function ya(t){return t!==null&&typeof t=="object"&&!Array.isArray(t)}function Ri(t){if(t&&"$el"in t){const e=t.$el;return(e==null?void 0:e.nodeType)===Node.TEXT_NODE?e.nextElementSibling:e}return t}const Au=Object.freeze({enter:13,tab:9,delete:46,esc:27,space:32,up:38,down:40,left:37,right:39,end:35,home:36,del:46,backspace:8,insert:45,pageup:33,pagedown:34,shift:16}),sp=Object.freeze({enter:"Enter",tab:"Tab",delete:"Delete",esc:"Escape",space:"Space",up:"ArrowUp",down:"ArrowDown",left:"ArrowLeft",right:"ArrowRight",end:"End",home:"Home",del:"Delete",backspace:"Backspace",insert:"Insert",pageup:"PageUp",pagedown:"PageDown",shift:"Shift"});function mf(t){return Object.keys(t)}function No(t,e){return e.every(n=>t.hasOwnProperty(n))}function rp(t,e){const n={},s=new Set(Object.keys(t));for(const r of e)s.has(r)&&(n[r]=t[r]);return n}function Ru(t,e,n){const s=Object.create(null),r=Object.create(null);for(const i in t)e.some(o=>o instanceof RegExp?o.test(i):o===i)&&!(n!=null&&n.some(o=>o===i))?s[i]=t[i]:r[i]=t[i];return[s,r]}function ul(t,e){const n={...t};return e.forEach(s=>delete n[s]),n}const gf=/^on[^a-z]/,vf=t=>gf.test(t),ip=["onAfterscriptexecute","onAnimationcancel","onAnimationend","onAnimationiteration","onAnimationstart","onAuxclick","onBeforeinput","onBeforescriptexecute","onChange","onClick","onCompositionend","onCompositionstart","onCompositionupdate","onContextmenu","onCopy","onCut","onDblclick","onFocusin","onFocusout","onFullscreenchange","onFullscreenerror","onGesturechange","onGestureend","onGesturestart","onGotpointercapture","onInput","onKeydown","onKeypress","onKeyup","onLostpointercapture","onMousedown","onMousemove","onMouseout","onMouseover","onMouseup","onMousewheel","onPaste","onPointercancel","onPointerdown","onPointerenter","onPointerleave","onPointermove","onPointerout","onPointerover","onPointerup","onReset","onSelect","onSubmit","onTouchcancel","onTouchend","onTouchmove","onTouchstart","onTransitioncancel","onTransitionend","onTransitionrun","onTransitionstart","onWheel"];function op(t){const[e,n]=Ru(t,[gf]),s=ul(e,ip),[r,i]=Ru(n,["class","style","id",/^data-/]);return Object.assign(r,e),Object.assign(i,s),[r,i]}function Fn(t){return t==null?[]:Array.isArray(t)?t:[t]}function Vw(t,e){let n=0;const s=function(){for(var r=arguments.length,i=new Array(r),o=0;o<r;o++)i[o]=arguments[o];clearTimeout(n),n=setTimeout(()=>t(...i),P(e))};return s.clear=()=>{clearTimeout(n)},s.immediate=t,s}function yr(t){let e=arguments.length>1&&arguments[1]!==void 0?arguments[1]:0,n=arguments.length>2&&arguments[2]!==void 0?arguments[2]:1;return Math.max(e,Math.min(n,t))}function Iu(t){const e=t.toString().trim();return e.includes(".")?e.length-e.indexOf(".")-1:0}function ku(t,e){let n=arguments.length>2&&arguments[2]!==void 0?arguments[2]:"0";return t+n.repeat(Math.max(0,e-t.length))}function Fu(t,e){return(arguments.length>2&&arguments[2]!==void 0?arguments[2]:"0").repeat(Math.max(0,e-t.length))+t}function ap(t){let e=arguments.length>1&&arguments[1]!==void 0?arguments[1]:1;const n=[];let s=0;for(;s<t.length;)n.push(t.substr(s,e)),s+=e;return n}function pt(){let t=arguments.length>0&&arguments[0]!==void 0?arguments[0]:{},e=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{},n=arguments.length>2?arguments[2]:void 0;const s={};for(const r in t)s[r]=t[r];for(const r in e){const i=t[r],o=e[r];if(ya(i)&&ya(o)){s[r]=pt(i,o,n);continue}if(Array.isArray(i)&&Array.isArray(o)&&n){s[r]=n(i,o);continue}s[r]=o}return s}function pf(t){return t.map(e=>e.type===Ie?pf(e.children):e).flat()}function Qn(){let t=arguments.length>0&&arguments[0]!==void 0?arguments[0]:"";if(Qn.cache.has(t))return Qn.cache.get(t);const e=t.replace(/[^a-z]/gi,"-").replace(/\B([A-Z])/g,"-$1").toLowerCase();return Qn.cache.set(t,e),e}Qn.cache=new Map;function ar(t,e){if(!e||typeof e!="object")return[];if(Array.isArray(e))return e.map(n=>ar(t,n)).flat(1);if(Array.isArray(e.children))return e.children.map(n=>ar(t,n)).flat(1);if(e.component){if(Object.getOwnPropertySymbols(e.component.provides).includes(t))return[e.component];if(e.component.subTree)return ar(t,e.component.subTree).flat(1)}return[]}var Wr=new WeakMap,gs=new WeakMap;class lp{constructor(e){Tu(this,Wr,{writable:!0,value:[]}),Tu(this,gs,{writable:!0,value:0}),this.size=e}push(e){Un(this,Wr)[Un(this,gs)]=e,ep(this,gs,(Un(this,gs)+1)%this.size)}values(){return Un(this,Wr).slice(Un(this,gs)).concat(Un(this,Wr).slice(0,Un(this,gs)))}}function cl(t){const e=tt({}),n=k(t);return Nn(()=>{for(const s in n.value)e[s]=n.value[s]},{flush:"sync"}),Qa(e)}function Ii(t,e){return t.includes(e)}function yf(t){return t[2].toLowerCase()+t.slice(3)}const ki=()=>[Function,Array];function Lu(t,e){return e="on"+dn(e),!!(t[e]||t[`${e}Once`]||t[`${e}Capture`]||t[`${e}OnceCapture`]||t[`${e}CaptureOnce`])}function Pw(t){for(var e=arguments.length,n=new Array(e>1?e-1:0),s=1;s<e;s++)n[s-1]=arguments[s];if(Array.isArray(t))for(const r of t)r(...n);else typeof t=="function"&&t(...n)}function bf(t){let e=arguments.length>1&&arguments[1]!==void 0?arguments[1]:!0;const n=["button","[href]",'input:not([type="hidden"])',"select","textarea","[tabindex]"].map(s=>`${s}${e?':not([tabindex="-1"])':""}:not([disabled])`).join(", ");return[...t.querySelectorAll(n)]}function up(t,e,n){let s,r=t.indexOf(document.activeElement);const i=e==="next"?1:-1;do r+=i,s=t[r];while((!s||s.offsetParent==null||!((n==null?void 0:n(s))??!0))&&r<t.length&&r>=0);return s}function cp(t,e){var s,r,i,o;const n=bf(t);if(!e)(t===document.activeElement||!t.contains(document.activeElement))&&((s=n[0])==null||s.focus());else if(e==="first")(r=n[0])==null||r.focus();else if(e==="last")(i=n.at(-1))==null||i.focus();else if(typeof e=="number")(o=n[e])==null||o.focus();else{const a=up(n,e);a?a.focus():cp(t,e==="next"?"first":"last")}}function xf(t,e){if(!(Oe&&typeof CSS<"u"&&typeof CSS.supports<"u"&&CSS.supports(`selector(${e})`)))return null;try{return!!t&&t.matches(e)}catch{return null}}function dp(t){return t.some(e=>pr(e)?e.type===yt?!1:e.type!==Ie||dp(e.children):!0)?t:null}function fp(t,e){if(!Oe||t===0)return e(),()=>{};const n=window.setTimeout(e,t);return()=>window.clearTimeout(n)}function Ow(t,e){const n=t.clientX,s=t.clientY,r=e.getBoundingClientRect(),i=r.left,o=r.top,a=r.right,l=r.bottom;return n>=i&&n<=a&&s>=o&&s<=l}const wf=["top","bottom"],hp=["start","end","left","right"];function ba(t,e){let[n,s]=t.split(" ");return s||(s=Ii(wf,n)?"start":Ii(hp,n)?"top":"center"),{side:xa(n,e),align:xa(s,e)}}function xa(t,e){return t==="start"?e?"right":"left":t==="end"?e?"left":"right":t}function Bo(t){return{side:{center:"center",top:"bottom",bottom:"top",left:"right",right:"left"}[t.side],align:t.align}}function Do(t){return{side:t.side,align:{center:"center",top:"bottom",bottom:"top",left:"right",right:"left"}[t.align]}}function Vu(t){return{side:t.align,align:t.side}}function Pu(t){return Ii(wf,t.side)?"y":"x"}class es{constructor(e){let{x:n,y:s,width:r,height:i}=e;this.x=n,this.y=s,this.width=r,this.height=i}get top(){return this.y}get bottom(){return this.y+this.height}get left(){return this.x}get right(){return this.x+this.width}}function Ou(t,e){return{x:{before:Math.max(0,e.left-t.left),after:Math.max(0,t.right-e.right)},y:{before:Math.max(0,e.top-t.top),after:Math.max(0,t.bottom-e.bottom)}}}function _f(t){return Array.isArray(t)?new es({x:t[0],y:t[1],width:0,height:0}):t.getBoundingClientRect()}function Sf(t){const e=t.getBoundingClientRect(),n=getComputedStyle(t),s=n.transform;if(s){let r,i,o,a,l;if(s.startsWith("matrix3d("))r=s.slice(9,-1).split(/, /),i=+r[0],o=+r[5],a=+r[12],l=+r[13];else if(s.startsWith("matrix("))r=s.slice(7,-1).split(/, /),i=+r[0],o=+r[3],a=+r[4],l=+r[5];else return new es(e);const u=n.transformOrigin,c=e.x-a-(1-i)*parseFloat(u),d=e.y-l-(1-o)*parseFloat(u.slice(u.indexOf(" ")+1)),f=i?e.width/i:t.offsetWidth+1,h=o?e.height/o:t.offsetHeight+1;return new es({x:c,y:d,width:f,height:h})}else return new es(e)}function Zs(t,e,n){if(typeof t.animate>"u")return{finished:Promise.resolve()};let s;try{s=t.animate(e,n)}catch{return{finished:Promise.resolve()}}return typeof s.finished>"u"&&(s.finished=new Promise(r=>{s.onfinish=()=>{r(s)}})),s}const hi=new WeakMap;function mp(t,e){Object.keys(e).forEach(n=>{if(vf(n)){const s=yf(n),r=hi.get(t);if(e[n]==null)r==null||r.forEach(i=>{const[o,a]=i;o===s&&(t.removeEventListener(s,a),r.delete(i))});else if(!r||![...r].some(i=>i[0]===s&&i[1]===e[n])){t.addEventListener(s,e[n]);const i=r||new Set;i.add([s,e[n]]),hi.has(t)||hi.set(t,i)}}else e[n]==null?t.removeAttribute(n):t.setAttribute(n,e[n])})}function gp(t,e){Object.keys(e).forEach(n=>{if(vf(n)){const s=yf(n),r=hi.get(t);r==null||r.forEach(i=>{const[o,a]=i;o===s&&(t.removeEventListener(s,a),r.delete(i))})}else t.removeAttribute(n)})}const vs=2.4,Nu=.2126729,Bu=.7151522,Du=.072175,vp=.55,pp=.58,yp=.57,bp=.62,Gr=.03,zu=1.45,xp=5e-4,wp=1.25,_p=1.25,Hu=.078,$u=12.82051282051282,qr=.06,Uu=.001;function ju(t,e){const n=(t.r/255)**vs,s=(t.g/255)**vs,r=(t.b/255)**vs,i=(e.r/255)**vs,o=(e.g/255)**vs,a=(e.b/255)**vs;let l=n*Nu+s*Bu+r*Du,u=i*Nu+o*Bu+a*Du;if(l<=Gr&&(l+=(Gr-l)**zu),u<=Gr&&(u+=(Gr-u)**zu),Math.abs(u-l)<xp)return 0;let c;if(u>l){const d=(u**vp-l**pp)*wp;c=d<Uu?0:d<Hu?d-d*$u*qr:d-qr}else{const d=(u**bp-l**yp)*_p;c=d>-Uu?0:d>-Hu?d-d*$u*qr:d+qr}return c*100}function Nw(t,e){e=Array.isArray(e)?e.slice(0,-1).map(n=>`'${n}'`).join(", ")+` or '${e.at(-1)}'`:`'${e}'`}const Fi=.20689655172413793,Sp=t=>t>Fi**3?Math.cbrt(t):t/(3*Fi**2)+4/29,Cp=t=>t>Fi?t**3:3*Fi**2*(t-4/29);function Cf(t){const e=Sp,n=e(t[1]);return[116*n-16,500*(e(t[0]/.95047)-n),200*(n-e(t[2]/1.08883))]}function Ef(t){const e=Cp,n=(t[0]+16)/116;return[e(n+t[1]/500)*.95047,e(n),e(n-t[2]/200)*1.08883]}const Ep=[[3.2406,-1.5372,-.4986],[-.9689,1.8758,.0415],[.0557,-.204,1.057]],Mp=t=>t<=.0031308?t*12.92:1.055*t**(1/2.4)-.055,Tp=[[.4124,.3576,.1805],[.2126,.7152,.0722],[.0193,.1192,.9505]],Ap=t=>t<=.04045?t/12.92:((t+.055)/1.055)**2.4;function Mf(t){const e=Array(3),n=Mp,s=Ep;for(let r=0;r<3;++r)e[r]=Math.round(yr(n(s[r][0]*t[0]+s[r][1]*t[1]+s[r][2]*t[2]))*255);return{r:e[0],g:e[1],b:e[2]}}function dl(t){let{r:e,g:n,b:s}=t;const r=[0,0,0],i=Ap,o=Tp;e=i(e/255),n=i(n/255),s=i(s/255);for(let a=0;a<3;++a)r[a]=o[a][0]*e+o[a][1]*n+o[a][2]*s;return r}function wa(t){return!!t&&/^(#|var\(--|(rgb|hsl)a?\()/.test(t)}function Rp(t){return wa(t)&&!/^((rgb|hsl)a?\()?var\(--/.test(t)}const Wu=/^(?<fn>(?:rgb|hsl)a?)\((?<values>.+)\)/,Ip={rgb:(t,e,n,s)=>({r:t,g:e,b:n,a:s}),rgba:(t,e,n,s)=>({r:t,g:e,b:n,a:s}),hsl:(t,e,n,s)=>Gu({h:t,s:e,l:n,a:s}),hsla:(t,e,n,s)=>Gu({h:t,s:e,l:n,a:s}),hsv:(t,e,n,s)=>br({h:t,s:e,v:n,a:s}),hsva:(t,e,n,s)=>br({h:t,s:e,v:n,a:s})};function jt(t){if(typeof t=="number")return{r:(t&16711680)>>16,g:(t&65280)>>8,b:t&255};if(typeof t=="string"&&Wu.test(t)){const{groups:e}=t.match(Wu),{fn:n,values:s}=e,r=s.split(/,\s*/).map(i=>i.endsWith("%")&&["hsl","hsla","hsv","hsva"].includes(n)?parseFloat(i)/100:parseFloat(i));return Ip[n](...r)}else if(typeof t=="string"){let e=t.startsWith("#")?t.slice(1):t;return[3,4].includes(e.length)?e=e.split("").map(n=>n+n).join(""):[6,8].includes(e.length),Fp(e)}else if(typeof t=="object"){if(No(t,["r","g","b"]))return t;if(No(t,["h","s","l"]))return br(Tf(t));if(No(t,["h","s","v"]))return br(t)}throw new TypeError(`Invalid color: ${t==null?t:String(t)||t.constructor.name}
Expected #hex, #hexa, rgb(), rgba(), hsl(), hsla(), object or number`)}function br(t){const{h:e,s:n,v:s,a:r}=t,i=a=>{const l=(a+e/60)%6;return s-s*n*Math.max(Math.min(l,4-l,1),0)},o=[i(5),i(3),i(1)].map(a=>Math.round(a*255));return{r:o[0],g:o[1],b:o[2],a:r}}function Gu(t){return br(Tf(t))}function Tf(t){const{h:e,s:n,l:s,a:r}=t,i=s+n*Math.min(s,1-s),o=i===0?0:2-2*s/i;return{h:e,s:o,v:i,a:r}}function Yr(t){const e=Math.round(t).toString(16);return("00".substr(0,2-e.length)+e).toUpperCase()}function kp(t){let{r:e,g:n,b:s,a:r}=t;return`#${[Yr(e),Yr(n),Yr(s),r!==void 0?Yr(Math.round(r*255)):""].join("")}`}function Fp(t){t=Lp(t);let[e,n,s,r]=ap(t,2).map(i=>parseInt(i,16));return r=r===void 0?r:r/255,{r:e,g:n,b:s,a:r}}function Lp(t){return t.startsWith("#")&&(t=t.slice(1)),t=t.replace(/([^0-9a-f])/gi,"F"),(t.length===3||t.length===4)&&(t=t.split("").map(e=>e+e).join("")),t.length!==6&&(t=ku(ku(t,6),8,"F")),t}function Vp(t,e){const n=Cf(dl(t));return n[0]=n[0]+e*10,Mf(Ef(n))}function Pp(t,e){const n=Cf(dl(t));return n[0]=n[0]-e*10,Mf(Ef(n))}function Op(t){const e=jt(t);return dl(e)[1]}function Af(t){const e=Math.abs(ju(jt(0),jt(t)));return Math.abs(ju(jt(16777215),jt(t)))>Math.min(e,50)?"#fff":"#000"}const Vs=Symbol.for("vuetify:defaults");function Np(t){return ee(t)}function fl(){const t=Ne(Vs);if(!t)throw new Error("[Vuetify] Could not find defaults instance");return t}function mn(t,e){const n=fl(),s=ee(t),r=k(()=>{if(P(e==null?void 0:e.disabled))return n.value;const o=P(e==null?void 0:e.scoped),a=P(e==null?void 0:e.reset),l=P(e==null?void 0:e.root);if(s.value==null&&!(o||a||l))return n.value;let u=pt(s.value,{prev:n.value});if(o)return u;if(a||l){const c=Number(a||1/0);for(let d=0;d<=c&&!(!u||!("prev"in u));d++)u=u.prev;return u&&typeof l=="string"&&l in u&&(u=pt(pt(u,{prev:u}),u[l])),u}return u.prev?pt(u.prev,u):u});return ut(Vs,r),r}function Bp(t,e){var n,s;return typeof((n=t.props)==null?void 0:n[e])<"u"||typeof((s=t.props)==null?void 0:s[Qn(e)])<"u"}function Dp(){let t=arguments.length>0&&arguments[0]!==void 0?arguments[0]:{},e=arguments.length>1?arguments[1]:void 0,n=arguments.length>2&&arguments[2]!==void 0?arguments[2]:fl();const s=qe("useDefaults");if(e=e??s.type.name??s.type.__name,!e)throw new Error("[Vuetify] Could not determine component name");const r=k(()=>{var l;return(l=n.value)==null?void 0:l[t._as??e]}),i=new Proxy(t,{get(l,u){var d,f,h,m;const c=Reflect.get(l,u);return u==="class"||u==="style"?[(d=r.value)==null?void 0:d[u],c].filter(p=>p!=null):typeof u=="string"&&!Bp(s.vnode,u)?((f=r.value)==null?void 0:f[u])??((m=(h=n.value)==null?void 0:h.global)==null?void 0:m[u])??c:c}}),o=be();Nn(()=>{if(r.value){const l=Object.entries(r.value).filter(u=>{let[c]=u;return c.startsWith(c[0].toUpperCase())});o.value=l.length?Object.fromEntries(l):void 0}else o.value=void 0});function a(){const l=jp(Vs,s);ut(Vs,k(()=>o.value?pt((l==null?void 0:l.value)??{},o.value):l==null?void 0:l.value))}return{props:i,provideSubDefaults:a}}function kr(t){if(t._setup=t._setup??t.setup,!t.name)return t;if(t._setup){t.props=Y(t.props??{},t.name)();const e=Object.keys(t.props).filter(n=>n!=="class"&&n!=="style");t.filterProps=function(s){return rp(s,e)},t.props._as=String,t.setup=function(s,r){const i=fl();if(!i.value)return t._setup(s,r);const{props:o,provideSubDefaults:a}=Dp(s,s._as??t.name,i),l=t._setup(o,r);return a(),l}}return t}function me(){let t=arguments.length>0&&arguments[0]!==void 0?arguments[0]:!0;return e=>(t?kr:fn)(e)}function fo(t){let e=arguments.length>1&&arguments[1]!==void 0?arguments[1]:"div",n=arguments.length>2?arguments[2]:void 0;return me()({name:n??dn(xt(t.replace(/__/g,"-"))),props:{tag:{type:String,default:e},...we()},setup(s,r){let{slots:i}=r;return()=>{var o;return Bn(s.tag,{class:[t,s.class],style:s.style},(o=i.default)==null?void 0:o.call(i))}}})}function Rf(t){if(typeof t.getRootNode!="function"){for(;t.parentNode;)t=t.parentNode;return t!==document?null:document}const e=t.getRootNode();return e!==document&&e.getRootNode({composed:!0})!==document?null:e}const _a="cubic-bezier(0.4, 0, 0.2, 1)",zp="cubic-bezier(0.0, 0, 0.2, 1)",Hp="cubic-bezier(0.4, 0, 1, 1)";function qe(t,e){const n=ao();if(!n)throw new Error(`[Vuetify] ${t} ${e||"must be called from inside a setup function"}`);return n}function Kt(){let t=arguments.length>0&&arguments[0]!==void 0?arguments[0]:"composables";const e=qe(t).type;return Qn((e==null?void 0:e.aliasName)||(e==null?void 0:e.name))}let If=0,mi=new WeakMap;function gn(){const t=qe("getUid");if(mi.has(t))return mi.get(t);{const e=If++;return mi.set(t,e),e}}gn.reset=()=>{If=0,mi=new WeakMap};function $p(t){let e=arguments.length>1&&arguments[1]!==void 0?arguments[1]:!1;for(;t;){if(e?Up(t):hl(t))return t;t=t.parentElement}return document.scrollingElement}function Li(t,e){const n=[];if(e&&t&&!e.contains(t))return n;for(;t&&(hl(t)&&n.push(t),t!==e);)t=t.parentElement;return n}function hl(t){if(!t||t.nodeType!==Node.ELEMENT_NODE)return!1;const e=window.getComputedStyle(t);return e.overflowY==="scroll"||e.overflowY==="auto"&&t.scrollHeight>t.clientHeight}function Up(t){if(!t||t.nodeType!==Node.ELEMENT_NODE)return!1;const e=window.getComputedStyle(t);return["scroll","auto"].includes(e.overflowY)}function jp(t){let e=arguments.length>1&&arguments[1]!==void 0?arguments[1]:qe("injectSelf");const{provides:n}=e;if(n&&t in n)return n[t]}function Wp(t){for(;t;){if(window.getComputedStyle(t).position==="fixed")return!0;t=t.offsetParent}return!1}function ye(t){const e=qe("useRender");e.render=t}function kf(t){let e=arguments.length>1&&arguments[1]!==void 0?arguments[1]:"content";const n=ee(),s=ee();if(Oe){const r=new ResizeObserver(i=>{t==null||t(i,r),i.length&&(e==="content"?s.value=i[0].contentRect:s.value=i[0].target.getBoundingClientRect())});kt(()=>{r.disconnect()}),le(n,(i,o)=>{o&&(r.unobserve(Ri(o)),s.value=void 0),i&&r.observe(Ri(i))},{flush:"post"})}return{resizeRef:n,contentRect:Cr(s)}}const Vi=Symbol.for("vuetify:layout"),Ff=Symbol.for("vuetify:layout-item"),qu=1e3,Gp=Y({overlaps:{type:Array,default:()=>[]},fullHeight:Boolean},"layout"),qp=Y({name:{type:String},order:{type:[Number,String],default:0},absolute:Boolean},"layout-item");function Yp(){const t=Ne(Vi);if(!t)throw new Error("[Vuetify] Could not find injected layout");return{getLayoutItem:t.getLayoutItem,mainRect:t.mainRect,mainStyles:t.mainStyles}}function Xp(t){const e=Ne(Vi);if(!e)throw new Error("[Vuetify] Could not find injected layout");const n=t.id??`layout-item-${gn()}`,s=qe("useLayoutItem");ut(Ff,{id:n});const r=be(!1);Dd(()=>r.value=!0),Bd(()=>r.value=!1);const{layoutItemStyles:i,layoutItemScrimStyles:o}=e.register(s,{...t,active:k(()=>r.value?!1:t.active.value),id:n});return kt(()=>e.unregister(n)),{layoutItemStyles:i,layoutRect:e.layoutRect,layoutItemScrimStyles:o}}const Zp=(t,e,n,s)=>{let r={top:0,left:0,right:0,bottom:0};const i=[{id:"",layer:{...r}}];for(const o of t){const a=e.get(o),l=n.get(o),u=s.get(o);if(!a||!l||!u)continue;const c={...r,[a.value]:parseInt(r[a.value],10)+(u.value?parseInt(l.value,10):0)};i.push({id:o,layer:c}),r=c}return i};function Kp(t){const e=Ne(Vi,null),n=k(()=>e?e.rootZIndex.value-100:qu),s=ee([]),r=tt(new Map),i=tt(new Map),o=tt(new Map),a=tt(new Map),l=tt(new Map),{resizeRef:u,contentRect:c}=kf(),d=k(()=>{const _=new Map,C=t.overlaps??[];for(const M of C.filter(R=>R.includes(":"))){const[R,F]=M.split(":");if(!s.value.includes(R)||!s.value.includes(F))continue;const V=r.get(R),T=r.get(F),L=i.get(R),A=i.get(F);!V||!T||!L||!A||(_.set(F,{position:V.value,amount:parseInt(L.value,10)}),_.set(R,{position:T.value,amount:-parseInt(A.value,10)}))}return _}),f=k(()=>{const _=[...new Set([...o.values()].map(M=>M.value))].sort((M,R)=>M-R),C=[];for(const M of _){const R=s.value.filter(F=>{var V;return((V=o.get(F))==null?void 0:V.value)===M});C.push(...R)}return Zp(C,r,i,a)}),h=k(()=>!Array.from(l.values()).some(_=>_.value)),m=k(()=>f.value[f.value.length-1].layer),p=k(()=>({"--v-layout-left":oe(m.value.left),"--v-layout-right":oe(m.value.right),"--v-layout-top":oe(m.value.top),"--v-layout-bottom":oe(m.value.bottom),...h.value?void 0:{transition:"none"}})),v=k(()=>f.value.slice(1).map((_,C)=>{let{id:M}=_;const{layer:R}=f.value[C],F=i.get(M),V=r.get(M);return{id:M,...R,size:Number(F.value),position:V.value}})),S=_=>v.value.find(C=>C.id===_),g=qe("createLayout"),y=be(!1);Zt(()=>{y.value=!0}),ut(Vi,{register:(_,C)=>{let{id:M,order:R,position:F,layoutSize:V,elementSize:T,active:L,disableTransitions:A,absolute:N}=C;o.set(M,R),r.set(M,F),i.set(M,V),a.set(M,L),A&&l.set(M,A);const z=ar(Ff,g==null?void 0:g.vnode).indexOf(_);z>-1?s.value.splice(z,0,M):s.value.push(M);const X=k(()=>v.value.findIndex(ce=>ce.id===M)),re=k(()=>n.value+f.value.length*2-X.value*2),te=k(()=>{const ce=F.value==="left"||F.value==="right",Te=F.value==="right",ge=F.value==="bottom",Le={[F.value]:0,zIndex:re.value,transform:`translate${ce?"X":"Y"}(${(L.value?0:-110)*(Te||ge?-1:1)}%)`,position:N.value||n.value!==qu?"absolute":"fixed",...h.value?void 0:{transition:"none"}};if(!y.value)return Le;const Z=v.value[X.value];if(!Z)throw new Error(`[Vuetify] Could not find layout item "${M}"`);const xe=d.value.get(M);return xe&&(Z[xe.position]+=xe.amount),{...Le,height:ce?`calc(100% - ${Z.top}px - ${Z.bottom}px)`:T.value?`${T.value}px`:void 0,left:Te?void 0:`${Z.left}px`,right:Te?`${Z.right}px`:void 0,top:F.value!=="bottom"?`${Z.top}px`:void 0,bottom:F.value!=="top"?`${Z.bottom}px`:void 0,width:ce?T.value?`${T.value}px`:void 0:`calc(100% - ${Z.left}px - ${Z.right}px)`}}),Q=k(()=>({zIndex:re.value-1}));return{layoutItemStyles:te,layoutItemScrimStyles:Q,zIndex:re}},unregister:_=>{o.delete(_),r.delete(_),i.delete(_),a.delete(_),l.delete(_),s.value=s.value.filter(C=>C!==_)},mainRect:m,mainStyles:p,getLayoutItem:S,items:v,layoutRect:c,rootZIndex:n});const b=k(()=>["v-layout",{"v-layout--full-height":t.fullHeight}]),x=k(()=>({zIndex:e?n.value:void 0,position:e?"relative":void 0,overflow:e?"hidden":void 0}));return{layoutClasses:b,layoutStyles:x,getLayoutItem:S,items:v,layoutRect:c,layoutRef:u}}function an(t,e){let n;function s(){n=ja(),n.run(()=>e.length?e(()=>{n==null||n.stop(),s()}):e())}le(t,r=>{r&&!n?s():r||(n==null||n.stop(),n=void 0)},{immediate:!0}),Rt(()=>{n==null||n.stop()})}function ct(t,e,n){let s=arguments.length>3&&arguments[3]!==void 0?arguments[3]:d=>d,r=arguments.length>4&&arguments[4]!==void 0?arguments[4]:d=>d;const i=qe("useProxiedModel"),o=ee(t[e]!==void 0?t[e]:n),a=Qn(e),u=k(a!==e?()=>{var d,f,h,m;return t[e],!!(((d=i.vnode.props)!=null&&d.hasOwnProperty(e)||(f=i.vnode.props)!=null&&f.hasOwnProperty(a))&&((h=i.vnode.props)!=null&&h.hasOwnProperty(`onUpdate:${e}`)||(m=i.vnode.props)!=null&&m.hasOwnProperty(`onUpdate:${a}`)))}:()=>{var d,f;return t[e],!!((d=i.vnode.props)!=null&&d.hasOwnProperty(e)&&((f=i.vnode.props)!=null&&f.hasOwnProperty(`onUpdate:${e}`)))});an(()=>!u.value,()=>{le(()=>t[e],d=>{o.value=d})});const c=k({get(){const d=t[e];return s(u.value?d:o.value)},set(d){const f=r(d),h=pe(u.value?t[e]:o.value);h===f||s(h)===d||(o.value=f,i==null||i.emit(`update:${e}`,f))}});return Object.defineProperty(c,"externalValue",{get:()=>u.value?t[e]:o.value}),c}const Jp={badge:"Badge",open:"Open",close:"Close",confirmEdit:{ok:"OK",cancel:"Cancel"},dataIterator:{noResultsText:"No matching records found",loadingText:"Loading items..."},dataTable:{itemsPerPageText:"Rows per page:",ariaLabel:{sortDescending:"Sorted descending.",sortAscending:"Sorted ascending.",sortNone:"Not sorted.",activateNone:"Activate to remove sorting.",activateDescending:"Activate to sort descending.",activateAscending:"Activate to sort ascending."},sortBy:"Sort by"},dataFooter:{itemsPerPageText:"Items per page:",itemsPerPageAll:"All",nextPage:"Next page",prevPage:"Previous page",firstPage:"First page",lastPage:"Last page",pageText:"{0}-{1} of {2}"},dateRangeInput:{divider:"to"},datePicker:{itemsSelected:"{0} selected",range:{title:"Select dates",header:"Enter dates"},title:"Select date",header:"Enter date",input:{placeholder:"Enter date"}},noDataText:"No data available",carousel:{prev:"Previous visual",next:"Next visual",ariaLabel:{delimiter:"Carousel slide {0} of {1}"}},calendar:{moreEvents:"{0} more",today:"Today"},input:{clear:"Clear {0}",prependAction:"{0} prepended action",appendAction:"{0} appended action",otp:"Please enter OTP character {0}"},fileInput:{counter:"{0} files",counterSize:"{0} files ({1} in total)"},timePicker:{am:"AM",pm:"PM",title:"Select Time"},pagination:{ariaLabel:{root:"Pagination Navigation",next:"Next page",previous:"Previous page",page:"Go to page {0}",currentPage:"Page {0}, Current page",first:"First page",last:"Last page"}},stepper:{next:"Next",prev:"Previous"},rating:{ariaLabel:{item:"Rating {0} of {1}"}},loading:"Loading...",infiniteScroll:{loadMore:"Load more",empty:"No more"}},Yu="$vuetify.",Xu=(t,e)=>t.replace(/\{(\d+)\}/g,(n,s)=>String(e[+s])),Lf=(t,e,n)=>function(s){for(var r=arguments.length,i=new Array(r>1?r-1:0),o=1;o<r;o++)i[o-1]=arguments[o];if(!s.startsWith(Yu))return Xu(s,i);const a=s.replace(Yu,""),l=t.value&&n.value[t.value],u=e.value&&n.value[e.value];let c=pa(l,a,null);return c||(`${s}${t.value}`,c=pa(u,a,null)),c||(c=s),typeof c!="string"&&(c=s),Xu(c,i)};function Vf(t,e){return(n,s)=>new Intl.NumberFormat([t.value,e.value],s).format(n)}function zo(t,e,n){const s=ct(t,e,t[e]??n.value);return s.value=t[e]??n.value,le(n,r=>{t[e]==null&&(s.value=n.value)}),s}function Pf(t){return e=>{const n=zo(e,"locale",t.current),s=zo(e,"fallback",t.fallback),r=zo(e,"messages",t.messages);return{name:"vuetify",current:n,fallback:s,messages:r,t:Lf(n,s,r),n:Vf(n,s),provide:Pf({current:n,fallback:s,messages:r})}}}function Qp(t){const e=be((t==null?void 0:t.locale)??"en"),n=be((t==null?void 0:t.fallback)??"en"),s=ee({en:Jp,...t==null?void 0:t.messages});return{name:"vuetify",current:e,fallback:n,messages:s,t:Lf(e,n,s),n:Vf(e,n),provide:Pf({current:e,fallback:n,messages:s})}}const Pi=Symbol.for("vuetify:locale");function ey(t){return t.name!=null}function ty(t){const e=t!=null&&t.adapter&&ey(t==null?void 0:t.adapter)?t==null?void 0:t.adapter:Qp(t),n=ry(e,t);return{...e,...n}}function ny(){const t=Ne(Pi);if(!t)throw new Error("[Vuetify] Could not find injected locale instance");return t}function sy(){return{af:!1,ar:!0,bg:!1,ca:!1,ckb:!1,cs:!1,de:!1,el:!1,en:!1,es:!1,et:!1,fa:!0,fi:!1,fr:!1,hr:!1,hu:!1,he:!0,id:!1,it:!1,ja:!1,km:!1,ko:!1,lv:!1,lt:!1,nl:!1,no:!1,pl:!1,pt:!1,ro:!1,ru:!1,sk:!1,sl:!1,srCyrl:!1,srLatn:!1,sv:!1,th:!1,tr:!1,az:!1,uk:!1,vi:!1,zhHans:!1,zhHant:!1}}function ry(t,e){const n=ee((e==null?void 0:e.rtl)??sy()),s=k(()=>n.value[t.current.value]??!1);return{isRtl:s,rtl:n,rtlClasses:k(()=>`v-locale--is-${s.value?"rtl":"ltr"}`)}}function Jt(){const t=Ne(Pi);if(!t)throw new Error("[Vuetify] Could not find injected rtl instance");return{isRtl:t.isRtl,rtlClasses:t.rtlClasses}}const Oi=Symbol.for("vuetify:theme"),dt=Y({theme:String},"theme");function Zu(){return{defaultTheme:"light",variations:{colors:[],lighten:0,darken:0},themes:{light:{dark:!1,colors:{background:"#FFFFFF",surface:"#FFFFFF","surface-bright":"#FFFFFF","surface-light":"#EEEEEE","surface-variant":"#424242","on-surface-variant":"#EEEEEE",primary:"#1867C0","primary-darken-1":"#1F5592",secondary:"#48A9A6","secondary-darken-1":"#018786",error:"#B00020",info:"#2196F3",success:"#4CAF50",warning:"#FB8C00"},variables:{"border-color":"#000000","border-opacity":.12,"high-emphasis-opacity":.87,"medium-emphasis-opacity":.6,"disabled-opacity":.38,"idle-opacity":.04,"hover-opacity":.04,"focus-opacity":.12,"selected-opacity":.08,"activated-opacity":.12,"pressed-opacity":.12,"dragged-opacity":.08,"theme-kbd":"#212529","theme-on-kbd":"#FFFFFF","theme-code":"#F5F5F5","theme-on-code":"#000000"}},dark:{dark:!0,colors:{background:"#121212",surface:"#212121","surface-bright":"#ccbfd6","surface-light":"#424242","surface-variant":"#a3a3a3","on-surface-variant":"#424242",primary:"#2196F3","primary-darken-1":"#277CC1",secondary:"#54B6B2","secondary-darken-1":"#48A9A6",error:"#CF6679",info:"#2196F3",success:"#4CAF50",warning:"#FB8C00"},variables:{"border-color":"#FFFFFF","border-opacity":.12,"high-emphasis-opacity":1,"medium-emphasis-opacity":.7,"disabled-opacity":.5,"idle-opacity":.1,"hover-opacity":.04,"focus-opacity":.12,"selected-opacity":.08,"activated-opacity":.12,"pressed-opacity":.16,"dragged-opacity":.08,"theme-kbd":"#212529","theme-on-kbd":"#FFFFFF","theme-code":"#343434","theme-on-code":"#CCCCCC"}}}}}function iy(){var s,r;let t=arguments.length>0&&arguments[0]!==void 0?arguments[0]:Zu();const e=Zu();if(!t)return{...e,isDisabled:!0};const n={};for(const[i,o]of Object.entries(t.themes??{})){const a=o.dark||i==="dark"?(s=e.themes)==null?void 0:s.dark:(r=e.themes)==null?void 0:r.light;n[i]=pt(a,o)}return pt(e,{...t,themes:n})}function oy(t){const e=iy(t),n=ee(e.defaultTheme),s=ee(e.themes),r=k(()=>{const c={};for(const[d,f]of Object.entries(s.value)){const h=c[d]={...f,colors:{...f.colors}};if(e.variations)for(const m of e.variations.colors){const p=h.colors[m];if(p)for(const v of["lighten","darken"]){const S=v==="lighten"?Vp:Pp;for(const g of ll(e.variations[v],1))h.colors[`${m}-${v}-${g}`]=kp(S(jt(p),g))}}for(const m of Object.keys(h.colors)){if(/^on-[a-z]/.test(m)||h.colors[`on-${m}`])continue;const p=`on-${m}`,v=jt(h.colors[m]);h.colors[p]=Af(v)}}return c}),i=k(()=>r.value[n.value]),o=k(()=>{var m;const c=[];(m=i.value)!=null&&m.dark&&jn(c,":root",["color-scheme: dark"]),jn(c,":root",Ku(i.value));for(const[p,v]of Object.entries(r.value))jn(c,`.v-theme--${p}`,[`color-scheme: ${v.dark?"dark":"normal"}`,...Ku(v)]);const d=[],f=[],h=new Set(Object.values(r.value).flatMap(p=>Object.keys(p.colors)));for(const p of h)/^on-[a-z]/.test(p)?jn(f,`.${p}`,[`color: rgb(var(--v-theme-${p})) !important`]):(jn(d,`.bg-${p}`,[`--v-theme-overlay-multiplier: var(--v-theme-${p}-overlay-multiplier)`,`background-color: rgb(var(--v-theme-${p})) !important`,`color: rgb(var(--v-theme-on-${p})) !important`]),jn(f,`.text-${p}`,[`color: rgb(var(--v-theme-${p})) !important`]),jn(f,`.border-${p}`,[`--v-border-color: var(--v-theme-${p})`]));return c.push(...d,...f),c.map((p,v)=>v===0?p:`    ${p}`).join("")});function a(){return{style:[{children:o.value,id:"vuetify-theme-stylesheet",nonce:e.cspNonce||!1}]}}function l(c){if(e.isDisabled)return;const d=c._context.provides.usehead;if(d)if(d.push){const f=d.push(a);Oe&&le(o,()=>{f.patch(a)})}else Oe?(d.addHeadObjs(k(a)),Nn(()=>d.updateDOM())):d.addHeadObjs(a());else{let h=function(){if(typeof document<"u"&&!f){const m=document.createElement("style");m.type="text/css",m.id="vuetify-theme-stylesheet",e.cspNonce&&m.setAttribute("nonce",e.cspNonce),f=m,document.head.appendChild(f)}f&&(f.innerHTML=o.value)},f=Oe?document.getElementById("vuetify-theme-stylesheet"):null;Oe?le(o,h,{immediate:!0}):h()}}const u=k(()=>e.isDisabled?void 0:`v-theme--${n.value}`);return{install:l,isDisabled:e.isDisabled,name:n,themes:s,current:i,computedThemes:r,themeClasses:u,styles:o,global:{name:n,current:i}}}function Ft(t){qe("provideTheme");const e=Ne(Oi,null);if(!e)throw new Error("Could not find Vuetify theme injection");const n=k(()=>t.theme??e.name.value),s=k(()=>e.themes.value[n.value]),r=k(()=>e.isDisabled?void 0:`v-theme--${n.value}`),i={...e,name:n,current:s,themeClasses:r};return ut(Oi,i),i}function jn(t,e,n){t.push(`${e} {
`,...n.map(s=>`  ${s};
`),`}
`)}function Ku(t){const e=t.dark?2:1,n=t.dark?1:2,s=[];for(const[r,i]of Object.entries(t.colors)){const o=jt(i);s.push(`--v-theme-${r}: ${o.r},${o.g},${o.b}`),r.startsWith("on-")||s.push(`--v-theme-${r}-overlay-multiplier: ${Op(i)>.18?e:n}`)}for(const[r,i]of Object.entries(t.variables)){const o=typeof i=="string"&&i.startsWith("#")?jt(i):void 0,a=o?`${o.r}, ${o.g}, ${o.b}`:void 0;s.push(`--v-${r}: ${a??i}`)}return s}const Je=Y({tag:{type:String,default:"div"}},"tag"),ay=Y({text:String,...we(),...Je()},"VToolbarTitle"),Ni=me()({name:"VToolbarTitle",props:ay(),setup(t,e){let{slots:n}=e;return ye(()=>{const s=!!(n.default||n.text||t.text);return w(t.tag,{class:["v-toolbar-title",t.class],style:t.style},{default:()=>{var r;return[s&&w("div",{class:"v-toolbar-title__placeholder"},[n.text?n.text():t.text,(r=n.default)==null?void 0:r.call(n)])]}})}),{}}}),ly=Y({disabled:Boolean,group:Boolean,hideOnLeave:Boolean,leaveAbsolute:Boolean,mode:String,origin:String},"transition");function wt(t,e,n){return me()({name:t,props:ly({mode:n,origin:e}),setup(s,r){let{slots:i}=r;const o={onBeforeEnter(a){s.origin&&(a.style.transformOrigin=s.origin)},onLeave(a){if(s.leaveAbsolute){const{offsetTop:l,offsetLeft:u,offsetWidth:c,offsetHeight:d}=a;a._transitionInitialStyles={position:a.style.position,top:a.style.top,left:a.style.left,width:a.style.width,height:a.style.height},a.style.position="absolute",a.style.top=`${l}px`,a.style.left=`${u}px`,a.style.width=`${c}px`,a.style.height=`${d}px`}s.hideOnLeave&&a.style.setProperty("display","none","important")},onAfterLeave(a){if(s.leaveAbsolute&&(a!=null&&a._transitionInitialStyles)){const{position:l,top:u,left:c,width:d,height:f}=a._transitionInitialStyles;delete a._transitionInitialStyles,a.style.position=l||"",a.style.top=u||"",a.style.left=c||"",a.style.width=d||"",a.style.height=f||""}}};return()=>{const a=s.group?df:Xt;return Bn(a,{name:s.disabled?"":t,css:!s.disabled,...s.group?void 0:{mode:s.mode},...s.disabled?{}:o},i.default)}}})}function Of(t,e){let n=arguments.length>2&&arguments[2]!==void 0?arguments[2]:"in-out";return me()({name:t,props:{mode:{type:String,default:n},disabled:Boolean},setup(s,r){let{slots:i}=r;return()=>Bn(Xt,{name:s.disabled?"":t,css:!s.disabled,...s.disabled?{}:e},i.default)}})}function Nf(){let t=arguments.length>0&&arguments[0]!==void 0?arguments[0]:"";const n=(arguments.length>1&&arguments[1]!==void 0?arguments[1]:!1)?"width":"height",s=xt(`offset-${n}`);return{onBeforeEnter(o){o._parent=o.parentNode,o._initialStyle={transition:o.style.transition,overflow:o.style.overflow,[n]:o.style[n]}},onEnter(o){const a=o._initialStyle;o.style.setProperty("transition","none","important"),o.style.overflow="hidden";const l=`${o[s]}px`;o.style[n]="0",o.offsetHeight,o.style.transition=a.transition,t&&o._parent&&o._parent.classList.add(t),requestAnimationFrame(()=>{o.style[n]=l})},onAfterEnter:i,onEnterCancelled:i,onLeave(o){o._initialStyle={transition:"",overflow:o.style.overflow,[n]:o.style[n]},o.style.overflow="hidden",o.style[n]=`${o[s]}px`,o.offsetHeight,requestAnimationFrame(()=>o.style[n]="0")},onAfterLeave:r,onLeaveCancelled:r};function r(o){t&&o._parent&&o._parent.classList.remove(t),i(o)}function i(o){const a=o._initialStyle[n];o.style.overflow=o._initialStyle.overflow,a!=null&&(o.style[n]=a),delete o._initialStyle}}const uy=Y({target:[Object,Array]},"v-dialog-transition"),cy=me()({name:"VDialogTransition",props:uy(),setup(t,e){let{slots:n}=e;const s={onBeforeEnter(r){r.style.pointerEvents="none",r.style.visibility="hidden"},async onEnter(r,i){var f;await new Promise(h=>requestAnimationFrame(h)),await new Promise(h=>requestAnimationFrame(h)),r.style.visibility="";const{x:o,y:a,sx:l,sy:u,speed:c}=Qu(t.target,r),d=Zs(r,[{transform:`translate(${o}px, ${a}px) scale(${l}, ${u})`,opacity:0},{}],{duration:225*c,easing:zp});(f=Ju(r))==null||f.forEach(h=>{Zs(h,[{opacity:0},{opacity:0,offset:.33},{}],{duration:225*2*c,easing:_a})}),d.finished.then(()=>i())},onAfterEnter(r){r.style.removeProperty("pointer-events")},onBeforeLeave(r){r.style.pointerEvents="none"},async onLeave(r,i){var f;await new Promise(h=>requestAnimationFrame(h));const{x:o,y:a,sx:l,sy:u,speed:c}=Qu(t.target,r);Zs(r,[{},{transform:`translate(${o}px, ${a}px) scale(${l}, ${u})`,opacity:0}],{duration:125*c,easing:Hp}).finished.then(()=>i()),(f=Ju(r))==null||f.forEach(h=>{Zs(h,[{},{opacity:0,offset:.2},{opacity:0}],{duration:125*2*c,easing:_a})})},onAfterLeave(r){r.style.removeProperty("pointer-events")}};return()=>t.target?w(Xt,Pe({name:"dialog-transition"},s,{css:!1}),n):w(Xt,{name:"dialog-transition"},n)}});function Ju(t){var n;const e=(n=t.querySelector(":scope > .v-card, :scope > .v-sheet, :scope > .v-list"))==null?void 0:n.children;return e&&[...e]}function Qu(t,e){const n=_f(t),s=Sf(e),[r,i]=getComputedStyle(e).transformOrigin.split(" ").map(S=>parseFloat(S)),[o,a]=getComputedStyle(e).getPropertyValue("--v-overlay-anchor-origin").split(" ");let l=n.left+n.width/2;o==="left"||a==="left"?l-=n.width/2:(o==="right"||a==="right")&&(l+=n.width/2);let u=n.top+n.height/2;o==="top"||a==="top"?u-=n.height/2:(o==="bottom"||a==="bottom")&&(u+=n.height/2);const c=n.width/s.width,d=n.height/s.height,f=Math.max(1,c,d),h=c/f||0,m=d/f||0,p=s.width*s.height/(window.innerWidth*window.innerHeight),v=p>.12?Math.min(1.5,(p-.12)*10+1):1;return{x:l-(r+s.left),y:u-(i+s.top),sx:h,sy:m,speed:v}}wt("fab-transition","center center","out-in");wt("dialog-bottom-transition");wt("dialog-top-transition");const Bw=wt("fade-transition"),dy=wt("scale-transition");wt("scroll-x-transition");wt("scroll-x-reverse-transition");wt("scroll-y-transition");wt("scroll-y-reverse-transition");wt("slide-x-transition");wt("slide-x-reverse-transition");const fy=wt("slide-y-transition");wt("slide-y-reverse-transition");const Bf=Of("expand-transition",Nf()),Dw=Of("expand-x-transition",Nf("",!0)),hy=Y({defaults:Object,disabled:Boolean,reset:[Number,String],root:[Boolean,String],scoped:Boolean},"VDefaultsProvider"),Tt=me(!1)({name:"VDefaultsProvider",props:hy(),setup(t,e){let{slots:n}=e;const{defaults:s,disabled:r,reset:i,root:o,scoped:a}=Qa(t);return mn(s,{reset:i,root:o,scoped:a,disabled:r}),()=>{var l;return(l=n.default)==null?void 0:l.call(n)}}}),ho=Y({height:[Number,String],maxHeight:[Number,String],maxWidth:[Number,String],minHeight:[Number,String],minWidth:[Number,String],width:[Number,String]},"dimension");function mo(t){return{dimensionStyles:k(()=>({height:oe(t.height),maxHeight:oe(t.maxHeight),maxWidth:oe(t.maxWidth),minHeight:oe(t.minHeight),minWidth:oe(t.minWidth),width:oe(t.width)}))}}function my(t){return{aspectStyles:k(()=>{const e=Number(t.aspectRatio);return e?{paddingBottom:String(1/e*100)+"%"}:void 0})}}const Df=Y({aspectRatio:[String,Number],contentClass:String,inline:Boolean,...we(),...ho()},"VResponsive"),ec=me()({name:"VResponsive",props:Df(),setup(t,e){let{slots:n}=e;const{aspectStyles:s}=my(t),{dimensionStyles:r}=mo(t);return ye(()=>{var i;return w("div",{class:["v-responsive",{"v-responsive--inline":t.inline},t.class],style:[r.value,t.style]},[w("div",{class:"v-responsive__sizer",style:s.value},null),(i=n.additional)==null?void 0:i.call(n),n.default&&w("div",{class:["v-responsive__content",t.contentClass]},[n.default()])])}),{}}});function ml(t){return cl(()=>{const e=[],n={};if(t.value.background)if(wa(t.value.background)){if(n.backgroundColor=t.value.background,!t.value.text&&Rp(t.value.background)){const s=jt(t.value.background);if(s.a==null||s.a===1){const r=Af(s);n.color=r,n.caretColor=r}}}else e.push(`bg-${t.value.background}`);return t.value.text&&(wa(t.value.text)?(n.color=t.value.text,n.caretColor=t.value.text):e.push(`text-${t.value.text}`)),{colorClasses:e,colorStyles:n}})}function On(t,e){const n=k(()=>({text:Me(t)?t.value:e?t[e]:null})),{colorClasses:s,colorStyles:r}=ml(n);return{textColorClasses:s,textColorStyles:r}}function At(t,e){const n=k(()=>({background:Me(t)?t.value:e?t[e]:null})),{colorClasses:s,colorStyles:r}=ml(n);return{backgroundColorClasses:s,backgroundColorStyles:r}}const Qt=Y({rounded:{type:[Boolean,Number,String],default:void 0},tile:Boolean},"rounded");function en(t){let e=arguments.length>1&&arguments[1]!==void 0?arguments[1]:Kt();return{roundedClasses:k(()=>{const s=Me(t)?t.value:t.rounded,r=Me(t)?t.value:t.tile,i=[];if(s===!0||s==="")i.push(`${e}--rounded`);else if(typeof s=="string"||s===0)for(const o of String(s).split(" "))i.push(`rounded-${o}`);else r&&i.push("rounded-0");return i})}}const gl=Y({transition:{type:[Boolean,String,Object],default:"fade-transition",validator:t=>t!==!0}},"transition"),Es=(t,e)=>{let{slots:n}=e;const{transition:s,disabled:r,group:i,...o}=t,{component:a=i?df:Xt,...l}=typeof s=="object"?s:{};return Bn(a,Pe(typeof s=="string"?{name:r?"":s}:l,typeof s=="string"?{}:Object.fromEntries(Object.entries({disabled:r,group:i}).filter(u=>{let[c,d]=u;return d!==void 0})),o),n)};function gy(t,e){if(!al)return;const n=e.modifiers||{},s=e.value,{handler:r,options:i}=typeof s=="object"?s:{handler:s,options:{}},o=new IntersectionObserver(function(){var d;let a=arguments.length>0&&arguments[0]!==void 0?arguments[0]:[],l=arguments.length>1?arguments[1]:void 0;const u=(d=t._observe)==null?void 0:d[e.instance.$.uid];if(!u)return;const c=a.some(f=>f.isIntersecting);r&&(!n.quiet||u.init)&&(!n.once||c||u.init)&&r(c,a,l),c&&n.once?zf(t,e):u.init=!0},i);t._observe=Object(t._observe),t._observe[e.instance.$.uid]={init:!1,observer:o},o.observe(t)}function zf(t,e){var s;const n=(s=t._observe)==null?void 0:s[e.instance.$.uid];n&&(n.observer.unobserve(t),delete t._observe[e.instance.$.uid])}const Hf={mounted:gy,unmounted:zf},vy=Y({alt:String,cover:Boolean,color:String,draggable:{type:[Boolean,String],default:void 0},eager:Boolean,gradient:String,lazySrc:String,options:{type:Object,default:()=>({root:void 0,rootMargin:void 0,threshold:void 0})},sizes:String,src:{type:[String,Object],default:""},crossorigin:String,referrerpolicy:String,srcset:String,position:String,...Df(),...we(),...Qt(),...gl()},"VImg"),go=me()({name:"VImg",directives:{intersect:Hf},props:vy(),emits:{loadstart:t=>!0,load:t=>!0,error:t=>!0},setup(t,e){let{emit:n,slots:s}=e;const{backgroundColorClasses:r,backgroundColorStyles:i}=At(fe(t,"color")),{roundedClasses:o}=en(t),a=qe("VImg"),l=be(""),u=ee(),c=be(t.eager?"loading":"idle"),d=be(),f=be(),h=k(()=>t.src&&typeof t.src=="object"?{src:t.src.src,srcset:t.srcset||t.src.srcset,lazySrc:t.lazySrc||t.src.lazySrc,aspect:Number(t.aspectRatio||t.src.aspect||0)}:{src:t.src,srcset:t.srcset,lazySrc:t.lazySrc,aspect:Number(t.aspectRatio||0)}),m=k(()=>h.value.aspect||d.value/f.value||0);le(()=>t.src,()=>{p(c.value!=="idle")}),le(m,(T,L)=>{!T&&L&&u.value&&b(u.value)}),io(()=>p());function p(T){if(!(t.eager&&T)&&!(al&&!T&&!t.eager)){if(c.value="loading",h.value.lazySrc){const L=new Image;L.src=h.value.lazySrc,b(L,null)}h.value.src&&It(()=>{var L;n("loadstart",((L=u.value)==null?void 0:L.currentSrc)||h.value.src),setTimeout(()=>{var A;if(!a.isUnmounted)if((A=u.value)!=null&&A.complete){if(u.value.naturalWidth||S(),c.value==="error")return;m.value||b(u.value,null),c.value==="loading"&&v()}else m.value||b(u.value),g()})})}}function v(){var T;a.isUnmounted||(g(),b(u.value),c.value="loaded",n("load",((T=u.value)==null?void 0:T.currentSrc)||h.value.src))}function S(){var T;a.isUnmounted||(c.value="error",n("error",((T=u.value)==null?void 0:T.currentSrc)||h.value.src))}function g(){const T=u.value;T&&(l.value=T.currentSrc||T.src)}let y=-1;kt(()=>{clearTimeout(y)});function b(T){let L=arguments.length>1&&arguments[1]!==void 0?arguments[1]:100;const A=()=>{if(clearTimeout(y),a.isUnmounted)return;const{naturalHeight:N,naturalWidth:$}=T;N||$?(d.value=$,f.value=N):!T.complete&&c.value==="loading"&&L!=null?y=window.setTimeout(A,L):(T.currentSrc.endsWith(".svg")||T.currentSrc.startsWith("data:image/svg+xml"))&&(d.value=1,f.value=1)};A()}const x=k(()=>({"v-img__img--cover":t.cover,"v-img__img--contain":!t.cover})),_=()=>{var A;if(!h.value.src||c.value==="idle")return null;const T=w("img",{class:["v-img__img",x.value],style:{objectPosition:t.position},src:h.value.src,srcset:h.value.srcset,alt:t.alt,crossorigin:t.crossorigin,referrerpolicy:t.referrerpolicy,draggable:t.draggable,sizes:t.sizes,ref:u,onLoad:v,onError:S},null),L=(A=s.sources)==null?void 0:A.call(s);return w(Es,{transition:t.transition,appear:!0},{default:()=>[qt(L?w("picture",{class:"v-img__picture"},[L,T]):T,[[uo,c.value==="loaded"]])]})},C=()=>w(Es,{transition:t.transition},{default:()=>[h.value.lazySrc&&c.value!=="loaded"&&w("img",{class:["v-img__img","v-img__img--preload",x.value],style:{objectPosition:t.position},src:h.value.lazySrc,alt:t.alt,crossorigin:t.crossorigin,referrerpolicy:t.referrerpolicy,draggable:t.draggable},null)]}),M=()=>s.placeholder?w(Es,{transition:t.transition,appear:!0},{default:()=>[(c.value==="loading"||c.value==="error"&&!s.error)&&w("div",{class:"v-img__placeholder"},[s.placeholder()])]}):null,R=()=>s.error?w(Es,{transition:t.transition,appear:!0},{default:()=>[c.value==="error"&&w("div",{class:"v-img__error"},[s.error()])]}):null,F=()=>t.gradient?w("div",{class:"v-img__gradient",style:{backgroundImage:`linear-gradient(${t.gradient})`}},null):null,V=be(!1);{const T=le(m,L=>{L&&(requestAnimationFrame(()=>{requestAnimationFrame(()=>{V.value=!0})}),T())})}return ye(()=>{const T=ec.filterProps(t);return qt(w(ec,Pe({class:["v-img",{"v-img--booting":!V.value},r.value,o.value,t.class],style:[{width:oe(t.width==="auto"?d.value:t.width)},i.value,t.style]},T,{aspectRatio:m.value,"aria-label":t.alt,role:t.alt?"img":void 0}),{additional:()=>w(Ie,null,[w(_,null,null),w(C,null,null),w(F,null,null),w(M,null,null),w(R,null,null)]),default:s.default}),[[ls("intersect"),{handler:p,options:t.options},null,{once:!0}]])}),{currentSrc:l,image:u,state:c,naturalWidth:d,naturalHeight:f}}}),Fr=Y({border:[Boolean,Number,String]},"border");function Lr(t){let e=arguments.length>1&&arguments[1]!==void 0?arguments[1]:Kt();return{borderClasses:k(()=>{const s=Me(t)?t.value:t.border,r=[];if(s===!0||s==="")r.push(`${e}--border`);else if(typeof s=="string"||s===0)for(const i of String(s).split(" "))r.push(`border-${i}`);return r})}}const us=Y({elevation:{type:[Number,String],validator(t){const e=parseInt(t);return!isNaN(e)&&e>=0&&e<=24}}},"elevation");function cs(t){return{elevationClasses:k(()=>{const n=Me(t)?t.value:t.elevation,s=[];return n==null||s.push(`elevation-${n}`),s})}}const py=[null,"prominent","default","comfortable","compact"],yy=Y({absolute:Boolean,collapse:Boolean,color:String,density:{type:String,default:"default",validator:t=>py.includes(t)},extended:Boolean,extensionHeight:{type:[Number,String],default:48},flat:Boolean,floating:Boolean,height:{type:[Number,String],default:64},image:String,title:String,...Fr(),...we(),...us(),...Qt(),...Je({tag:"header"}),...dt()},"VToolbar"),$f=me()({name:"VToolbar",props:yy(),setup(t,e){var h;let{slots:n}=e;const{backgroundColorClasses:s,backgroundColorStyles:r}=At(fe(t,"color")),{borderClasses:i}=Lr(t),{elevationClasses:o}=cs(t),{roundedClasses:a}=en(t),{themeClasses:l}=Ft(t),{rtlClasses:u}=Jt(),c=be(!!(t.extended||(h=n.extension)!=null&&h.call(n))),d=k(()=>parseInt(Number(t.height)+(t.density==="prominent"?Number(t.height):0)-(t.density==="comfortable"?8:0)-(t.density==="compact"?16:0),10)),f=k(()=>c.value?parseInt(Number(t.extensionHeight)+(t.density==="prominent"?Number(t.extensionHeight):0)-(t.density==="comfortable"?4:0)-(t.density==="compact"?8:0),10):0);return mn({VBtn:{variant:"text"}}),ye(()=>{var S;const m=!!(t.title||n.title),p=!!(n.image||t.image),v=(S=n.extension)==null?void 0:S.call(n);return c.value=!!(t.extended||v),w(t.tag,{class:["v-toolbar",{"v-toolbar--absolute":t.absolute,"v-toolbar--collapse":t.collapse,"v-toolbar--flat":t.flat,"v-toolbar--floating":t.floating,[`v-toolbar--density-${t.density}`]:!0},s.value,i.value,o.value,a.value,l.value,u.value,t.class],style:[r.value,t.style]},{default:()=>[p&&w("div",{key:"image",class:"v-toolbar__image"},[n.image?w(Tt,{key:"image-defaults",disabled:!t.image,defaults:{VImg:{cover:!0,src:t.image}}},n.image):w(go,{key:"image-img",cover:!0,src:t.image},null)]),w(Tt,{defaults:{VTabs:{height:oe(d.value)}}},{default:()=>{var g,y,b;return[w("div",{class:"v-toolbar__content",style:{height:oe(d.value)}},[n.prepend&&w("div",{class:"v-toolbar__prepend"},[(g=n.prepend)==null?void 0:g.call(n)]),m&&w(Ni,{key:"title",text:t.title},{text:n.title}),(y=n.default)==null?void 0:y.call(n),n.append&&w("div",{class:"v-toolbar__append"},[(b=n.append)==null?void 0:b.call(n)])])]}}),w(Tt,{defaults:{VTabs:{height:oe(f.value)}}},{default:()=>[w(Bf,null,{default:()=>[c.value&&w("div",{class:"v-toolbar__extension",style:{height:oe(f.value)}},[v])]})]})]})}),{contentHeight:d,extensionHeight:f}}});function Uf(){const t=be(!1);return Zt(()=>{window.requestAnimationFrame(()=>{t.value=!0})}),{ssrBootStyles:k(()=>t.value?void 0:{transition:"none !important"}),isBooted:Cr(t)}}const by=[null,"default","comfortable","compact"],ds=Y({density:{type:String,default:"default",validator:t=>by.includes(t)}},"density");function Bs(t){let e=arguments.length>1&&arguments[1]!==void 0?arguments[1]:Kt();return{densityClasses:k(()=>`${e}--density-${t.density}`)}}const xy=["elevated","flat","tonal","outlined","text","plain"];function vl(t,e){return w(Ie,null,[t&&w("span",{key:"overlay",class:`${e}__overlay`},null),w("span",{key:"underlay",class:`${e}__underlay`},null)])}const Vr=Y({color:String,variant:{type:String,default:"elevated",validator:t=>xy.includes(t)}},"variant");function pl(t){let e=arguments.length>1&&arguments[1]!==void 0?arguments[1]:Kt();const n=k(()=>{const{variant:i}=P(t);return`${e}--variant-${i}`}),{colorClasses:s,colorStyles:r}=ml(k(()=>{const{variant:i,color:o}=P(t);return{[["elevated","flat"].includes(i)?"background":"text"]:o}}));return{colorClasses:s,colorStyles:r,variantClasses:n}}const jf=Y({divided:Boolean,...Fr(),...we(),...ds(),...us(),...Qt(),...Je(),...dt(),...Vr()},"VBtnGroup"),tc=me()({name:"VBtnGroup",props:jf(),setup(t,e){let{slots:n}=e;const{themeClasses:s}=Ft(t),{densityClasses:r}=Bs(t),{borderClasses:i}=Lr(t),{elevationClasses:o}=cs(t),{roundedClasses:a}=en(t);mn({VBtn:{height:"auto",color:fe(t,"color"),density:fe(t,"density"),flat:!0,variant:fe(t,"variant")}}),ye(()=>w(t.tag,{class:["v-btn-group",{"v-btn-group--divided":t.divided},s.value,i.value,r.value,o.value,a.value,t.class],style:t.style},n))}}),Wf=Y({modelValue:{type:null,default:void 0},multiple:Boolean,mandatory:[Boolean,String],max:Number,selectedClass:String,disabled:Boolean},"group"),Gf=Y({value:null,disabled:Boolean,selectedClass:String},"group-item");function qf(t,e){let n=arguments.length>2&&arguments[2]!==void 0?arguments[2]:!0;const s=qe("useGroupItem");if(!s)throw new Error("[Vuetify] useGroupItem composable must be used inside a component setup function");const r=gn();ut(Symbol.for(`${e.description}:id`),r);const i=Ne(e,null);if(!i){if(!n)return i;throw new Error(`[Vuetify] Could not find useGroup injection with symbol ${e.description}`)}const o=fe(t,"value"),a=k(()=>!!(i.disabled.value||t.disabled));i.register({id:r,value:o,disabled:a},s),kt(()=>{i.unregister(r)});const l=k(()=>i.isSelected(r)),u=k(()=>l.value&&[i.selectedClass.value,t.selectedClass]);return le(l,c=>{s.emit("group:selected",{value:c})},{flush:"sync"}),{id:r,isSelected:l,toggle:()=>i.select(r,!l.value),select:c=>i.select(r,c),selectedClass:u,value:o,disabled:a,group:i}}function Yf(t,e){let n=!1;const s=tt([]),r=ct(t,"modelValue",[],f=>f==null?[]:Xf(s,Fn(f)),f=>{const h=_y(s,f);return t.multiple?h:h[0]}),i=qe("useGroup");function o(f,h){const m=f,p=Symbol.for(`${e.description}:id`),S=ar(p,i==null?void 0:i.vnode).indexOf(h);P(m.value)==null&&(m.value=S),S>-1?s.splice(S,0,m):s.push(m)}function a(f){if(n)return;l();const h=s.findIndex(m=>m.id===f);s.splice(h,1)}function l(){const f=s.find(h=>!h.disabled);f&&t.mandatory==="force"&&!r.value.length&&(r.value=[f.id])}Zt(()=>{l()}),kt(()=>{n=!0});function u(f,h){const m=s.find(p=>p.id===f);if(!(h&&(m!=null&&m.disabled)))if(t.multiple){const p=r.value.slice(),v=p.findIndex(g=>g===f),S=~v;if(h=h??!S,S&&t.mandatory&&p.length<=1||!S&&t.max!=null&&p.length+1>t.max)return;v<0&&h?p.push(f):v>=0&&!h&&p.splice(v,1),r.value=p}else{const p=r.value.includes(f);if(t.mandatory&&p)return;r.value=h??!p?[f]:[]}}function c(f){if(t.multiple,r.value.length){const h=r.value[0],m=s.findIndex(S=>S.id===h);let p=(m+f)%s.length,v=s[p];for(;v.disabled&&p!==m;)p=(p+f)%s.length,v=s[p];if(v.disabled)return;r.value=[s[p].id]}else{const h=s.find(m=>!m.disabled);h&&(r.value=[h.id])}}const d={register:o,unregister:a,selected:r,select:u,disabled:fe(t,"disabled"),prev:()=>c(s.length-1),next:()=>c(1),isSelected:f=>r.value.includes(f),selectedClass:k(()=>t.selectedClass),items:k(()=>s),getItemIndex:f=>wy(s,f)};return ut(e,d),d}function wy(t,e){const n=Xf(t,[e]);return n.length?t.findIndex(s=>s.id===n[0]):-1}function Xf(t,e){const n=[];return e.forEach(s=>{const r=t.find(o=>co(s,o.value)),i=t[s];(r==null?void 0:r.value)!=null?n.push(r.id):i!=null&&n.push(i.id)}),n}function _y(t,e){const n=[];return e.forEach(s=>{const r=t.findIndex(i=>i.id===s);if(~r){const i=t[r];n.push(i.value!=null?i.value:r)}}),n}const Zf=Symbol.for("vuetify:v-btn-toggle"),Sy=Y({...jf(),...Wf()},"VBtnToggle"),Cy=me()({name:"VBtnToggle",props:Sy(),emits:{"update:modelValue":t=>!0},setup(t,e){let{slots:n}=e;const{isSelected:s,next:r,prev:i,select:o,selected:a}=Yf(t,Zf);return ye(()=>{const l=tc.filterProps(t);return w(tc,Pe({class:["v-btn-toggle",t.class]},l,{style:t.style}),{default:()=>{var u;return[(u=n.default)==null?void 0:u.call(n,{isSelected:s,next:r,prev:i,select:o,selected:a})]}})}),{next:r,prev:i,select:o}}}),Ey={collapse:"mdi-chevron-up",complete:"mdi-check",cancel:"mdi-close-circle",close:"mdi-close",delete:"mdi-close-circle",clear:"mdi-close-circle",success:"mdi-check-circle",info:"mdi-information",warning:"mdi-alert-circle",error:"mdi-close-circle",prev:"mdi-chevron-left",next:"mdi-chevron-right",checkboxOn:"mdi-checkbox-marked",checkboxOff:"mdi-checkbox-blank-outline",checkboxIndeterminate:"mdi-minus-box",delimiter:"mdi-circle",sortAsc:"mdi-arrow-up",sortDesc:"mdi-arrow-down",expand:"mdi-chevron-down",menu:"mdi-menu",subgroup:"mdi-menu-down",dropdown:"mdi-menu-down",radioOn:"mdi-radiobox-marked",radioOff:"mdi-radiobox-blank",edit:"mdi-pencil",ratingEmpty:"mdi-star-outline",ratingFull:"mdi-star",ratingHalf:"mdi-star-half-full",loading:"mdi-cached",first:"mdi-page-first",last:"mdi-page-last",unfold:"mdi-unfold-more-horizontal",file:"mdi-paperclip",plus:"mdi-plus",minus:"mdi-minus",calendar:"mdi-calendar",treeviewCollapse:"mdi-menu-down",treeviewExpand:"mdi-menu-right",eyeDropper:"mdi-eyedropper"},My={component:t=>Bn(Jf,{...t,class:"mdi"})},Ke=[String,Function,Object,Array],Sa=Symbol.for("vuetify:icons"),vo=Y({icon:{type:Ke},tag:{type:String,required:!0}},"icon"),nc=me()({name:"VComponentIcon",props:vo(),setup(t,e){let{slots:n}=e;return()=>{const s=t.icon;return w(t.tag,null,{default:()=>{var r;return[t.icon?w(s,null,null):(r=n.default)==null?void 0:r.call(n)]}})}}}),Kf=kr({name:"VSvgIcon",inheritAttrs:!1,props:vo(),setup(t,e){let{attrs:n}=e;return()=>w(t.tag,Pe(n,{style:null}),{default:()=>[w("svg",{class:"v-icon__svg",xmlns:"http://www.w3.org/2000/svg",viewBox:"0 0 24 24",role:"img","aria-hidden":"true"},[Array.isArray(t.icon)?t.icon.map(s=>Array.isArray(s)?w("path",{d:s[0],"fill-opacity":s[1]},null):w("path",{d:s},null)):w("path",{d:t.icon},null)])]})}});kr({name:"VLigatureIcon",props:vo(),setup(t){return()=>w(t.tag,null,{default:()=>[t.icon]})}});const Jf=kr({name:"VClassIcon",props:vo(),setup(t){return()=>w(t.tag,{class:t.icon},null)}});function Ty(){return{svg:{component:Kf},class:{component:Jf}}}function Ay(t){const e=Ty(),n=(t==null?void 0:t.defaultSet)??"mdi";return n==="mdi"&&!e.mdi&&(e.mdi=My),pt({defaultSet:n,sets:e,aliases:{...Ey,vuetify:["M8.2241 14.2009L12 21L22 3H14.4459L8.2241 14.2009Z",["M7.26303 12.4733L7.00113 12L2 3H12.5261C12.5261 3 12.5261 3 12.5261 3L7.26303 12.4733Z",.6]],"vuetify-outline":"svg:M7.26 12.47 12.53 3H2L7.26 12.47ZM14.45 3 8.22 14.2 12 21 22 3H14.45ZM18.6 5 12 16.88 10.51 14.2 15.62 5ZM7.26 8.35 5.4 5H9.13L7.26 8.35Z"}},t)}const Ry=t=>{const e=Ne(Sa);if(!e)throw new Error("Missing Vuetify Icons provide!");return{iconData:k(()=>{var l;const s=P(t);if(!s)return{component:nc};let r=s;if(typeof r=="string"&&(r=r.trim(),r.startsWith("$")&&(r=(l=e.aliases)==null?void 0:l[r.slice(1)])),!r)throw new Error(`Could not find aliased icon "${s}"`);if(Array.isArray(r))return{component:Kf,icon:r};if(typeof r!="string")return{component:nc,icon:r};const i=Object.keys(e.sets).find(u=>typeof r=="string"&&r.startsWith(`${u}:`)),o=i?r.slice(i.length+1):r;return{component:e.sets[i??e.defaultSet].component,icon:o}})}},Iy=["x-small","small","default","large","x-large"],po=Y({size:{type:[String,Number],default:"default"}},"size");function yo(t){let e=arguments.length>1&&arguments[1]!==void 0?arguments[1]:Kt();return cl(()=>{let n,s;return Ii(Iy,t.size)?n=`${e}--size-${t.size}`:t.size&&(s={width:oe(t.size),height:oe(t.size)}),{sizeClasses:n,sizeStyles:s}})}const ky=Y({color:String,disabled:Boolean,start:Boolean,end:Boolean,icon:Ke,...we(),...po(),...Je({tag:"i"}),...dt()},"VIcon"),ln=me()({name:"VIcon",props:ky(),setup(t,e){let{attrs:n,slots:s}=e;const r=ee(),{themeClasses:i}=Ft(t),{iconData:o}=Ry(k(()=>r.value||t.icon)),{sizeClasses:a}=yo(t),{textColorClasses:l,textColorStyles:u}=On(fe(t,"color"));return ye(()=>{var f,h;const c=(f=s.default)==null?void 0:f.call(s);c&&(r.value=(h=pf(c).filter(m=>m.type===Tr&&m.children&&typeof m.children=="string")[0])==null?void 0:h.children);const d=!!(n.onClick||n.onClickOnce);return w(o.value.component,{tag:t.tag,icon:o.value.icon,class:["v-icon","notranslate",i.value,a.value,l.value,{"v-icon--clickable":d,"v-icon--disabled":t.disabled,"v-icon--start":t.start,"v-icon--end":t.end},t.class],style:[a.value?void 0:{fontSize:oe(t.size),height:oe(t.size),width:oe(t.size)},u.value,t.style],role:d?"button":void 0,"aria-hidden":!d,tabindex:d?t.disabled?-1:0:void 0},{default:()=>[c]})}),{}}});function Qf(t,e){const n=ee(),s=be(!1);if(al){const r=new IntersectionObserver(i=>{t==null||t(i,r),s.value=!!i.find(o=>o.isIntersecting)},e);kt(()=>{r.disconnect()}),le(n,(i,o)=>{o&&(r.unobserve(o),s.value=!1),i&&r.observe(i)},{flush:"post"})}return{intersectionRef:n,isIntersecting:s}}const Fy=Y({bgColor:String,color:String,indeterminate:[Boolean,String],modelValue:{type:[Number,String],default:0},rotate:{type:[Number,String],default:0},width:{type:[Number,String],default:4},...we(),...po(),...Je({tag:"div"}),...dt()},"VProgressCircular"),eh=me()({name:"VProgressCircular",props:Fy(),setup(t,e){let{slots:n}=e;const s=20,r=2*Math.PI*s,i=ee(),{themeClasses:o}=Ft(t),{sizeClasses:a,sizeStyles:l}=yo(t),{textColorClasses:u,textColorStyles:c}=On(fe(t,"color")),{textColorClasses:d,textColorStyles:f}=On(fe(t,"bgColor")),{intersectionRef:h,isIntersecting:m}=Qf(),{resizeRef:p,contentRect:v}=kf(),S=k(()=>Math.max(0,Math.min(100,parseFloat(t.modelValue)))),g=k(()=>Number(t.width)),y=k(()=>l.value?Number(t.size):v.value?v.value.width:Math.max(g.value,32)),b=k(()=>s/(1-g.value/y.value)*2),x=k(()=>g.value/y.value*b.value),_=k(()=>oe((100-S.value)/100*r));return Nn(()=>{h.value=i.value,p.value=i.value}),ye(()=>w(t.tag,{ref:i,class:["v-progress-circular",{"v-progress-circular--indeterminate":!!t.indeterminate,"v-progress-circular--visible":m.value,"v-progress-circular--disable-shrink":t.indeterminate==="disable-shrink"},o.value,a.value,u.value,t.class],style:[l.value,c.value,t.style],role:"progressbar","aria-valuemin":"0","aria-valuemax":"100","aria-valuenow":t.indeterminate?void 0:S.value},{default:()=>[w("svg",{style:{transform:`rotate(calc(-90deg + ${Number(t.rotate)}deg))`},xmlns:"http://www.w3.org/2000/svg",viewBox:`0 0 ${b.value} ${b.value}`},[w("circle",{class:["v-progress-circular__underlay",d.value],style:f.value,fill:"transparent",cx:"50%",cy:"50%",r:s,"stroke-width":x.value,"stroke-dasharray":r,"stroke-dashoffset":0},null),w("circle",{class:"v-progress-circular__overlay",fill:"transparent",cx:"50%",cy:"50%",r:s,"stroke-width":x.value,"stroke-dasharray":r,"stroke-dashoffset":_.value},null)]),n.default&&w("div",{class:"v-progress-circular__content"},[n.default({value:S.value})])]})),{}}}),sc={center:"center",top:"bottom",bottom:"top",left:"right",right:"left"},yl=Y({location:String},"location");function bl(t){let e=arguments.length>1&&arguments[1]!==void 0?arguments[1]:!1,n=arguments.length>2?arguments[2]:void 0;const{isRtl:s}=Jt();return{locationStyles:k(()=>{if(!t.location)return{};const{side:i,align:o}=ba(t.location.split(" ").length>1?t.location:`${t.location} center`,s.value);function a(u){return n?n(u):0}const l={};return i!=="center"&&(e?l[sc[i]]=`calc(100% - ${a(i)}px)`:l[i]=0),o!=="center"?e?l[sc[o]]=`calc(100% - ${a(o)}px)`:l[o]=0:(i==="center"?l.top=l.left="50%":l[{top:"left",bottom:"left",left:"top",right:"top"}[i]]="50%",l.transform={top:"translateX(-50%)",bottom:"translateX(-50%)",left:"translateY(-50%)",right:"translateY(-50%)",center:"translate(-50%, -50%)"}[i]),l})}}const Ly=Y({absolute:Boolean,active:{type:Boolean,default:!0},bgColor:String,bgOpacity:[Number,String],bufferValue:{type:[Number,String],default:0},clickable:Boolean,color:String,height:{type:[Number,String],default:4},indeterminate:Boolean,max:{type:[Number,String],default:100},modelValue:{type:[Number,String],default:0},reverse:Boolean,stream:Boolean,striped:Boolean,roundedBar:Boolean,...we(),...yl({location:"top"}),...Qt(),...Je(),...dt()},"VProgressLinear"),Vy=me()({name:"VProgressLinear",props:Ly(),emits:{"update:modelValue":t=>!0},setup(t,e){let{slots:n}=e;const s=ct(t,"modelValue"),{isRtl:r,rtlClasses:i}=Jt(),{themeClasses:o}=Ft(t),{locationStyles:a}=bl(t),{textColorClasses:l,textColorStyles:u}=On(t,"color"),{backgroundColorClasses:c,backgroundColorStyles:d}=At(k(()=>t.bgColor||t.color)),{backgroundColorClasses:f,backgroundColorStyles:h}=At(t,"color"),{roundedClasses:m}=en(t),{intersectionRef:p,isIntersecting:v}=Qf(),S=k(()=>parseInt(t.max,10)),g=k(()=>parseInt(t.height,10)),y=k(()=>parseFloat(t.bufferValue)/S.value*100),b=k(()=>parseFloat(s.value)/S.value*100),x=k(()=>r.value!==t.reverse),_=k(()=>t.indeterminate?"fade-transition":"slide-x-transition"),C=k(()=>t.bgOpacity==null?t.bgOpacity:parseFloat(t.bgOpacity));function M(R){if(!p.value)return;const{left:F,right:V,width:T}=p.value.getBoundingClientRect(),L=x.value?T-R.clientX+(V-T):R.clientX-F;s.value=Math.round(L/T*S.value)}return ye(()=>w(t.tag,{ref:p,class:["v-progress-linear",{"v-progress-linear--absolute":t.absolute,"v-progress-linear--active":t.active&&v.value,"v-progress-linear--reverse":x.value,"v-progress-linear--rounded":t.rounded,"v-progress-linear--rounded-bar":t.roundedBar,"v-progress-linear--striped":t.striped},m.value,o.value,i.value,t.class],style:[{bottom:t.location==="bottom"?0:void 0,top:t.location==="top"?0:void 0,height:t.active?oe(g.value):0,"--v-progress-linear-height":oe(g.value),...a.value},t.style],role:"progressbar","aria-hidden":t.active?"false":"true","aria-valuemin":"0","aria-valuemax":t.max,"aria-valuenow":t.indeterminate?void 0:b.value,onClick:t.clickable&&M},{default:()=>[t.stream&&w("div",{key:"stream",class:["v-progress-linear__stream",l.value],style:{...u.value,[x.value?"left":"right"]:oe(-g.value),borderTop:`${oe(g.value/2)} dotted`,opacity:C.value,top:`calc(50% - ${oe(g.value/4)})`,width:oe(100-y.value,"%"),"--v-progress-linear-stream-to":oe(g.value*(x.value?1:-1))}},null),w("div",{class:["v-progress-linear__background",c.value],style:[d.value,{opacity:C.value,width:oe(t.stream?y.value:100,"%")}]},null),w(Xt,{name:_.value},{default:()=>[t.indeterminate?w("div",{class:"v-progress-linear__indeterminate"},[["long","short"].map(R=>w("div",{key:R,class:["v-progress-linear__indeterminate",R,f.value],style:h.value},null))]):w("div",{class:["v-progress-linear__determinate",f.value],style:[h.value,{width:oe(b.value,"%")}]},null)]}),n.default&&w("div",{class:"v-progress-linear__content"},[n.default({value:b.value,buffer:y.value})])]})),{}}}),th=Y({loading:[Boolean,String]},"loader");function nh(t){let e=arguments.length>1&&arguments[1]!==void 0?arguments[1]:Kt();return{loaderClasses:k(()=>({[`${e}--loading`]:t.loading}))}}function Py(t,e){var s;let{slots:n}=e;return w("div",{class:`${t.name}__loader`},[((s=n.default)==null?void 0:s.call(n,{color:t.color,isActive:t.active}))||w(Vy,{absolute:t.absolute,active:t.active,color:t.color,height:"2",indeterminate:!0},null)])}const Oy=["static","relative","fixed","absolute","sticky"],sh=Y({position:{type:String,validator:t=>Oy.includes(t)}},"position");function rh(t){let e=arguments.length>1&&arguments[1]!==void 0?arguments[1]:Kt();return{positionClasses:k(()=>t.position?`${e}--${t.position}`:void 0)}}function Ny(){const t=qe("useRoute");return k(()=>{var e;return(e=t==null?void 0:t.proxy)==null?void 0:e.$route})}function ih(){var t,e;return(e=(t=qe("useRouter"))==null?void 0:t.proxy)==null?void 0:e.$router}function oh(t,e){var u,c;const n=bg("RouterLink"),s=k(()=>!!(t.href||t.to)),r=k(()=>(s==null?void 0:s.value)||Lu(e,"click")||Lu(t,"click"));if(typeof n=="string"||!("useLink"in n))return{isLink:s,isClickable:r,href:fe(t,"href")};const i=k(()=>({...t,to:t.to?t.to:{}})),o=n.useLink(i.value),a=k(()=>t.to?o:void 0),l=Ny();return{isLink:s,isClickable:r,route:(u=a.value)==null?void 0:u.route,navigate:(c=a.value)==null?void 0:c.navigate,isActive:k(()=>{var d,f,h;return a.value?t.exact?l.value?((h=a.value.isExactActive)==null?void 0:h.value)&&co(a.value.route.value.query,l.value.query):((f=a.value.isExactActive)==null?void 0:f.value)??!1:((d=a.value.isActive)==null?void 0:d.value)??!1:!1}),href:k(()=>{var d;return t.to?(d=a.value)==null?void 0:d.route.value.href:t.href})}}const ah=Y({href:String,replace:Boolean,to:[String,Object],exact:Boolean},"router");let Ho=!1;function By(t,e){let n=!1,s,r;Oe&&(It(()=>{window.addEventListener("popstate",i),s=t==null?void 0:t.beforeEach((o,a,l)=>{Ho?n?e(l):l():setTimeout(()=>n?e(l):l()),Ho=!0}),r=t==null?void 0:t.afterEach(()=>{Ho=!1})}),Rt(()=>{window.removeEventListener("popstate",i),s==null||s(),r==null||r()}));function i(o){var a;(a=o.state)!=null&&a.replaced||(n=!0,setTimeout(()=>n=!1))}}function Dy(t,e){le(()=>{var n;return(n=t.isActive)==null?void 0:n.value},n=>{t.isLink.value&&n&&e&&It(()=>{e(!0)})},{immediate:!0})}const Ca=Symbol("rippleStop"),zy=80;function rc(t,e){t.style.transform=e,t.style.webkitTransform=e}function Ea(t){return t.constructor.name==="TouchEvent"}function lh(t){return t.constructor.name==="KeyboardEvent"}const Hy=function(t,e){var d;let n=arguments.length>2&&arguments[2]!==void 0?arguments[2]:{},s=0,r=0;if(!lh(t)){const f=e.getBoundingClientRect(),h=Ea(t)?t.touches[t.touches.length-1]:t;s=h.clientX-f.left,r=h.clientY-f.top}let i=0,o=.3;(d=e._ripple)!=null&&d.circle?(o=.15,i=e.clientWidth/2,i=n.center?i:i+Math.sqrt((s-i)**2+(r-i)**2)/4):i=Math.sqrt(e.clientWidth**2+e.clientHeight**2)/2;const a=`${(e.clientWidth-i*2)/2}px`,l=`${(e.clientHeight-i*2)/2}px`,u=n.center?a:`${s-i}px`,c=n.center?l:`${r-i}px`;return{radius:i,scale:o,x:u,y:c,centerX:a,centerY:l}},Bi={show(t,e){var h;let n=arguments.length>2&&arguments[2]!==void 0?arguments[2]:{};if(!((h=e==null?void 0:e._ripple)!=null&&h.enabled))return;const s=document.createElement("span"),r=document.createElement("span");s.appendChild(r),s.className="v-ripple__container",n.class&&(s.className+=` ${n.class}`);const{radius:i,scale:o,x:a,y:l,centerX:u,centerY:c}=Hy(t,e,n),d=`${i*2}px`;r.className="v-ripple__animation",r.style.width=d,r.style.height=d,e.appendChild(s);const f=window.getComputedStyle(e);f&&f.position==="static"&&(e.style.position="relative",e.dataset.previousPosition="static"),r.classList.add("v-ripple__animation--enter"),r.classList.add("v-ripple__animation--visible"),rc(r,`translate(${a}, ${l}) scale3d(${o},${o},${o})`),r.dataset.activated=String(performance.now()),setTimeout(()=>{r.classList.remove("v-ripple__animation--enter"),r.classList.add("v-ripple__animation--in"),rc(r,`translate(${u}, ${c}) scale3d(1,1,1)`)},0)},hide(t){var i;if(!((i=t==null?void 0:t._ripple)!=null&&i.enabled))return;const e=t.getElementsByClassName("v-ripple__animation");if(e.length===0)return;const n=e[e.length-1];if(n.dataset.isHiding)return;n.dataset.isHiding="true";const s=performance.now()-Number(n.dataset.activated),r=Math.max(250-s,0);setTimeout(()=>{n.classList.remove("v-ripple__animation--in"),n.classList.add("v-ripple__animation--out"),setTimeout(()=>{var a;t.getElementsByClassName("v-ripple__animation").length===1&&t.dataset.previousPosition&&(t.style.position=t.dataset.previousPosition,delete t.dataset.previousPosition),((a=n.parentNode)==null?void 0:a.parentNode)===t&&t.removeChild(n.parentNode)},300)},r)}};function uh(t){return typeof t>"u"||!!t}function xr(t){const e={},n=t.currentTarget;if(!(!(n!=null&&n._ripple)||n._ripple.touched||t[Ca])){if(t[Ca]=!0,Ea(t))n._ripple.touched=!0,n._ripple.isTouch=!0;else if(n._ripple.isTouch)return;if(e.center=n._ripple.centered||lh(t),n._ripple.class&&(e.class=n._ripple.class),Ea(t)){if(n._ripple.showTimerCommit)return;n._ripple.showTimerCommit=()=>{Bi.show(t,n,e)},n._ripple.showTimer=window.setTimeout(()=>{var s;(s=n==null?void 0:n._ripple)!=null&&s.showTimerCommit&&(n._ripple.showTimerCommit(),n._ripple.showTimerCommit=null)},zy)}else Bi.show(t,n,e)}}function ic(t){t[Ca]=!0}function vt(t){const e=t.currentTarget;if(e!=null&&e._ripple){if(window.clearTimeout(e._ripple.showTimer),t.type==="touchend"&&e._ripple.showTimerCommit){e._ripple.showTimerCommit(),e._ripple.showTimerCommit=null,e._ripple.showTimer=window.setTimeout(()=>{vt(t)});return}window.setTimeout(()=>{e._ripple&&(e._ripple.touched=!1)}),Bi.hide(e)}}function ch(t){const e=t.currentTarget;e!=null&&e._ripple&&(e._ripple.showTimerCommit&&(e._ripple.showTimerCommit=null),window.clearTimeout(e._ripple.showTimer))}let wr=!1;function dh(t){!wr&&(t.keyCode===Au.enter||t.keyCode===Au.space)&&(wr=!0,xr(t))}function fh(t){wr=!1,vt(t)}function hh(t){wr&&(wr=!1,vt(t))}function mh(t,e,n){const{value:s,modifiers:r}=e,i=uh(s);if(i||Bi.hide(t),t._ripple=t._ripple??{},t._ripple.enabled=i,t._ripple.centered=r.center,t._ripple.circle=r.circle,ya(s)&&s.class&&(t._ripple.class=s.class),i&&!n){if(r.stop){t.addEventListener("touchstart",ic,{passive:!0}),t.addEventListener("mousedown",ic);return}t.addEventListener("touchstart",xr,{passive:!0}),t.addEventListener("touchend",vt,{passive:!0}),t.addEventListener("touchmove",ch,{passive:!0}),t.addEventListener("touchcancel",vt),t.addEventListener("mousedown",xr),t.addEventListener("mouseup",vt),t.addEventListener("mouseleave",vt),t.addEventListener("keydown",dh),t.addEventListener("keyup",fh),t.addEventListener("blur",hh),t.addEventListener("dragstart",vt,{passive:!0})}else!i&&n&&gh(t)}function gh(t){t.removeEventListener("mousedown",xr),t.removeEventListener("touchstart",xr),t.removeEventListener("touchend",vt),t.removeEventListener("touchmove",ch),t.removeEventListener("touchcancel",vt),t.removeEventListener("mouseup",vt),t.removeEventListener("mouseleave",vt),t.removeEventListener("keydown",dh),t.removeEventListener("keyup",fh),t.removeEventListener("dragstart",vt),t.removeEventListener("blur",hh)}function $y(t,e){mh(t,e,!1)}function Uy(t){delete t._ripple,gh(t)}function jy(t,e){if(e.value===e.oldValue)return;const n=uh(e.oldValue);mh(t,e,n)}const Ds={mounted:$y,unmounted:Uy,updated:jy},Wy=Y({active:{type:Boolean,default:void 0},symbol:{type:null,default:Zf},flat:Boolean,icon:[Boolean,String,Function,Object],prependIcon:Ke,appendIcon:Ke,block:Boolean,slim:Boolean,stacked:Boolean,ripple:{type:[Boolean,Object],default:!0},text:String,...Fr(),...we(),...ds(),...ho(),...us(),...Gf(),...th(),...yl(),...sh(),...Qt(),...ah(),...po(),...Je({tag:"button"}),...dt(),...Vr({variant:"elevated"})},"VBtn"),ot=me()({name:"VBtn",directives:{Ripple:Ds},props:Wy(),emits:{"group:selected":t=>!0},setup(t,e){let{attrs:n,slots:s}=e;const{themeClasses:r}=Ft(t),{borderClasses:i}=Lr(t),{colorClasses:o,colorStyles:a,variantClasses:l}=pl(t),{densityClasses:u}=Bs(t),{dimensionStyles:c}=mo(t),{elevationClasses:d}=cs(t),{loaderClasses:f}=nh(t),{locationStyles:h}=bl(t),{positionClasses:m}=rh(t),{roundedClasses:p}=en(t),{sizeClasses:v,sizeStyles:S}=yo(t),g=qf(t,t.symbol,!1),y=oh(t,n),b=k(()=>{var R;return t.active!==void 0?t.active:y.isLink.value?(R=y.isActive)==null?void 0:R.value:g==null?void 0:g.isSelected.value}),x=k(()=>(g==null?void 0:g.disabled.value)||t.disabled),_=k(()=>t.variant==="elevated"&&!(t.disabled||t.flat||t.border)),C=k(()=>{if(!(t.value===void 0||typeof t.value=="symbol"))return Object(t.value)===t.value?JSON.stringify(t.value,null,0):t.value});function M(R){var F;x.value||y.isLink.value&&(R.metaKey||R.ctrlKey||R.shiftKey||R.button!==0||n.target==="_blank")||((F=y.navigate)==null||F.call(y,R),g==null||g.toggle())}return Dy(y,g==null?void 0:g.select),ye(()=>{var A,N;const R=y.isLink.value?"a":t.tag,F=!!(t.prependIcon||s.prepend),V=!!(t.appendIcon||s.append),T=!!(t.icon&&t.icon!==!0),L=(g==null?void 0:g.isSelected.value)&&(!y.isLink.value||((A=y.isActive)==null?void 0:A.value))||!g||((N=y.isActive)==null?void 0:N.value);return qt(w(R,{type:R==="a"?void 0:"button",class:["v-btn",g==null?void 0:g.selectedClass.value,{"v-btn--active":b.value,"v-btn--block":t.block,"v-btn--disabled":x.value,"v-btn--elevated":_.value,"v-btn--flat":t.flat,"v-btn--icon":!!t.icon,"v-btn--loading":t.loading,"v-btn--slim":t.slim,"v-btn--stacked":t.stacked},r.value,i.value,L?o.value:void 0,u.value,d.value,f.value,m.value,p.value,v.value,l.value,t.class],style:[L?a.value:void 0,c.value,h.value,S.value,t.style],disabled:x.value||void 0,href:y.href.value,onClick:M,value:C.value},{default:()=>{var $;return[vl(!0,"v-btn"),!t.icon&&F&&w("span",{key:"prepend",class:"v-btn__prepend"},[s.prepend?w(Tt,{key:"prepend-defaults",disabled:!t.prependIcon,defaults:{VIcon:{icon:t.prependIcon}}},s.prepend):w(ln,{key:"prepend-icon",icon:t.prependIcon},null)]),w("span",{class:"v-btn__content","data-no-activator":""},[!s.default&&T?w(ln,{key:"content-icon",icon:t.icon},null):w(Tt,{key:"content-defaults",disabled:!T,defaults:{VIcon:{icon:t.icon}}},{default:()=>{var z;return[((z=s.default)==null?void 0:z.call(s))??t.text]}})]),!t.icon&&V&&w("span",{key:"append",class:"v-btn__append"},[s.append?w(Tt,{key:"append-defaults",disabled:!t.appendIcon,defaults:{VIcon:{icon:t.appendIcon}}},s.append):w(ln,{key:"append-icon",icon:t.appendIcon},null)]),!!t.loading&&w("span",{key:"loader",class:"v-btn__loader"},[(($=s.loader)==null?void 0:$.call(s))??w(eh,{color:typeof t.loading=="boolean"?void 0:t.loading,indeterminate:!0,size:"23",width:"2"},null)])]}}),[[ls("ripple"),!x.value&&t.ripple,null]])}),{group:g}}}),Gy=Y({start:Boolean,end:Boolean,icon:Ke,image:String,text:String,...we(),...ds(),...Qt(),...po(),...Je(),...dt(),...Vr({variant:"flat"})},"VAvatar"),oc=me()({name:"VAvatar",props:Gy(),setup(t,e){let{slots:n}=e;const{themeClasses:s}=Ft(t),{colorClasses:r,colorStyles:i,variantClasses:o}=pl(t),{densityClasses:a}=Bs(t),{roundedClasses:l}=en(t),{sizeClasses:u,sizeStyles:c}=yo(t);return ye(()=>w(t.tag,{class:["v-avatar",{"v-avatar--start":t.start,"v-avatar--end":t.end},s.value,r.value,a.value,l.value,u.value,o.value,t.class],style:[i.value,c.value,t.style]},{default:()=>[n.default?w(Tt,{key:"content-defaults",defaults:{VImg:{cover:!0,image:t.image},VIcon:{icon:t.icon}}},{default:()=>[n.default()]}):t.image?w(go,{key:"image",src:t.image,alt:"",cover:!0},null):t.icon?w(ln,{key:"icon",icon:t.icon},null):t.text,vl(!1,"v-avatar")]})),{}}}),qy=Y({text:String,onClick:ki(),...we(),...dt()},"VLabel"),vh=me()({name:"VLabel",props:qy(),setup(t,e){let{slots:n}=e;return ye(()=>{var s;return w("label",{class:["v-label",{"v-label--clickable":!!t.onClick},t.class],style:t.style,onClick:t.onClick},[t.text,(s=n.default)==null?void 0:s.call(n)])}),{}}}),ph=Symbol.for("vuetify:selection-control-group"),yh=Y({color:String,disabled:{type:Boolean,default:null},defaultsTarget:String,error:Boolean,id:String,inline:Boolean,falseIcon:Ke,trueIcon:Ke,ripple:{type:[Boolean,Object],default:!0},multiple:{type:Boolean,default:null},name:String,readonly:{type:Boolean,default:null},modelValue:null,type:String,valueComparator:{type:Function,default:co},...we(),...ds(),...dt()},"SelectionControlGroup"),Yy=Y({...yh({defaultsTarget:"VSelectionControl"})},"VSelectionControlGroup");me()({name:"VSelectionControlGroup",props:Yy(),emits:{"update:modelValue":t=>!0},setup(t,e){let{slots:n}=e;const s=ct(t,"modelValue"),r=gn(),i=k(()=>t.id||`v-selection-control-group-${r}`),o=k(()=>t.name||i.value),a=new Set;return ut(ph,{modelValue:s,forceUpdate:()=>{a.forEach(l=>l())},onForceUpdate:l=>{a.add(l),Rt(()=>{a.delete(l)})}}),mn({[t.defaultsTarget]:{color:fe(t,"color"),disabled:fe(t,"disabled"),density:fe(t,"density"),error:fe(t,"error"),inline:fe(t,"inline"),modelValue:s,multiple:k(()=>!!t.multiple||t.multiple==null&&Array.isArray(s.value)),name:o,falseIcon:fe(t,"falseIcon"),trueIcon:fe(t,"trueIcon"),readonly:fe(t,"readonly"),ripple:fe(t,"ripple"),type:fe(t,"type"),valueComparator:fe(t,"valueComparator")}}),ye(()=>{var l;return w("div",{class:["v-selection-control-group",{"v-selection-control-group--inline":t.inline},t.class],style:t.style,role:t.type==="radio"?"radiogroup":void 0},[(l=n.default)==null?void 0:l.call(n)])}),{}}});const bh=Y({label:String,baseColor:String,trueValue:null,falseValue:null,value:null,...we(),...yh()},"VSelectionControl");function Xy(t){const e=Ne(ph,void 0),{densityClasses:n}=Bs(t),s=ct(t,"modelValue"),r=k(()=>t.trueValue!==void 0?t.trueValue:t.value!==void 0?t.value:!0),i=k(()=>t.falseValue!==void 0?t.falseValue:!1),o=k(()=>!!t.multiple||t.multiple==null&&Array.isArray(s.value)),a=k({get(){const h=e?e.modelValue.value:s.value;return o.value?Fn(h).some(m=>t.valueComparator(m,r.value)):t.valueComparator(h,r.value)},set(h){if(t.readonly)return;const m=h?r.value:i.value;let p=m;o.value&&(p=h?[...Fn(s.value),m]:Fn(s.value).filter(v=>!t.valueComparator(v,r.value))),e?e.modelValue.value=p:s.value=p}}),{textColorClasses:l,textColorStyles:u}=On(k(()=>{if(!(t.error||t.disabled))return a.value?t.color:t.baseColor})),{backgroundColorClasses:c,backgroundColorStyles:d}=At(k(()=>a.value&&!t.error&&!t.disabled?t.color:void 0)),f=k(()=>a.value?t.trueIcon:t.falseIcon);return{group:e,densityClasses:n,trueValue:r,falseValue:i,model:a,textColorClasses:l,textColorStyles:u,backgroundColorClasses:c,backgroundColorStyles:d,icon:f}}const ac=me()({name:"VSelectionControl",directives:{Ripple:Ds},inheritAttrs:!1,props:bh(),emits:{"update:modelValue":t=>!0},setup(t,e){let{attrs:n,slots:s}=e;const{group:r,densityClasses:i,icon:o,model:a,textColorClasses:l,textColorStyles:u,backgroundColorClasses:c,backgroundColorStyles:d,trueValue:f}=Xy(t),h=gn(),m=be(!1),p=be(!1),v=ee(),S=k(()=>t.id||`input-${h}`),g=k(()=>!t.disabled&&!t.readonly);r==null||r.onForceUpdate(()=>{v.value&&(v.value.checked=a.value)});function y(C){g.value&&(m.value=!0,xf(C.target,":focus-visible")!==!1&&(p.value=!0))}function b(){m.value=!1,p.value=!1}function x(C){C.stopPropagation()}function _(C){g.value&&(t.readonly&&r&&It(()=>r.forceUpdate()),a.value=C.target.checked)}return ye(()=>{var V,T;const C=s.label?s.label({label:t.label,props:{for:S.value}}):t.label,[M,R]=op(n),F=w("input",Pe({ref:v,checked:a.value,disabled:!!t.disabled,id:S.value,onBlur:b,onFocus:y,onInput:_,"aria-disabled":!!t.disabled,"aria-label":t.label,type:t.type,value:f.value,name:t.name,"aria-checked":t.type==="checkbox"?a.value:void 0},R),null);return w("div",Pe({class:["v-selection-control",{"v-selection-control--dirty":a.value,"v-selection-control--disabled":t.disabled,"v-selection-control--error":t.error,"v-selection-control--focused":m.value,"v-selection-control--focus-visible":p.value,"v-selection-control--inline":t.inline},i.value,t.class]},M,{style:t.style}),[w("div",{class:["v-selection-control__wrapper",l.value],style:u.value},[(V=s.default)==null?void 0:V.call(s,{backgroundColorClasses:c,backgroundColorStyles:d}),qt(w("div",{class:["v-selection-control__input"]},[((T=s.input)==null?void 0:T.call(s,{model:a,textColorClasses:l,textColorStyles:u,backgroundColorClasses:c,backgroundColorStyles:d,inputNode:F,icon:o.value,props:{onFocus:y,onBlur:b,id:S.value}}))??w(Ie,null,[o.value&&w(ln,{key:"icon",icon:o.value},null),F])]),[[ls("ripple"),t.ripple&&[!t.disabled&&!t.readonly,null,["center","circle"]]]])]),C&&w(vh,{for:S.value,onClick:x},{default:()=>[C]})])}),{isFocused:m,input:v}}}),Zy=Y({indeterminate:Boolean,indeterminateIcon:{type:Ke,default:"$checkboxIndeterminate"},...bh({falseIcon:"$checkboxOff",trueIcon:"$checkboxOn"})},"VCheckboxBtn"),Xr=me()({name:"VCheckboxBtn",props:Zy(),emits:{"update:modelValue":t=>!0,"update:indeterminate":t=>!0},setup(t,e){let{slots:n}=e;const s=ct(t,"indeterminate"),r=ct(t,"modelValue");function i(l){s.value&&(s.value=!1)}const o=k(()=>s.value?t.indeterminateIcon:t.falseIcon),a=k(()=>s.value?t.indeterminateIcon:t.trueIcon);return ye(()=>{const l=ul(ac.filterProps(t),["modelValue"]);return w(ac,Pe(l,{modelValue:r.value,"onUpdate:modelValue":[u=>r.value=u,i],class:["v-checkbox-btn",t.class],style:t.style,type:"checkbox",falseIcon:o.value,trueIcon:a.value,"aria-checked":s.value?"mixed":void 0}),n)}),{}}});function Ky(t){const{t:e}=ny();function n(s){let{name:r}=s;const i={prepend:"prependAction",prependInner:"prependAction",append:"appendAction",appendInner:"appendAction",clear:"clear"}[r],o=t[`onClick:${r}`],a=o&&i?e(`$vuetify.input.${i}`,t.label??""):void 0;return w(ln,{icon:t[`${r}Icon`],"aria-label":a,onClick:o},null)}return{InputIcon:n}}const Jy=Y({active:Boolean,color:String,messages:{type:[Array,String],default:()=>[]},...we(),...gl({transition:{component:fy,leaveAbsolute:!0,group:!0}})},"VMessages"),Qy=me()({name:"VMessages",props:Jy(),setup(t,e){let{slots:n}=e;const s=k(()=>Fn(t.messages)),{textColorClasses:r,textColorStyles:i}=On(k(()=>t.color));return ye(()=>w(Es,{transition:t.transition,tag:"div",class:["v-messages",r.value,t.class],style:[i.value,t.style],role:"alert","aria-live":"polite"},{default:()=>[t.active&&s.value.map((o,a)=>w("div",{class:"v-messages__message",key:`${a}-${s.value}`},[n.message?n.message({message:o}):o]))]})),{}}}),xh=Y({focused:Boolean,"onUpdate:focused":ki()},"focus");function e0(t){let e=arguments.length>1&&arguments[1]!==void 0?arguments[1]:Kt();const n=ct(t,"focused"),s=k(()=>({[`${e}--focused`]:n.value}));function r(){n.value=!0}function i(){n.value=!1}return{focusClasses:s,isFocused:n,focus:r,blur:i}}const t0=Symbol.for("vuetify:form");function n0(){return Ne(t0,null)}const s0=Y({disabled:{type:Boolean,default:null},error:Boolean,errorMessages:{type:[Array,String],default:()=>[]},maxErrors:{type:[Number,String],default:1},name:String,label:String,readonly:{type:Boolean,default:null},rules:{type:Array,default:()=>[]},modelValue:null,validateOn:String,validationValue:null,...xh()},"validation");function r0(t){let e=arguments.length>1&&arguments[1]!==void 0?arguments[1]:Kt(),n=arguments.length>2&&arguments[2]!==void 0?arguments[2]:gn();const s=ct(t,"modelValue"),r=k(()=>t.validationValue===void 0?s.value:t.validationValue),i=n0(),o=ee([]),a=be(!0),l=k(()=>!!(Fn(s.value===""?null:s.value).length||Fn(r.value===""?null:r.value).length)),u=k(()=>!!(t.disabled??(i==null?void 0:i.isDisabled.value))),c=k(()=>!!(t.readonly??(i==null?void 0:i.isReadonly.value))),d=k(()=>{var b;return(b=t.errorMessages)!=null&&b.length?Fn(t.errorMessages).concat(o.value).slice(0,Math.max(0,+t.maxErrors)):o.value}),f=k(()=>{let b=(t.validateOn??(i==null?void 0:i.validateOn.value))||"input";b==="lazy"&&(b="input lazy");const x=new Set((b==null?void 0:b.split(" "))??[]);return{blur:x.has("blur")||x.has("input"),input:x.has("input"),submit:x.has("submit"),lazy:x.has("lazy")}}),h=k(()=>{var b;return t.error||(b=t.errorMessages)!=null&&b.length?!1:t.rules.length?a.value?o.value.length||f.value.lazy?null:!0:!o.value.length:!0}),m=be(!1),p=k(()=>({[`${e}--error`]:h.value===!1,[`${e}--dirty`]:l.value,[`${e}--disabled`]:u.value,[`${e}--readonly`]:c.value})),v=k(()=>t.name??P(n));io(()=>{i==null||i.register({id:v.value,validate:y,reset:S,resetValidation:g})}),kt(()=>{i==null||i.unregister(v.value)}),Zt(async()=>{f.value.lazy||await y(!0),i==null||i.update(v.value,h.value,d.value)}),an(()=>f.value.input,()=>{le(r,()=>{if(r.value!=null)y();else if(t.focused){const b=le(()=>t.focused,x=>{x||y(),b()})}})}),an(()=>f.value.blur,()=>{le(()=>t.focused,b=>{b||y()})}),le([h,d],()=>{i==null||i.update(v.value,h.value,d.value)});async function S(){s.value=null,await It(),await g()}async function g(){a.value=!0,f.value.lazy?o.value=[]:await y(!0)}async function y(){let b=arguments.length>0&&arguments[0]!==void 0?arguments[0]:!1;const x=[];m.value=!0;for(const _ of t.rules){if(x.length>=+(t.maxErrors??1))break;const M=await(typeof _=="function"?_:()=>_)(r.value);if(M!==!0){if(M!==!1&&typeof M!="string"){console.warn(`${M} is not a valid value. Rule functions must return boolean true or a string.`);continue}x.push(M||"")}}return o.value=x,m.value=!1,a.value=b,o.value}return{errorMessages:d,isDirty:l,isDisabled:u,isReadonly:c,isPristine:a,isValid:h,isValidating:m,reset:S,resetValidation:g,validate:y,validationClasses:p}}const wh=Y({id:String,appendIcon:Ke,centerAffix:{type:Boolean,default:!0},prependIcon:Ke,hideDetails:[Boolean,String],hideSpinButtons:Boolean,hint:String,persistentHint:Boolean,messages:{type:[Array,String],default:()=>[]},direction:{type:String,default:"horizontal",validator:t=>["horizontal","vertical"].includes(t)},"onClick:prepend":ki(),"onClick:append":ki(),...we(),...ds(),...s0()},"VInput"),lc=me()({name:"VInput",props:{...wh()},emits:{"update:modelValue":t=>!0},setup(t,e){let{attrs:n,slots:s,emit:r}=e;const{densityClasses:i}=Bs(t),{rtlClasses:o}=Jt(),{InputIcon:a}=Ky(t),l=gn(),u=k(()=>t.id||`input-${l}`),c=k(()=>`${u.value}-messages`),{errorMessages:d,isDirty:f,isDisabled:h,isReadonly:m,isPristine:p,isValid:v,isValidating:S,reset:g,resetValidation:y,validate:b,validationClasses:x}=r0(t,"v-input",u),_=k(()=>({id:u,messagesId:c,isDirty:f,isDisabled:h,isReadonly:m,isPristine:p,isValid:v,isValidating:S,reset:g,resetValidation:y,validate:b})),C=k(()=>{var M;return(M=t.errorMessages)!=null&&M.length||!p.value&&d.value.length?d.value:t.hint&&(t.persistentHint||t.focused)?t.hint:t.messages});return ye(()=>{var T,L,A,N;const M=!!(s.prepend||t.prependIcon),R=!!(s.append||t.appendIcon),F=C.value.length>0,V=!t.hideDetails||t.hideDetails==="auto"&&(F||!!s.details);return w("div",{class:["v-input",`v-input--${t.direction}`,{"v-input--center-affix":t.centerAffix,"v-input--hide-spin-buttons":t.hideSpinButtons},i.value,o.value,x.value,t.class],style:t.style},[M&&w("div",{key:"prepend",class:"v-input__prepend"},[(T=s.prepend)==null?void 0:T.call(s,_.value),t.prependIcon&&w(a,{key:"prepend-icon",name:"prepend"},null)]),s.default&&w("div",{class:"v-input__control"},[(L=s.default)==null?void 0:L.call(s,_.value)]),R&&w("div",{key:"append",class:"v-input__append"},[t.appendIcon&&w(a,{key:"append-icon",name:"append"},null),(A=s.append)==null?void 0:A.call(s,_.value)]),V&&w("div",{class:"v-input__details"},[w(Qy,{id:c.value,active:F,messages:C.value},{message:s.message}),(N=s.details)==null?void 0:N.call(s,_.value)])])}),{reset:g,resetValidation:y,validate:b,isValid:v,errorMessages:d}}}),bo=["sm","md","lg","xl","xxl"],Ma=Symbol.for("vuetify:display"),uc={mobileBreakpoint:"lg",thresholds:{xs:0,sm:600,md:960,lg:1280,xl:1920,xxl:2560}},i0=function(){let t=arguments.length>0&&arguments[0]!==void 0?arguments[0]:uc;return pt(uc,t)};function cc(t){return Oe&&!t?window.innerWidth:typeof t=="object"&&t.clientWidth||0}function dc(t){return Oe&&!t?window.innerHeight:typeof t=="object"&&t.clientHeight||0}function fc(t){const e=Oe&&!t?window.navigator.userAgent:"ssr";function n(m){return!!e.match(m)}const s=n(/android/i),r=n(/iphone|ipad|ipod/i),i=n(/cordova/i),o=n(/electron/i),a=n(/chrome/i),l=n(/edge/i),u=n(/firefox/i),c=n(/opera/i),d=n(/win/i),f=n(/mac/i),h=n(/linux/i);return{android:s,ios:r,cordova:i,electron:o,chrome:a,edge:l,firefox:u,opera:c,win:d,mac:f,linux:h,touch:Jv,ssr:e==="ssr"}}function o0(t,e){const{thresholds:n,mobileBreakpoint:s}=i0(t),r=be(dc(e)),i=be(fc(e)),o=tt({}),a=be(cc(e));function l(){r.value=dc(),a.value=cc()}function u(){l(),i.value=fc()}return Nn(()=>{const c=a.value<n.sm,d=a.value<n.md&&!c,f=a.value<n.lg&&!(d||c),h=a.value<n.xl&&!(f||d||c),m=a.value<n.xxl&&!(h||f||d||c),p=a.value>=n.xxl,v=c?"xs":d?"sm":f?"md":h?"lg":m?"xl":"xxl",S=typeof s=="number"?s:n[s],g=a.value<S;o.xs=c,o.sm=d,o.md=f,o.lg=h,o.xl=m,o.xxl=p,o.smAndUp=!c,o.mdAndUp=!(c||d),o.lgAndUp=!(c||d||f),o.xlAndUp=!(c||d||f||h),o.smAndDown=!(f||h||m||p),o.mdAndDown=!(h||m||p),o.lgAndDown=!(m||p),o.xlAndDown=!p,o.name=v,o.height=r.value,o.width=a.value,o.mobile=g,o.mobileBreakpoint=s,o.platform=i.value,o.thresholds=n}),Oe&&window.addEventListener("resize",l,{passive:!0}),{...Qa(o),update:u,ssr:!!e}}const a0=Y({mobileBreakpoint:[Number,String]},"display");function _h(){let t=arguments.length>0&&arguments[0]!==void 0?arguments[0]:{},e=arguments.length>1&&arguments[1]!==void 0?arguments[1]:Kt();const n=Ne(Ma);if(!n)throw new Error("Could not find Vuetify display injection");const s=k(()=>{if(!t.mobileBreakpoint)return n.mobile.value;const i=typeof t.mobileBreakpoint=="number"?t.mobileBreakpoint:n.thresholds.value[t.mobileBreakpoint];return n.width.value<i}),r=k(()=>e?{[`${e}--mobile`]:s.value}:{});return{...n,displayClasses:r,mobile:s}}const l0=Y({color:String,inset:Boolean,length:[Number,String],thickness:[Number,String],vertical:Boolean,...we(),...dt()},"VDivider"),gi=me()({name:"VDivider",props:l0(),setup(t,e){let{attrs:n}=e;const{themeClasses:s}=Ft(t),{textColorClasses:r,textColorStyles:i}=On(fe(t,"color")),o=k(()=>{const a={};return t.length&&(a[t.vertical?"maxHeight":"maxWidth"]=oe(t.length)),t.thickness&&(a[t.vertical?"borderRightWidth":"borderTopWidth"]=oe(t.thickness)),a});return ye(()=>w("hr",{class:[{"v-divider":!0,"v-divider--inset":t.inset,"v-divider--vertical":t.vertical},s.value,r.value,t.class],style:[o.value,i.value,t.style],"aria-orientation":!n.role||n.role==="separator"?t.vertical?"vertical":"horizontal":void 0,role:`${n.role||"separator"}`},null)),{}}});function $o(t,e){return{x:t.x+e.x,y:t.y+e.y}}function u0(t,e){return{x:t.x-e.x,y:t.y-e.y}}function hc(t,e){if(t.side==="top"||t.side==="bottom"){const{side:n,align:s}=t,r=s==="left"?0:s==="center"?e.width/2:s==="right"?e.width:s,i=n==="top"?0:n==="bottom"?e.height:n;return $o({x:r,y:i},e)}else if(t.side==="left"||t.side==="right"){const{side:n,align:s}=t,r=n==="left"?0:n==="right"?e.width:n,i=s==="top"?0:s==="center"?e.height/2:s==="bottom"?e.height:s;return $o({x:r,y:i},e)}return $o({x:e.width/2,y:e.height/2},e)}const Sh={static:f0,connected:m0},c0=Y({locationStrategy:{type:[String,Function],default:"static",validator:t=>typeof t=="function"||t in Sh},location:{type:String,default:"bottom"},origin:{type:String,default:"auto"},offset:[Number,String,Array]},"VOverlay-location-strategies");function d0(t,e){const n=ee({}),s=ee();Oe&&an(()=>!!(e.isActive.value&&t.locationStrategy),i=>{var o,a;le(()=>t.locationStrategy,i),Rt(()=>{window.removeEventListener("resize",r),s.value=void 0}),window.addEventListener("resize",r,{passive:!0}),typeof t.locationStrategy=="function"?s.value=(o=t.locationStrategy(e,t,n))==null?void 0:o.updateLocation:s.value=(a=Sh[t.locationStrategy](e,t,n))==null?void 0:a.updateLocation});function r(i){var o;(o=s.value)==null||o.call(s,i)}return{contentStyles:n,updateLocation:s}}function f0(){}function h0(t,e){e?t.style.removeProperty("left"):t.style.removeProperty("right");const n=Sf(t);return e?n.x+=parseFloat(t.style.right||0):n.x-=parseFloat(t.style.left||0),n.y-=parseFloat(t.style.top||0),n}function m0(t,e,n){(Array.isArray(t.target.value)||Wp(t.target.value))&&Object.assign(n.value,{position:"fixed",top:0,[t.isRtl.value?"right":"left"]:0});const{preferredAnchor:r,preferredOrigin:i}=cl(()=>{const m=ba(e.location,t.isRtl.value),p=e.origin==="overlap"?m:e.origin==="auto"?Bo(m):ba(e.origin,t.isRtl.value);return m.side===p.side&&m.align===Do(p).align?{preferredAnchor:Vu(m),preferredOrigin:Vu(p)}:{preferredAnchor:m,preferredOrigin:p}}),[o,a,l,u]=["minWidth","minHeight","maxWidth","maxHeight"].map(m=>k(()=>{const p=parseFloat(e[m]);return isNaN(p)?1/0:p})),c=k(()=>{if(Array.isArray(e.offset))return e.offset;if(typeof e.offset=="string"){const m=e.offset.split(" ").map(parseFloat);return m.length<2&&m.push(0),m}return typeof e.offset=="number"?[e.offset,0]:[0,0]});let d=!1;const f=new ResizeObserver(()=>{d&&h()});le([t.target,t.contentEl],(m,p)=>{let[v,S]=m,[g,y]=p;g&&!Array.isArray(g)&&f.unobserve(g),v&&!Array.isArray(v)&&f.observe(v),y&&f.unobserve(y),S&&f.observe(S)},{immediate:!0}),Rt(()=>{f.disconnect()});function h(){if(d=!1,requestAnimationFrame(()=>d=!0),!t.target.value||!t.contentEl.value)return;const m=_f(t.target.value),p=h0(t.contentEl.value,t.isRtl.value),v=Li(t.contentEl.value),S=12;v.length||(v.push(document.documentElement),t.contentEl.value.style.top&&t.contentEl.value.style.left||(p.x-=parseFloat(document.documentElement.style.getPropertyValue("--v-body-scroll-x")||0),p.y-=parseFloat(document.documentElement.style.getPropertyValue("--v-body-scroll-y")||0)));const g=v.reduce((V,T)=>{const L=T.getBoundingClientRect(),A=new es({x:T===document.documentElement?0:L.x,y:T===document.documentElement?0:L.y,width:T.clientWidth,height:T.clientHeight});return V?new es({x:Math.max(V.left,A.left),y:Math.max(V.top,A.top),width:Math.min(V.right,A.right)-Math.max(V.left,A.left),height:Math.min(V.bottom,A.bottom)-Math.max(V.top,A.top)}):A},void 0);g.x+=S,g.y+=S,g.width-=S*2,g.height-=S*2;let y={anchor:r.value,origin:i.value};function b(V){const T=new es(p),L=hc(V.anchor,m),A=hc(V.origin,T);let{x:N,y:$}=u0(L,A);switch(V.anchor.side){case"top":$-=c.value[0];break;case"bottom":$+=c.value[0];break;case"left":N-=c.value[0];break;case"right":N+=c.value[0];break}switch(V.anchor.align){case"top":$-=c.value[1];break;case"bottom":$+=c.value[1];break;case"left":N-=c.value[1];break;case"right":N+=c.value[1];break}return T.x+=N,T.y+=$,T.width=Math.min(T.width,l.value),T.height=Math.min(T.height,u.value),{overflows:Ou(T,g),x:N,y:$}}let x=0,_=0;const C={x:0,y:0},M={x:!1,y:!1};let R=-1;for(;!(R++>10);){const{x:V,y:T,overflows:L}=b(y);x+=V,_+=T,p.x+=V,p.y+=T;{const A=Pu(y.anchor),N=L.x.before||L.x.after,$=L.y.before||L.y.after;let z=!1;if(["x","y"].forEach(X=>{if(X==="x"&&N&&!M.x||X==="y"&&$&&!M.y){const re={anchor:{...y.anchor},origin:{...y.origin}},te=X==="x"?A==="y"?Do:Bo:A==="y"?Bo:Do;re.anchor=te(re.anchor),re.origin=te(re.origin);const{overflows:Q}=b(re);(Q[X].before<=L[X].before&&Q[X].after<=L[X].after||Q[X].before+Q[X].after<(L[X].before+L[X].after)/2)&&(y=re,z=M[X]=!0)}}),z)continue}L.x.before&&(x+=L.x.before,p.x+=L.x.before),L.x.after&&(x-=L.x.after,p.x-=L.x.after),L.y.before&&(_+=L.y.before,p.y+=L.y.before),L.y.after&&(_-=L.y.after,p.y-=L.y.after);{const A=Ou(p,g);C.x=g.width-A.x.before-A.x.after,C.y=g.height-A.y.before-A.y.after,x+=A.x.before,p.x+=A.x.before,_+=A.y.before,p.y+=A.y.before}break}const F=Pu(y.anchor);return Object.assign(n.value,{"--v-overlay-anchor-origin":`${y.anchor.side} ${y.anchor.align}`,transformOrigin:`${y.origin.side} ${y.origin.align}`,top:oe(Uo(_)),left:t.isRtl.value?void 0:oe(Uo(x)),right:t.isRtl.value?oe(Uo(-x)):void 0,minWidth:oe(F==="y"?Math.min(o.value,m.width):o.value),maxWidth:oe(mc(yr(C.x,o.value===1/0?0:o.value,l.value))),maxHeight:oe(mc(yr(C.y,a.value===1/0?0:a.value,u.value)))}),{available:C,contentBox:p}}return le(()=>[r.value,i.value,e.offset,e.minWidth,e.minHeight,e.maxWidth,e.maxHeight],()=>h()),It(()=>{const m=h();if(!m)return;const{available:p,contentBox:v}=m;v.height>p.y&&requestAnimationFrame(()=>{h(),requestAnimationFrame(()=>{h()})})}),{updateLocation:h}}function Uo(t){return Math.round(t*devicePixelRatio)/devicePixelRatio}function mc(t){return Math.ceil(t*devicePixelRatio)/devicePixelRatio}let Ta=!0;const Di=[];function g0(t){!Ta||Di.length?(Di.push(t),Aa()):(Ta=!1,t(),Aa())}let gc=-1;function Aa(){cancelAnimationFrame(gc),gc=requestAnimationFrame(()=>{const t=Di.shift();t&&t(),Di.length?Aa():Ta=!0})}const vi={none:null,close:y0,block:b0,reposition:x0},v0=Y({scrollStrategy:{type:[String,Function],default:"block",validator:t=>typeof t=="function"||t in vi}},"VOverlay-scroll-strategies");function p0(t,e){if(!Oe)return;let n;Nn(async()=>{n==null||n.stop(),e.isActive.value&&t.scrollStrategy&&(n=ja(),await new Promise(s=>setTimeout(s)),n.active&&n.run(()=>{var s;typeof t.scrollStrategy=="function"?t.scrollStrategy(e,t,n):(s=vi[t.scrollStrategy])==null||s.call(vi,e,t,n)}))}),Rt(()=>{n==null||n.stop()})}function y0(t){function e(n){t.isActive.value=!1}Ch(t.targetEl.value??t.contentEl.value,e)}function b0(t,e){var o;const n=(o=t.root.value)==null?void 0:o.offsetParent,s=[...new Set([...Li(t.targetEl.value,e.contained?n:void 0),...Li(t.contentEl.value,e.contained?n:void 0)])].filter(a=>!a.classList.contains("v-overlay-scroll-blocked")),r=window.innerWidth-document.documentElement.offsetWidth,i=(a=>hl(a)&&a)(n||document.documentElement);i&&t.root.value.classList.add("v-overlay--scroll-blocked"),s.forEach((a,l)=>{a.style.setProperty("--v-body-scroll-x",oe(-a.scrollLeft)),a.style.setProperty("--v-body-scroll-y",oe(-a.scrollTop)),a!==document.documentElement&&a.style.setProperty("--v-scrollbar-offset",oe(r)),a.classList.add("v-overlay-scroll-blocked")}),Rt(()=>{s.forEach((a,l)=>{const u=parseFloat(a.style.getPropertyValue("--v-body-scroll-x")),c=parseFloat(a.style.getPropertyValue("--v-body-scroll-y")),d=a.style.scrollBehavior;a.style.scrollBehavior="auto",a.style.removeProperty("--v-body-scroll-x"),a.style.removeProperty("--v-body-scroll-y"),a.style.removeProperty("--v-scrollbar-offset"),a.classList.remove("v-overlay-scroll-blocked"),a.scrollLeft=-u,a.scrollTop=-c,a.style.scrollBehavior=d}),i&&t.root.value.classList.remove("v-overlay--scroll-blocked")})}function x0(t,e,n){let s=!1,r=-1,i=-1;function o(a){g0(()=>{var c,d;const l=performance.now();(d=(c=t.updateLocation).value)==null||d.call(c,a),s=(performance.now()-l)/(1e3/60)>2})}i=(typeof requestIdleCallback>"u"?a=>a():requestIdleCallback)(()=>{n.run(()=>{Ch(t.targetEl.value??t.contentEl.value,a=>{s?(cancelAnimationFrame(r),r=requestAnimationFrame(()=>{r=requestAnimationFrame(()=>{o(a)})})):o(a)})})}),Rt(()=>{typeof cancelIdleCallback<"u"&&cancelIdleCallback(i),cancelAnimationFrame(r)})}function Ch(t,e){const n=[document,...Li(t)];n.forEach(s=>{s.addEventListener("scroll",e,{passive:!0})}),Rt(()=>{n.forEach(s=>{s.removeEventListener("scroll",e)})})}const w0=Symbol.for("vuetify:v-menu"),_0=Y({closeDelay:[Number,String],openDelay:[Number,String]},"delay");function S0(t,e){let n=()=>{};function s(o){n==null||n();const a=Number(o?t.openDelay:t.closeDelay);return new Promise(l=>{n=fp(a,()=>{e==null||e(o),l(o)})})}function r(){return s(!0)}function i(){return s(!1)}return{clearDelay:n,runOpenDelay:r,runCloseDelay:i}}const C0=Y({target:[String,Object],activator:[String,Object],activatorProps:{type:Object,default:()=>({})},openOnClick:{type:Boolean,default:void 0},openOnHover:Boolean,openOnFocus:{type:Boolean,default:void 0},closeOnContentClick:Boolean,..._0()},"VOverlay-activator");function E0(t,e){let{isActive:n,isTop:s}=e;const r=qe("useActivator"),i=ee();let o=!1,a=!1,l=!0;const u=k(()=>t.openOnFocus||t.openOnFocus==null&&t.openOnHover),c=k(()=>t.openOnClick||t.openOnClick==null&&!t.openOnHover&&!u.value),{runOpenDelay:d,runCloseDelay:f}=S0(t,C=>{C===(t.openOnHover&&o||u.value&&a)&&!(t.openOnHover&&n.value&&!s.value)&&(n.value!==C&&(l=!0),n.value=C)}),h=ee(),m={onClick:C=>{C.stopPropagation(),i.value=C.currentTarget||C.target,n.value||(h.value=[C.clientX,C.clientY]),n.value=!n.value},onMouseenter:C=>{var M;(M=C.sourceCapabilities)!=null&&M.firesTouchEvents||(o=!0,i.value=C.currentTarget||C.target,d())},onMouseleave:C=>{o=!1,f()},onFocus:C=>{xf(C.target,":focus-visible")!==!1&&(a=!0,C.stopPropagation(),i.value=C.currentTarget||C.target,d())},onBlur:C=>{a=!1,C.stopPropagation(),f()}},p=k(()=>{const C={};return c.value&&(C.onClick=m.onClick),t.openOnHover&&(C.onMouseenter=m.onMouseenter,C.onMouseleave=m.onMouseleave),u.value&&(C.onFocus=m.onFocus,C.onBlur=m.onBlur),C}),v=k(()=>{const C={};if(t.openOnHover&&(C.onMouseenter=()=>{o=!0,d()},C.onMouseleave=()=>{o=!1,f()}),u.value&&(C.onFocusin=()=>{a=!0,d()},C.onFocusout=()=>{a=!1,f()}),t.closeOnContentClick){const M=Ne(w0,null);C.onClick=()=>{n.value=!1,M==null||M.closeParents()}}return C}),S=k(()=>{const C={};return t.openOnHover&&(C.onMouseenter=()=>{l&&(o=!0,l=!1,d())},C.onMouseleave=()=>{o=!1,f()}),C});le(s,C=>{C&&(t.openOnHover&&!o&&(!u.value||!a)||u.value&&!a&&(!t.openOnHover||!o))&&(n.value=!1)}),le(n,C=>{C||setTimeout(()=>{h.value=void 0})},{flush:"post"});const g=ee();Nn(()=>{g.value&&It(()=>{i.value=Ri(g.value)})});const y=ee(),b=k(()=>t.target==="cursor"&&h.value?h.value:y.value?Ri(y.value):Eh(t.target,r)||i.value),x=k(()=>Array.isArray(b.value)?void 0:b.value);let _;return le(()=>!!t.activator,C=>{C&&Oe?(_=ja(),_.run(()=>{M0(t,r,{activatorEl:i,activatorEvents:p})})):_&&_.stop()},{flush:"post",immediate:!0}),Rt(()=>{_==null||_.stop()}),{activatorEl:i,activatorRef:g,target:b,targetEl:x,targetRef:y,activatorEvents:p,contentEvents:v,scrimEvents:S}}function M0(t,e,n){let{activatorEl:s,activatorEvents:r}=n;le(()=>t.activator,(l,u)=>{if(u&&l!==u){const c=a(u);c&&o(c)}l&&It(()=>i())},{immediate:!0}),le(()=>t.activatorProps,()=>{i()}),Rt(()=>{o()});function i(){let l=arguments.length>0&&arguments[0]!==void 0?arguments[0]:a(),u=arguments.length>1&&arguments[1]!==void 0?arguments[1]:t.activatorProps;l&&mp(l,Pe(r.value,u))}function o(){let l=arguments.length>0&&arguments[0]!==void 0?arguments[0]:a(),u=arguments.length>1&&arguments[1]!==void 0?arguments[1]:t.activatorProps;l&&gp(l,Pe(r.value,u))}function a(){let l=arguments.length>0&&arguments[0]!==void 0?arguments[0]:t.activator;const u=Eh(l,e);return s.value=(u==null?void 0:u.nodeType)===Node.ELEMENT_NODE?u:void 0,s.value}}function Eh(t,e){var s,r;if(!t)return;let n;if(t==="parent"){let i=(r=(s=e==null?void 0:e.proxy)==null?void 0:s.$el)==null?void 0:r.parentNode;for(;i!=null&&i.hasAttribute("data-no-activator");)i=i.parentNode;n=i}else typeof t=="string"?n=document.querySelector(t):"$el"in t?n=t.$el:n=t;return n}function T0(){if(!Oe)return be(!1);const{ssr:t}=_h();if(t){const e=be(!1);return Zt(()=>{e.value=!0}),e}else return be(!0)}const xl=Y({eager:Boolean},"lazy");function Mh(t,e){const n=be(!1),s=k(()=>n.value||t.eager||e.value);le(e,()=>n.value=!0);function r(){t.eager||(n.value=!1)}return{isBooted:n,hasContent:s,onAfterLeave:r}}function xo(){const e=qe("useScopeId").vnode.scopeId;return{scopeId:e?{[e]:""}:void 0}}const vc=Symbol.for("vuetify:stack"),Us=tt([]);function A0(t,e,n){const s=qe("useStack"),r=!n,i=Ne(vc,void 0),o=tt({activeChildren:new Set});ut(vc,o);const a=be(+e.value);an(t,()=>{var d;const c=(d=Us.at(-1))==null?void 0:d[1];a.value=c?c+10:+e.value,r&&Us.push([s.uid,a.value]),i==null||i.activeChildren.add(s.uid),Rt(()=>{if(r){const f=pe(Us).findIndex(h=>h[0]===s.uid);Us.splice(f,1)}i==null||i.activeChildren.delete(s.uid)})});const l=be(!0);r&&Nn(()=>{var d;const c=((d=Us.at(-1))==null?void 0:d[0])===s.uid;setTimeout(()=>l.value=c)});const u=k(()=>!o.activeChildren.size);return{globalTop:Cr(l),localTop:u,stackStyles:k(()=>({zIndex:a.value}))}}function R0(t){return{teleportTarget:k(()=>{const n=t.value;if(n===!0||!Oe)return;const s=n===!1?document.body:typeof n=="string"?document.querySelector(n):n;if(s==null)return;let r=s.querySelector(":scope > .v-overlay-container");return r||(r=document.createElement("div"),r.className="v-overlay-container",s.appendChild(r)),r})}}function I0(){return!0}function Th(t,e,n){if(!t||Ah(t,n)===!1)return!1;const s=Rf(e);if(typeof ShadowRoot<"u"&&s instanceof ShadowRoot&&s.host===t.target)return!1;const r=(typeof n.value=="object"&&n.value.include||(()=>[]))();return r.push(e),!r.some(i=>i==null?void 0:i.contains(t.target))}function Ah(t,e){return(typeof e.value=="object"&&e.value.closeConditional||I0)(t)}function k0(t,e,n){const s=typeof n.value=="function"?n.value:n.value.handler;e._clickOutside.lastMousedownWasOutside&&Th(t,e,n)&&setTimeout(()=>{Ah(t,n)&&s&&s(t)},0)}function pc(t,e){const n=Rf(t);e(document),typeof ShadowRoot<"u"&&n instanceof ShadowRoot&&e(n)}const Rh={mounted(t,e){const n=r=>k0(r,t,e),s=r=>{t._clickOutside.lastMousedownWasOutside=Th(r,t,e)};pc(t,r=>{r.addEventListener("click",n,!0),r.addEventListener("mousedown",s,!0)}),t._clickOutside||(t._clickOutside={lastMousedownWasOutside:!1}),t._clickOutside[e.instance.$.uid]={onClick:n,onMousedown:s}},unmounted(t,e){t._clickOutside&&(pc(t,n=>{var i;if(!n||!((i=t._clickOutside)!=null&&i[e.instance.$.uid]))return;const{onClick:s,onMousedown:r}=t._clickOutside[e.instance.$.uid];n.removeEventListener("click",s,!0),n.removeEventListener("mousedown",r,!0)}),delete t._clickOutside[e.instance.$.uid])}};function F0(t){const{modelValue:e,color:n,...s}=t;return w(Xt,{name:"fade-transition",appear:!0},{default:()=>[t.modelValue&&w("div",Pe({class:["v-overlay__scrim",t.color.backgroundColorClasses.value],style:t.color.backgroundColorStyles.value},s),null)]})}const wl=Y({absolute:Boolean,attach:[Boolean,String,Object],closeOnBack:{type:Boolean,default:!0},contained:Boolean,contentClass:null,contentProps:null,disabled:Boolean,opacity:[Number,String],noClickAnimation:Boolean,modelValue:Boolean,persistent:Boolean,scrim:{type:[Boolean,String],default:!0},zIndex:{type:[Number,String],default:2e3},...C0(),...we(),...ho(),...xl(),...c0(),...v0(),...dt(),...gl()},"VOverlay"),zi=me()({name:"VOverlay",directives:{ClickOutside:Rh},inheritAttrs:!1,props:{_disableGlobalStack:Boolean,...wl()},emits:{"click:outside":t=>!0,"update:modelValue":t=>!0,afterLeave:()=>!0},setup(t,e){let{slots:n,attrs:s,emit:r}=e;const i=ct(t,"modelValue"),o=k({get:()=>i.value,set:ge=>{ge&&t.disabled||(i.value=ge)}}),{teleportTarget:a}=R0(k(()=>t.attach||t.contained)),{themeClasses:l}=Ft(t),{rtlClasses:u,isRtl:c}=Jt(),{hasContent:d,onAfterLeave:f}=Mh(t,o),h=At(k(()=>typeof t.scrim=="string"?t.scrim:null)),{globalTop:m,localTop:p,stackStyles:v}=A0(o,fe(t,"zIndex"),t._disableGlobalStack),{activatorEl:S,activatorRef:g,target:y,targetEl:b,targetRef:x,activatorEvents:_,contentEvents:C,scrimEvents:M}=E0(t,{isActive:o,isTop:p}),{dimensionStyles:R}=mo(t),F=T0(),{scopeId:V}=xo();le(()=>t.disabled,ge=>{ge&&(o.value=!1)});const T=ee(),L=ee(),A=ee(),{contentStyles:N,updateLocation:$}=d0(t,{isRtl:c,contentEl:A,target:y,isActive:o});p0(t,{root:T,contentEl:A,targetEl:b,isActive:o,updateLocation:$});function z(ge){r("click:outside",ge),t.persistent?ce():o.value=!1}function X(ge){return o.value&&m.value&&(!t.scrim||ge.target===L.value)}Oe&&le(o,ge=>{ge?window.addEventListener("keydown",re):window.removeEventListener("keydown",re)},{immediate:!0}),kt(()=>{Oe&&window.removeEventListener("keydown",re)});function re(ge){var Le,Z;ge.key==="Escape"&&m.value&&(t.persistent?ce():(o.value=!1,(Le=A.value)!=null&&Le.contains(document.activeElement)&&((Z=S.value)==null||Z.focus())))}const te=ih();an(()=>t.closeOnBack,()=>{By(te,ge=>{m.value&&o.value?(ge(!1),t.persistent?ce():o.value=!1):ge()})});const Q=ee();le(()=>o.value&&(t.absolute||t.contained)&&a.value==null,ge=>{if(ge){const Le=$p(T.value);Le&&Le!==document.scrollingElement&&(Q.value=Le.scrollTop)}});function ce(){t.noClickAnimation||A.value&&Zs(A.value,[{transformOrigin:"center"},{transform:"scale(1.03)"},{transformOrigin:"center"}],{duration:150,easing:_a})}function Te(){f(),r("afterLeave")}return ye(()=>{var ge;return w(Ie,null,[(ge=n.activator)==null?void 0:ge.call(n,{isActive:o.value,props:Pe({ref:g,targetRef:x},_.value,t.activatorProps)}),F.value&&d.value&&w(tv,{disabled:!a.value,to:a.value},{default:()=>[w("div",Pe({class:["v-overlay",{"v-overlay--absolute":t.absolute||t.contained,"v-overlay--active":o.value,"v-overlay--contained":t.contained},l.value,u.value,t.class],style:[v.value,{"--v-overlay-opacity":t.opacity,top:oe(Q.value)},t.style],ref:T},V,s),[w(F0,Pe({color:h,modelValue:o.value&&!!t.scrim,ref:L},M.value),null),w(Es,{appear:!0,persisted:!0,transition:t.transition,target:y.value,onAfterLeave:Te},{default:()=>{var Le;return[qt(w("div",Pe({ref:A,class:["v-overlay__content",t.contentClass],style:[R.value,N.value]},C.value,t.contentProps),[(Le=n.default)==null?void 0:Le.call(n,{isActive:o})]),[[uo,o.value],[ls("click-outside"),{handler:z,closeConditional:X,include:()=>[S.value]}]])]}})])]})])}),{activatorEl:S,scrimEl:L,target:y,animateClick:ce,contentEl:A,globalTop:m,localTop:p,updateLocation:$}}}),jo=Symbol("Forwarded refs");function Wo(t,e){let n=t;for(;n;){const s=Reflect.getOwnPropertyDescriptor(n,e);if(s)return s;n=Object.getPrototypeOf(n)}}function Ih(t){for(var e=arguments.length,n=new Array(e>1?e-1:0),s=1;s<e;s++)n[s-1]=arguments[s];return t[jo]=n,new Proxy(t,{get(r,i){if(Reflect.has(r,i))return Reflect.get(r,i);if(!(typeof i=="symbol"||i.startsWith("$")||i.startsWith("__"))){for(const o of n)if(o.value&&Reflect.has(o.value,i)){const a=Reflect.get(o.value,i);return typeof a=="function"?a.bind(o.value):a}}},has(r,i){if(Reflect.has(r,i))return!0;if(typeof i=="symbol"||i.startsWith("$")||i.startsWith("__"))return!1;for(const o of n)if(o.value&&Reflect.has(o.value,i))return!0;return!1},set(r,i,o){if(Reflect.has(r,i))return Reflect.set(r,i,o);if(typeof i=="symbol"||i.startsWith("$")||i.startsWith("__"))return!1;for(const a of n)if(a.value&&Reflect.has(a.value,i))return Reflect.set(a.value,i,o);return!1},getOwnPropertyDescriptor(r,i){var a;const o=Reflect.getOwnPropertyDescriptor(r,i);if(o)return o;if(!(typeof i=="symbol"||i.startsWith("$")||i.startsWith("__"))){for(const l of n){if(!l.value)continue;const u=Wo(l.value,i)??("_"in l.value?Wo((a=l.value._)==null?void 0:a.setupState,i):void 0);if(u)return u}for(const l of n){const u=l.value&&l.value[jo];if(!u)continue;const c=u.slice();for(;c.length;){const d=c.shift(),f=Wo(d.value,i);if(f)return f;const h=d.value&&d.value[jo];h&&c.push(...h)}}}}})}const L0=Y({fullscreen:Boolean,retainFocus:{type:Boolean,default:!0},scrollable:Boolean,...wl({origin:"center center",scrollStrategy:"block",transition:{component:cy},zIndex:2400})},"VDialog"),V0=me()({name:"VDialog",props:L0(),emits:{"update:modelValue":t=>!0},setup(t,e){let{slots:n}=e;const s=ct(t,"modelValue"),{scopeId:r}=xo(),i=ee();function o(l){var d,f;const u=l.relatedTarget,c=l.target;if(u!==c&&((d=i.value)!=null&&d.contentEl)&&((f=i.value)!=null&&f.globalTop)&&![document,i.value.contentEl].includes(c)&&!i.value.contentEl.contains(c)){const h=bf(i.value.contentEl);if(!h.length)return;const m=h[0],p=h[h.length-1];u===m?p.focus():m.focus()}}Oe&&le(()=>s.value&&t.retainFocus,l=>{l?document.addEventListener("focusin",o):document.removeEventListener("focusin",o)},{immediate:!0}),le(s,async l=>{var u,c;await It(),l?(u=i.value.contentEl)==null||u.focus({preventScroll:!0}):(c=i.value.activatorEl)==null||c.focus({preventScroll:!0})});const a=k(()=>Pe({"aria-haspopup":"dialog","aria-expanded":String(s.value)},t.activatorProps));return ye(()=>{const l=zi.filterProps(t);return w(zi,Pe({ref:i,class:["v-dialog",{"v-dialog--fullscreen":t.fullscreen,"v-dialog--scrollable":t.scrollable},t.class],style:t.style},l,{modelValue:s.value,"onUpdate:modelValue":u=>s.value=u,"aria-modal":"true",activatorProps:a.value,role:"dialog"},r),{activator:n.activator,default:function(){for(var u=arguments.length,c=new Array(u),d=0;d<u;d++)c[d]=arguments[d];return w(Tt,{root:"VDialog"},{default:()=>{var f;return[(f=n.default)==null?void 0:f.call(n,...c)]}})}})}),Ih({},i)}}),P0=me()({name:"VCardActions",props:we(),setup(t,e){let{slots:n}=e;return mn({VBtn:{slim:!0,variant:"text"}}),ye(()=>{var s;return w("div",{class:["v-card-actions",t.class],style:t.style},[(s=n.default)==null?void 0:s.call(n)])}),{}}}),O0=fo("v-card-subtitle"),N0=fo("v-card-title"),B0=Y({appendAvatar:String,appendIcon:Ke,prependAvatar:String,prependIcon:Ke,subtitle:[String,Number],title:[String,Number],...we(),...ds()},"VCardItem"),D0=me()({name:"VCardItem",props:B0(),setup(t,e){let{slots:n}=e;return ye(()=>{var u;const s=!!(t.prependAvatar||t.prependIcon),r=!!(s||n.prepend),i=!!(t.appendAvatar||t.appendIcon),o=!!(i||n.append),a=!!(t.title!=null||n.title),l=!!(t.subtitle!=null||n.subtitle);return w("div",{class:["v-card-item",t.class],style:t.style},[r&&w("div",{key:"prepend",class:"v-card-item__prepend"},[n.prepend?w(Tt,{key:"prepend-defaults",disabled:!s,defaults:{VAvatar:{density:t.density,image:t.prependAvatar},VIcon:{density:t.density,icon:t.prependIcon}}},n.prepend):w(Ie,null,[t.prependAvatar&&w(oc,{key:"prepend-avatar",density:t.density,image:t.prependAvatar},null),t.prependIcon&&w(ln,{key:"prepend-icon",density:t.density,icon:t.prependIcon},null)])]),w("div",{class:"v-card-item__content"},[a&&w(N0,{key:"title"},{default:()=>{var c;return[((c=n.title)==null?void 0:c.call(n))??t.title]}}),l&&w(O0,{key:"subtitle"},{default:()=>{var c;return[((c=n.subtitle)==null?void 0:c.call(n))??t.subtitle]}}),(u=n.default)==null?void 0:u.call(n)]),o&&w("div",{key:"append",class:"v-card-item__append"},[n.append?w(Tt,{key:"append-defaults",disabled:!i,defaults:{VAvatar:{density:t.density,image:t.appendAvatar},VIcon:{density:t.density,icon:t.appendIcon}}},n.append):w(Ie,null,[t.appendIcon&&w(ln,{key:"append-icon",density:t.density,icon:t.appendIcon},null),t.appendAvatar&&w(oc,{key:"append-avatar",density:t.density,image:t.appendAvatar},null)])])])}),{}}}),kh=fo("v-card-text"),z0=Y({appendAvatar:String,appendIcon:Ke,disabled:Boolean,flat:Boolean,hover:Boolean,image:String,link:{type:Boolean,default:void 0},prependAvatar:String,prependIcon:Ke,ripple:{type:[Boolean,Object],default:!0},subtitle:[String,Number],text:[String,Number],title:[String,Number],...Fr(),...we(),...ds(),...ho(),...us(),...th(),...yl(),...sh(),...Qt(),...ah(),...Je(),...dt(),...Vr({variant:"elevated"})},"VCard"),H0=me()({name:"VCard",directives:{Ripple:Ds},props:z0(),setup(t,e){let{attrs:n,slots:s}=e;const{themeClasses:r}=Ft(t),{borderClasses:i}=Lr(t),{colorClasses:o,colorStyles:a,variantClasses:l}=pl(t),{densityClasses:u}=Bs(t),{dimensionStyles:c}=mo(t),{elevationClasses:d}=cs(t),{loaderClasses:f}=nh(t),{locationStyles:h}=bl(t),{positionClasses:m}=rh(t),{roundedClasses:p}=en(t),v=oh(t,n),S=k(()=>t.link!==!1&&v.isLink.value),g=k(()=>!t.disabled&&t.link!==!1&&(t.link||v.isClickable.value));return ye(()=>{const y=S.value?"a":t.tag,b=!!(s.title||t.title!=null),x=!!(s.subtitle||t.subtitle!=null),_=b||x,C=!!(s.append||t.appendAvatar||t.appendIcon),M=!!(s.prepend||t.prependAvatar||t.prependIcon),R=!!(s.image||t.image),F=_||M||C,V=!!(s.text||t.text!=null);return qt(w(y,{class:["v-card",{"v-card--disabled":t.disabled,"v-card--flat":t.flat,"v-card--hover":t.hover&&!(t.disabled||t.flat),"v-card--link":g.value},r.value,i.value,o.value,u.value,d.value,f.value,m.value,p.value,l.value,t.class],style:[a.value,c.value,h.value,t.style],href:v.href.value,onClick:g.value&&v.navigate,tabindex:t.disabled?-1:void 0},{default:()=>{var T;return[R&&w("div",{key:"image",class:"v-card__image"},[s.image?w(Tt,{key:"image-defaults",disabled:!t.image,defaults:{VImg:{cover:!0,src:t.image}}},s.image):w(go,{key:"image-img",cover:!0,src:t.image},null)]),w(Py,{name:"v-card",active:!!t.loading,color:typeof t.loading=="boolean"?void 0:t.loading},{default:s.loader}),F&&w(D0,{key:"item",prependAvatar:t.prependAvatar,prependIcon:t.prependIcon,title:t.title,subtitle:t.subtitle,appendAvatar:t.appendAvatar,appendIcon:t.appendIcon},{default:s.item,prepend:s.prepend,title:s.title,subtitle:s.subtitle,append:s.append}),V&&w(kh,{key:"text"},{default:()=>{var L;return[((L=s.text)==null?void 0:L.call(s))??t.text]}}),(T=s.default)==null?void 0:T.call(s),s.actions&&w(P0,null,{default:s.actions}),vl(g.value,"v-card")]}}),[[ls("ripple"),g.value&&t.ripple]])}),{}}}),$0=t=>{const{touchstartX:e,touchendX:n,touchstartY:s,touchendY:r}=t,i=.5,o=16;t.offsetX=n-e,t.offsetY=r-s,Math.abs(t.offsetY)<i*Math.abs(t.offsetX)&&(t.left&&n<e-o&&t.left(t),t.right&&n>e+o&&t.right(t)),Math.abs(t.offsetX)<i*Math.abs(t.offsetY)&&(t.up&&r<s-o&&t.up(t),t.down&&r>s+o&&t.down(t))};function U0(t,e){var s;const n=t.changedTouches[0];e.touchstartX=n.clientX,e.touchstartY=n.clientY,(s=e.start)==null||s.call(e,{originalEvent:t,...e})}function j0(t,e){var s;const n=t.changedTouches[0];e.touchendX=n.clientX,e.touchendY=n.clientY,(s=e.end)==null||s.call(e,{originalEvent:t,...e}),$0(e)}function W0(t,e){var s;const n=t.changedTouches[0];e.touchmoveX=n.clientX,e.touchmoveY=n.clientY,(s=e.move)==null||s.call(e,{originalEvent:t,...e})}function G0(){let t=arguments.length>0&&arguments[0]!==void 0?arguments[0]:{};const e={touchstartX:0,touchstartY:0,touchendX:0,touchendY:0,touchmoveX:0,touchmoveY:0,offsetX:0,offsetY:0,left:t.left,right:t.right,up:t.up,down:t.down,start:t.start,move:t.move,end:t.end};return{touchstart:n=>U0(n,e),touchend:n=>j0(n,e),touchmove:n=>W0(n,e)}}function q0(t,e){var a;const n=e.value,s=n!=null&&n.parent?t.parentElement:t,r=(n==null?void 0:n.options)??{passive:!0},i=(a=e.instance)==null?void 0:a.$.uid;if(!s||!i)return;const o=G0(e.value);s._touchHandlers=s._touchHandlers??Object.create(null),s._touchHandlers[i]=o,mf(o).forEach(l=>{s.addEventListener(l,o[l],r)})}function Y0(t,e){var i,o;const n=(i=e.value)!=null&&i.parent?t.parentElement:t,s=(o=e.instance)==null?void 0:o.$.uid;if(!(n!=null&&n._touchHandlers)||!s)return;const r=n._touchHandlers[s];mf(r).forEach(a=>{n.removeEventListener(a,r[a])}),delete n._touchHandlers[s]}const X0={mounted:q0,unmounted:Y0},_l=Symbol.for("vuetify:v-slider");function Z0(t,e,n){const s=n==="vertical",r=e.getBoundingClientRect(),i="touches"in t?t.touches[0]:t;return s?i.clientY-(r.top+r.height/2):i.clientX-(r.left+r.width/2)}function K0(t,e){return"touches"in t&&t.touches.length?t.touches[0][e]:"changedTouches"in t&&t.changedTouches.length?t.changedTouches[0][e]:t[e]}const J0=Y({disabled:{type:Boolean,default:null},error:Boolean,readonly:{type:Boolean,default:null},max:{type:[Number,String],default:100},min:{type:[Number,String],default:0},step:{type:[Number,String],default:0},thumbColor:String,thumbLabel:{type:[Boolean,String],default:void 0,validator:t=>typeof t=="boolean"||t==="always"},thumbSize:{type:[Number,String],default:20},showTicks:{type:[Boolean,String],default:!1,validator:t=>typeof t=="boolean"||t==="always"},ticks:{type:[Array,Object]},tickSize:{type:[Number,String],default:2},color:String,trackColor:String,trackFillColor:String,trackSize:{type:[Number,String],default:4},direction:{type:String,default:"horizontal",validator:t=>["vertical","horizontal"].includes(t)},reverse:Boolean,...Qt(),...us({elevation:2}),ripple:{type:Boolean,default:!0}},"Slider"),Q0=t=>{const e=k(()=>parseFloat(t.min)),n=k(()=>parseFloat(t.max)),s=k(()=>+t.step>0?parseFloat(t.step):0),r=k(()=>Math.max(Iu(s.value),Iu(e.value)));function i(o){if(o=parseFloat(o),s.value<=0)return o;const a=yr(o,e.value,n.value),l=e.value%s.value,u=Math.round((a-l)/s.value)*s.value+l;return parseFloat(Math.min(u,n.value).toFixed(r.value))}return{min:e,max:n,step:s,decimals:r,roundValue:i}},e1=t=>{let{props:e,steps:n,onSliderStart:s,onSliderMove:r,onSliderEnd:i,getActiveThumb:o}=t;const{isRtl:a}=Jt(),l=fe(e,"reverse"),u=k(()=>e.direction==="vertical"),c=k(()=>u.value!==l.value),{min:d,max:f,step:h,decimals:m,roundValue:p}=n,v=k(()=>parseInt(e.thumbSize,10)),S=k(()=>parseInt(e.tickSize,10)),g=k(()=>parseInt(e.trackSize,10)),y=k(()=>(f.value-d.value)/h.value),b=fe(e,"disabled"),x=k(()=>e.error||e.disabled?void 0:e.thumbColor??e.color),_=k(()=>e.error||e.disabled?void 0:e.trackColor??e.color),C=k(()=>e.error||e.disabled?void 0:e.trackFillColor??e.color),M=be(!1),R=be(0),F=ee(),V=ee();function T(Z){var O;const xe=e.direction==="vertical",vn=xe?"top":"left",Fe=xe?"height":"width",_t=xe?"clientY":"clientX",{[vn]:Qe,[Fe]:Dn}=(O=F.value)==null?void 0:O.$el.getBoundingClientRect(),E=K0(Z,_t);let I=Math.min(Math.max((E-Qe-R.value)/Dn,0),1)||0;return(xe?c.value:c.value!==a.value)&&(I=1-I),p(d.value+I*(f.value-d.value))}const L=Z=>{i({value:T(Z)}),M.value=!1,R.value=0},A=Z=>{V.value=o(Z),V.value&&(V.value.focus(),M.value=!0,V.value.contains(Z.target)?R.value=Z0(Z,V.value,e.direction):(R.value=0,r({value:T(Z)})),s({value:T(Z)}))},N={passive:!0,capture:!0};function $(Z){r({value:T(Z)})}function z(Z){Z.stopPropagation(),Z.preventDefault(),L(Z),window.removeEventListener("mousemove",$,N),window.removeEventListener("mouseup",z)}function X(Z){var xe;L(Z),window.removeEventListener("touchmove",$,N),(xe=Z.target)==null||xe.removeEventListener("touchend",X)}function re(Z){var xe;A(Z),window.addEventListener("touchmove",$,N),(xe=Z.target)==null||xe.addEventListener("touchend",X,{passive:!1})}function te(Z){Z.preventDefault(),A(Z),window.addEventListener("mousemove",$,N),window.addEventListener("mouseup",z,{passive:!1})}const Q=Z=>{const xe=(Z-d.value)/(f.value-d.value)*100;return yr(isNaN(xe)?0:xe,0,100)},ce=fe(e,"showTicks"),Te=k(()=>ce.value?e.ticks?Array.isArray(e.ticks)?e.ticks.map(Z=>({value:Z,position:Q(Z),label:Z.toString()})):Object.keys(e.ticks).map(Z=>({value:parseFloat(Z),position:Q(parseFloat(Z)),label:e.ticks[Z]})):y.value!==1/0?ll(y.value+1).map(Z=>{const xe=d.value+Z*h.value;return{value:xe,position:Q(xe)}}):[]:[]),ge=k(()=>Te.value.some(Z=>{let{label:xe}=Z;return!!xe})),Le={activeThumbRef:V,color:fe(e,"color"),decimals:m,disabled:b,direction:fe(e,"direction"),elevation:fe(e,"elevation"),hasLabels:ge,isReversed:l,indexFromEnd:c,min:d,max:f,mousePressed:M,numTicks:y,onSliderMousedown:te,onSliderTouchstart:re,parsedTicks:Te,parseMouseMove:T,position:Q,readonly:fe(e,"readonly"),rounded:fe(e,"rounded"),roundValue:p,showTicks:ce,startOffset:R,step:h,thumbSize:v,thumbColor:x,thumbLabel:fe(e,"thumbLabel"),ticks:fe(e,"ticks"),tickSize:S,trackColor:_,trackContainerRef:F,trackFillColor:C,trackSize:g,vertical:u};return ut(_l,Le),Le},t1=Y({focused:Boolean,max:{type:Number,required:!0},min:{type:Number,required:!0},modelValue:{type:Number,required:!0},position:{type:Number,required:!0},ripple:{type:[Boolean,Object],default:!0},...we()},"VSliderThumb"),n1=me()({name:"VSliderThumb",directives:{Ripple:Ds},props:t1(),emits:{"update:modelValue":t=>!0},setup(t,e){let{slots:n,emit:s}=e;const r=Ne(_l),{isRtl:i,rtlClasses:o}=Jt();if(!r)throw new Error("[Vuetify] v-slider-thumb must be used inside v-slider or v-range-slider");const{thumbColor:a,step:l,disabled:u,thumbSize:c,thumbLabel:d,direction:f,isReversed:h,vertical:m,readonly:p,elevation:v,mousePressed:S,decimals:g,indexFromEnd:y}=r,b=k(()=>u.value?void 0:v.value),{elevationClasses:x}=cs(b),{textColorClasses:_,textColorStyles:C}=On(a),{pageup:M,pagedown:R,end:F,home:V,left:T,right:L,down:A,up:N}=sp,$=[M,R,F,V,T,L,A,N],z=k(()=>l.value?[1,2,3]:[1,5,10]);function X(te,Q){if(!$.includes(te.key))return;te.preventDefault();const ce=l.value||.1,Te=(t.max-t.min)/ce;if([T,L,A,N].includes(te.key)){const Le=(m.value?[i.value?T:L,h.value?A:N]:y.value!==i.value?[T,N]:[L,N]).includes(te.key)?1:-1,Z=te.shiftKey?2:te.ctrlKey?1:0;Q=Q+Le*ce*z.value[Z]}else if(te.key===V)Q=t.min;else if(te.key===F)Q=t.max;else{const ge=te.key===R?1:-1;Q=Q-ge*ce*(Te>100?Te/10:10)}return Math.max(t.min,Math.min(t.max,Q))}function re(te){const Q=X(te,t.modelValue);Q!=null&&s("update:modelValue",Q)}return ye(()=>{const te=oe(y.value?100-t.position:t.position,"%");return w("div",{class:["v-slider-thumb",{"v-slider-thumb--focused":t.focused,"v-slider-thumb--pressed":t.focused&&S.value},t.class,o.value],style:[{"--v-slider-thumb-position":te,"--v-slider-thumb-size":oe(c.value)},t.style],role:"slider",tabindex:u.value?-1:0,"aria-valuemin":t.min,"aria-valuemax":t.max,"aria-valuenow":t.modelValue,"aria-readonly":!!p.value,"aria-orientation":f.value,onKeydown:p.value?void 0:re},[w("div",{class:["v-slider-thumb__surface",_.value,x.value],style:{...C.value}},null),qt(w("div",{class:["v-slider-thumb__ripple",_.value],style:C.value},null),[[ls("ripple"),t.ripple,null,{circle:!0,center:!0}]]),w(dy,{origin:"bottom center"},{default:()=>{var Q;return[qt(w("div",{class:"v-slider-thumb__label-container"},[w("div",{class:["v-slider-thumb__label"]},[w("div",null,[((Q=n["thumb-label"])==null?void 0:Q.call(n,{modelValue:t.modelValue}))??t.modelValue.toFixed(l.value?g.value:1)])])]),[[uo,d.value&&t.focused||d.value==="always"]])]}})])}),{}}}),s1=Y({start:{type:Number,required:!0},stop:{type:Number,required:!0},...we()},"VSliderTrack"),r1=me()({name:"VSliderTrack",props:s1(),emits:{},setup(t,e){let{slots:n}=e;const s=Ne(_l);if(!s)throw new Error("[Vuetify] v-slider-track must be inside v-slider or v-range-slider");const{color:r,parsedTicks:i,rounded:o,showTicks:a,tickSize:l,trackColor:u,trackFillColor:c,trackSize:d,vertical:f,min:h,max:m,indexFromEnd:p}=s,{roundedClasses:v}=en(o),{backgroundColorClasses:S,backgroundColorStyles:g}=At(c),{backgroundColorClasses:y,backgroundColorStyles:b}=At(u),x=k(()=>`inset-${f.value?"block":"inline"}-${p.value?"end":"start"}`),_=k(()=>f.value?"height":"width"),C=k(()=>({[x.value]:"0%",[_.value]:"100%"})),M=k(()=>t.stop-t.start),R=k(()=>({[x.value]:oe(t.start,"%"),[_.value]:oe(M.value,"%")})),F=k(()=>a.value?(f.value?i.value.slice().reverse():i.value).map((T,L)=>{var N;const A=T.value!==h.value&&T.value!==m.value?oe(T.position,"%"):void 0;return w("div",{key:T.value,class:["v-slider-track__tick",{"v-slider-track__tick--filled":T.position>=t.start&&T.position<=t.stop,"v-slider-track__tick--first":T.value===h.value,"v-slider-track__tick--last":T.value===m.value}],style:{[x.value]:A}},[(T.label||n["tick-label"])&&w("div",{class:"v-slider-track__tick-label"},[((N=n["tick-label"])==null?void 0:N.call(n,{tick:T,index:L}))??T.label])])}):[]);return ye(()=>w("div",{class:["v-slider-track",v.value,t.class],style:[{"--v-slider-track-size":oe(d.value),"--v-slider-tick-size":oe(l.value)},t.style]},[w("div",{class:["v-slider-track__background",y.value,{"v-slider-track__background--opacity":!!r.value||!c.value}],style:{...C.value,...b.value}},null),w("div",{class:["v-slider-track__fill",S.value],style:{...R.value,...g.value}},null),a.value&&w("div",{class:["v-slider-track__ticks",{"v-slider-track__ticks--always-show":a.value==="always"}]},[F.value])])),{}}}),i1=Y({...xh(),...J0(),...wh(),modelValue:{type:[Number,String],default:0}},"VSlider"),js=me()({name:"VSlider",props:i1(),emits:{"update:focused":t=>!0,"update:modelValue":t=>!0,start:t=>!0,end:t=>!0},setup(t,e){let{slots:n,emit:s}=e;const r=ee(),{rtlClasses:i}=Jt(),o=Q0(t),a=ct(t,"modelValue",void 0,_=>o.roundValue(_??o.min.value)),{min:l,max:u,mousePressed:c,roundValue:d,onSliderMousedown:f,onSliderTouchstart:h,trackContainerRef:m,position:p,hasLabels:v,readonly:S}=e1({props:t,steps:o,onSliderStart:()=>{s("start",a.value)},onSliderEnd:_=>{let{value:C}=_;const M=d(C);a.value=M,s("end",M)},onSliderMove:_=>{let{value:C}=_;return a.value=d(C)},getActiveThumb:()=>{var _;return(_=r.value)==null?void 0:_.$el}}),{isFocused:g,focus:y,blur:b}=e0(t),x=k(()=>p(a.value));return ye(()=>{const _=lc.filterProps(t),C=!!(t.label||n.label||n.prepend);return w(lc,Pe({class:["v-slider",{"v-slider--has-labels":!!n["tick-label"]||v.value,"v-slider--focused":g.value,"v-slider--pressed":c.value,"v-slider--disabled":t.disabled},i.value,t.class],style:t.style},_,{focused:g.value}),{...n,prepend:C?M=>{var R,F;return w(Ie,null,[((R=n.label)==null?void 0:R.call(n,M))??(t.label?w(vh,{id:M.id.value,class:"v-slider__label",text:t.label},null):void 0),(F=n.prepend)==null?void 0:F.call(n,M)])}:void 0,default:M=>{let{id:R,messagesId:F}=M;return w("div",{class:"v-slider__container",onMousedown:S.value?void 0:f,onTouchstartPassive:S.value?void 0:h},[w("input",{id:R.value,name:t.name||R.value,disabled:!!t.disabled,readonly:!!t.readonly,tabindex:"-1",value:a.value},null),w(r1,{ref:m,start:0,stop:x.value},{"tick-label":n["tick-label"]}),w(n1,{ref:r,"aria-describedby":F.value,focused:g.value,min:l.value,max:u.value,modelValue:a.value,"onUpdate:modelValue":V=>a.value=V,position:x.value,elevation:t.elevation,onFocus:y,onBlur:b,ripple:t.ripple},{"thumb-label":n["thumb-label"]})])}})}),{}}}),Ra={"001":1,AD:1,AE:6,AF:6,AG:0,AI:1,AL:1,AM:1,AN:1,AR:1,AS:0,AT:1,AU:1,AX:1,AZ:1,BA:1,BD:0,BE:1,BG:1,BH:6,BM:1,BN:1,BR:0,BS:0,BT:0,BW:0,BY:1,BZ:0,CA:0,CH:1,CL:1,CM:1,CN:1,CO:0,CR:1,CY:1,CZ:1,DE:1,DJ:6,DK:1,DM:0,DO:0,DZ:6,EC:1,EE:1,EG:6,ES:1,ET:0,FI:1,FJ:1,FO:1,FR:1,GB:1,"GB-alt-variant":0,GE:1,GF:1,GP:1,GR:1,GT:0,GU:0,HK:0,HN:0,HR:1,HU:1,ID:0,IE:1,IL:0,IN:0,IQ:6,IR:6,IS:1,IT:1,JM:0,JO:6,JP:0,KE:0,KG:1,KH:0,KR:0,KW:6,KZ:1,LA:0,LB:1,LI:1,LK:1,LT:1,LU:1,LV:1,LY:6,MC:1,MD:1,ME:1,MH:0,MK:1,MM:0,MN:1,MO:0,MQ:1,MT:0,MV:5,MX:0,MY:1,MZ:0,NI:0,NL:1,NO:1,NP:0,NZ:1,OM:6,PA:0,PE:0,PH:0,PK:0,PL:1,PR:0,PT:0,PY:0,QA:6,RE:1,RO:1,RS:1,RU:1,SA:0,SD:6,SE:1,SG:0,SI:1,SK:1,SM:1,SV:0,SY:6,TH:0,TJ:1,TM:1,TR:1,TT:0,TW:0,UA:1,UM:0,US:0,UY:1,UZ:1,VA:1,VE:0,VI:0,VN:1,WS:0,XK:1,YE:0,ZA:0,ZW:0};function o1(t,e){const n=[];let s=[];const r=Fh(t),i=Lh(t),o=(r.getDay()-Ra[e.slice(-2).toUpperCase()]+7)%7,a=(i.getDay()-Ra[e.slice(-2).toUpperCase()]+7)%7;for(let l=0;l<o;l++){const u=new Date(r);u.setDate(u.getDate()-(o-l)),s.push(u)}for(let l=1;l<=i.getDate();l++){const u=new Date(t.getFullYear(),t.getMonth(),l);s.push(u),s.length===7&&(n.push(s),s=[])}for(let l=1;l<7-a;l++){const u=new Date(i);u.setDate(u.getDate()+l),s.push(u)}return s.length>0&&n.push(s),n}function a1(t){const e=new Date(t);for(;e.getDay()!==0;)e.setDate(e.getDate()-1);return e}function l1(t){const e=new Date(t);for(;e.getDay()!==6;)e.setDate(e.getDate()+1);return e}function Fh(t){return new Date(t.getFullYear(),t.getMonth(),1)}function Lh(t){return new Date(t.getFullYear(),t.getMonth()+1,0)}function u1(t){const e=t.split("-").map(Number);return new Date(e[0],e[1]-1,e[2])}const c1=/^([12]\d{3}-([1-9]|0[1-9]|1[0-2])-([1-9]|0[1-9]|[12]\d|3[01]))$/;function Vh(t){if(t==null)return new Date;if(t instanceof Date)return t;if(typeof t=="string"){let e;if(c1.test(t))return u1(t);if(e=Date.parse(t),!isNaN(e))return new Date(e)}return null}const yc=new Date(2e3,0,2);function d1(t){const e=Ra[t.slice(-2).toUpperCase()];return ll(7).map(n=>{const s=new Date(yc);return s.setDate(yc.getDate()+e+n),new Intl.DateTimeFormat(t,{weekday:"narrow"}).format(s)})}function f1(t,e,n,s){const r=Vh(t)??new Date,i=s==null?void 0:s[e];if(typeof i=="function")return i(r,e,n);let o={};switch(e){case"fullDateWithWeekday":o={weekday:"long",day:"numeric",month:"long",year:"numeric"};break;case"hours12h":o={hour:"numeric",hour12:!0};break;case"normalDateWithWeekday":o={weekday:"short",day:"numeric",month:"short"};break;case"keyboardDate":o={day:"2-digit",month:"2-digit",year:"numeric"};break;case"monthAndDate":o={month:"long",day:"numeric"};break;case"monthAndYear":o={month:"long",year:"numeric"};break;case"month":o={month:"long"};break;case"monthShort":o={month:"short"};break;case"dayOfMonth":return new Intl.NumberFormat(n).format(r.getDate());case"shortDate":o={year:"2-digit",month:"numeric",day:"numeric"};break;case"weekdayShort":o={weekday:"short"};break;case"year":o={year:"numeric"};break;default:o=i??{timeZone:"UTC",timeZoneName:"short"}}return new Intl.DateTimeFormat(n,o).format(r)}function h1(t,e){const n=t.toJsDate(e),s=n.getFullYear(),r=Fu(String(n.getMonth()+1),2,"0"),i=Fu(String(n.getDate()),2,"0");return`${s}-${r}-${i}`}function m1(t){const[e,n,s]=t.split("-").map(Number);return new Date(e,n-1,s)}function g1(t,e){const n=new Date(t);return n.setMinutes(n.getMinutes()+e),n}function v1(t,e){const n=new Date(t);return n.setHours(n.getHours()+e),n}function p1(t,e){const n=new Date(t);return n.setDate(n.getDate()+e),n}function y1(t,e){const n=new Date(t);return n.setDate(n.getDate()+e*7),n}function b1(t,e){const n=new Date(t);return n.setMonth(n.getMonth()+e),n}function x1(t){return t.getFullYear()}function w1(t){return t.getMonth()}function _1(t){return new Date(t.getFullYear(),t.getMonth()+1,1)}function S1(t){return t.getHours()}function C1(t){return t.getMinutes()}function E1(t){return new Date(t.getFullYear(),0,1)}function M1(t){return new Date(t.getFullYear(),11,31)}function T1(t,e){return Ia(t,e[0])&&R1(t,e[1])}function A1(t){const e=new Date(t);return e instanceof Date&&!isNaN(e.getTime())}function Ia(t,e){return t.getTime()>e.getTime()}function R1(t,e){return t.getTime()<e.getTime()}function bc(t,e){return t.getTime()===e.getTime()}function I1(t,e){return t.getDate()===e.getDate()&&t.getMonth()===e.getMonth()&&t.getFullYear()===e.getFullYear()}function k1(t,e){return t.getMonth()===e.getMonth()&&t.getFullYear()===e.getFullYear()}function F1(t,e,n){const s=new Date(t),r=new Date(e);switch(n){case"years":return s.getFullYear()-r.getFullYear();case"quarters":return Math.floor((s.getMonth()-r.getMonth()+(s.getFullYear()-r.getFullYear())*12)/4);case"months":return s.getMonth()-r.getMonth()+(s.getFullYear()-r.getFullYear())*12;case"weeks":return Math.floor((s.getTime()-r.getTime())/(1e3*60*60*24*7));case"days":return Math.floor((s.getTime()-r.getTime())/(1e3*60*60*24));case"hours":return Math.floor((s.getTime()-r.getTime())/(1e3*60*60));case"minutes":return Math.floor((s.getTime()-r.getTime())/(1e3*60));case"seconds":return Math.floor((s.getTime()-r.getTime())/1e3);default:return s.getTime()-r.getTime()}}function L1(t,e){const n=new Date(t);return n.setHours(e),n}function V1(t,e){const n=new Date(t);return n.setMinutes(e),n}function P1(t,e){const n=new Date(t);return n.setMonth(e),n}function O1(t,e){const n=new Date(t);return n.setFullYear(e),n}function N1(t){return new Date(t.getFullYear(),t.getMonth(),t.getDate(),0,0,0,0)}function B1(t){return new Date(t.getFullYear(),t.getMonth(),t.getDate(),23,59,59,999)}class D1{constructor(e){this.locale=e.locale,this.formats=e.formats}date(e){return Vh(e)}toJsDate(e){return e}toISO(e){return h1(this,e)}parseISO(e){return m1(e)}addMinutes(e,n){return g1(e,n)}addHours(e,n){return v1(e,n)}addDays(e,n){return p1(e,n)}addWeeks(e,n){return y1(e,n)}addMonths(e,n){return b1(e,n)}getWeekArray(e){return o1(e,this.locale)}startOfWeek(e){return a1(e)}endOfWeek(e){return l1(e)}startOfMonth(e){return Fh(e)}endOfMonth(e){return Lh(e)}format(e,n){return f1(e,n,this.locale,this.formats)}isEqual(e,n){return bc(e,n)}isValid(e){return A1(e)}isWithinRange(e,n){return T1(e,n)}isAfter(e,n){return Ia(e,n)}isBefore(e,n){return!Ia(e,n)&&!bc(e,n)}isSameDay(e,n){return I1(e,n)}isSameMonth(e,n){return k1(e,n)}setMinutes(e,n){return V1(e,n)}setHours(e,n){return L1(e,n)}setMonth(e,n){return P1(e,n)}setYear(e,n){return O1(e,n)}getDiff(e,n,s){return F1(e,n,s)}getWeekdays(){return d1(this.locale)}getYear(e){return x1(e)}getMonth(e){return w1(e)}getNextMonth(e){return _1(e)}getHours(e){return S1(e)}getMinutes(e){return C1(e)}startOfDay(e){return N1(e)}endOfDay(e){return B1(e)}startOfYear(e){return E1(e)}endOfYear(e){return M1(e)}}const z1=Symbol.for("vuetify:date-options"),xc=Symbol.for("vuetify:date-adapter");function H1(t,e){const n=pt({adapter:D1,locale:{af:"af-ZA",bg:"bg-BG",ca:"ca-ES",ckb:"",cs:"cs-CZ",de:"de-DE",el:"el-GR",en:"en-US",et:"et-EE",fa:"fa-IR",fi:"fi-FI",hr:"hr-HR",hu:"hu-HU",he:"he-IL",id:"id-ID",it:"it-IT",ja:"ja-JP",ko:"ko-KR",lv:"lv-LV",lt:"lt-LT",nl:"nl-NL",no:"no-NO",pl:"pl-PL",pt:"pt-PT",ro:"ro-RO",ru:"ru-RU",sk:"sk-SK",sl:"sl-SI",srCyrl:"sr-SP",srLatn:"sr-SP",sv:"sv-SE",th:"th-TH",tr:"tr-TR",az:"az-AZ",uk:"uk-UA",vi:"vi-VN",zhHans:"zh-CN",zhHant:"zh-TW"}},t);return{options:n,instance:$1(n,e)}}function $1(t,e){const n=tt(typeof t.adapter=="function"?new t.adapter({locale:t.locale[e.current.value]??e.current.value,formats:t.formats}):t.adapter);return le(e.current,s=>{n.locale=t.locale[s]??s??n.locale}),n}const U1=Symbol.for("vuetify:goto");function j1(){return{container:void 0,duration:300,layout:!1,offset:0,easing:"easeInOutCubic",patterns:{linear:t=>t,easeInQuad:t=>t**2,easeOutQuad:t=>t*(2-t),easeInOutQuad:t=>t<.5?2*t**2:-1+(4-2*t)*t,easeInCubic:t=>t**3,easeOutCubic:t=>--t**3+1,easeInOutCubic:t=>t<.5?4*t**3:(t-1)*(2*t-2)*(2*t-2)+1,easeInQuart:t=>t**4,easeOutQuart:t=>1- --t**4,easeInOutQuart:t=>t<.5?8*t**4:1-8*--t**4,easeInQuint:t=>t**5,easeOutQuint:t=>1+--t**5,easeInOutQuint:t=>t<.5?16*t**5:1+16*--t**5}}}function W1(t,e){return{rtl:e.isRtl,options:pt(j1(),t)}}const G1=Y({fluid:{type:Boolean,default:!1},...we(),...Je()},"VContainer"),q1=me()({name:"VContainer",props:G1(),setup(t,e){let{slots:n}=e;const{rtlClasses:s}=Jt();return ye(()=>w(t.tag,{class:["v-container",{"v-container--fluid":t.fluid},s.value,t.class],style:t.style},n)),{}}}),Ph=bo.reduce((t,e)=>(t[e]={type:[Boolean,String,Number],default:!1},t),{}),Oh=bo.reduce((t,e)=>{const n="offset"+dn(e);return t[n]={type:[String,Number],default:null},t},{}),Nh=bo.reduce((t,e)=>{const n="order"+dn(e);return t[n]={type:[String,Number],default:null},t},{}),wc={col:Object.keys(Ph),offset:Object.keys(Oh),order:Object.keys(Nh)};function Y1(t,e,n){let s=t;if(!(n==null||n===!1)){if(e){const r=e.replace(t,"");s+=`-${r}`}return t==="col"&&(s="v-"+s),t==="col"&&(n===""||n===!0)||(s+=`-${n}`),s.toLowerCase()}}const X1=["auto","start","end","center","baseline","stretch"],Z1=Y({cols:{type:[Boolean,String,Number],default:!1},...Ph,offset:{type:[String,Number],default:null},...Oh,order:{type:[String,Number],default:null},...Nh,alignSelf:{type:String,default:null,validator:t=>X1.includes(t)},...we(),...Je()},"VCol"),K1=me()({name:"VCol",props:Z1(),setup(t,e){let{slots:n}=e;const s=k(()=>{const r=[];let i;for(i in wc)wc[i].forEach(a=>{const l=t[a],u=Y1(i,a,l);u&&r.push(u)});const o=r.some(a=>a.startsWith("v-col-"));return r.push({"v-col":!o||!t.cols,[`v-col-${t.cols}`]:t.cols,[`offset-${t.offset}`]:t.offset,[`order-${t.order}`]:t.order,[`align-self-${t.alignSelf}`]:t.alignSelf}),r});return()=>{var r;return Bn(t.tag,{class:[s.value,t.class],style:t.style},(r=n.default)==null?void 0:r.call(n))}}}),Sl=["start","end","center"],Bh=["space-between","space-around","space-evenly"];function Cl(t,e){return bo.reduce((n,s)=>{const r=t+dn(s);return n[r]=e(),n},{})}const J1=[...Sl,"baseline","stretch"],Dh=t=>J1.includes(t),zh=Cl("align",()=>({type:String,default:null,validator:Dh})),Q1=[...Sl,...Bh],Hh=t=>Q1.includes(t),$h=Cl("justify",()=>({type:String,default:null,validator:Hh})),eb=[...Sl,...Bh,"stretch"],Uh=t=>eb.includes(t),jh=Cl("alignContent",()=>({type:String,default:null,validator:Uh})),_c={align:Object.keys(zh),justify:Object.keys($h),alignContent:Object.keys(jh)},tb={align:"align",justify:"justify",alignContent:"align-content"};function nb(t,e,n){let s=tb[t];if(n!=null){if(e){const r=e.replace(t,"");s+=`-${r}`}return s+=`-${n}`,s.toLowerCase()}}const sb=Y({dense:Boolean,noGutters:Boolean,align:{type:String,default:null,validator:Dh},...zh,justify:{type:String,default:null,validator:Hh},...$h,alignContent:{type:String,default:null,validator:Uh},...jh,...we(),...Je()},"VRow"),rb=me()({name:"VRow",props:sb(),setup(t,e){let{slots:n}=e;const s=k(()=>{const r=[];let i;for(i in _c)_c[i].forEach(o=>{const a=t[o],l=nb(i,o,a);l&&r.push(l)});return r.push({"v-row--no-gutters":t.noGutters,"v-row--dense":t.dense,[`align-${t.align}`]:t.align,[`justify-${t.justify}`]:t.justify,[`align-content-${t.alignContent}`]:t.alignContent}),r});return()=>{var r;return Bn(t.tag,{class:["v-row",s.value,t.class],style:t.style},(r=n.default)==null?void 0:r.call(n))}}}),ka=fo("v-spacer","div","VSpacer"),_r=Symbol.for("vuetify:v-expansion-panel"),ib=["default","accordion","inset","popout"],ob=Y({color:String,flat:Boolean,focusable:Boolean,static:Boolean,tile:Boolean,variant:{type:String,default:"default",validator:t=>ib.includes(t)},readonly:Boolean,...we(),...Wf(),...Je(),...dt()},"VExpansionPanels"),ab=me()({name:"VExpansionPanels",props:ob(),emits:{"update:modelValue":t=>!0},setup(t,e){let{slots:n}=e;Yf(t,_r);const{themeClasses:s}=Ft(t),r=k(()=>t.variant&&`v-expansion-panels--variant-${t.variant}`);return mn({VExpansionPanel:{color:fe(t,"color"),readonly:fe(t,"readonly")},VExpansionPanelTitle:{focusable:fe(t,"focusable"),static:fe(t,"static")}}),ye(()=>w(t.tag,{class:["v-expansion-panels",{"v-expansion-panels--flat":t.flat,"v-expansion-panels--tile":t.tile},s.value,r.value,t.class],style:t.style},n)),{}}}),lb=Y({...we(),...xl()},"VExpansionPanelText"),Wh=me()({name:"VExpansionPanelText",props:lb(),setup(t,e){let{slots:n}=e;const s=Ne(_r);if(!s)throw new Error("[Vuetify] v-expansion-panel-text needs to be placed inside v-expansion-panel");const{hasContent:r,onAfterLeave:i}=Mh(t,s.isSelected);return ye(()=>w(Bf,{onAfterLeave:i},{default:()=>{var o;return[qt(w("div",{class:["v-expansion-panel-text",t.class],style:t.style},[n.default&&r.value&&w("div",{class:"v-expansion-panel-text__wrapper"},[(o=n.default)==null?void 0:o.call(n)])]),[[uo,s.isSelected.value]])]}})),{}}}),Gh=Y({color:String,expandIcon:{type:Ke,default:"$expand"},collapseIcon:{type:Ke,default:"$collapse"},hideActions:Boolean,focusable:Boolean,static:Boolean,ripple:{type:[Boolean,Object],default:!1},readonly:Boolean,...we()},"VExpansionPanelTitle"),qh=me()({name:"VExpansionPanelTitle",directives:{Ripple:Ds},props:Gh(),setup(t,e){let{slots:n}=e;const s=Ne(_r);if(!s)throw new Error("[Vuetify] v-expansion-panel-title needs to be placed inside v-expansion-panel");const{backgroundColorClasses:r,backgroundColorStyles:i}=At(t,"color"),o=k(()=>({collapseIcon:t.collapseIcon,disabled:s.disabled.value,expanded:s.isSelected.value,expandIcon:t.expandIcon,readonly:t.readonly}));return ye(()=>{var a;return qt(w("button",{class:["v-expansion-panel-title",{"v-expansion-panel-title--active":s.isSelected.value,"v-expansion-panel-title--focusable":t.focusable,"v-expansion-panel-title--static":t.static},r.value,t.class],style:[i.value,t.style],type:"button",tabindex:s.disabled.value?-1:void 0,disabled:s.disabled.value,"aria-expanded":s.isSelected.value,onClick:t.readonly?void 0:s.toggle},[w("span",{class:"v-expansion-panel-title__overlay"},null),(a=n.default)==null?void 0:a.call(n,o.value),!t.hideActions&&w("span",{class:"v-expansion-panel-title__icon"},[n.actions?n.actions(o.value):w(ln,{icon:s.isSelected.value?t.collapseIcon:t.expandIcon},null)])]),[[ls("ripple"),t.ripple]])}),{}}}),ub=Y({title:String,text:String,bgColor:String,...we(),...us(),...Gf(),...xl(),...Qt(),...Je(),...Gh()},"VExpansionPanel"),cb=me()({name:"VExpansionPanel",props:ub(),emits:{"group:selected":t=>!0},setup(t,e){let{slots:n}=e;const s=qf(t,_r),{backgroundColorClasses:r,backgroundColorStyles:i}=At(t,"bgColor"),{elevationClasses:o}=cs(t),{roundedClasses:a}=en(t),l=k(()=>(s==null?void 0:s.disabled.value)||t.disabled),u=k(()=>s.group.items.value.reduce((f,h,m)=>(s.group.selected.value.includes(h.id)&&f.push(m),f),[])),c=k(()=>{const f=s.group.items.value.findIndex(h=>h.id===s.id);return!s.isSelected.value&&u.value.some(h=>h-f===1)}),d=k(()=>{const f=s.group.items.value.findIndex(h=>h.id===s.id);return!s.isSelected.value&&u.value.some(h=>h-f===-1)});return ut(_r,s),mn({VExpansionPanelText:{eager:fe(t,"eager")},VExpansionPanelTitle:{readonly:fe(t,"readonly")}}),ye(()=>{const f=!!(n.text||t.text),h=!!(n.title||t.title);return w(t.tag,{class:["v-expansion-panel",{"v-expansion-panel--active":s.isSelected.value,"v-expansion-panel--before-active":c.value,"v-expansion-panel--after-active":d.value,"v-expansion-panel--disabled":l.value},a.value,r.value,t.class],style:[i.value,t.style]},{default:()=>{var m;return[w("div",{class:["v-expansion-panel__shadow",...o.value]},null),h&&w(qh,{key:"title",collapseIcon:t.collapseIcon,color:t.color,expandIcon:t.expandIcon,hideActions:t.hideActions,ripple:t.ripple},{default:()=>[n.title?n.title():t.title]}),f&&w(Wh,{key:"text"},{default:()=>[n.text?n.text():t.text]}),(m=n.default)==null?void 0:m.call(n)]}})}),{}}}),db=Y({...we(),...Gp()},"VLayout"),fb=me()({name:"VLayout",props:db(),setup(t,e){let{slots:n}=e;const{layoutClasses:s,layoutStyles:r,getLayoutItem:i,items:o,layoutRef:a}=Kp(t);return ye(()=>{var l;return w("div",{ref:a,class:[s.value,t.class],style:[r.value,t.style]},[(l=n.default)==null?void 0:l.call(n)])}),{getLayoutItem:i,items:o}}}),hb=Y({scrollable:Boolean,...we(),...Je({tag:"main"})},"VMain"),mb=me()({name:"VMain",props:hb(),setup(t,e){let{slots:n}=e;const{mainStyles:s}=Yp(),{ssrBootStyles:r}=Uf();return ye(()=>w(t.tag,{class:["v-main",{"v-main--scrollable":t.scrollable},t.class],style:[s.value,r.value,t.style]},{default:()=>{var i,o;return[t.scrollable?w("div",{class:"v-main__scroller"},[(i=n.default)==null?void 0:i.call(n)]):(o=n.default)==null?void 0:o.call(n)]}})),{}}});function gb(t){let{rootEl:e,isSticky:n,layoutItemStyles:s}=t;const r=be(!1),i=be(0),o=k(()=>{const u=typeof r.value=="boolean"?"top":r.value;return[n.value?{top:"auto",bottom:"auto",height:void 0}:void 0,r.value?{[u]:oe(i.value)}:{top:s.value.top}]});Zt(()=>{le(n,u=>{u?window.addEventListener("scroll",l,{passive:!0}):window.removeEventListener("scroll",l)},{immediate:!0})}),kt(()=>{window.removeEventListener("scroll",l)});let a=0;function l(){const u=a>window.scrollY?"up":"down",c=e.value.getBoundingClientRect(),d=parseFloat(s.value.top??0),f=window.scrollY-Math.max(0,i.value-d),h=c.height+Math.max(i.value,d)-window.scrollY-window.innerHeight,m=parseFloat(getComputedStyle(e.value).getPropertyValue("--v-body-scroll-y"))||0;c.height<window.innerHeight-d?(r.value="top",i.value=d):u==="up"&&r.value==="bottom"||u==="down"&&r.value==="top"?(i.value=window.scrollY+c.top-m,r.value=!0):u==="down"&&h<=0?(i.value=0,r.value="bottom"):u==="up"&&f<=0&&(m?r.value!=="top"&&(i.value=-f+m+d,r.value="top"):(i.value=c.top+f,r.value="top")),a=window.scrollY}return{isStuck:r,stickyStyles:o}}const vb=100,pb=20;function Sc(t){return(t<0?-1:1)*Math.sqrt(Math.abs(t))*1.41421356237}function Cc(t){if(t.length<2)return 0;if(t.length===2)return t[1].t===t[0].t?0:(t[1].d-t[0].d)/(t[1].t-t[0].t);let e=0;for(let n=t.length-1;n>0;n--){if(t[n].t===t[n-1].t)continue;const s=Sc(e),r=(t[n].d-t[n-1].d)/(t[n].t-t[n-1].t);e+=(r-s)*Math.abs(r),n===t.length-1&&(e*=.5)}return Sc(e)*1e3}function yb(){const t={};function e(r){Array.from(r.changedTouches).forEach(i=>{(t[i.identifier]??(t[i.identifier]=new lp(pb))).push([r.timeStamp,i])})}function n(r){Array.from(r.changedTouches).forEach(i=>{delete t[i.identifier]})}function s(r){var u;const i=(u=t[r])==null?void 0:u.values().reverse();if(!i)throw new Error(`No samples for touch id ${r}`);const o=i[0],a=[],l=[];for(const c of i){if(o[0]-c[0]>vb)break;a.push({t:c[0],d:c[1].clientX}),l.push({t:c[0],d:c[1].clientY})}return{x:Cc(a),y:Cc(l),get direction(){const{x:c,y:d}=this,[f,h]=[Math.abs(c),Math.abs(d)];return f>h&&c>=0?"right":f>h&&c<=0?"left":h>f&&d>=0?"down":h>f&&d<=0?"up":bb()}}}return{addMovement:e,endTouch:n,getVelocity:s}}function bb(){throw new Error}function xb(t){let{isActive:e,isTemporary:n,width:s,touchless:r,position:i}=t;Zt(()=>{window.addEventListener("touchstart",S,{passive:!0}),window.addEventListener("touchmove",g,{passive:!1}),window.addEventListener("touchend",y,{passive:!0})}),kt(()=>{window.removeEventListener("touchstart",S),window.removeEventListener("touchmove",g),window.removeEventListener("touchend",y)});const o=k(()=>["left","right"].includes(i.value)),{addMovement:a,endTouch:l,getVelocity:u}=yb();let c=!1;const d=be(!1),f=be(0),h=be(0);let m;function p(x,_){return(i.value==="left"?x:i.value==="right"?document.documentElement.clientWidth-x:i.value==="top"?x:i.value==="bottom"?document.documentElement.clientHeight-x:ps())-(_?s.value:0)}function v(x){let _=arguments.length>1&&arguments[1]!==void 0?arguments[1]:!0;const C=i.value==="left"?(x-h.value)/s.value:i.value==="right"?(document.documentElement.clientWidth-x-h.value)/s.value:i.value==="top"?(x-h.value)/s.value:i.value==="bottom"?(document.documentElement.clientHeight-x-h.value)/s.value:ps();return _?Math.max(0,Math.min(1,C)):C}function S(x){if(r.value)return;const _=x.changedTouches[0].clientX,C=x.changedTouches[0].clientY,M=25,R=i.value==="left"?_<M:i.value==="right"?_>document.documentElement.clientWidth-M:i.value==="top"?C<M:i.value==="bottom"?C>document.documentElement.clientHeight-M:ps(),F=e.value&&(i.value==="left"?_<s.value:i.value==="right"?_>document.documentElement.clientWidth-s.value:i.value==="top"?C<s.value:i.value==="bottom"?C>document.documentElement.clientHeight-s.value:ps());(R||F||e.value&&n.value)&&(c=!0,m=[_,C],h.value=p(o.value?_:C,e.value),f.value=v(o.value?_:C),l(x),a(x))}function g(x){const _=x.changedTouches[0].clientX,C=x.changedTouches[0].clientY;if(c){if(!x.cancelable){c=!1;return}const R=Math.abs(_-m[0]),F=Math.abs(C-m[1]);(o.value?R>F&&R>3:F>R&&F>3)?(d.value=!0,c=!1):(o.value?F:R)>3&&(c=!1)}if(!d.value)return;x.preventDefault(),a(x);const M=v(o.value?_:C,!1);f.value=Math.max(0,Math.min(1,M)),M>1?h.value=p(o.value?_:C,!0):M<0&&(h.value=p(o.value?_:C,!1))}function y(x){if(c=!1,!d.value)return;a(x),d.value=!1;const _=u(x.changedTouches[0].identifier),C=Math.abs(_.x),M=Math.abs(_.y);(o.value?C>M&&C>400:M>C&&M>3)?e.value=_.direction===({left:"right",right:"left",top:"down",bottom:"up"}[i.value]||ps()):e.value=f.value>.5}const b=k(()=>d.value?{transform:i.value==="left"?`translateX(calc(-100% + ${f.value*s.value}px))`:i.value==="right"?`translateX(calc(100% - ${f.value*s.value}px))`:i.value==="top"?`translateY(calc(-100% + ${f.value*s.value}px))`:i.value==="bottom"?`translateY(calc(100% - ${f.value*s.value}px))`:ps(),transition:"none"}:void 0);return{isDragging:d,dragProgress:f,dragStyles:b}}function ps(){throw new Error}const wb=["start","end","left","right","top","bottom"],_b=Y({color:String,disableResizeWatcher:Boolean,disableRouteWatcher:Boolean,expandOnHover:Boolean,floating:Boolean,modelValue:{type:Boolean,default:null},permanent:Boolean,rail:{type:Boolean,default:null},railWidth:{type:[Number,String],default:56},scrim:{type:[Boolean,String],default:!0},image:String,temporary:Boolean,touchless:Boolean,width:{type:[Number,String],default:256},location:{type:String,default:"start",validator:t=>wb.includes(t)},sticky:Boolean,...Fr(),...we(),...a0(),...us(),...qp(),...Qt(),...Je({tag:"nav"}),...dt()},"VNavigationDrawer"),Sb=me()({name:"VNavigationDrawer",props:_b(),emits:{"update:modelValue":t=>!0,"update:rail":t=>!0},setup(t,e){let{attrs:n,emit:s,slots:r}=e;const{isRtl:i}=Jt(),{themeClasses:o}=Ft(t),{borderClasses:a}=Lr(t),{backgroundColorClasses:l,backgroundColorStyles:u}=At(fe(t,"color")),{elevationClasses:c}=cs(t),{displayClasses:d,mobile:f}=_h(t),{roundedClasses:h}=en(t),m=ih(),p=ct(t,"modelValue",null,te=>!!te),{ssrBootStyles:v}=Uf(),{scopeId:S}=xo(),g=ee(),y=be(!1),b=k(()=>t.rail&&t.expandOnHover&&y.value?Number(t.width):Number(t.rail?t.railWidth:t.width)),x=k(()=>xa(t.location,i.value)),_=k(()=>!t.permanent&&(f.value||t.temporary)),C=k(()=>t.sticky&&!_.value&&x.value!=="bottom");an(()=>t.expandOnHover&&t.rail!=null,()=>{le(y,te=>s("update:rail",!te))}),an(()=>!t.disableResizeWatcher,()=>{le(_,te=>!t.permanent&&It(()=>p.value=!te))}),an(()=>!t.disableRouteWatcher&&!!m,()=>{le(m.currentRoute,()=>_.value&&(p.value=!1))}),le(()=>t.permanent,te=>{te&&(p.value=!0)}),io(()=>{t.modelValue!=null||_.value||(p.value=t.permanent||!f.value)});const{isDragging:M,dragProgress:R,dragStyles:F}=xb({isActive:p,isTemporary:_,width:b,touchless:fe(t,"touchless"),position:x}),V=k(()=>{const te=_.value?0:t.rail&&t.expandOnHover?Number(t.railWidth):b.value;return M.value?te*R.value:te}),{layoutItemStyles:T,layoutItemScrimStyles:L}=Xp({id:t.name,order:k(()=>parseInt(t.order,10)),position:x,layoutSize:V,elementSize:b,active:k(()=>p.value||M.value),disableTransitions:k(()=>M.value),absolute:k(()=>t.absolute||C.value&&typeof A.value!="string")}),{isStuck:A,stickyStyles:N}=gb({rootEl:g,isSticky:C,layoutItemStyles:T}),$=At(k(()=>typeof t.scrim=="string"?t.scrim:null)),z=k(()=>({...M.value?{opacity:R.value*.2,transition:"none"}:void 0,...L.value}));mn({VList:{bgColor:"transparent"}});function X(){y.value=!0}function re(){y.value=!1}return ye(()=>{const te=r.image||t.image;return w(Ie,null,[w(t.tag,Pe({ref:g,onMouseenter:X,onMouseleave:re,class:["v-navigation-drawer",`v-navigation-drawer--${x.value}`,{"v-navigation-drawer--expand-on-hover":t.expandOnHover,"v-navigation-drawer--floating":t.floating,"v-navigation-drawer--is-hovering":y.value,"v-navigation-drawer--rail":t.rail,"v-navigation-drawer--temporary":_.value,"v-navigation-drawer--active":p.value,"v-navigation-drawer--sticky":C.value},o.value,l.value,a.value,d.value,c.value,h.value,t.class],style:[u.value,T.value,F.value,v.value,N.value,t.style]},S,n),{default:()=>{var Q,ce,Te;return[te&&w("div",{key:"image",class:"v-navigation-drawer__img"},[r.image?w(Tt,{key:"image-defaults",disabled:!t.image,defaults:{VImg:{alt:"",cover:!0,height:"inherit",src:t.image}}},r.image):w(go,{key:"image-img",alt:"",cover:!0,height:"inherit",src:t.image},null)]),r.prepend&&w("div",{class:"v-navigation-drawer__prepend"},[(Q=r.prepend)==null?void 0:Q.call(r)]),w("div",{class:"v-navigation-drawer__content"},[(ce=r.default)==null?void 0:ce.call(r)]),r.append&&w("div",{class:"v-navigation-drawer__append"},[(Te=r.append)==null?void 0:Te.call(r)])]}}),w(Xt,{name:"fade-transition"},{default:()=>[_.value&&(M.value||p.value)&&!!t.scrim&&w("div",Pe({class:["v-navigation-drawer__scrim",$.backgroundColorClasses.value],style:[z.value,$.backgroundColorStyles.value],onClick:()=>p.value=!1},S),null)]})])}),{isStuck:A}}}),Cb=Y({...we(),...Vr({variant:"text"})},"VToolbarItems"),Ec=me()({name:"VToolbarItems",props:Cb(),setup(t,e){let{slots:n}=e;return mn({VBtn:{color:fe(t,"color"),height:"inherit",variant:fe(t,"variant")}}),ye(()=>{var s;return w("div",{class:["v-toolbar-items",t.class],style:t.style},[(s=n.default)==null?void 0:s.call(n)])}),{}}}),Eb=Y({id:String,text:String,...ul(wl({closeOnBack:!1,location:"end",locationStrategy:"connected",eager:!0,minWidth:0,offset:10,openOnClick:!1,openOnHover:!0,origin:"auto",scrim:!1,scrollStrategy:"reposition",transition:!1}),["absolute","persistent"])},"VTooltip"),We=me()({name:"VTooltip",props:Eb(),emits:{"update:modelValue":t=>!0},setup(t,e){let{slots:n}=e;const s=ct(t,"modelValue"),{scopeId:r}=xo(),i=gn(),o=k(()=>t.id||`v-tooltip-${i}`),a=ee(),l=k(()=>t.location.split(" ").length>1?t.location:t.location+" center"),u=k(()=>t.origin==="auto"||t.origin==="overlap"||t.origin.split(" ").length>1||t.location.split(" ").length>1?t.origin:t.origin+" center"),c=k(()=>t.transition?t.transition:s.value?"scale-transition":"fade-transition"),d=k(()=>Pe({"aria-describedby":o.value},t.activatorProps));return ye(()=>{const f=zi.filterProps(t);return w(zi,Pe({ref:a,class:["v-tooltip",t.class],style:t.style,id:o.value},f,{modelValue:s.value,"onUpdate:modelValue":h=>s.value=h,transition:c.value,absolute:!0,location:l.value,origin:u.value,persistent:!0,role:"tooltip",activatorProps:d.value,_disableGlobalStack:!0},r),{activator:n.activator,default:function(){var v;for(var h=arguments.length,m=new Array(h),p=0;p<h;p++)m[p]=arguments[p];return((v=n.default)==null?void 0:v.call(n,...m))??t.text}})}),Ih({},a)}});var Mb="M15.41,16.58L10.83,12L15.41,7.41L14,6L8,12L14,18L15.41,16.58Z",Tb="M8.59,16.58L13.17,12L8.59,7.41L10,6L16,12L10,18L8.59,16.58Z",Ab="M18 10V8H20V10H18M18 12V10H16V12H18M18 8V6H16V8H18M16 2.84V4H18C17.37 3.54 16.71 3.15 16 2.84M18 4V6H20C19.42 5.25 18.75 4.58 18 4M20 6V8H21.16C20.85 7.29 20.46 6.63 20 6M22 12C22 11.32 21.93 10.65 21.8 10H20V12H22M16 6V4H14V6H16M16 16H18V14H16V16M18 18H20L20 18V16H18V18M16 20H18L18 20V18H16V20M14 21.8C14.7 21.66 15.36 21.44 16 21.16V20H14V21.8M18 14H20V12H18V14M16 8H14V10H16V8M20 16H21.16C21.44 15.36 21.66 14.7 21.8 14H20V16M16 12H14V14H16V12M12 18V16H14V14H12V12H14V10H12V8H14V6H12V4H14V2.2C13.35 2.07 12.69 2 12 2C6.5 2 2 6.5 2 12S6.5 22 12 22V20H14V18H12M14 18H16V16H14V18Z",Fa="M19,6.41L17.59,5L12,10.59L6.41,5L5,6.41L10.59,12L5,17.59L6.41,19L12,13.41L17.59,19L19,17.59L13.41,12L19,6.41Z",Rb="M12,8A4,4 0 0,1 16,12A4,4 0 0,1 12,16A4,4 0 0,1 8,12A4,4 0 0,1 12,8M3.05,13H1V11H3.05C3.5,6.83 6.83,3.5 11,3.05V1H13V3.05C17.17,3.5 20.5,6.83 20.95,11H23V13H20.95C20.5,17.17 17.17,20.5 13,20.95V23H11V20.95C6.83,20.5 3.5,17.17 3.05,13M12,5A7,7 0 0,0 5,12A7,7 0 0,0 12,19A7,7 0 0,0 19,12A7,7 0 0,0 12,5Z",Go="M21,16.5C21,16.88 20.79,17.21 20.47,17.38L12.57,21.82C12.41,21.94 12.21,22 12,22C11.79,22 11.59,21.94 11.43,21.82L3.53,17.38C3.21,17.21 3,16.88 3,16.5V7.5C3,7.12 3.21,6.79 3.53,6.62L11.43,2.18C11.59,2.06 11.79,2 12,2C12.21,2 12.41,2.06 12.57,2.18L20.47,6.62C20.79,6.79 21,7.12 21,7.5V16.5M12,4.15L6.04,7.5L12,10.85L17.96,7.5L12,4.15Z",zw="M21,16.5C21,16.88 20.79,17.21 20.47,17.38L12.57,21.82C12.41,21.94 12.21,22 12,22C11.79,22 11.59,21.94 11.43,21.82L3.53,17.38C3.21,17.21 3,16.88 3,16.5V7.5C3,7.12 3.21,6.79 3.53,6.62L11.43,2.18C11.59,2.06 11.79,2 12,2C12.21,2 12.41,2.06 12.57,2.18L20.47,6.62C20.79,6.79 21,7.12 21,7.5V16.5M12,4.15L6.04,7.5L12,10.85L17.96,7.5L12,4.15M5,15.91L11,19.29V12.58L5,9.21V15.91M19,15.91V9.21L13,12.58V19.29L19,15.91Z",Hw="M10.76,8.69A0.76,0.76 0 0,0 10,9.45V20.9C10,21.32 10.34,21.66 10.76,21.66C10.95,21.66 11.11,21.6 11.24,21.5L13.15,19.95L14.81,23.57C14.94,23.84 15.21,24 15.5,24C15.61,24 15.72,24 15.83,23.92L18.59,22.64C18.97,22.46 19.15,22 18.95,21.63L17.28,18L19.69,17.55C19.85,17.5 20,17.43 20.12,17.29C20.39,16.97 20.35,16.5 20,16.21L11.26,8.86L11.25,8.87C11.12,8.76 10.95,8.69 10.76,8.69M15,10V8H20V10H15M13.83,4.76L16.66,1.93L18.07,3.34L15.24,6.17L13.83,4.76M10,0H12V5H10V0M3.93,14.66L6.76,11.83L8.17,13.24L5.34,16.07L3.93,14.66M3.93,3.34L5.34,1.93L8.17,4.76L6.76,6.17L3.93,3.34M7,10H2V8H7V10",Ib="M19,4H15.5L14.5,3H9.5L8.5,4H5V6H19M6,19A2,2 0 0,0 8,21H16A2,2 0 0,0 18,19V7H6V19Z",kb="M5,20H19V18H5M19,9H15V3H9V9H5L12,16L19,9Z",$w="M20,20A2,2 0 0,1 18,22H4A2,2 0 0,1 2,20V6A2,2 0 0,1 4,4H9.5C9.2,4.8 9,5.6 9,6.5C9,10.1 11.9,13 15.5,13C16.3,13 17,12.9 17.6,12.6L20,15V20M19.3,8.9C19.7,8.2 20,7.4 20,6.5C20,4 18,2 15.5,2C13,2 11,4 11,6.5C11,9 13,11 15.5,11C16.4,11 17.2,10.7 17.9,10.3L21,13.4L22.4,12L19.3,8.9M15.5,9C14.1,9 13,7.9 13,6.5C13,5.1 14.1,4 15.5,4C16.9,4 18,5.1 18,6.5C18,7.9 16.9,9 15.5,9Z",Fb="M12,2A10,10 0 0,0 2,12C2,16.42 4.87,20.17 8.84,21.5C9.34,21.58 9.5,21.27 9.5,21C9.5,20.77 9.5,20.14 9.5,19.31C6.73,19.91 6.14,17.97 6.14,17.97C5.68,16.81 5.03,16.5 5.03,16.5C4.12,15.88 5.1,15.9 5.1,15.9C6.1,15.97 6.63,16.93 6.63,16.93C7.5,18.45 8.97,18 9.54,17.76C9.63,17.11 9.89,16.67 10.17,16.42C7.95,16.17 5.62,15.31 5.62,11.5C5.62,10.39 6,9.5 6.65,8.79C6.55,8.54 6.2,7.5 6.75,6.15C6.75,6.15 7.59,5.88 9.5,7.17C10.29,6.95 11.15,6.84 12,6.84C12.85,6.84 13.71,6.95 14.5,7.17C16.41,5.88 17.25,6.15 17.25,6.15C17.8,7.5 17.45,8.54 17.35,8.79C18,9.5 18.38,10.39 18.38,11.5C18.38,15.32 16.04,16.16 13.81,16.41C14.17,16.72 14.5,17.33 14.5,18.26C14.5,19.6 14.5,20.68 14.5,21C14.5,21.27 14.66,21.59 15.17,21.5C19.14,20.16 22,16.42 22,12A10,10 0 0,0 12,2Z",Lb="M9 10A3.04 3.04 0 0 1 12 7A3.04 3.04 0 0 1 15 10A3.04 3.04 0 0 1 12 13A3.04 3.04 0 0 1 9 10M12 19L16 20V16.92A7.54 7.54 0 0 1 12 18A7.54 7.54 0 0 1 8 16.92V20M12 4A5.78 5.78 0 0 0 7.76 5.74A5.78 5.78 0 0 0 6 10A5.78 5.78 0 0 0 7.76 14.23A5.78 5.78 0 0 0 12 16A5.78 5.78 0 0 0 16.24 14.23A5.78 5.78 0 0 0 18 10A5.78 5.78 0 0 0 16.24 5.74A5.78 5.78 0 0 0 12 4M20 10A8.04 8.04 0 0 1 19.43 12.8A7.84 7.84 0 0 1 18 15.28V23L12 21L6 23V15.28A7.9 7.9 0 0 1 4 10A7.68 7.68 0 0 1 6.33 4.36A7.73 7.73 0 0 1 12 2A7.73 7.73 0 0 1 17.67 4.36A7.68 7.68 0 0 1 20 10Z",Vb="M19,13H13V19H11V13H5V11H11V5H13V11H19V13Z",Pb="M16,6C14.87,6 13.77,6.35 12.84,7H4C2.89,7 2,7.89 2,9V15C2,16.11 2.89,17 4,17H5V18A1,1 0 0,0 6,19H8A1,1 0 0,0 9,18V17H15V18A1,1 0 0,0 16,19H18A1,1 0 0,0 19,18V17H20C21.11,17 22,16.11 22,15V9C22,7.89 21.11,7 20,7H19.15C18.23,6.35 17.13,6 16,6M16,7.5A3.5,3.5 0 0,1 19.5,11A3.5,3.5 0 0,1 16,14.5A3.5,3.5 0 0,1 12.5,11A3.5,3.5 0 0,1 16,7.5M4,9H8V10H4V9M16,9A2,2 0 0,0 14,11A2,2 0 0,0 16,13A2,2 0 0,0 18,11A2,2 0 0,0 16,9M4,11H8V12H4V11M4,13H8V14H4V13Z",Ob="M4,6V19H20V6H4Z",Nb="M4,6V19H20V6H4M18,17H6V8H18V17Z",Uw="M1.39,18.36L3.16,16.6L4.58,18L5.64,16.95L4.22,15.54L5.64,14.12L8.11,16.6L9.17,15.54L6.7,13.06L8.11,11.65L9.53,13.06L10.59,12L9.17,10.59L10.59,9.17L13.06,11.65L14.12,10.59L11.65,8.11L13.06,6.7L14.47,8.11L15.54,7.05L14.12,5.64L15.54,4.22L18,6.7L19.07,5.64L16.6,3.16L18.36,1.39L22.61,5.64L5.64,22.61L1.39,18.36Z",qo="M21,9L17,5V8H10V10H17V13M7,11L3,15L7,19V16H14V14H7V11Z",Bb="M15,3V7.59L7.59,15H3V21H9V16.42L16.42,9H21V3M17,5H19V7H17M5,17H7V19H5",Db="M2,4H8V6H16V4H22V10H20V14H22V20H16V18H8V20H2V14H4V10H2V4M16,10V8H8V10H6V14H8V16H16V14H18V10H16M4,6V8H6V6H4M18,6V8H20V6H18M4,16V18H6V16H4M18,16V18H20V16H18Z";const El=(t,e)=>{const n=t.__vccOpts||t;for(const[s,r]of e)n[s]=r;return n},Mc={mdi:{size:24,viewbox:"0 0 24 24"},"simple-icons":{size:24,viewbox:"0 0 24 24"},default:{size:0,viewbox:"0 0 0 0"}},zb={name:"icon",props:{type:String,path:{type:String,required:!0},size:{type:[String,Number],default:24},viewbox:String,flip:{type:String,validator:t=>["horizontal","vertical","both","none"].includes(t)},rotate:{type:Number,default:0}},computed:{styles(){return{"--sx":["both","horizontal"].includes(this.flip)?"-1":"1","--sy":["both","vertical"].includes(this.flip)?"-1":"1","--r":isNaN(this.rotate)?this.rotate:this.rotate+"deg"}},defaults(){return Mc[this.type]||Mc.default},sizeValue(){return this.size||this.defaults.size},viewboxValue(){return this.viewbox||this.defaults.viewbox}}},Hb=["width","height","viewBox"],$b=["d"];function Ub(t,e,n,s,r,i){return st(),Ar("svg",{width:i.sizeValue,height:i.sizeValue,viewBox:i.viewboxValue,style:Ji(i.styles)},[Wt("path",{d:n.path},null,8,$b)],12,Hb)}const Be=El(zb,[["render",Ub],["__scopeId","data-v-76aa6b74"]]),jb=fn({__name:"Sidebar",props:{openedInit:{},side:{},width:{}},setup(t){const e=t;let n=ee(e.openedInit.valueOf());const s=e.side==="left"?Tb:Mb;return(r,i)=>(st(),Ar(Ie,null,[w(P(ot),{icon:"",onClick:i[0]||(i[0]=o=>Me(n)?n.value=!P(n):n=!P(n)),class:Qi(["open-button",r.side])},{default:q(()=>[w(P(Be),{type:"mdi",path:P(s)},null,8,["path"])]),_:1},8,["class"]),w(P(Sb),{modelValue:P(n),"onUpdate:modelValue":i[3]||(i[3]=o=>Me(n)?n.value=o:n=o),permanent:"",location:r.side,width:e.width},{default:q(()=>[w(P($f),{density:"compact"},{default:q(()=>[r.side=="right"?(st(),Yt(P(Ec),{key:0},{default:q(()=>[Ys(r.$slots,"toolbar-items",{},void 0,!0),w(P(ot),{icon:"",onClick:i[1]||(i[1]=o=>Me(n)?n.value=!P(n):n=!P(n))},{default:q(()=>[w(P(Be),{type:"mdi",path:P(Fa)},null,8,["path"])]),_:1})]),_:3})):Ei("",!0),Ys(r.$slots,"toolbar",{},void 0,!0),r.side=="left"?(st(),Yt(P(Ec),{key:1},{default:q(()=>[Ys(r.$slots,"toolbar-items",{},void 0,!0),w(P(ot),{icon:"",onClick:i[2]||(i[2]=o=>Me(n)?n.value=!P(n):n=!P(n))},{default:q(()=>[w(P(Be),{type:"mdi",path:P(Fa)},null,8,["path"])]),_:1})]),_:3})):Ei("",!0)]),_:3}),Ys(r.$slots,"default",{},void 0,!0)]),_:3},8,["modelValue","location","width"])],64))}}),Tc=El(jb,[["__scopeId","data-v-7849b5fc"]]),Yh=fn({__name:"Loading",setup(t){return(e,n)=>(st(),Yt(P(q1),null,{default:q(()=>[w(P(rb),{justify:"center",style:{height:"100%"}},{default:q(()=>[w(P(K1),{"align-self":"center"},{default:q(()=>[w(P(eh),{indeterminate:"",style:{display:"block",margin:"0 auto"}}),Ys(e.$slots,"default")]),_:3})]),_:3})]),_:3}))}});function Wb(t){return t&&t.__esModule&&Object.prototype.hasOwnProperty.call(t,"default")?t.default:t}function jw(t){if(t.__esModule)return t;var e=t.default;if(typeof e=="function"){var n=function s(){return this instanceof s?Reflect.construct(e,arguments,this.constructor):e.apply(this,arguments)};n.prototype=e.prototype}else n={};return Object.defineProperty(n,"__esModule",{value:!0}),Object.keys(t).forEach(function(s){var r=Object.getOwnPropertyDescriptor(t,s);Object.defineProperty(n,s,r.get?r:{enumerable:!0,get:function(){return t[s]}})}),n}class Xh extends HTMLElement{constructor(e,n){super(),this.camera=e,this.options=Object.assign({size:90,padding:8,bubbleSizePrimary:8,bubbleSizeSeconday:6,showSecondary:!0,lineWidth:2,fontSize:"11px",fontFamily:"arial",fontWeight:"bold",fontColor:"#151515",fontYAdjust:0,colors:{x:["#f73c3c","#942424"],y:["#6ccb26","#417a17"],z:["#178cf0","#0e5490"]}},n),this.onAxisSelected=null,this.bubbles=[{axis:"x",direction:new THREE.Vector3(1,0,0),size:this.options.bubbleSizePrimary,color:this.options.colors.x,line:this.options.lineWidth,label:"X"},{axis:"y",direction:new THREE.Vector3(0,1,0),size:this.options.bubbleSizePrimary,color:this.options.colors.y,line:this.options.lineWidth,label:"Y"},{axis:"z",direction:new THREE.Vector3(0,0,1),size:this.options.bubbleSizePrimary,color:this.options.colors.z,line:this.options.lineWidth,label:"Z"},{axis:"-x",direction:new THREE.Vector3(-1,0,0),size:this.options.bubbleSizeSeconday,color:this.options.colors.x},{axis:"-y",direction:new THREE.Vector3(0,-1,0),size:this.options.bubbleSizeSeconday,color:this.options.colors.y},{axis:"-z",direction:new THREE.Vector3(0,0,-1),size:this.options.bubbleSizeSeconday,color:this.options.colors.z}],this.center=new THREE.Vector3(this.options.size/2,this.options.size/2,0),this.selectedAxis=null,this.innerHTML="<canvas width='"+this.options.size+"' height='"+this.options.size+"'></canvas>",this.onMouseMove=this.onMouseMove.bind(this),this.onMouseOut=this.onMouseOut.bind(this),this.onMouseClick=this.onMouseClick.bind(this)}connectedCallback(){this.canvas=this.querySelector("canvas"),this.context=this.canvas.getContext("2d"),this.canvas.addEventListener("mousemove",this.onMouseMove,!1),this.canvas.addEventListener("mouseout",this.onMouseOut,!1),this.canvas.addEventListener("click",this.onMouseClick,!1)}disconnectedCallback(){this.canvas.removeEventListener("mousemove",this.onMouseMove,!1),this.canvas.removeEventListener("mouseout",this.onMouseOut,!1),this.canvas.removeEventListener("click",this.onMouseClick,!1)}onMouseMove(e){const n=this.canvas.getBoundingClientRect();this.mouse=new THREE.Vector3(e.clientX-n.left,e.clientY-n.top,0)}onMouseOut(e){this.mouse=null}onMouseClick(e){this.onAxisSelected&&typeof this.onAxisSelected=="function"&&this.onAxisSelected({axis:this.selectedAxis.axis,direction:this.selectedAxis.direction.clone()})}clear(){this.context.clearRect(0,0,this.canvas.width,this.canvas.height)}drawCircle(e,n=10,s="#FF0000"){this.context.beginPath(),this.context.arc(e.x,e.y,n,0,2*Math.PI,!1),this.context.fillStyle=s,this.context.fill(),this.context.closePath()}drawLine(e,n,s=1,r="#FF0000"){this.context.beginPath(),this.context.moveTo(e.x,e.y),this.context.lineTo(n.x,n.y),this.context.lineWidth=s,this.context.strokeStyle=r,this.context.stroke(),this.context.closePath()}update(){this.clear();let n=new THREE.Matrix4().makeRotationFromEuler(this.camera.rotation).clone().invert();for(var s of this.bubbles)s.position=this.getBubblePosition(s.direction.clone().applyMatrix4(n));const r=[];for(let i in this.bubbles)(this.options.showSecondary===!0||i[0]!=="-")&&r.push(this.bubbles[i]);if(r.sort((i,o)=>i.position.z>o.position.z?1:-1),this.selectedAxis=null,this.mouse){let i=1/0;for(var s of r){const a=this.mouse.distanceTo(s.position);(a<i||a<s.size)&&(i=a,this.selectedAxis=s)}}this.drawLayers(r)}drawLayers(e){for(let n of e){let s=n.color;this.selectedAxis===n?s="#FFFFFF":n.position.z>=-.01?s=n.color[0]:s=n.color[1],this.drawCircle(n.position,n.size,s),n.line&&this.drawLine(this.center,n.position,n.line,s),n.label&&(this.context.font=[this.options.fontWeight,this.options.fontSize,this.options.fontFamily].join(" "),this.context.fillStyle=this.options.fontColor,this.context.textBaseline="middle",this.context.textAlign="center",this.context.fillText(n.label,n.position.x,n.position.y+this.options.fontYAdjust))}}getBubblePosition(e){return new THREE.Vector3(e.x*(this.center.x-this.options.bubbleSizePrimary/2-this.options.padding)+this.center.x,this.center.y-e.y*(this.center.y-this.options.bubbleSizePrimary/2-this.options.padding),e.z)}}window.customElements.define("orientation-gizmo",Xh);var Gb=Xh;const qb=Wb(Gb),Ye=["00","01","02","03","04","05","06","07","08","09","0a","0b","0c","0d","0e","0f","10","11","12","13","14","15","16","17","18","19","1a","1b","1c","1d","1e","1f","20","21","22","23","24","25","26","27","28","29","2a","2b","2c","2d","2e","2f","30","31","32","33","34","35","36","37","38","39","3a","3b","3c","3d","3e","3f","40","41","42","43","44","45","46","47","48","49","4a","4b","4c","4d","4e","4f","50","51","52","53","54","55","56","57","58","59","5a","5b","5c","5d","5e","5f","60","61","62","63","64","65","66","67","68","69","6a","6b","6c","6d","6e","6f","70","71","72","73","74","75","76","77","78","79","7a","7b","7c","7d","7e","7f","80","81","82","83","84","85","86","87","88","89","8a","8b","8c","8d","8e","8f","90","91","92","93","94","95","96","97","98","99","9a","9b","9c","9d","9e","9f","a0","a1","a2","a3","a4","a5","a6","a7","a8","a9","aa","ab","ac","ad","ae","af","b0","b1","b2","b3","b4","b5","b6","b7","b8","b9","ba","bb","bc","bd","be","bf","c0","c1","c2","c3","c4","c5","c6","c7","c8","c9","ca","cb","cc","cd","ce","cf","d0","d1","d2","d3","d4","d5","d6","d7","d8","d9","da","db","dc","dd","de","df","e0","e1","e2","e3","e4","e5","e6","e7","e8","e9","ea","eb","ec","ed","ee","ef","f0","f1","f2","f3","f4","f5","f6","f7","f8","f9","fa","fb","fc","fd","fe","ff"];function Yb(){const t=Math.random()*4294967295|0,e=Math.random()*4294967295|0,n=Math.random()*4294967295|0,s=Math.random()*4294967295|0;return(Ye[t&255]+Ye[t>>8&255]+Ye[t>>16&255]+Ye[t>>24&255]+"-"+Ye[e&255]+Ye[e>>8&255]+"-"+Ye[e>>16&15|64]+Ye[e>>24&255]+"-"+Ye[n&63|128]+Ye[n>>8&255]+"-"+Ye[n>>16&255]+Ye[n>>24&255]+Ye[s&255]+Ye[s>>8&255]+Ye[s>>16&255]+Ye[s>>24&255]).toLowerCase()}function at(t,e,n){return Math.max(e,Math.min(n,t))}function Xb(t,e){return(t%e+e)%e}function Yo(t,e,n){return(1-n)*t+n*e}class Pr{constructor(e=0,n=0,s=0,r=1){this.isQuaternion=!0,this._x=e,this._y=n,this._z=s,this._w=r}static slerpFlat(e,n,s,r,i,o,a){let l=s[r+0],u=s[r+1],c=s[r+2],d=s[r+3];const f=i[o+0],h=i[o+1],m=i[o+2],p=i[o+3];if(a===0){e[n+0]=l,e[n+1]=u,e[n+2]=c,e[n+3]=d;return}if(a===1){e[n+0]=f,e[n+1]=h,e[n+2]=m,e[n+3]=p;return}if(d!==p||l!==f||u!==h||c!==m){let v=1-a;const S=l*f+u*h+c*m+d*p,g=S>=0?1:-1,y=1-S*S;if(y>Number.EPSILON){const x=Math.sqrt(y),_=Math.atan2(x,S*g);v=Math.sin(v*_)/x,a=Math.sin(a*_)/x}const b=a*g;if(l=l*v+f*b,u=u*v+h*b,c=c*v+m*b,d=d*v+p*b,v===1-a){const x=1/Math.sqrt(l*l+u*u+c*c+d*d);l*=x,u*=x,c*=x,d*=x}}e[n]=l,e[n+1]=u,e[n+2]=c,e[n+3]=d}static multiplyQuaternionsFlat(e,n,s,r,i,o){const a=s[r],l=s[r+1],u=s[r+2],c=s[r+3],d=i[o],f=i[o+1],h=i[o+2],m=i[o+3];return e[n]=a*m+c*d+l*h-u*f,e[n+1]=l*m+c*f+u*d-a*h,e[n+2]=u*m+c*h+a*f-l*d,e[n+3]=c*m-a*d-l*f-u*h,e}get x(){return this._x}set x(e){this._x=e,this._onChangeCallback()}get y(){return this._y}set y(e){this._y=e,this._onChangeCallback()}get z(){return this._z}set z(e){this._z=e,this._onChangeCallback()}get w(){return this._w}set w(e){this._w=e,this._onChangeCallback()}set(e,n,s,r){return this._x=e,this._y=n,this._z=s,this._w=r,this._onChangeCallback(),this}clone(){return new this.constructor(this._x,this._y,this._z,this._w)}copy(e){return this._x=e.x,this._y=e.y,this._z=e.z,this._w=e.w,this._onChangeCallback(),this}setFromEuler(e,n=!0){const s=e._x,r=e._y,i=e._z,o=e._order,a=Math.cos,l=Math.sin,u=a(s/2),c=a(r/2),d=a(i/2),f=l(s/2),h=l(r/2),m=l(i/2);switch(o){case"XYZ":this._x=f*c*d+u*h*m,this._y=u*h*d-f*c*m,this._z=u*c*m+f*h*d,this._w=u*c*d-f*h*m;break;case"YXZ":this._x=f*c*d+u*h*m,this._y=u*h*d-f*c*m,this._z=u*c*m-f*h*d,this._w=u*c*d+f*h*m;break;case"ZXY":this._x=f*c*d-u*h*m,this._y=u*h*d+f*c*m,this._z=u*c*m+f*h*d,this._w=u*c*d-f*h*m;break;case"ZYX":this._x=f*c*d-u*h*m,this._y=u*h*d+f*c*m,this._z=u*c*m-f*h*d,this._w=u*c*d+f*h*m;break;case"YZX":this._x=f*c*d+u*h*m,this._y=u*h*d+f*c*m,this._z=u*c*m-f*h*d,this._w=u*c*d-f*h*m;break;case"XZY":this._x=f*c*d-u*h*m,this._y=u*h*d-f*c*m,this._z=u*c*m+f*h*d,this._w=u*c*d+f*h*m;break;default:console.warn("THREE.Quaternion: .setFromEuler() encountered an unknown order: "+o)}return n===!0&&this._onChangeCallback(),this}setFromAxisAngle(e,n){const s=n/2,r=Math.sin(s);return this._x=e.x*r,this._y=e.y*r,this._z=e.z*r,this._w=Math.cos(s),this._onChangeCallback(),this}setFromRotationMatrix(e){const n=e.elements,s=n[0],r=n[4],i=n[8],o=n[1],a=n[5],l=n[9],u=n[2],c=n[6],d=n[10],f=s+a+d;if(f>0){const h=.5/Math.sqrt(f+1);this._w=.25/h,this._x=(c-l)*h,this._y=(i-u)*h,this._z=(o-r)*h}else if(s>a&&s>d){const h=2*Math.sqrt(1+s-a-d);this._w=(c-l)/h,this._x=.25*h,this._y=(r+o)/h,this._z=(i+u)/h}else if(a>d){const h=2*Math.sqrt(1+a-s-d);this._w=(i-u)/h,this._x=(r+o)/h,this._y=.25*h,this._z=(l+c)/h}else{const h=2*Math.sqrt(1+d-s-a);this._w=(o-r)/h,this._x=(i+u)/h,this._y=(l+c)/h,this._z=.25*h}return this._onChangeCallback(),this}setFromUnitVectors(e,n){let s=e.dot(n)+1;return s<Number.EPSILON?(s=0,Math.abs(e.x)>Math.abs(e.z)?(this._x=-e.y,this._y=e.x,this._z=0,this._w=s):(this._x=0,this._y=-e.z,this._z=e.y,this._w=s)):(this._x=e.y*n.z-e.z*n.y,this._y=e.z*n.x-e.x*n.z,this._z=e.x*n.y-e.y*n.x,this._w=s),this.normalize()}angleTo(e){return 2*Math.acos(Math.abs(at(this.dot(e),-1,1)))}rotateTowards(e,n){const s=this.angleTo(e);if(s===0)return this;const r=Math.min(1,n/s);return this.slerp(e,r),this}identity(){return this.set(0,0,0,1)}invert(){return this.conjugate()}conjugate(){return this._x*=-1,this._y*=-1,this._z*=-1,this._onChangeCallback(),this}dot(e){return this._x*e._x+this._y*e._y+this._z*e._z+this._w*e._w}lengthSq(){return this._x*this._x+this._y*this._y+this._z*this._z+this._w*this._w}length(){return Math.sqrt(this._x*this._x+this._y*this._y+this._z*this._z+this._w*this._w)}normalize(){let e=this.length();return e===0?(this._x=0,this._y=0,this._z=0,this._w=1):(e=1/e,this._x=this._x*e,this._y=this._y*e,this._z=this._z*e,this._w=this._w*e),this._onChangeCallback(),this}multiply(e){return this.multiplyQuaternions(this,e)}premultiply(e){return this.multiplyQuaternions(e,this)}multiplyQuaternions(e,n){const s=e._x,r=e._y,i=e._z,o=e._w,a=n._x,l=n._y,u=n._z,c=n._w;return this._x=s*c+o*a+r*u-i*l,this._y=r*c+o*l+i*a-s*u,this._z=i*c+o*u+s*l-r*a,this._w=o*c-s*a-r*l-i*u,this._onChangeCallback(),this}slerp(e,n){if(n===0)return this;if(n===1)return this.copy(e);const s=this._x,r=this._y,i=this._z,o=this._w;let a=o*e._w+s*e._x+r*e._y+i*e._z;if(a<0?(this._w=-e._w,this._x=-e._x,this._y=-e._y,this._z=-e._z,a=-a):this.copy(e),a>=1)return this._w=o,this._x=s,this._y=r,this._z=i,this;const l=1-a*a;if(l<=Number.EPSILON){const h=1-n;return this._w=h*o+n*this._w,this._x=h*s+n*this._x,this._y=h*r+n*this._y,this._z=h*i+n*this._z,this.normalize(),this}const u=Math.sqrt(l),c=Math.atan2(u,a),d=Math.sin((1-n)*c)/u,f=Math.sin(n*c)/u;return this._w=o*d+this._w*f,this._x=s*d+this._x*f,this._y=r*d+this._y*f,this._z=i*d+this._z*f,this._onChangeCallback(),this}slerpQuaternions(e,n,s){return this.copy(e).slerp(n,s)}random(){const e=Math.random(),n=Math.sqrt(1-e),s=Math.sqrt(e),r=2*Math.PI*Math.random(),i=2*Math.PI*Math.random();return this.set(n*Math.cos(r),s*Math.sin(i),s*Math.cos(i),n*Math.sin(r))}equals(e){return e._x===this._x&&e._y===this._y&&e._z===this._z&&e._w===this._w}fromArray(e,n=0){return this._x=e[n],this._y=e[n+1],this._z=e[n+2],this._w=e[n+3],this._onChangeCallback(),this}toArray(e=[],n=0){return e[n]=this._x,e[n+1]=this._y,e[n+2]=this._z,e[n+3]=this._w,e}fromBufferAttribute(e,n){return this._x=e.getX(n),this._y=e.getY(n),this._z=e.getZ(n),this._w=e.getW(n),this._onChangeCallback(),this}toJSON(){return this.toArray()}_onChange(e){return this._onChangeCallback=e,this}_onChangeCallback(){}*[Symbol.iterator](){yield this._x,yield this._y,yield this._z,yield this._w}}class J{constructor(e=0,n=0,s=0){J.prototype.isVector3=!0,this.x=e,this.y=n,this.z=s}set(e,n,s){return s===void 0&&(s=this.z),this.x=e,this.y=n,this.z=s,this}setScalar(e){return this.x=e,this.y=e,this.z=e,this}setX(e){return this.x=e,this}setY(e){return this.y=e,this}setZ(e){return this.z=e,this}setComponent(e,n){switch(e){case 0:this.x=n;break;case 1:this.y=n;break;case 2:this.z=n;break;default:throw new Error("index is out of range: "+e)}return this}getComponent(e){switch(e){case 0:return this.x;case 1:return this.y;case 2:return this.z;default:throw new Error("index is out of range: "+e)}}clone(){return new this.constructor(this.x,this.y,this.z)}copy(e){return this.x=e.x,this.y=e.y,this.z=e.z,this}add(e){return this.x+=e.x,this.y+=e.y,this.z+=e.z,this}addScalar(e){return this.x+=e,this.y+=e,this.z+=e,this}addVectors(e,n){return this.x=e.x+n.x,this.y=e.y+n.y,this.z=e.z+n.z,this}addScaledVector(e,n){return this.x+=e.x*n,this.y+=e.y*n,this.z+=e.z*n,this}sub(e){return this.x-=e.x,this.y-=e.y,this.z-=e.z,this}subScalar(e){return this.x-=e,this.y-=e,this.z-=e,this}subVectors(e,n){return this.x=e.x-n.x,this.y=e.y-n.y,this.z=e.z-n.z,this}multiply(e){return this.x*=e.x,this.y*=e.y,this.z*=e.z,this}multiplyScalar(e){return this.x*=e,this.y*=e,this.z*=e,this}multiplyVectors(e,n){return this.x=e.x*n.x,this.y=e.y*n.y,this.z=e.z*n.z,this}applyEuler(e){return this.applyQuaternion(Ac.setFromEuler(e))}applyAxisAngle(e,n){return this.applyQuaternion(Ac.setFromAxisAngle(e,n))}applyMatrix3(e){const n=this.x,s=this.y,r=this.z,i=e.elements;return this.x=i[0]*n+i[3]*s+i[6]*r,this.y=i[1]*n+i[4]*s+i[7]*r,this.z=i[2]*n+i[5]*s+i[8]*r,this}applyNormalMatrix(e){return this.applyMatrix3(e).normalize()}applyMatrix4(e){const n=this.x,s=this.y,r=this.z,i=e.elements,o=1/(i[3]*n+i[7]*s+i[11]*r+i[15]);return this.x=(i[0]*n+i[4]*s+i[8]*r+i[12])*o,this.y=(i[1]*n+i[5]*s+i[9]*r+i[13])*o,this.z=(i[2]*n+i[6]*s+i[10]*r+i[14])*o,this}applyQuaternion(e){const n=this.x,s=this.y,r=this.z,i=e.x,o=e.y,a=e.z,l=e.w,u=2*(o*r-a*s),c=2*(a*n-i*r),d=2*(i*s-o*n);return this.x=n+l*u+o*d-a*c,this.y=s+l*c+a*u-i*d,this.z=r+l*d+i*c-o*u,this}project(e){return this.applyMatrix4(e.matrixWorldInverse).applyMatrix4(e.projectionMatrix)}unproject(e){return this.applyMatrix4(e.projectionMatrixInverse).applyMatrix4(e.matrixWorld)}transformDirection(e){const n=this.x,s=this.y,r=this.z,i=e.elements;return this.x=i[0]*n+i[4]*s+i[8]*r,this.y=i[1]*n+i[5]*s+i[9]*r,this.z=i[2]*n+i[6]*s+i[10]*r,this.normalize()}divide(e){return this.x/=e.x,this.y/=e.y,this.z/=e.z,this}divideScalar(e){return this.multiplyScalar(1/e)}min(e){return this.x=Math.min(this.x,e.x),this.y=Math.min(this.y,e.y),this.z=Math.min(this.z,e.z),this}max(e){return this.x=Math.max(this.x,e.x),this.y=Math.max(this.y,e.y),this.z=Math.max(this.z,e.z),this}clamp(e,n){return this.x=Math.max(e.x,Math.min(n.x,this.x)),this.y=Math.max(e.y,Math.min(n.y,this.y)),this.z=Math.max(e.z,Math.min(n.z,this.z)),this}clampScalar(e,n){return this.x=Math.max(e,Math.min(n,this.x)),this.y=Math.max(e,Math.min(n,this.y)),this.z=Math.max(e,Math.min(n,this.z)),this}clampLength(e,n){const s=this.length();return this.divideScalar(s||1).multiplyScalar(Math.max(e,Math.min(n,s)))}floor(){return this.x=Math.floor(this.x),this.y=Math.floor(this.y),this.z=Math.floor(this.z),this}ceil(){return this.x=Math.ceil(this.x),this.y=Math.ceil(this.y),this.z=Math.ceil(this.z),this}round(){return this.x=Math.round(this.x),this.y=Math.round(this.y),this.z=Math.round(this.z),this}roundToZero(){return this.x=Math.trunc(this.x),this.y=Math.trunc(this.y),this.z=Math.trunc(this.z),this}negate(){return this.x=-this.x,this.y=-this.y,this.z=-this.z,this}dot(e){return this.x*e.x+this.y*e.y+this.z*e.z}lengthSq(){return this.x*this.x+this.y*this.y+this.z*this.z}length(){return Math.sqrt(this.x*this.x+this.y*this.y+this.z*this.z)}manhattanLength(){return Math.abs(this.x)+Math.abs(this.y)+Math.abs(this.z)}normalize(){return this.divideScalar(this.length()||1)}setLength(e){return this.normalize().multiplyScalar(e)}lerp(e,n){return this.x+=(e.x-this.x)*n,this.y+=(e.y-this.y)*n,this.z+=(e.z-this.z)*n,this}lerpVectors(e,n,s){return this.x=e.x+(n.x-e.x)*s,this.y=e.y+(n.y-e.y)*s,this.z=e.z+(n.z-e.z)*s,this}cross(e){return this.crossVectors(this,e)}crossVectors(e,n){const s=e.x,r=e.y,i=e.z,o=n.x,a=n.y,l=n.z;return this.x=r*l-i*a,this.y=i*o-s*l,this.z=s*a-r*o,this}projectOnVector(e){const n=e.lengthSq();if(n===0)return this.set(0,0,0);const s=e.dot(this)/n;return this.copy(e).multiplyScalar(s)}projectOnPlane(e){return Xo.copy(this).projectOnVector(e),this.sub(Xo)}reflect(e){return this.sub(Xo.copy(e).multiplyScalar(2*this.dot(e)))}angleTo(e){const n=Math.sqrt(this.lengthSq()*e.lengthSq());if(n===0)return Math.PI/2;const s=this.dot(e)/n;return Math.acos(at(s,-1,1))}distanceTo(e){return Math.sqrt(this.distanceToSquared(e))}distanceToSquared(e){const n=this.x-e.x,s=this.y-e.y,r=this.z-e.z;return n*n+s*s+r*r}manhattanDistanceTo(e){return Math.abs(this.x-e.x)+Math.abs(this.y-e.y)+Math.abs(this.z-e.z)}setFromSpherical(e){return this.setFromSphericalCoords(e.radius,e.phi,e.theta)}setFromSphericalCoords(e,n,s){const r=Math.sin(n)*e;return this.x=r*Math.sin(s),this.y=Math.cos(n)*e,this.z=r*Math.cos(s),this}setFromCylindrical(e){return this.setFromCylindricalCoords(e.radius,e.theta,e.y)}setFromCylindricalCoords(e,n,s){return this.x=e*Math.sin(n),this.y=s,this.z=e*Math.cos(n),this}setFromMatrixPosition(e){const n=e.elements;return this.x=n[12],this.y=n[13],this.z=n[14],this}setFromMatrixScale(e){const n=this.setFromMatrixColumn(e,0).length(),s=this.setFromMatrixColumn(e,1).length(),r=this.setFromMatrixColumn(e,2).length();return this.x=n,this.y=s,this.z=r,this}setFromMatrixColumn(e,n){return this.fromArray(e.elements,n*4)}setFromMatrix3Column(e,n){return this.fromArray(e.elements,n*3)}setFromEuler(e){return this.x=e._x,this.y=e._y,this.z=e._z,this}setFromColor(e){return this.x=e.r,this.y=e.g,this.z=e.b,this}equals(e){return e.x===this.x&&e.y===this.y&&e.z===this.z}fromArray(e,n=0){return this.x=e[n],this.y=e[n+1],this.z=e[n+2],this}toArray(e=[],n=0){return e[n]=this.x,e[n+1]=this.y,e[n+2]=this.z,e}fromBufferAttribute(e,n){return this.x=e.getX(n),this.y=e.getY(n),this.z=e.getZ(n),this}random(){return this.x=Math.random(),this.y=Math.random(),this.z=Math.random(),this}randomDirection(){const e=(Math.random()-.5)*2,n=Math.random()*Math.PI*2,s=Math.sqrt(1-e**2);return this.x=s*Math.cos(n),this.y=s*Math.sin(n),this.z=e,this}*[Symbol.iterator](){yield this.x,yield this.y,yield this.z}}const Xo=new J,Ac=new Pr,Zb=0,Kb=1,Jb="",Cn="srgb",Ml="srgb-linear",Qb="display-p3",Zh="display-p3-linear",La="linear",Rc="srgb",Ic="rec709",kc="p3",Ks=2e3,Fc=2001;class He{constructor(e,n,s,r,i,o,a,l,u,c,d,f,h,m,p,v){He.prototype.isMatrix4=!0,this.elements=[1,0,0,0,0,1,0,0,0,0,1,0,0,0,0,1],e!==void 0&&this.set(e,n,s,r,i,o,a,l,u,c,d,f,h,m,p,v)}set(e,n,s,r,i,o,a,l,u,c,d,f,h,m,p,v){const S=this.elements;return S[0]=e,S[4]=n,S[8]=s,S[12]=r,S[1]=i,S[5]=o,S[9]=a,S[13]=l,S[2]=u,S[6]=c,S[10]=d,S[14]=f,S[3]=h,S[7]=m,S[11]=p,S[15]=v,this}identity(){return this.set(1,0,0,0,0,1,0,0,0,0,1,0,0,0,0,1),this}clone(){return new He().fromArray(this.elements)}copy(e){const n=this.elements,s=e.elements;return n[0]=s[0],n[1]=s[1],n[2]=s[2],n[3]=s[3],n[4]=s[4],n[5]=s[5],n[6]=s[6],n[7]=s[7],n[8]=s[8],n[9]=s[9],n[10]=s[10],n[11]=s[11],n[12]=s[12],n[13]=s[13],n[14]=s[14],n[15]=s[15],this}copyPosition(e){const n=this.elements,s=e.elements;return n[12]=s[12],n[13]=s[13],n[14]=s[14],this}setFromMatrix3(e){const n=e.elements;return this.set(n[0],n[3],n[6],0,n[1],n[4],n[7],0,n[2],n[5],n[8],0,0,0,0,1),this}extractBasis(e,n,s){return e.setFromMatrixColumn(this,0),n.setFromMatrixColumn(this,1),s.setFromMatrixColumn(this,2),this}makeBasis(e,n,s){return this.set(e.x,n.x,s.x,0,e.y,n.y,s.y,0,e.z,n.z,s.z,0,0,0,0,1),this}extractRotation(e){const n=this.elements,s=e.elements,r=1/ys.setFromMatrixColumn(e,0).length(),i=1/ys.setFromMatrixColumn(e,1).length(),o=1/ys.setFromMatrixColumn(e,2).length();return n[0]=s[0]*r,n[1]=s[1]*r,n[2]=s[2]*r,n[3]=0,n[4]=s[4]*i,n[5]=s[5]*i,n[6]=s[6]*i,n[7]=0,n[8]=s[8]*o,n[9]=s[9]*o,n[10]=s[10]*o,n[11]=0,n[12]=0,n[13]=0,n[14]=0,n[15]=1,this}makeRotationFromEuler(e){const n=this.elements,s=e.x,r=e.y,i=e.z,o=Math.cos(s),a=Math.sin(s),l=Math.cos(r),u=Math.sin(r),c=Math.cos(i),d=Math.sin(i);if(e.order==="XYZ"){const f=o*c,h=o*d,m=a*c,p=a*d;n[0]=l*c,n[4]=-l*d,n[8]=u,n[1]=h+m*u,n[5]=f-p*u,n[9]=-a*l,n[2]=p-f*u,n[6]=m+h*u,n[10]=o*l}else if(e.order==="YXZ"){const f=l*c,h=l*d,m=u*c,p=u*d;n[0]=f+p*a,n[4]=m*a-h,n[8]=o*u,n[1]=o*d,n[5]=o*c,n[9]=-a,n[2]=h*a-m,n[6]=p+f*a,n[10]=o*l}else if(e.order==="ZXY"){const f=l*c,h=l*d,m=u*c,p=u*d;n[0]=f-p*a,n[4]=-o*d,n[8]=m+h*a,n[1]=h+m*a,n[5]=o*c,n[9]=p-f*a,n[2]=-o*u,n[6]=a,n[10]=o*l}else if(e.order==="ZYX"){const f=o*c,h=o*d,m=a*c,p=a*d;n[0]=l*c,n[4]=m*u-h,n[8]=f*u+p,n[1]=l*d,n[5]=p*u+f,n[9]=h*u-m,n[2]=-u,n[6]=a*l,n[10]=o*l}else if(e.order==="YZX"){const f=o*l,h=o*u,m=a*l,p=a*u;n[0]=l*c,n[4]=p-f*d,n[8]=m*d+h,n[1]=d,n[5]=o*c,n[9]=-a*c,n[2]=-u*c,n[6]=h*d+m,n[10]=f-p*d}else if(e.order==="XZY"){const f=o*l,h=o*u,m=a*l,p=a*u;n[0]=l*c,n[4]=-d,n[8]=u*c,n[1]=f*d+p,n[5]=o*c,n[9]=h*d-m,n[2]=m*d-h,n[6]=a*c,n[10]=p*d+f}return n[3]=0,n[7]=0,n[11]=0,n[12]=0,n[13]=0,n[14]=0,n[15]=1,this}makeRotationFromQuaternion(e){return this.compose(ex,e,tx)}lookAt(e,n,s){const r=this.elements;return ht.subVectors(e,n),ht.lengthSq()===0&&(ht.z=1),ht.normalize(),bn.crossVectors(s,ht),bn.lengthSq()===0&&(Math.abs(s.z)===1?ht.x+=1e-4:ht.z+=1e-4,ht.normalize(),bn.crossVectors(s,ht)),bn.normalize(),Zr.crossVectors(ht,bn),r[0]=bn.x,r[4]=Zr.x,r[8]=ht.x,r[1]=bn.y,r[5]=Zr.y,r[9]=ht.y,r[2]=bn.z,r[6]=Zr.z,r[10]=ht.z,this}multiply(e){return this.multiplyMatrices(this,e)}premultiply(e){return this.multiplyMatrices(e,this)}multiplyMatrices(e,n){const s=e.elements,r=n.elements,i=this.elements,o=s[0],a=s[4],l=s[8],u=s[12],c=s[1],d=s[5],f=s[9],h=s[13],m=s[2],p=s[6],v=s[10],S=s[14],g=s[3],y=s[7],b=s[11],x=s[15],_=r[0],C=r[4],M=r[8],R=r[12],F=r[1],V=r[5],T=r[9],L=r[13],A=r[2],N=r[6],$=r[10],z=r[14],X=r[3],re=r[7],te=r[11],Q=r[15];return i[0]=o*_+a*F+l*A+u*X,i[4]=o*C+a*V+l*N+u*re,i[8]=o*M+a*T+l*$+u*te,i[12]=o*R+a*L+l*z+u*Q,i[1]=c*_+d*F+f*A+h*X,i[5]=c*C+d*V+f*N+h*re,i[9]=c*M+d*T+f*$+h*te,i[13]=c*R+d*L+f*z+h*Q,i[2]=m*_+p*F+v*A+S*X,i[6]=m*C+p*V+v*N+S*re,i[10]=m*M+p*T+v*$+S*te,i[14]=m*R+p*L+v*z+S*Q,i[3]=g*_+y*F+b*A+x*X,i[7]=g*C+y*V+b*N+x*re,i[11]=g*M+y*T+b*$+x*te,i[15]=g*R+y*L+b*z+x*Q,this}multiplyScalar(e){const n=this.elements;return n[0]*=e,n[4]*=e,n[8]*=e,n[12]*=e,n[1]*=e,n[5]*=e,n[9]*=e,n[13]*=e,n[2]*=e,n[6]*=e,n[10]*=e,n[14]*=e,n[3]*=e,n[7]*=e,n[11]*=e,n[15]*=e,this}determinant(){const e=this.elements,n=e[0],s=e[4],r=e[8],i=e[12],o=e[1],a=e[5],l=e[9],u=e[13],c=e[2],d=e[6],f=e[10],h=e[14],m=e[3],p=e[7],v=e[11],S=e[15];return m*(+i*l*d-r*u*d-i*a*f+s*u*f+r*a*h-s*l*h)+p*(+n*l*h-n*u*f+i*o*f-r*o*h+r*u*c-i*l*c)+v*(+n*u*d-n*a*h-i*o*d+s*o*h+i*a*c-s*u*c)+S*(-r*a*c-n*l*d+n*a*f+r*o*d-s*o*f+s*l*c)}transpose(){const e=this.elements;let n;return n=e[1],e[1]=e[4],e[4]=n,n=e[2],e[2]=e[8],e[8]=n,n=e[6],e[6]=e[9],e[9]=n,n=e[3],e[3]=e[12],e[12]=n,n=e[7],e[7]=e[13],e[13]=n,n=e[11],e[11]=e[14],e[14]=n,this}setPosition(e,n,s){const r=this.elements;return e.isVector3?(r[12]=e.x,r[13]=e.y,r[14]=e.z):(r[12]=e,r[13]=n,r[14]=s),this}invert(){const e=this.elements,n=e[0],s=e[1],r=e[2],i=e[3],o=e[4],a=e[5],l=e[6],u=e[7],c=e[8],d=e[9],f=e[10],h=e[11],m=e[12],p=e[13],v=e[14],S=e[15],g=d*v*u-p*f*u+p*l*h-a*v*h-d*l*S+a*f*S,y=m*f*u-c*v*u-m*l*h+o*v*h+c*l*S-o*f*S,b=c*p*u-m*d*u+m*a*h-o*p*h-c*a*S+o*d*S,x=m*d*l-c*p*l-m*a*f+o*p*f+c*a*v-o*d*v,_=n*g+s*y+r*b+i*x;if(_===0)return this.set(0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0);const C=1/_;return e[0]=g*C,e[1]=(p*f*i-d*v*i-p*r*h+s*v*h+d*r*S-s*f*S)*C,e[2]=(a*v*i-p*l*i+p*r*u-s*v*u-a*r*S+s*l*S)*C,e[3]=(d*l*i-a*f*i-d*r*u+s*f*u+a*r*h-s*l*h)*C,e[4]=y*C,e[5]=(c*v*i-m*f*i+m*r*h-n*v*h-c*r*S+n*f*S)*C,e[6]=(m*l*i-o*v*i-m*r*u+n*v*u+o*r*S-n*l*S)*C,e[7]=(o*f*i-c*l*i+c*r*u-n*f*u-o*r*h+n*l*h)*C,e[8]=b*C,e[9]=(m*d*i-c*p*i-m*s*h+n*p*h+c*s*S-n*d*S)*C,e[10]=(o*p*i-m*a*i+m*s*u-n*p*u-o*s*S+n*a*S)*C,e[11]=(c*a*i-o*d*i-c*s*u+n*d*u+o*s*h-n*a*h)*C,e[12]=x*C,e[13]=(c*p*r-m*d*r+m*s*f-n*p*f-c*s*v+n*d*v)*C,e[14]=(m*a*r-o*p*r-m*s*l+n*p*l+o*s*v-n*a*v)*C,e[15]=(o*d*r-c*a*r+c*s*l-n*d*l-o*s*f+n*a*f)*C,this}scale(e){const n=this.elements,s=e.x,r=e.y,i=e.z;return n[0]*=s,n[4]*=r,n[8]*=i,n[1]*=s,n[5]*=r,n[9]*=i,n[2]*=s,n[6]*=r,n[10]*=i,n[3]*=s,n[7]*=r,n[11]*=i,this}getMaxScaleOnAxis(){const e=this.elements,n=e[0]*e[0]+e[1]*e[1]+e[2]*e[2],s=e[4]*e[4]+e[5]*e[5]+e[6]*e[6],r=e[8]*e[8]+e[9]*e[9]+e[10]*e[10];return Math.sqrt(Math.max(n,s,r))}makeTranslation(e,n,s){return e.isVector3?this.set(1,0,0,e.x,0,1,0,e.y,0,0,1,e.z,0,0,0,1):this.set(1,0,0,e,0,1,0,n,0,0,1,s,0,0,0,1),this}makeRotationX(e){const n=Math.cos(e),s=Math.sin(e);return this.set(1,0,0,0,0,n,-s,0,0,s,n,0,0,0,0,1),this}makeRotationY(e){const n=Math.cos(e),s=Math.sin(e);return this.set(n,0,s,0,0,1,0,0,-s,0,n,0,0,0,0,1),this}makeRotationZ(e){const n=Math.cos(e),s=Math.sin(e);return this.set(n,-s,0,0,s,n,0,0,0,0,1,0,0,0,0,1),this}makeRotationAxis(e,n){const s=Math.cos(n),r=Math.sin(n),i=1-s,o=e.x,a=e.y,l=e.z,u=i*o,c=i*a;return this.set(u*o+s,u*a-r*l,u*l+r*a,0,u*a+r*l,c*a+s,c*l-r*o,0,u*l-r*a,c*l+r*o,i*l*l+s,0,0,0,0,1),this}makeScale(e,n,s){return this.set(e,0,0,0,0,n,0,0,0,0,s,0,0,0,0,1),this}makeShear(e,n,s,r,i,o){return this.set(1,s,i,0,e,1,o,0,n,r,1,0,0,0,0,1),this}compose(e,n,s){const r=this.elements,i=n._x,o=n._y,a=n._z,l=n._w,u=i+i,c=o+o,d=a+a,f=i*u,h=i*c,m=i*d,p=o*c,v=o*d,S=a*d,g=l*u,y=l*c,b=l*d,x=s.x,_=s.y,C=s.z;return r[0]=(1-(p+S))*x,r[1]=(h+b)*x,r[2]=(m-y)*x,r[3]=0,r[4]=(h-b)*_,r[5]=(1-(f+S))*_,r[6]=(v+g)*_,r[7]=0,r[8]=(m+y)*C,r[9]=(v-g)*C,r[10]=(1-(f+p))*C,r[11]=0,r[12]=e.x,r[13]=e.y,r[14]=e.z,r[15]=1,this}decompose(e,n,s){const r=this.elements;let i=ys.set(r[0],r[1],r[2]).length();const o=ys.set(r[4],r[5],r[6]).length(),a=ys.set(r[8],r[9],r[10]).length();this.determinant()<0&&(i=-i),e.x=r[12],e.y=r[13],e.z=r[14],Vt.copy(this);const u=1/i,c=1/o,d=1/a;return Vt.elements[0]*=u,Vt.elements[1]*=u,Vt.elements[2]*=u,Vt.elements[4]*=c,Vt.elements[5]*=c,Vt.elements[6]*=c,Vt.elements[8]*=d,Vt.elements[9]*=d,Vt.elements[10]*=d,n.setFromRotationMatrix(Vt),s.x=i,s.y=o,s.z=a,this}makePerspective(e,n,s,r,i,o,a=Ks){const l=this.elements,u=2*i/(n-e),c=2*i/(s-r),d=(n+e)/(n-e),f=(s+r)/(s-r);let h,m;if(a===Ks)h=-(o+i)/(o-i),m=-2*o*i/(o-i);else if(a===Fc)h=-o/(o-i),m=-o*i/(o-i);else throw new Error("THREE.Matrix4.makePerspective(): Invalid coordinate system: "+a);return l[0]=u,l[4]=0,l[8]=d,l[12]=0,l[1]=0,l[5]=c,l[9]=f,l[13]=0,l[2]=0,l[6]=0,l[10]=h,l[14]=m,l[3]=0,l[7]=0,l[11]=-1,l[15]=0,this}makeOrthographic(e,n,s,r,i,o,a=Ks){const l=this.elements,u=1/(n-e),c=1/(s-r),d=1/(o-i),f=(n+e)*u,h=(s+r)*c;let m,p;if(a===Ks)m=(o+i)*d,p=-2*d;else if(a===Fc)m=i*d,p=-1*d;else throw new Error("THREE.Matrix4.makeOrthographic(): Invalid coordinate system: "+a);return l[0]=2*u,l[4]=0,l[8]=0,l[12]=-f,l[1]=0,l[5]=2*c,l[9]=0,l[13]=-h,l[2]=0,l[6]=0,l[10]=p,l[14]=-m,l[3]=0,l[7]=0,l[11]=0,l[15]=1,this}equals(e){const n=this.elements,s=e.elements;for(let r=0;r<16;r++)if(n[r]!==s[r])return!1;return!0}fromArray(e,n=0){for(let s=0;s<16;s++)this.elements[s]=e[s+n];return this}toArray(e=[],n=0){const s=this.elements;return e[n]=s[0],e[n+1]=s[1],e[n+2]=s[2],e[n+3]=s[3],e[n+4]=s[4],e[n+5]=s[5],e[n+6]=s[6],e[n+7]=s[7],e[n+8]=s[8],e[n+9]=s[9],e[n+10]=s[10],e[n+11]=s[11],e[n+12]=s[12],e[n+13]=s[13],e[n+14]=s[14],e[n+15]=s[15],e}}const ys=new J,Vt=new He,ex=new J(0,0,0),tx=new J(1,1,1),bn=new J,Zr=new J,ht=new J,nx=fn({__name:"OrientationGizmo",props:{elem:{},scene:{}},setup(t){globalThis.THREE={Vector3:J,Matrix4:He};const e=t;function n(a,l){let u=new qb(l.camera,{size:a.clientWidth,bubbleSizePrimary:a.clientWidth/12,bubbleSizeSeconday:a.clientWidth/14,fontSize:a.clientWidth/10+"px"});for(let c of[["y","-z"],["z","-y"],["z","-z"]]){let d=u.bubbles.findIndex(p=>p.axis==c[0]),f=u.bubbles.findIndex(p=>p.axis==c[1]),h=u.bubbles[d].direction.clone(),m=u.bubbles[f].direction.clone();u.bubbles[d].direction.copy(m),u.bubbles[f].direction.copy(h)}return u.onAxisSelected=c=>{let d=l.getCamera().position.clone(),f=l.getTarget().clone().add(l.target.position),h=d.clone().sub(f).length(),m=new J(c.direction.x,c.direction.y,c.direction.z),p=f.clone().add(m.clone().multiplyScalar(h));l.getCamera().position.copy(p),l.getCamera().lookAt(f),l.__perspectiveCamera&&(l.__perspectiveCamera.position.copy(p),l.__perspectiveCamera.lookAt(f)),l.queueRender(),requestIdleCallback(()=>{var v;return(v=e.elem)==null?void 0:v.dispatchEvent(new CustomEvent("camera-change",{detail:{source:"none"}}))},{timeout:100})},u}let s=ee(null),r;function i(){r.isConnected&&(r.update(),requestIdleCallback(i,{timeout:250}))}let o=()=>{s.value&&(r&&s.value.removeChild(r),r=n(s.value,e.scene),s.value.appendChild(r),requestIdleCallback(i,{timeout:250}))};return Zt(o),nl(o),(a,l)=>(st(),Ar("div",{ref_key:"container",ref:s,class:"orientation-gizmo"},null,512))}});let sx=class{addEventListener(e,n){this._listeners===void 0&&(this._listeners={});const s=this._listeners;s[e]===void 0&&(s[e]=[]),s[e].indexOf(n)===-1&&s[e].push(n)}hasEventListener(e,n){if(this._listeners===void 0)return!1;const s=this._listeners;return s[e]!==void 0&&s[e].indexOf(n)!==-1}removeEventListener(e,n){if(this._listeners===void 0)return;const r=this._listeners[e];if(r!==void 0){const i=r.indexOf(n);i!==-1&&r.splice(i,1)}}dispatchEvent(e){if(this._listeners===void 0)return;const s=this._listeners[e.type];if(s!==void 0){e.target=this;const r=s.slice(0);for(let i=0,o=r.length;i<o;i++)r[i].call(this,e);e.target=null}}};const Lc=new He,Vc=new Pr;class wo{constructor(e=0,n=0,s=0,r=wo.DEFAULT_ORDER){this.isEuler=!0,this._x=e,this._y=n,this._z=s,this._order=r}get x(){return this._x}set x(e){this._x=e,this._onChangeCallback()}get y(){return this._y}set y(e){this._y=e,this._onChangeCallback()}get z(){return this._z}set z(e){this._z=e,this._onChangeCallback()}get order(){return this._order}set order(e){this._order=e,this._onChangeCallback()}set(e,n,s,r=this._order){return this._x=e,this._y=n,this._z=s,this._order=r,this._onChangeCallback(),this}clone(){return new this.constructor(this._x,this._y,this._z,this._order)}copy(e){return this._x=e._x,this._y=e._y,this._z=e._z,this._order=e._order,this._onChangeCallback(),this}setFromRotationMatrix(e,n=this._order,s=!0){const r=e.elements,i=r[0],o=r[4],a=r[8],l=r[1],u=r[5],c=r[9],d=r[2],f=r[6],h=r[10];switch(n){case"XYZ":this._y=Math.asin(at(a,-1,1)),Math.abs(a)<.9999999?(this._x=Math.atan2(-c,h),this._z=Math.atan2(-o,i)):(this._x=Math.atan2(f,u),this._z=0);break;case"YXZ":this._x=Math.asin(-at(c,-1,1)),Math.abs(c)<.9999999?(this._y=Math.atan2(a,h),this._z=Math.atan2(l,u)):(this._y=Math.atan2(-d,i),this._z=0);break;case"ZXY":this._x=Math.asin(at(f,-1,1)),Math.abs(f)<.9999999?(this._y=Math.atan2(-d,h),this._z=Math.atan2(-o,u)):(this._y=0,this._z=Math.atan2(l,i));break;case"ZYX":this._y=Math.asin(-at(d,-1,1)),Math.abs(d)<.9999999?(this._x=Math.atan2(f,h),this._z=Math.atan2(l,i)):(this._x=0,this._z=Math.atan2(-o,u));break;case"YZX":this._z=Math.asin(at(l,-1,1)),Math.abs(l)<.9999999?(this._x=Math.atan2(-c,u),this._y=Math.atan2(-d,i)):(this._x=0,this._y=Math.atan2(a,h));break;case"XZY":this._z=Math.asin(-at(o,-1,1)),Math.abs(o)<.9999999?(this._x=Math.atan2(f,u),this._y=Math.atan2(a,i)):(this._x=Math.atan2(-c,h),this._y=0);break;default:console.warn("THREE.Euler: .setFromRotationMatrix() encountered an unknown order: "+n)}return this._order=n,s===!0&&this._onChangeCallback(),this}setFromQuaternion(e,n,s){return Lc.makeRotationFromQuaternion(e),this.setFromRotationMatrix(Lc,n,s)}setFromVector3(e,n=this._order){return this.set(e.x,e.y,e.z,n)}reorder(e){return Vc.setFromEuler(this),this.setFromQuaternion(Vc,e)}equals(e){return e._x===this._x&&e._y===this._y&&e._z===this._z&&e._order===this._order}fromArray(e){return this._x=e[0],this._y=e[1],this._z=e[2],e[3]!==void 0&&(this._order=e[3]),this._onChangeCallback(),this}toArray(e=[],n=0){return e[n]=this._x,e[n+1]=this._y,e[n+2]=this._z,e[n+3]=this._order,e}_onChange(e){return this._onChangeCallback=e,this}_onChangeCallback(){}*[Symbol.iterator](){yield this._x,yield this._y,yield this._z,yield this._order}}wo.DEFAULT_ORDER="XYZ";class rx{constructor(){this.mask=1}set(e){this.mask=(1<<e|0)>>>0}enable(e){this.mask|=1<<e|0}enableAll(){this.mask=-1}toggle(e){this.mask^=1<<e|0}disable(e){this.mask&=~(1<<e|0)}disableAll(){this.mask=0}test(e){return(this.mask&e.mask)!==0}isEnabled(e){return(this.mask&(1<<e|0))!==0}}class fs{constructor(e,n,s,r,i,o,a,l,u){fs.prototype.isMatrix3=!0,this.elements=[1,0,0,0,1,0,0,0,1],e!==void 0&&this.set(e,n,s,r,i,o,a,l,u)}set(e,n,s,r,i,o,a,l,u){const c=this.elements;return c[0]=e,c[1]=r,c[2]=a,c[3]=n,c[4]=i,c[5]=l,c[6]=s,c[7]=o,c[8]=u,this}identity(){return this.set(1,0,0,0,1,0,0,0,1),this}copy(e){const n=this.elements,s=e.elements;return n[0]=s[0],n[1]=s[1],n[2]=s[2],n[3]=s[3],n[4]=s[4],n[5]=s[5],n[6]=s[6],n[7]=s[7],n[8]=s[8],this}extractBasis(e,n,s){return e.setFromMatrix3Column(this,0),n.setFromMatrix3Column(this,1),s.setFromMatrix3Column(this,2),this}setFromMatrix4(e){const n=e.elements;return this.set(n[0],n[4],n[8],n[1],n[5],n[9],n[2],n[6],n[10]),this}multiply(e){return this.multiplyMatrices(this,e)}premultiply(e){return this.multiplyMatrices(e,this)}multiplyMatrices(e,n){const s=e.elements,r=n.elements,i=this.elements,o=s[0],a=s[3],l=s[6],u=s[1],c=s[4],d=s[7],f=s[2],h=s[5],m=s[8],p=r[0],v=r[3],S=r[6],g=r[1],y=r[4],b=r[7],x=r[2],_=r[5],C=r[8];return i[0]=o*p+a*g+l*x,i[3]=o*v+a*y+l*_,i[6]=o*S+a*b+l*C,i[1]=u*p+c*g+d*x,i[4]=u*v+c*y+d*_,i[7]=u*S+c*b+d*C,i[2]=f*p+h*g+m*x,i[5]=f*v+h*y+m*_,i[8]=f*S+h*b+m*C,this}multiplyScalar(e){const n=this.elements;return n[0]*=e,n[3]*=e,n[6]*=e,n[1]*=e,n[4]*=e,n[7]*=e,n[2]*=e,n[5]*=e,n[8]*=e,this}determinant(){const e=this.elements,n=e[0],s=e[1],r=e[2],i=e[3],o=e[4],a=e[5],l=e[6],u=e[7],c=e[8];return n*o*c-n*a*u-s*i*c+s*a*l+r*i*u-r*o*l}invert(){const e=this.elements,n=e[0],s=e[1],r=e[2],i=e[3],o=e[4],a=e[5],l=e[6],u=e[7],c=e[8],d=c*o-a*u,f=a*l-c*i,h=u*i-o*l,m=n*d+s*f+r*h;if(m===0)return this.set(0,0,0,0,0,0,0,0,0);const p=1/m;return e[0]=d*p,e[1]=(r*u-c*s)*p,e[2]=(a*s-r*o)*p,e[3]=f*p,e[4]=(c*n-r*l)*p,e[5]=(r*i-a*n)*p,e[6]=h*p,e[7]=(s*l-u*n)*p,e[8]=(o*n-s*i)*p,this}transpose(){let e;const n=this.elements;return e=n[1],n[1]=n[3],n[3]=e,e=n[2],n[2]=n[6],n[6]=e,e=n[5],n[5]=n[7],n[7]=e,this}getNormalMatrix(e){return this.setFromMatrix4(e).invert().transpose()}transposeIntoArray(e){const n=this.elements;return e[0]=n[0],e[1]=n[3],e[2]=n[6],e[3]=n[1],e[4]=n[4],e[5]=n[7],e[6]=n[2],e[7]=n[5],e[8]=n[8],this}setUvTransform(e,n,s,r,i,o,a){const l=Math.cos(i),u=Math.sin(i);return this.set(s*l,s*u,-s*(l*o+u*a)+o+e,-r*u,r*l,-r*(-u*o+l*a)+a+n,0,0,1),this}scale(e,n){return this.premultiply(Zo.makeScale(e,n)),this}rotate(e){return this.premultiply(Zo.makeRotation(-e)),this}translate(e,n){return this.premultiply(Zo.makeTranslation(e,n)),this}makeTranslation(e,n){return e.isVector2?this.set(1,0,e.x,0,1,e.y,0,0,1):this.set(1,0,e,0,1,n,0,0,1),this}makeRotation(e){const n=Math.cos(e),s=Math.sin(e);return this.set(n,-s,0,s,n,0,0,0,1),this}makeScale(e,n){return this.set(e,0,0,0,n,0,0,0,1),this}equals(e){const n=this.elements,s=e.elements;for(let r=0;r<9;r++)if(n[r]!==s[r])return!1;return!0}fromArray(e,n=0){for(let s=0;s<9;s++)this.elements[s]=e[s+n];return this}toArray(e=[],n=0){const s=this.elements;return e[n]=s[0],e[n+1]=s[1],e[n+2]=s[2],e[n+3]=s[3],e[n+4]=s[4],e[n+5]=s[5],e[n+6]=s[6],e[n+7]=s[7],e[n+8]=s[8],e}clone(){return new this.constructor().fromArray(this.elements)}}const Zo=new fs;let ix=0;const Pc=new J,bs=new Pr,tn=new He,Kr=new J,Ws=new J,ox=new J,ax=new Pr,Oc=new J(1,0,0),Nc=new J(0,1,0),Bc=new J(0,0,1),lx={type:"added"},ux={type:"removed"};class Ln extends sx{constructor(){super(),this.isObject3D=!0,Object.defineProperty(this,"id",{value:ix++}),this.uuid=Yb(),this.name="",this.type="Object3D",this.parent=null,this.children=[],this.up=Ln.DEFAULT_UP.clone();const e=new J,n=new wo,s=new Pr,r=new J(1,1,1);function i(){s.setFromEuler(n,!1)}function o(){n.setFromQuaternion(s,void 0,!1)}n._onChange(i),s._onChange(o),Object.defineProperties(this,{position:{configurable:!0,enumerable:!0,value:e},rotation:{configurable:!0,enumerable:!0,value:n},quaternion:{configurable:!0,enumerable:!0,value:s},scale:{configurable:!0,enumerable:!0,value:r},modelViewMatrix:{value:new He},normalMatrix:{value:new fs}}),this.matrix=new He,this.matrixWorld=new He,this.matrixAutoUpdate=Ln.DEFAULT_MATRIX_AUTO_UPDATE,this.matrixWorldAutoUpdate=Ln.DEFAULT_MATRIX_WORLD_AUTO_UPDATE,this.matrixWorldNeedsUpdate=!1,this.layers=new rx,this.visible=!0,this.castShadow=!1,this.receiveShadow=!1,this.frustumCulled=!0,this.renderOrder=0,this.animations=[],this.userData={}}onBeforeShadow(){}onAfterShadow(){}onBeforeRender(){}onAfterRender(){}applyMatrix4(e){this.matrixAutoUpdate&&this.updateMatrix(),this.matrix.premultiply(e),this.matrix.decompose(this.position,this.quaternion,this.scale)}applyQuaternion(e){return this.quaternion.premultiply(e),this}setRotationFromAxisAngle(e,n){this.quaternion.setFromAxisAngle(e,n)}setRotationFromEuler(e){this.quaternion.setFromEuler(e,!0)}setRotationFromMatrix(e){this.quaternion.setFromRotationMatrix(e)}setRotationFromQuaternion(e){this.quaternion.copy(e)}rotateOnAxis(e,n){return bs.setFromAxisAngle(e,n),this.quaternion.multiply(bs),this}rotateOnWorldAxis(e,n){return bs.setFromAxisAngle(e,n),this.quaternion.premultiply(bs),this}rotateX(e){return this.rotateOnAxis(Oc,e)}rotateY(e){return this.rotateOnAxis(Nc,e)}rotateZ(e){return this.rotateOnAxis(Bc,e)}translateOnAxis(e,n){return Pc.copy(e).applyQuaternion(this.quaternion),this.position.add(Pc.multiplyScalar(n)),this}translateX(e){return this.translateOnAxis(Oc,e)}translateY(e){return this.translateOnAxis(Nc,e)}translateZ(e){return this.translateOnAxis(Bc,e)}localToWorld(e){return this.updateWorldMatrix(!0,!1),e.applyMatrix4(this.matrixWorld)}worldToLocal(e){return this.updateWorldMatrix(!0,!1),e.applyMatrix4(tn.copy(this.matrixWorld).invert())}lookAt(e,n,s){e.isVector3?Kr.copy(e):Kr.set(e,n,s);const r=this.parent;this.updateWorldMatrix(!0,!1),Ws.setFromMatrixPosition(this.matrixWorld),this.isCamera||this.isLight?tn.lookAt(Ws,Kr,this.up):tn.lookAt(Kr,Ws,this.up),this.quaternion.setFromRotationMatrix(tn),r&&(tn.extractRotation(r.matrixWorld),bs.setFromRotationMatrix(tn),this.quaternion.premultiply(bs.invert()))}add(e){if(arguments.length>1){for(let n=0;n<arguments.length;n++)this.add(arguments[n]);return this}return e===this?(console.error("THREE.Object3D.add: object can't be added as a child of itself.",e),this):(e&&e.isObject3D?(e.parent!==null&&e.parent.remove(e),e.parent=this,this.children.push(e),e.dispatchEvent(lx)):console.error("THREE.Object3D.add: object not an instance of THREE.Object3D.",e),this)}remove(e){if(arguments.length>1){for(let s=0;s<arguments.length;s++)this.remove(arguments[s]);return this}const n=this.children.indexOf(e);return n!==-1&&(e.parent=null,this.children.splice(n,1),e.dispatchEvent(ux)),this}removeFromParent(){const e=this.parent;return e!==null&&e.remove(this),this}clear(){return this.remove(...this.children)}attach(e){return this.updateWorldMatrix(!0,!1),tn.copy(this.matrixWorld).invert(),e.parent!==null&&(e.parent.updateWorldMatrix(!0,!1),tn.multiply(e.parent.matrixWorld)),e.applyMatrix4(tn),this.add(e),e.updateWorldMatrix(!1,!0),this}getObjectById(e){return this.getObjectByProperty("id",e)}getObjectByName(e){return this.getObjectByProperty("name",e)}getObjectByProperty(e,n){if(this[e]===n)return this;for(let s=0,r=this.children.length;s<r;s++){const o=this.children[s].getObjectByProperty(e,n);if(o!==void 0)return o}}getObjectsByProperty(e,n,s=[]){this[e]===n&&s.push(this);const r=this.children;for(let i=0,o=r.length;i<o;i++)r[i].getObjectsByProperty(e,n,s);return s}getWorldPosition(e){return this.updateWorldMatrix(!0,!1),e.setFromMatrixPosition(this.matrixWorld)}getWorldQuaternion(e){return this.updateWorldMatrix(!0,!1),this.matrixWorld.decompose(Ws,e,ox),e}getWorldScale(e){return this.updateWorldMatrix(!0,!1),this.matrixWorld.decompose(Ws,ax,e),e}getWorldDirection(e){this.updateWorldMatrix(!0,!1);const n=this.matrixWorld.elements;return e.set(n[8],n[9],n[10]).normalize()}raycast(){}traverse(e){e(this);const n=this.children;for(let s=0,r=n.length;s<r;s++)n[s].traverse(e)}traverseVisible(e){if(this.visible===!1)return;e(this);const n=this.children;for(let s=0,r=n.length;s<r;s++)n[s].traverseVisible(e)}traverseAncestors(e){const n=this.parent;n!==null&&(e(n),n.traverseAncestors(e))}updateMatrix(){this.matrix.compose(this.position,this.quaternion,this.scale),this.matrixWorldNeedsUpdate=!0}updateMatrixWorld(e){this.matrixAutoUpdate&&this.updateMatrix(),(this.matrixWorldNeedsUpdate||e)&&(this.parent===null?this.matrixWorld.copy(this.matrix):this.matrixWorld.multiplyMatrices(this.parent.matrixWorld,this.matrix),this.matrixWorldNeedsUpdate=!1,e=!0);const n=this.children;for(let s=0,r=n.length;s<r;s++){const i=n[s];(i.matrixWorldAutoUpdate===!0||e===!0)&&i.updateMatrixWorld(e)}}updateWorldMatrix(e,n){const s=this.parent;if(e===!0&&s!==null&&s.matrixWorldAutoUpdate===!0&&s.updateWorldMatrix(!0,!1),this.matrixAutoUpdate&&this.updateMatrix(),this.parent===null?this.matrixWorld.copy(this.matrix):this.matrixWorld.multiplyMatrices(this.parent.matrixWorld,this.matrix),n===!0){const r=this.children;for(let i=0,o=r.length;i<o;i++){const a=r[i];a.matrixWorldAutoUpdate===!0&&a.updateWorldMatrix(!1,!0)}}}toJSON(e){const n=e===void 0||typeof e=="string",s={};n&&(e={geometries:{},materials:{},textures:{},images:{},shapes:{},skeletons:{},animations:{},nodes:{}},s.metadata={version:4.6,type:"Object",generator:"Object3D.toJSON"});const r={};r.uuid=this.uuid,r.type=this.type,this.name!==""&&(r.name=this.name),this.castShadow===!0&&(r.castShadow=!0),this.receiveShadow===!0&&(r.receiveShadow=!0),this.visible===!1&&(r.visible=!1),this.frustumCulled===!1&&(r.frustumCulled=!1),this.renderOrder!==0&&(r.renderOrder=this.renderOrder),Object.keys(this.userData).length>0&&(r.userData=this.userData),r.layers=this.layers.mask,r.matrix=this.matrix.toArray(),r.up=this.up.toArray(),this.matrixAutoUpdate===!1&&(r.matrixAutoUpdate=!1),this.isInstancedMesh&&(r.type="InstancedMesh",r.count=this.count,r.instanceMatrix=this.instanceMatrix.toJSON(),this.instanceColor!==null&&(r.instanceColor=this.instanceColor.toJSON())),this.isBatchedMesh&&(r.type="BatchedMesh",r.perObjectFrustumCulled=this.perObjectFrustumCulled,r.sortObjects=this.sortObjects,r.drawRanges=this._drawRanges,r.reservedRanges=this._reservedRanges,r.visibility=this._visibility,r.active=this._active,r.bounds=this._bounds.map(a=>({boxInitialized:a.boxInitialized,boxMin:a.box.min.toArray(),boxMax:a.box.max.toArray(),sphereInitialized:a.sphereInitialized,sphereRadius:a.sphere.radius,sphereCenter:a.sphere.center.toArray()})),r.maxGeometryCount=this._maxGeometryCount,r.maxVertexCount=this._maxVertexCount,r.maxIndexCount=this._maxIndexCount,r.geometryInitialized=this._geometryInitialized,r.geometryCount=this._geometryCount,r.matricesTexture=this._matricesTexture.toJSON(e),this.boundingSphere!==null&&(r.boundingSphere={center:r.boundingSphere.center.toArray(),radius:r.boundingSphere.radius}),this.boundingBox!==null&&(r.boundingBox={min:r.boundingBox.min.toArray(),max:r.boundingBox.max.toArray()}));function i(a,l){return a[l.uuid]===void 0&&(a[l.uuid]=l.toJSON(e)),l.uuid}if(this.isScene)this.background&&(this.background.isColor?r.background=this.background.toJSON():this.background.isTexture&&(r.background=this.background.toJSON(e).uuid)),this.environment&&this.environment.isTexture&&this.environment.isRenderTargetTexture!==!0&&(r.environment=this.environment.toJSON(e).uuid);else if(this.isMesh||this.isLine||this.isPoints){r.geometry=i(e.geometries,this.geometry);const a=this.geometry.parameters;if(a!==void 0&&a.shapes!==void 0){const l=a.shapes;if(Array.isArray(l))for(let u=0,c=l.length;u<c;u++){const d=l[u];i(e.shapes,d)}else i(e.shapes,l)}}if(this.isSkinnedMesh&&(r.bindMode=this.bindMode,r.bindMatrix=this.bindMatrix.toArray(),this.skeleton!==void 0&&(i(e.skeletons,this.skeleton),r.skeleton=this.skeleton.uuid)),this.material!==void 0)if(Array.isArray(this.material)){const a=[];for(let l=0,u=this.material.length;l<u;l++)a.push(i(e.materials,this.material[l]));r.material=a}else r.material=i(e.materials,this.material);if(this.children.length>0){r.children=[];for(let a=0;a<this.children.length;a++)r.children.push(this.children[a].toJSON(e).object)}if(this.animations.length>0){r.animations=[];for(let a=0;a<this.animations.length;a++){const l=this.animations[a];r.animations.push(i(e.animations,l))}}if(n){const a=o(e.geometries),l=o(e.materials),u=o(e.textures),c=o(e.images),d=o(e.shapes),f=o(e.skeletons),h=o(e.animations),m=o(e.nodes);a.length>0&&(s.geometries=a),l.length>0&&(s.materials=l),u.length>0&&(s.textures=u),c.length>0&&(s.images=c),d.length>0&&(s.shapes=d),f.length>0&&(s.skeletons=f),h.length>0&&(s.animations=h),m.length>0&&(s.nodes=m)}return s.object=r,s;function o(a){const l=[];for(const u in a){const c=a[u];delete c.metadata,l.push(c)}return l}}clone(e){return new this.constructor().copy(this,e)}copy(e,n=!0){if(this.name=e.name,this.up.copy(e.up),this.position.copy(e.position),this.rotation.order=e.rotation.order,this.quaternion.copy(e.quaternion),this.scale.copy(e.scale),this.matrix.copy(e.matrix),this.matrixWorld.copy(e.matrixWorld),this.matrixAutoUpdate=e.matrixAutoUpdate,this.matrixWorldAutoUpdate=e.matrixWorldAutoUpdate,this.matrixWorldNeedsUpdate=e.matrixWorldNeedsUpdate,this.layers.mask=e.layers.mask,this.visible=e.visible,this.castShadow=e.castShadow,this.receiveShadow=e.receiveShadow,this.frustumCulled=e.frustumCulled,this.renderOrder=e.renderOrder,this.animations=e.animations.slice(),this.userData=JSON.parse(JSON.stringify(e.userData)),n===!0)for(let s=0;s<e.children.length;s++){const r=e.children[s];this.add(r.clone())}return this}}Ln.DEFAULT_UP=new J(0,1,0);Ln.DEFAULT_MATRIX_AUTO_UPDATE=!0;Ln.DEFAULT_MATRIX_WORLD_AUTO_UPDATE=!0;class cx extends Ln{constructor(){super(),this.isCamera=!0,this.type="Camera",this.matrixWorldInverse=new He,this.projectionMatrix=new He,this.projectionMatrixInverse=new He,this.coordinateSystem=Ks}copy(e,n){return super.copy(e,n),this.matrixWorldInverse.copy(e.matrixWorldInverse),this.projectionMatrix.copy(e.projectionMatrix),this.projectionMatrixInverse.copy(e.projectionMatrixInverse),this.coordinateSystem=e.coordinateSystem,this}getWorldDirection(e){return super.getWorldDirection(e).negate()}updateMatrixWorld(e){super.updateMatrixWorld(e),this.matrixWorldInverse.copy(this.matrixWorld).invert()}updateWorldMatrix(e,n){super.updateWorldMatrix(e,n),this.matrixWorldInverse.copy(this.matrixWorld).invert()}clone(){return new this.constructor().copy(this)}}class dx extends cx{constructor(e=-1,n=1,s=1,r=-1,i=.1,o=2e3){super(),this.isOrthographicCamera=!0,this.type="OrthographicCamera",this.zoom=1,this.view=null,this.left=e,this.right=n,this.top=s,this.bottom=r,this.near=i,this.far=o,this.updateProjectionMatrix()}copy(e,n){return super.copy(e,n),this.left=e.left,this.right=e.right,this.top=e.top,this.bottom=e.bottom,this.near=e.near,this.far=e.far,this.zoom=e.zoom,this.view=e.view===null?null:Object.assign({},e.view),this}setViewOffset(e,n,s,r,i,o){this.view===null&&(this.view={enabled:!0,fullWidth:1,fullHeight:1,offsetX:0,offsetY:0,width:1,height:1}),this.view.enabled=!0,this.view.fullWidth=e,this.view.fullHeight=n,this.view.offsetX=s,this.view.offsetY=r,this.view.width=i,this.view.height=o,this.updateProjectionMatrix()}clearViewOffset(){this.view!==null&&(this.view.enabled=!1),this.updateProjectionMatrix()}updateProjectionMatrix(){const e=(this.right-this.left)/(2*this.zoom),n=(this.top-this.bottom)/(2*this.zoom),s=(this.right+this.left)/2,r=(this.top+this.bottom)/2;let i=s-e,o=s+e,a=r+n,l=r-n;if(this.view!==null&&this.view.enabled){const u=(this.right-this.left)/this.view.fullWidth/this.zoom,c=(this.top-this.bottom)/this.view.fullHeight/this.zoom;i+=u*this.view.offsetX,o=i+u*this.view.width,a-=c*this.view.offsetY,l=a-c*this.view.height}this.projectionMatrix.makeOrthographic(i,o,a,l,this.near,this.far,this.coordinateSystem),this.projectionMatrixInverse.copy(this.projectionMatrix).invert()}toJSON(e){const n=super.toJSON(e);return n.object.zoom=this.zoom,n.object.left=this.left,n.object.right=this.right,n.object.top=this.top,n.object.bottom=this.bottom,n.object.near=this.near,n.object.far=this.far,this.view!==null&&(n.object.view=Object.assign({},this.view)),n}}const fx=Wt("h5",null,"Camera",-1),hx={class:"icon-detail"},mx=Wt("br",null,null,-1),gx=Wt("h5",null,"Extras",-1),vx={ref:"statsHolder"},px=fn({__name:"Tools",props:{viewer:{}},emits:["findModel"],setup(t,{expose:e,emit:n}){const s=ua({loader:()=>Ut(()=>import("./Selection-GYCMl90M.js"),__vite__mapDeps([0,1,2]),import.meta.url),loadingComponent:()=>"Loading...",delay:0});let r=ee(null);const i=ua({loader:()=>Ut(()=>import("./LicensesDialogContent-C8ll0biF.js"),[],import.meta.url),loadingComponent:Yh,delay:0});let o=t;const a=n;let l=ee([]),u=()=>l.value.filter(y=>y.kind=="face").length,c=()=>l.value.filter(y=>y.kind=="edge").length,d=()=>l.value.filter(y=>y.kind=="vertex").length;function f(y){var _;let b=(_=o.viewer)==null?void 0:_.scene;if(!b)return;let x=b.__perspectiveCamera;if(y||x&&b.camera!=x){let C=b.getTarget().clone().add(b.target.position),M=2*Math.tan(x.fov*Math.PI/180/2)*x.position.distanceTo(C),R=M,F=M/b.aspect;b.camera=new dx(-R,R,F,-F,x.near,x.far),b.camera.position.copy(x.position),b.camera.lookAt(C),y&&b.queueRender(),requestAnimationFrame(()=>f(!1))}}let h=ee("PERSP");function m(){var _;let y=(_=o.viewer)==null?void 0:_.scene;if(!y)return;let b=y.camera,x=b.isPerspectiveCamera;x?(y.__perspectiveCamera=b,f(!0)):(y.camera=y.__perspectiveCamera,y.queueRender()),h.value=x?"ORTHO":"PERSP",requestIdleCallback(()=>{var C,M;return(M=(C=o.viewer)==null?void 0:C.elem)==null?void 0:M.dispatchEvent(new CustomEvent("camera-change",{detail:{source:"none"}}))},{timeout:100})}async function p(){var b;let y=(b=o.viewer)==null?void 0:b.elem;y&&(await y.updateFraming(),y.zoom(3))}async function v(){var C;let y=(C=o.viewer)==null?void 0:C.elem;if(!y)return;const b=await y.exportScene({onlyVisible:!0,binary:!0}),x=new File([b],"export.glb"),_=document.createElement("a");_.download=x.name,_.href=URL.createObjectURL(x),_.click()}async function S(){window.open("https://github.com/yeicor-3d/yet-another-cad-viewer","_blank")}function g(y){var b,x,_;for(let C of l.value.filter(M=>M.getObjectName()===y))(b=r.value)==null||b.deselect(C);(x=r.value)==null||x.updateBoundingBox(),(_=r.value)==null||_.updateDistances()}return e({removeObjectSelections:g}),window.addEventListener("keydown",y=>{y.key==="p"?m():y.key==="c"?p():y.key==="d"?v():y.key==="g"&&S()}),(y,b)=>{var x;return st(),Ar(Ie,null,[(x=P(o).viewer)!=null&&x.scene?(st(),Yt(nx,{key:0,scene:P(o).viewer.scene,elem:P(o).viewer.elem},null,8,["scene","elem"])):Ei("",!0),w(P(gi)),fx,w(P(ot),{icon:"",onClick:m},{default:q(()=>[Wt("span",hx,rn(P(h)),1),w(P(We),{activator:"parent"},{default:q(()=>[Ae("Toggle (P)rojection"),mx,Ae("(currently "+rn(P(h)==="PERSP"?"perspective":"orthographic")+") ",1)]),_:1}),w(P(Be),{type:"mdi",path:P(Pb)},null,8,["path"])]),_:1}),w(P(ot),{icon:"",onClick:p},{default:q(()=>[w(P(We),{activator:"parent"},{default:q(()=>[Ae("Re(c)enter Camera")]),_:1}),w(P(Be),{type:"mdi",path:P(Rb)},null,8,["path"])]),_:1}),w(P(gi)),Wt("h5",null,"Selection ("+rn(P(u)())+"F "+rn(P(c)())+"E "+rn(P(d)())+"V)",1),w(P(s),{ref_key:"selectionComp",ref:r,viewer:P(o).viewer,modelValue:P(l),"onUpdate:modelValue":b[0]||(b[0]=_=>Me(l)?l.value=_:l=_),onFindModel:b[1]||(b[1]=_=>a("findModel",_))},null,8,["viewer","modelValue"]),w(P(gi)),w(P(ka)),gx,w(P(ot),{icon:"",onClick:v},{default:q(()=>[w(P(We),{activator:"parent"},{default:q(()=>[Ae("(D)ownload Scene")]),_:1}),w(P(Be),{type:"mdi",path:P(kb)},null,8,["path"])]),_:1}),w(P(V0),{id:"licenses-dialog",fullscreen:""},{activator:q(({props:_})=>[w(P(ot),Pe({icon:""},_),{default:q(()=>[w(P(We),{activator:"parent"},{default:q(()=>[Ae("Show Licenses")]),_:1}),w(P(Be),{type:"mdi",path:P(Lb)},null,8,["path"])]),_:2},1040)]),default:q(({isActive:_})=>[w(P(H0),null,{default:q(()=>[w(P($f),null,{default:q(()=>[w(P(Ni),null,{default:q(()=>[Ae("Licenses")]),_:1}),w(P(ka)),w(P(ot),{icon:"",onClick:C=>_.value=!1},{default:q(()=>[w(P(Be),{type:"mdi",path:P(Fa)},null,8,["path"])]),_:2},1032,["onClick"])]),_:2},1024),w(P(kh),null,{default:q(()=>[w(P(i))]),_:1})]),_:2},1024)]),_:1}),w(P(ot),{icon:"",onClick:S},{default:q(()=>[w(P(We),{activator:"parent"},{default:q(()=>[Ae("Open (G)itHub")]),_:1}),w(P(Be),{type:"mdi",path:P(Fb)},null,8,["path"])]),_:1}),Wt("div",vx,null,512)],64)}}});class Tl{constructor(){this._listeners={}}addEventListener(e,n){const s=this._listeners;return s[e]===void 0&&(s[e]=[]),s[e].indexOf(n)===-1&&s[e].push(n),this}removeEventListener(e,n){if(this._listeners===void 0)return this;const r=this._listeners[e];if(r!==void 0){const i=r.indexOf(n);i!==-1&&r.splice(i,1)}return this}dispatchEvent(e){if(this._listeners===void 0)return this;const s=this._listeners[e.type];if(s!==void 0){const r=s.slice(0);for(let i=0,o=r.length;i<o;i++)r[i].call(this,e)}return this}dispose(){for(const e in this._listeners)delete this._listeners[e]}}class Ps extends Tl{constructor(e,n,s,r={}){if(super(),this._name=void 0,this._parent=void 0,this._child=void 0,this._attributes=void 0,this._disposed=!1,this._name=e,this._parent=n,this._child=s,this._attributes=r,!n.isOnGraph(s))throw new Error("Cannot connect disconnected graphs.")}getName(){return this._name}getParent(){return this._parent}getChild(){return this._child}setChild(e){return this._child=e,this}getAttributes(){return this._attributes}dispose(){this._disposed||(this._disposed=!0,this.dispatchEvent({type:"dispose",target:this}),super.dispose())}isDisposed(){return this._disposed}}class yx extends Tl{constructor(...e){super(...e),this._emptySet=new Set,this._edges=new Set,this._parentEdges=new Map,this._childEdges=new Map}listEdges(){return Array.from(this._edges)}listParentEdges(e){return Array.from(this._childEdges.get(e)||this._emptySet)}listParents(e){return this.listParentEdges(e).map(n=>n.getParent())}listChildEdges(e){return Array.from(this._parentEdges.get(e)||this._emptySet)}listChildren(e){return this.listChildEdges(e).map(n=>n.getChild())}disconnectParents(e,n){let s=this.listParentEdges(e);return n&&(s=s.filter(r=>n(r.getParent()))),s.forEach(r=>r.dispose()),this}createEdge(e,n,s,r){return this._registerEdge(new Ps(e,n,s,r))}_registerEdge(e){this._edges.add(e);const n=e.getParent();this._parentEdges.has(n)||this._parentEdges.set(n,new Set),this._parentEdges.get(n).add(e);const s=e.getChild();return this._childEdges.has(s)||this._childEdges.set(s,new Set),this._childEdges.get(s).add(e),e.addEventListener("dispose",()=>this._removeEdge(e)),e}_removeEdge(e){return this._edges.delete(e),this._parentEdges.get(e.getParent()).delete(e),this._childEdges.get(e.getChild()).delete(e),this}}function lr(){return lr=Object.assign||function(t){for(var e=1;e<arguments.length;e++){var n=arguments[e];for(var s in n)Object.prototype.hasOwnProperty.call(n,s)&&(t[s]=n[s])}return t},lr.apply(this,arguments)}function Va(t){return t instanceof Ps}function Js(t){return Array.isArray(t)&&t[0]instanceof Ps}function Qs(t){return!!(xx(t)&&bx(t)instanceof Ps)}function bx(t){for(const e in t)return t[e]}function xx(t){return!!t&&Object.getPrototypeOf(t)===Object.prototype}const Ce=Symbol("attributes"),Ms=Symbol("immutableKeys");class Al extends Tl{constructor(e){super(),this._disposed=!1,this.graph=void 0,this[Ce]=void 0,this[Ms]=void 0,this.graph=e,this[Ms]=new Set,this[Ce]=this._createAttributes()}getDefaults(){return{}}_createAttributes(){const e=this.getDefaults(),n={};for(const s in e){const r=e[s];if(r instanceof Al){const i=this.graph.createEdge(s,this,r);i.addEventListener("dispose",()=>r.dispose()),this[Ms].add(s),n[s]=i}else n[s]=r}return n}isOnGraph(e){return this.graph===e.graph}isDisposed(){return this._disposed}dispose(){this._disposed||(this.graph.listChildEdges(this).forEach(e=>e.dispose()),this.graph.disconnectParents(this),this._disposed=!0,this.dispatchEvent({type:"dispose"}))}detach(){return this.graph.disconnectParents(this),this}swap(e,n){for(const s in this[Ce]){const r=this[Ce][s];if(Va(r)){const i=r;i.getChild()===e&&this.setRef(s,n,i.getAttributes())}else if(Js(r)){const o=r.find(a=>a.getChild()===e);if(o){const a=o.getAttributes();this.removeRef(s,e).addRef(s,n,a)}}else if(Qs(r)){const i=r;for(const o in i){const a=i[o];a.getChild()===e&&this.setRefMap(s,o,n,a.getAttributes())}}}return this}get(e){return this[Ce][e]}set(e,n){return this[Ce][e]=n,this.dispatchEvent({type:"change",attribute:e})}getRef(e){const n=this[Ce][e];return n?n.getChild():null}setRef(e,n,s){if(this[Ms].has(e))throw new Error(`Cannot overwrite immutable attribute, "${e}".`);const r=this[Ce][e];if(r&&r.dispose(),!n)return this;const i=this.graph.createEdge(e,this,n,s);return i.addEventListener("dispose",()=>{delete this[Ce][e],this.dispatchEvent({type:"change",attribute:e})}),this[Ce][e]=i,this.dispatchEvent({type:"change",attribute:e})}listRefs(e){return this[Ce][e].map(s=>s.getChild())}addRef(e,n,s){const r=this.graph.createEdge(e,this,n,s),i=this[Ce][e];return i.push(r),r.addEventListener("dispose",()=>{let o;for(;(o=i.indexOf(r))!==-1;)i.splice(o,1);this.dispatchEvent({type:"change",attribute:e})}),this.dispatchEvent({type:"change",attribute:e})}removeRef(e,n){return this[Ce][e].filter(i=>i.getChild()===n).forEach(i=>i.dispose()),this}listRefMapKeys(e){return Object.keys(this[Ce][e])}listRefMapValues(e){return Object.values(this[Ce][e]).map(n=>n.getChild())}getRefMap(e,n){const s=this[Ce][e];return s[n]?s[n].getChild():null}setRefMap(e,n,s,r){const i=this[Ce][e],o=i[n];if(o&&o.dispose(),!s)return this;r=Object.assign(r||{},{key:n});const a=this.graph.createEdge(e,this,s,lr({},r,{key:n}));return a.addEventListener("dispose",()=>{delete i[n],this.dispatchEvent({type:"change",attribute:e,key:n})}),i[n]=a,this.dispatchEvent({type:"change",attribute:e,key:n})}dispatchEvent(e){return super.dispatchEvent(lr({},e,{target:this})),this.graph.dispatchEvent(lr({},e,{target:this,type:`node:${e.type}`})),this}}const Kh="v3.10.1",Hi="@glb.bin";var se,$i,bt,Pa,ts;(function(t){t.ACCESSOR="Accessor",t.ANIMATION="Animation",t.ANIMATION_CHANNEL="AnimationChannel",t.ANIMATION_SAMPLER="AnimationSampler",t.BUFFER="Buffer",t.CAMERA="Camera",t.MATERIAL="Material",t.MESH="Mesh",t.PRIMITIVE="Primitive",t.PRIMITIVE_TARGET="PrimitiveTarget",t.NODE="Node",t.ROOT="Root",t.SCENE="Scene",t.SKIN="Skin",t.TEXTURE="Texture",t.TEXTURE_INFO="TextureInfo"})(se||(se={})),function(t){t.INTERLEAVED="interleaved",t.SEPARATE="separate"}($i||($i={})),function(t){t.ARRAY_BUFFER="ARRAY_BUFFER",t.ELEMENT_ARRAY_BUFFER="ELEMENT_ARRAY_BUFFER",t.INVERSE_BIND_MATRICES="INVERSE_BIND_MATRICES",t.OTHER="OTHER",t.SPARSE="SPARSE"}(bt||(bt={})),function(t){t[t.R=4096]="R",t[t.G=256]="G",t[t.B=16]="B",t[t.A=1]="A"}(Pa||(Pa={})),function(t){t.GLTF="GLTF",t.GLB="GLB"}(ts||(ts={}));const Ui={5120:Int8Array,5121:Uint8Array,5122:Int16Array,5123:Uint16Array,5125:Uint32Array,5126:Float32Array};var Jr,Oa=typeof Float32Array<"u"?Float32Array:Array;function Ko(t){return Math.hypot(t[0],t[1],t[2])}function wx(t,e,n){var s=e[0],r=e[1],i=e[2],o=n[3]*s+n[7]*r+n[11]*i+n[15];return t[0]=(n[0]*s+n[4]*r+n[8]*i+n[12])/(o=o||1),t[1]=(n[1]*s+n[5]*r+n[9]*i+n[13])/o,t[2]=(n[2]*s+n[6]*r+n[10]*i+n[14])/o,t}function Gw(t){const e={min:[1/0,1/0,1/0],max:[-1/0,-1/0,-1/0]},n=t.propertyType===se.NODE?[t]:t.listChildren();for(const s of n)s.traverse(r=>{const i=r.getMesh();if(!i)return;const o=_x(i,r.getWorldMatrix());Na(o.min,e),Na(o.max,e)});return e}Math.hypot||(Math.hypot=function(){for(var t=0,e=arguments.length;e--;)t+=arguments[e]*arguments[e];return Math.sqrt(t)}),Jr=new Oa(3),Oa!=Float32Array&&(Jr[0]=0,Jr[1]=0,Jr[2]=0);function _x(t,e){const n={min:[1/0,1/0,1/0],max:[-1/0,-1/0,-1/0]};for(const s of t.listPrimitives()){const r=s.getAttribute("POSITION");if(!r)continue;let i=[0,0,0],o=[0,0,0];for(let a=0;a<r.getCount();a++)i=r.getElement(a,i),o=wx(o,i,e),Na(o,n)}return n}function Na(t,e){for(let n=0;n<3;n++)e.min[n]=Math.min(t[n],e.min[n]),e.max[n]=Math.max(t[n],e.max[n])}class Se{static createBufferFromDataURI(e){if(typeof Buffer>"u"){const n=atob(e.split(",")[1]),s=new Uint8Array(n.length);for(let r=0;r<n.length;r++)s[r]=n.charCodeAt(r);return s}{const n=e.split(",")[1],s=e.indexOf("base64")>=0;return Buffer.from(n,s?"base64":"utf8")}}static encodeText(e){return typeof TextEncoder<"u"?new TextEncoder().encode(e):Buffer.from(e)}static decodeText(e){return typeof TextDecoder<"u"?new TextDecoder().decode(e):Buffer.from(e).toString("utf8")}static concat(e){let n=0;for(const i of e)n+=i.byteLength;const s=new Uint8Array(n);let r=0;for(const i of e)s.set(i,r),r+=i.byteLength;return s}static pad(e,n=0){const s=this.padNumber(e.byteLength);if(s===e.byteLength)return e;const r=new Uint8Array(s);if(r.set(e),n!==0)for(let i=e.byteLength;i<s;i++)r[i]=n;return r}static padNumber(e){return 4*Math.ceil(e/4)}static equals(e,n){if(e===n)return!0;if(e.byteLength!==n.byteLength)return!1;let s=e.byteLength;for(;s--;)if(e[s]!==n[s])return!1;return!0}static toView(e,n=0,s=1/0){return new Uint8Array(e.buffer,e.byteOffset+n,Math.min(e.byteLength,s))}static assertView(e){if(e&&!ArrayBuffer.isView(e))throw new Error(`Method requires Uint8Array parameter; received "${typeof e}".`);return e}}class Qr{static hexToFactor(e,n){e=Math.floor(e);const s=n;return s[0]=(e>>16&255)/255,s[1]=(e>>8&255)/255,s[2]=(255&e)/255,this.convertSRGBToLinear(n,n)}static factorToHex(e){const n=[...e],[s,r,i]=this.convertLinearToSRGB(e,n);return 255*s<<16^255*r<<8^255*i<<0}static convertSRGBToLinear(e,n){const s=e,r=n;for(let i=0;i<3;i++)r[i]=s[i]<.04045?.0773993808*s[i]:Math.pow(.9478672986*s[i]+.0521327014,2.4);return n}static convertLinearToSRGB(e,n){const s=e,r=n;for(let i=0;i<3;i++)r[i]=s[i]<.0031308?12.92*s[i]:1.055*Math.pow(s[i],.41666)-.055;return n}}class _o{match(e){return e.length>=8&&e[0]===137&&e[1]===80&&e[2]===78&&e[3]===71&&e[4]===13&&e[5]===10&&e[6]===26&&e[7]===10}getSize(e){const n=new DataView(e.buffer,e.byteOffset);return Se.decodeText(e.slice(12,16))===_o.PNG_FRIED_CHUNK_NAME?[n.getUint32(32,!1),n.getUint32(36,!1)]:[n.getUint32(16,!1),n.getUint32(20,!1)]}getChannels(e){return 4}}_o.PNG_FRIED_CHUNK_NAME="CgBI";class ns{static registerFormat(e,n){this.impls[e]=n}static getMimeType(e){for(const n in this.impls)if(this.impls[n].match(e))return n;return null}static getSize(e,n){return this.impls[n]?this.impls[n].getSize(e):null}static getChannels(e,n){return this.impls[n]?this.impls[n].getChannels(e):null}static getVRAMByteLength(e,n){if(!this.impls[n])return null;if(this.impls[n].getVRAMByteLength)return this.impls[n].getVRAMByteLength(e);let s=0;const r=this.getSize(e,n);if(!r)return null;for(;r[0]>1||r[1]>1;)s+=r[0]*r[1]*4,r[0]=Math.max(Math.floor(r[0]/2),1),r[1]=Math.max(Math.floor(r[1]/2),1);return s+=4,s}static mimeTypeToExtension(e){return e==="image/jpeg"?"jpg":e.split("/").pop()}static extensionToMimeType(e){return e==="jpg"?"image/jpeg":e?`image/${e}`:""}}function Sx(t,e){if(e>t.byteLength)throw new TypeError("Corrupt JPG, exceeded buffer limits");if(t.getUint8(e)!==255)throw new TypeError("Invalid JPG, marker table corrupted");return t}ns.impls={"image/jpeg":new class{match(t){return t.length>=3&&t[0]===255&&t[1]===216&&t[2]===255}getSize(t){let e,n,s=new DataView(t.buffer,t.byteOffset+4);for(;s.byteLength;){if(e=s.getUint16(0,!1),Sx(s,e),n=s.getUint8(e+1),n===192||n===193||n===194)return[s.getUint16(e+7,!1),s.getUint16(e+5,!1)];s=new DataView(t.buffer,s.byteOffset+e+2)}throw new TypeError("Invalid JPG, no size found")}getChannels(t){return 3}},"image/png":new _o};class Os{static basename(e){const n=e.split(/[\\/]/).pop();return n.substring(0,n.lastIndexOf("."))}static extension(e){if(e.startsWith("data:image/")){const n=e.match(/data:(image\/\w+)/)[1];return ns.mimeTypeToExtension(n)}return e.startsWith("data:model/gltf+json")?"gltf":e.startsWith("data:model/gltf-binary")?"glb":e.startsWith("data:application/")?"bin":e.split(/[\\/]/).pop().split(/[.]/).pop()}}function Dc(t){return Object.prototype.toString.call(t)==="[object Object]"}function Zn(t){if(Dc(t)===!1)return!1;const e=t.constructor;if(e===void 0)return!0;const n=e.prototype;return Dc(n)!==!1&&Object.prototype.hasOwnProperty.call(n,"isPrototypeOf")!==!1}var Jo,Ba;(function(t){t[t.SILENT=4]="SILENT",t[t.ERROR=3]="ERROR",t[t.WARN=2]="WARN",t[t.INFO=1]="INFO",t[t.DEBUG=0]="DEBUG"})(Ba||(Ba={}));class Et{constructor(e){this.verbosity=void 0,this.verbosity=e}debug(e){this.verbosity<=Et.Verbosity.DEBUG&&console.debug(e)}info(e){this.verbosity<=Et.Verbosity.INFO&&console.info(e)}warn(e){this.verbosity<=Et.Verbosity.WARN&&console.warn(e)}error(e){this.verbosity<=Et.Verbosity.ERROR&&console.error(e)}}function Cx(t,e,n){var s=e[0],r=e[1],i=e[2],o=e[3],a=e[4],l=e[5],u=e[6],c=e[7],d=e[8],f=e[9],h=e[10],m=e[11],p=e[12],v=e[13],S=e[14],g=e[15],y=n[0],b=n[1],x=n[2],_=n[3];return t[0]=y*s+b*a+x*d+_*p,t[1]=y*r+b*l+x*f+_*v,t[2]=y*i+b*u+x*h+_*S,t[3]=y*o+b*c+x*m+_*g,t[4]=(y=n[4])*s+(b=n[5])*a+(x=n[6])*d+(_=n[7])*p,t[5]=y*r+b*l+x*f+_*v,t[6]=y*i+b*u+x*h+_*S,t[7]=y*o+b*c+x*m+_*g,t[8]=(y=n[8])*s+(b=n[9])*a+(x=n[10])*d+(_=n[11])*p,t[9]=y*r+b*l+x*f+_*v,t[10]=y*i+b*u+x*h+_*S,t[11]=y*o+b*c+x*m+_*g,t[12]=(y=n[12])*s+(b=n[13])*a+(x=n[14])*d+(_=n[15])*p,t[13]=y*r+b*l+x*f+_*v,t[14]=y*i+b*u+x*h+_*S,t[15]=y*o+b*c+x*m+_*g,t}Jo=Et,Et.Verbosity=Ba,Et.DEFAULT_INSTANCE=new Jo(Jo.Verbosity.INFO);class ze{static identity(e){return e}static eq(e,n,s=1e-5){if(e.length!==n.length)return!1;for(let r=0;r<e.length;r++)if(Math.abs(e[r]-n[r])>s)return!1;return!0}static decodeNormalizedInt(e,n){switch(n){case 5126:return e;case 5123:return e/65535;case 5121:return e/255;case 5122:return Math.max(e/32767,-1);case 5120:return Math.max(e/127,-1);default:throw new Error("Invalid component type.")}}static denormalize(e,n){return ze.decodeNormalizedInt(e,n)}static encodeNormalizedInt(e,n){switch(n){case 5126:return e;case 5123:return Math.round(65535*e);case 5121:return Math.round(255*e);case 5122:return Math.round(32767*e);case 5120:return Math.round(127*e);default:throw new Error("Invalid component type.")}}static normalize(e,n){return ze.encodeNormalizedInt(e,n)}static decompose(e,n,s,r){let i=Ko([e[0],e[1],e[2]]);const o=Ko([e[4],e[5],e[6]]),a=Ko([e[8],e[9],e[10]]);var l,u,c,d,f,h,m,p,v,S,g,y,b,x,_,C,M;((u=(l=e)[0])*(m=l[5])-(c=l[1])*(h=l[4]))*((y=l[10])*(M=l[15])-(b=l[11])*(C=l[14]))-(u*(p=l[6])-(d=l[2])*h)*((g=l[9])*M-b*(_=l[13]))+(u*(v=l[7])-(f=l[3])*h)*(g*C-y*_)+(c*p-d*m)*((S=l[8])*M-b*(x=l[12]))-(c*v-f*m)*(S*C-y*x)+(d*v-f*p)*(S*_-g*x)<0&&(i=-i),n[0]=e[12],n[1]=e[13],n[2]=e[14];const R=e.slice(),F=1/i,V=1/o,T=1/a;R[0]*=F,R[1]*=F,R[2]*=F,R[4]*=V,R[5]*=V,R[6]*=V,R[8]*=T,R[9]*=T,R[10]*=T,function(L,A){var N=new Oa(3);(function(_t,Qe){var Dn=Qe[4],E=Qe[5],I=Qe[6],O=Qe[8],B=Qe[9],D=Qe[10];_t[0]=Math.hypot(Qe[0],Qe[1],Qe[2]),_t[1]=Math.hypot(Dn,E,I),_t[2]=Math.hypot(O,B,D)})(N,A);var $=1/N[0],z=1/N[1],X=1/N[2],re=A[0]*$,te=A[1]*z,Q=A[2]*X,ce=A[4]*$,Te=A[5]*z,ge=A[6]*X,Le=A[8]*$,Z=A[9]*z,xe=A[10]*X,vn=re+Te+xe,Fe=0;vn>0?(Fe=2*Math.sqrt(vn+1),L[3]=.25*Fe,L[0]=(ge-Z)/Fe,L[1]=(Le-Q)/Fe,L[2]=(te-ce)/Fe):re>Te&&re>xe?(Fe=2*Math.sqrt(1+re-Te-xe),L[3]=(ge-Z)/Fe,L[0]=.25*Fe,L[1]=(te+ce)/Fe,L[2]=(Le+Q)/Fe):Te>xe?(Fe=2*Math.sqrt(1+Te-re-xe),L[3]=(Le-Q)/Fe,L[0]=(te+ce)/Fe,L[1]=.25*Fe,L[2]=(ge+Z)/Fe):(Fe=2*Math.sqrt(1+xe-re-Te),L[3]=(te-ce)/Fe,L[0]=(Le+Q)/Fe,L[1]=(ge+Z)/Fe,L[2]=.25*Fe)}(s,R),r[0]=i,r[1]=o,r[2]=a}static compose(e,n,s,r){const i=r,o=n[0],a=n[1],l=n[2],u=n[3],c=o+o,d=a+a,f=l+l,h=o*c,m=o*d,p=o*f,v=a*d,S=a*f,g=l*f,y=u*c,b=u*d,x=u*f,_=s[0],C=s[1],M=s[2];return i[0]=(1-(v+g))*_,i[1]=(m+x)*_,i[2]=(p-b)*_,i[3]=0,i[4]=(m-x)*C,i[5]=(1-(h+g))*C,i[6]=(S+y)*C,i[7]=0,i[8]=(p+b)*M,i[9]=(S-y)*M,i[10]=(1-(h+v))*M,i[11]=0,i[12]=e[0],i[13]=e[1],i[14]=e[2],i[15]=1,i}}function Ex(t,e){if(!!t!=!!e)return!1;const n=t.getChild(),s=e.getChild();return n===s||n.equals(s)}function Mx(t,e){if(!!t!=!!e||t.length!==e.length)return!1;for(let n=0;n<t.length;n++){const s=t[n],r=e[n];if(s.getChild()!==r.getChild()&&!s.getChild().equals(r.getChild()))return!1}return!0}function Tx(t,e){if(!!t!=!!e)return!1;const n=Object.keys(t),s=Object.keys(e);if(n.length!==s.length)return!1;for(const r in t){const i=t[r],o=e[r];if(!!i!=!!o)return!1;const a=i.getChild(),l=o.getChild();if(a!==l&&!a.equals(l))return!1}return!0}function Jh(t,e){if(t===e)return!0;if(!!t!=!!e||!t||!e||t.length!==e.length)return!1;for(let n=0;n<t.length;n++)if(t[n]!==e[n])return!1;return!0}function Qh(t,e){if(t===e)return!0;if(!!t!=!!e)return!1;if(!Zn(t)||!Zn(e))return t===e;const n=t,s=e;let r,i=0,o=0;for(r in n)i++;for(r in s)o++;if(i!==o)return!1;for(r in n){const a=n[r],l=s[r];if(ji(a)&&ji(l)){if(!Jh(a,l))return!1}else if(Zn(a)&&Zn(l)){if(!Qh(a,l))return!1}else if(a!==l)return!1}return!0}function ji(t){return Array.isArray(t)||ArrayBuffer.isView(t)}const zc=new Set,Ax=function(){let t="";for(let e=0;e<6;e++)t+="23456789abdegjkmnpqrvwxyzABDEGJKMNPQRVWXYZ".charAt(Math.floor(42*Math.random()));return t},Rx=function(){for(let t=0;t<999;t++){const e=Ax();if(!zc.has(e))return zc.add(e),e}return""},Hc="https://null.example";let ur=class{static dirname(e){const n=e.lastIndexOf("/");return n===-1?"./":e.substring(0,n+1)}static basename(e){return Os.basename(new URL(e,Hc).pathname)}static extension(e){return Os.extension(new URL(e,Hc).pathname)}static resolve(e,n){if(!this.isRelativePath(n))return n;const s=e.split("/"),r=n.split("/");s.pop();for(let i=0;i<r.length;i++)r[i]!=="."&&(r[i]===".."?s.pop():s.push(r[i]));return s.join("/")}static isAbsoluteURL(e){return this.PROTOCOL_REGEXP.test(e)}static isRelativePath(e){return!/^(?:[a-zA-Z]+:)?\//.test(e)}};ur.DEFAULT_INIT={},ur.PROTOCOL_REGEXP=/^[a-zA-Z]+:\/\//;const cn=t=>t,Ix=new Set;class Rl extends Al{constructor(e,n=""){super(e),this[Ce].name=n,this.init(),this.dispatchEvent({type:"create"})}getGraph(){return this.graph}getDefaults(){return Object.assign(super.getDefaults(),{name:"",extras:{}})}set(e,n){return Array.isArray(n)&&(n=n.slice()),super.set(e,n)}getName(){return this.get("name")}setName(e){return this.set("name",e)}getExtras(){return this.get("extras")}setExtras(e){return this.set("extras",e)}clone(){return new this.constructor(this.graph).copy(this,cn)}copy(e,n=cn){for(const s in this[Ce]){const r=this[Ce][s];if(r instanceof Ps)this[Ms].has(s)||r.dispose();else if(Js(r))for(const i of r)i.dispose();else if(Qs(r))for(const i in r)r[i].dispose()}for(const s in e[Ce]){const r=this[Ce][s],i=e[Ce][s];if(i instanceof Ps)this[Ms].has(s)?r.getChild().copy(n(i.getChild()),n):this.setRef(s,n(i.getChild()),i.getAttributes());else if(Js(i))for(const o of i)this.addRef(s,n(o.getChild()),o.getAttributes());else if(Qs(i))for(const o in i){const a=i[o];this.setRefMap(s,o,n(a.getChild()),a.getAttributes())}else this[Ce][s]=Zn(i)?JSON.parse(JSON.stringify(i)):Array.isArray(i)||i instanceof ArrayBuffer||ArrayBuffer.isView(i)?i.slice():i}return this}equals(e,n=Ix){if(this===e)return!0;if(this.propertyType!==e.propertyType)return!1;for(const s in this[Ce]){if(n.has(s))continue;const r=this[Ce][s],i=e[Ce][s];if(Va(r)||Va(i)){if(!Ex(r,i))return!1}else if(Js(r)||Js(i)){if(!Mx(r,i))return!1}else if(Qs(r)||Qs(i)){if(!Tx(r,i))return!1}else if(Zn(r)||Zn(i)){if(!Qh(r,i))return!1}else if(ji(r)||ji(i)){if(!Jh(r,i))return!1}else if(r!==i)return!1}return!0}detach(){return this.graph.disconnectParents(this,e=>e.propertyType!=="Root"),this}listParents(){return this.graph.listParents(this)}}class rt extends Rl{getDefaults(){return Object.assign(super.getDefaults(),{extensions:{}})}getExtension(e){return this.getRefMap("extensions",e)}setExtension(e,n){return n&&n.t(this),this.setRefMap("extensions",e,n)}listExtensions(){return this.listRefMapValues("extensions")}}class ue extends rt{constructor(...e){super(...e),this.i=ze.identity,this.o=ze.identity}init(){this.propertyType=se.ACCESSOR}getDefaults(){return Object.assign(super.getDefaults(),{array:null,type:ue.Type.SCALAR,componentType:ue.ComponentType.FLOAT,normalized:!1,sparse:!1,buffer:null})}copy(e,n=cn){return super.copy(e,n),this.i=e.i,this.o=e.o,this}static getElementSize(e){switch(e){case ue.Type.SCALAR:return 1;case ue.Type.VEC2:return 2;case ue.Type.VEC3:return 3;case ue.Type.VEC4:case ue.Type.MAT2:return 4;case ue.Type.MAT3:return 9;case ue.Type.MAT4:return 16;default:throw new Error("Unexpected type: "+e)}}static getComponentSize(e){switch(e){case ue.ComponentType.BYTE:case ue.ComponentType.UNSIGNED_BYTE:return 1;case ue.ComponentType.SHORT:case ue.ComponentType.UNSIGNED_SHORT:return 2;case ue.ComponentType.UNSIGNED_INT:case ue.ComponentType.FLOAT:return 4;default:throw new Error("Unexpected component type: "+e)}}getMinNormalized(e){const n=this.getElementSize();this.getMin(e);for(let s=0;s<n;s++)e[s]=this.o(e[s]);return e}getMin(e){const n=this.get("array"),s=this.getCount(),r=this.getElementSize();for(let i=0;i<r;i++)e[i]=1/0;for(let i=0;i<s*r;i+=r)for(let o=0;o<r;o++){const a=n[i+o];Number.isFinite(a)&&(e[o]=Math.min(e[o],a))}return e}getMaxNormalized(e){const n=this.getElementSize();this.getMax(e);for(let s=0;s<n;s++)e[s]=this.o(e[s]);return e}getMax(e){const n=this.get("array"),s=this.getCount(),r=this.getElementSize();for(let i=0;i<r;i++)e[i]=-1/0;for(let i=0;i<s*r;i+=r)for(let o=0;o<r;o++){const a=n[i+o];Number.isFinite(a)&&(e[o]=Math.max(e[o],a))}return e}getCount(){const e=this.get("array");return e?e.length/this.getElementSize():0}getType(){return this.get("type")}setType(e){return this.set("type",e)}getElementSize(){return ue.getElementSize(this.get("type"))}getComponentSize(){return this.get("array").BYTES_PER_ELEMENT}getComponentType(){return this.get("componentType")}getNormalized(){return this.get("normalized")}setNormalized(e){return this.set("normalized",e),e?(this.o=n=>ze.decodeNormalizedInt(n,this.get("componentType")),this.i=n=>ze.encodeNormalizedInt(n,this.get("componentType"))):(this.o=ze.identity,this.i=ze.identity),this}getScalar(e){const n=this.getElementSize();return this.o(this.get("array")[e*n])}setScalar(e,n){return this.get("array")[e*this.getElementSize()]=this.i(n),this}getElement(e,n){const s=this.getElementSize(),r=this.get("array");for(let i=0;i<s;i++)n[i]=this.o(r[e*s+i]);return n}setElement(e,n){const s=this.getElementSize(),r=this.get("array");for(let i=0;i<s;i++)r[e*s+i]=this.i(n[i]);return this}getSparse(){return this.get("sparse")}setSparse(e){return this.set("sparse",e)}getBuffer(){return this.getRef("buffer")}setBuffer(e){return this.setRef("buffer",e)}getArray(){return this.get("array")}setArray(e){return this.set("componentType",e?function(n){switch(n.constructor){case Float32Array:return ue.ComponentType.FLOAT;case Uint32Array:return ue.ComponentType.UNSIGNED_INT;case Uint16Array:return ue.ComponentType.UNSIGNED_SHORT;case Uint8Array:return ue.ComponentType.UNSIGNED_BYTE;case Int16Array:return ue.ComponentType.SHORT;case Int8Array:return ue.ComponentType.BYTE;default:throw new Error("Unknown accessor componentType.")}}(e):ue.ComponentType.FLOAT),this.set("array",e),this}getByteLength(){const e=this.get("array");return e?e.byteLength:0}}ue.Type={SCALAR:"SCALAR",VEC2:"VEC2",VEC3:"VEC3",VEC4:"VEC4",MAT2:"MAT2",MAT3:"MAT3",MAT4:"MAT4"},ue.ComponentType={BYTE:5120,UNSIGNED_BYTE:5121,SHORT:5122,UNSIGNED_SHORT:5123,UNSIGNED_INT:5125,FLOAT:5126};class em extends rt{init(){this.propertyType=se.ANIMATION}getDefaults(){return Object.assign(super.getDefaults(),{channels:[],samplers:[]})}addChannel(e){return this.addRef("channels",e)}removeChannel(e){return this.removeRef("channels",e)}listChannels(){return this.listRefs("channels")}addSampler(e){return this.addRef("samplers",e)}removeSampler(e){return this.removeRef("samplers",e)}listSamplers(){return this.listRefs("samplers")}}class Il extends rt{init(){this.propertyType=se.ANIMATION_CHANNEL}getDefaults(){return Object.assign(super.getDefaults(),{targetPath:null,targetNode:null,sampler:null})}getTargetPath(){return this.get("targetPath")}setTargetPath(e){return this.set("targetPath",e)}getTargetNode(){return this.getRef("targetNode")}setTargetNode(e){return this.setRef("targetNode",e)}getSampler(){return this.getRef("sampler")}setSampler(e){return this.setRef("sampler",e)}}Il.TargetPath={TRANSLATION:"translation",ROTATION:"rotation",SCALE:"scale",WEIGHTS:"weights"};class Or extends rt{init(){this.propertyType=se.ANIMATION_SAMPLER}getDefaultAttributes(){return Object.assign(super.getDefaults(),{interpolation:Or.Interpolation.LINEAR,input:null,output:null})}getInterpolation(){return this.get("interpolation")}setInterpolation(e){return this.set("interpolation",e)}getInput(){return this.getRef("input")}setInput(e){return this.setRef("input",e,{usage:bt.OTHER})}getOutput(){return this.getRef("output")}setOutput(e){return this.setRef("output",e,{usage:bt.OTHER})}}Or.Interpolation={LINEAR:"LINEAR",STEP:"STEP",CUBICSPLINE:"CUBICSPLINE"};class kl extends rt{init(){this.propertyType=se.BUFFER}getDefaults(){return Object.assign(super.getDefaults(),{uri:""})}getURI(){return this.get("uri")}setURI(e){return this.set("uri",e)}}class hs extends rt{init(){this.propertyType=se.CAMERA}getDefaults(){return Object.assign(super.getDefaults(),{type:hs.Type.PERSPECTIVE,znear:.1,zfar:100,aspectRatio:null,yfov:2*Math.PI*50/360,xmag:1,ymag:1})}getType(){return this.get("type")}setType(e){return this.set("type",e)}getZNear(){return this.get("znear")}setZNear(e){return this.set("znear",e)}getZFar(){return this.get("zfar")}setZFar(e){return this.set("zfar",e)}getAspectRatio(){return this.get("aspectRatio")}setAspectRatio(e){return this.set("aspectRatio",e)}getYFov(){return this.get("yfov")}setYFov(e){return this.set("yfov",e)}getXMag(){return this.get("xmag")}setXMag(e){return this.set("xmag",e)}getYMag(){return this.get("ymag")}setYMag(e){return this.set("ymag",e)}}hs.Type={PERSPECTIVE:"perspective",ORTHOGRAPHIC:"orthographic"};class tm extends Rl{t(e){if(!this.parentTypes.includes(e.propertyType))throw new Error(`Parent "${e.propertyType}" invalid for child "${this.propertyType}".`)}}tm.EXTENSION_NAME=void 0;let Rn=class Da extends rt{init(){this.propertyType=se.TEXTURE_INFO}getDefaults(){return Object.assign(super.getDefaults(),{texCoord:0,magFilter:null,minFilter:null,wrapS:Da.WrapMode.REPEAT,wrapT:Da.WrapMode.REPEAT})}getTexCoord(){return this.get("texCoord")}setTexCoord(e){return this.set("texCoord",e)}getMagFilter(){return this.get("magFilter")}setMagFilter(e){return this.set("magFilter",e)}getMinFilter(){return this.get("minFilter")}setMinFilter(e){return this.set("minFilter",e)}getWrapS(){return this.get("wrapS")}setWrapS(e){return this.set("wrapS",e)}getWrapT(){return this.get("wrapT")}setWrapT(e){return this.set("wrapT",e)}};Rn.WrapMode={CLAMP_TO_EDGE:33071,MIRRORED_REPEAT:33648,REPEAT:10497},Rn.MagFilter={NEAREST:9728,LINEAR:9729},Rn.MinFilter={NEAREST:9728,LINEAR:9729,NEAREST_MIPMAP_NEAREST:9984,LINEAR_MIPMAP_NEAREST:9985,NEAREST_MIPMAP_LINEAR:9986,LINEAR_MIPMAP_LINEAR:9987};const{R:ei,G:ti,B:ni,A:kx}=Pa;class rs extends rt{init(){this.propertyType=se.MATERIAL}getDefaults(){return Object.assign(super.getDefaults(),{alphaMode:rs.AlphaMode.OPAQUE,alphaCutoff:.5,doubleSided:!1,baseColorFactor:[1,1,1,1],baseColorTexture:null,baseColorTextureInfo:new Rn(this.graph,"baseColorTextureInfo"),emissiveFactor:[0,0,0],emissiveTexture:null,emissiveTextureInfo:new Rn(this.graph,"emissiveTextureInfo"),normalScale:1,normalTexture:null,normalTextureInfo:new Rn(this.graph,"normalTextureInfo"),occlusionStrength:1,occlusionTexture:null,occlusionTextureInfo:new Rn(this.graph,"occlusionTextureInfo"),roughnessFactor:1,metallicFactor:1,metallicRoughnessTexture:null,metallicRoughnessTextureInfo:new Rn(this.graph,"metallicRoughnessTextureInfo")})}getDoubleSided(){return this.get("doubleSided")}setDoubleSided(e){return this.set("doubleSided",e)}getAlpha(){return this.get("baseColorFactor")[3]}setAlpha(e){const n=this.get("baseColorFactor").slice();return n[3]=e,this.set("baseColorFactor",n)}getAlphaMode(){return this.get("alphaMode")}setAlphaMode(e){return this.set("alphaMode",e)}getAlphaCutoff(){return this.get("alphaCutoff")}setAlphaCutoff(e){return this.set("alphaCutoff",e)}getBaseColorFactor(){return this.get("baseColorFactor")}setBaseColorFactor(e){return this.set("baseColorFactor",e)}getBaseColorHex(){return Qr.factorToHex(this.get("baseColorFactor"))}setBaseColorHex(e){const n=this.get("baseColorFactor").slice();return this.set("baseColorFactor",Qr.hexToFactor(e,n))}getBaseColorTexture(){return this.getRef("baseColorTexture")}getBaseColorTextureInfo(){return this.getRef("baseColorTexture")?this.getRef("baseColorTextureInfo"):null}setBaseColorTexture(e){return this.setRef("baseColorTexture",e,{channels:ei|ti|ni|kx,isColor:!0})}getEmissiveFactor(){return this.get("emissiveFactor")}setEmissiveFactor(e){return this.set("emissiveFactor",e)}getEmissiveHex(){return Qr.factorToHex(this.get("emissiveFactor"))}setEmissiveHex(e){const n=this.get("emissiveFactor").slice();return this.set("emissiveFactor",Qr.hexToFactor(e,n))}getEmissiveTexture(){return this.getRef("emissiveTexture")}getEmissiveTextureInfo(){return this.getRef("emissiveTexture")?this.getRef("emissiveTextureInfo"):null}setEmissiveTexture(e){return this.setRef("emissiveTexture",e,{channels:ei|ti|ni,isColor:!0})}getNormalScale(){return this.get("normalScale")}setNormalScale(e){return this.set("normalScale",e)}getNormalTexture(){return this.getRef("normalTexture")}getNormalTextureInfo(){return this.getRef("normalTexture")?this.getRef("normalTextureInfo"):null}setNormalTexture(e){return this.setRef("normalTexture",e,{channels:ei|ti|ni})}getOcclusionStrength(){return this.get("occlusionStrength")}setOcclusionStrength(e){return this.set("occlusionStrength",e)}getOcclusionTexture(){return this.getRef("occlusionTexture")}getOcclusionTextureInfo(){return this.getRef("occlusionTexture")?this.getRef("occlusionTextureInfo"):null}setOcclusionTexture(e){return this.setRef("occlusionTexture",e,{channels:ei})}getRoughnessFactor(){return this.get("roughnessFactor")}setRoughnessFactor(e){return this.set("roughnessFactor",e)}getMetallicFactor(){return this.get("metallicFactor")}setMetallicFactor(e){return this.set("metallicFactor",e)}getMetallicRoughnessTexture(){return this.getRef("metallicRoughnessTexture")}getMetallicRoughnessTextureInfo(){return this.getRef("metallicRoughnessTexture")?this.getRef("metallicRoughnessTextureInfo"):null}setMetallicRoughnessTexture(e){return this.setRef("metallicRoughnessTexture",e,{channels:ti|ni})}}rs.AlphaMode={OPAQUE:"OPAQUE",MASK:"MASK",BLEND:"BLEND"};class nm extends rt{init(){this.propertyType=se.MESH}getDefaults(){return Object.assign(super.getDefaults(),{weights:[],primitives:[]})}addPrimitive(e){return this.addRef("primitives",e)}removePrimitive(e){return this.removeRef("primitives",e)}listPrimitives(){return this.listRefs("primitives")}getWeights(){return this.get("weights")}setWeights(e){return this.set("weights",e)}}class sm extends rt{constructor(...e){super(...e),this.u=null,this.h=new Set}init(){this.propertyType=se.NODE}getDefaults(){return Object.assign(super.getDefaults(),{translation:[0,0,0],rotation:[0,0,0,1],scale:[1,1,1],weights:[],camera:null,mesh:null,skin:null,children:[]})}copy(e,n=cn){if(n===cn)throw new Error("Node cannot be copied.");return super.copy(e,n)}getTranslation(){return this.get("translation")}getRotation(){return this.get("rotation")}getScale(){return this.get("scale")}setTranslation(e){return this.set("translation",e)}setRotation(e){return this.set("rotation",e)}setScale(e){return this.set("scale",e)}getMatrix(){return ze.compose(this.get("translation"),this.get("rotation"),this.get("scale"),[])}setMatrix(e){const n=this.get("translation").slice(),s=this.get("rotation").slice(),r=this.get("scale").slice();return ze.decompose(e,n,s,r),this.set("translation",n).set("rotation",s).set("scale",r)}getWorldTranslation(){const e=[0,0,0];return ze.decompose(this.getWorldMatrix(),e,[0,0,0,1],[1,1,1]),e}getWorldRotation(){const e=[0,0,0,1];return ze.decompose(this.getWorldMatrix(),[0,0,0],e,[1,1,1]),e}getWorldScale(){const e=[1,1,1];return ze.decompose(this.getWorldMatrix(),[0,0,0],[0,0,0,1],e),e}getWorldMatrix(){const e=[];for(let r=this;r!=null;r=r.u)e.push(r);let n;const s=e.pop().getMatrix();for(;n=e.pop();)Cx(s,s,n.getMatrix());return s}addChild(e){if(e.u&&e.u.removeChild(e),e.h.size)for(const s of e.h)s.removeChild(e);this.addRef("children",e),e.u=this;const n=this[Ce].children;return n[n.length-1].addEventListener("dispose",()=>e.u=null),this}removeChild(e){return this.removeRef("children",e)}listChildren(){return this.listRefs("children")}getParent(){return this.u?this.u:this.listParents().find(e=>e.propertyType===se.SCENE)||null}getParentNode(){return this.u}getMesh(){return this.getRef("mesh")}setMesh(e){return this.setRef("mesh",e)}getCamera(){return this.getRef("camera")}setCamera(e){return this.setRef("camera",e)}getSkin(){return this.getRef("skin")}setSkin(e){return this.setRef("skin",e)}getWeights(){return this.get("weights")}setWeights(e){return this.set("weights",e)}traverse(e){e(this);for(const n of this.listChildren())n.traverse(e);return this}}class So extends rt{init(){this.propertyType=se.PRIMITIVE}getDefaults(){return Object.assign(super.getDefaults(),{mode:So.Mode.TRIANGLES,material:null,indices:null,attributes:{},targets:[]})}getIndices(){return this.getRef("indices")}setIndices(e){return this.setRef("indices",e,{usage:bt.ELEMENT_ARRAY_BUFFER})}getAttribute(e){return this.getRefMap("attributes",e)}setAttribute(e,n){return this.setRefMap("attributes",e,n,{usage:bt.ARRAY_BUFFER})}listAttributes(){return this.listRefMapValues("attributes")}listSemantics(){return this.listRefMapKeys("attributes")}getMaterial(){return this.getRef("material")}setMaterial(e){return this.setRef("material",e)}getMode(){return this.get("mode")}setMode(e){return this.set("mode",e)}listTargets(){return this.listRefs("targets")}addTarget(e){return this.addRef("targets",e)}removeTarget(e){return this.removeRef("targets",e)}}So.Mode={POINTS:0,LINES:1,LINE_LOOP:2,LINE_STRIP:3,TRIANGLES:4,TRIANGLE_STRIP:5,TRIANGLE_FAN:6};class Fx extends Rl{init(){this.propertyType=se.PRIMITIVE_TARGET}getDefaults(){return Object.assign(super.getDefaults(),{attributes:{}})}getAttribute(e){return this.getRefMap("attributes",e)}setAttribute(e,n){return this.setRefMap("attributes",e,n,{usage:bt.ARRAY_BUFFER})}listAttributes(){return this.listRefMapValues("attributes")}listSemantics(){return this.listRefMapKeys("attributes")}}function nt(){return nt=Object.assign?Object.assign.bind():function(t){for(var e=1;e<arguments.length;e++){var n=arguments[e];for(var s in n)Object.prototype.hasOwnProperty.call(n,s)&&(t[s]=n[s])}return t},nt.apply(this,arguments)}class Fl extends rt{init(){this.propertyType=se.SCENE}getDefaults(){return Object.assign(super.getDefaults(),{children:[]})}copy(e,n=cn){if(n===cn)throw new Error("Scene cannot be copied.");return super.copy(e,n)}addChild(e){e.u&&e.u.removeChild(e),this.addRef("children",e),e.h.add(this);const n=this[Ce].children;return n[n.length-1].addEventListener("dispose",()=>e.h.delete(this)),this}removeChild(e){return this.removeRef("children",e)}listChildren(){return this.listRefs("children")}traverse(e){for(const n of this.listChildren())n.traverse(e);return this}}class rm extends rt{init(){this.propertyType=se.SKIN}getDefaults(){return Object.assign(super.getDefaults(),{skeleton:null,inverseBindMatrices:null,joints:[]})}getSkeleton(){return this.getRef("skeleton")}setSkeleton(e){return this.setRef("skeleton",e)}getInverseBindMatrices(){return this.getRef("inverseBindMatrices")}setInverseBindMatrices(e){return this.setRef("inverseBindMatrices",e,{usage:bt.INVERSE_BIND_MATRICES})}addJoint(e){return this.addRef("joints",e)}removeJoint(e){return this.removeRef("joints",e)}listJoints(){return this.listRefs("joints")}}class im extends rt{init(){this.propertyType=se.TEXTURE}getDefaults(){return Object.assign(super.getDefaults(),{image:null,mimeType:"",uri:""})}getMimeType(){return this.get("mimeType")||ns.extensionToMimeType(Os.extension(this.get("uri")))}setMimeType(e){return this.set("mimeType",e)}getURI(){return this.get("uri")}setURI(e){this.set("uri",e);const n=ns.extensionToMimeType(Os.extension(e));return n&&this.set("mimeType",n),this}getImage(){return this.get("image")}setImage(e){return this.set("image",Se.assertView(e))}getSize(){const e=this.get("image");return e?ns.getSize(e,this.getMimeType()):null}}class Lx extends rt{init(){this.propertyType=se.ROOT}getDefaults(){return Object.assign(super.getDefaults(),{asset:{generator:`glTF-Transform ${Kh}`,version:"2.0"},defaultScene:null,accessors:[],animations:[],buffers:[],cameras:[],materials:[],meshes:[],nodes:[],scenes:[],skins:[],textures:[]})}constructor(e){super(e),this.l=new Set,e.addEventListener("node:create",n=>{this.g(n.target)})}clone(){throw new Error("Root cannot be cloned.")}copy(e,n=cn){if(n===cn)throw new Error("Root cannot be copied.");this.set("asset",nt({},e.get("asset"))),this.setName(e.getName()),this.setExtras(nt({},e.getExtras())),this.setDefaultScene(e.getDefaultScene()?n(e.getDefaultScene()):null);for(const s of e.listRefMapKeys("extensions")){const r=e.getExtension(s);this.setExtension(s,n(r))}return this}g(e){return e instanceof Fl?this.addRef("scenes",e):e instanceof sm?this.addRef("nodes",e):e instanceof hs?this.addRef("cameras",e):e instanceof rm?this.addRef("skins",e):e instanceof nm?this.addRef("meshes",e):e instanceof rs?this.addRef("materials",e):e instanceof im?this.addRef("textures",e):e instanceof em?this.addRef("animations",e):e instanceof ue?this.addRef("accessors",e):e instanceof kl&&this.addRef("buffers",e),this}getAsset(){return this.get("asset")}listExtensionsUsed(){return Array.from(this.l)}listExtensionsRequired(){return this.listExtensionsUsed().filter(e=>e.isRequired())}p(e){return this.l.add(e),this}m(e){return this.l.delete(e),this}listScenes(){return this.listRefs("scenes")}setDefaultScene(e){return this.setRef("defaultScene",e)}getDefaultScene(){return this.getRef("defaultScene")}listNodes(){return this.listRefs("nodes")}listCameras(){return this.listRefs("cameras")}listSkins(){return this.listRefs("skins")}listMeshes(){return this.listRefs("meshes")}listMaterials(){return this.listRefs("materials")}listTextures(){return this.listRefs("textures")}listAnimations(){return this.listRefs("animations")}listAccessors(){return this.listRefs("accessors")}listBuffers(){return this.listRefs("buffers")}}class Vn{static fromGraph(e){return Vn.v.get(e)||null}constructor(){this.T=new yx,this.S=new Lx(this.T),this.M=Et.DEFAULT_INSTANCE,Vn.v.set(this.T,this)}getRoot(){return this.S}getGraph(){return this.T}getLogger(){return this.M}setLogger(e){return this.M=e,this}clone(){return new Vn().setLogger(this.M).merge(this)}merge(e){for(const i of e.getRoot().listExtensionsUsed()){const o=this.createExtension(i.constructor);i.isRequired()&&o.setRequired(!0)}const n=new Set,s=new Map;n.add(e.S),s.set(e.S,this.S);for(const i of e.T.listEdges())for(const o of[i.getParent(),i.getChild()]){if(n.has(o))continue;let a;a=o.propertyType===se.TEXTURE_INFO?o:new o.constructor(this.T),s.set(o,a),n.add(o)}const r=i=>{const o=s.get(i);if(!o)throw new Error("Could resolve property.");return o};for(const i of n){const o=s.get(i);if(!o)throw new Error("Could resolve property.");o.propertyType!==se.TEXTURE_INFO&&o.copy(i,r)}return this}async transform(...e){const n=e.map(s=>s.name);for(const s of e)await s(this,{stack:n});return this}createExtension(e){const n=e.EXTENSION_NAME;return this.getRoot().listExtensionsUsed().find(s=>s.extensionName===n)||new e(this)}createScene(e=""){return new Fl(this.T,e)}createNode(e=""){return new sm(this.T,e)}createCamera(e=""){return new hs(this.T,e)}createSkin(e=""){return new rm(this.T,e)}createMesh(e=""){return new nm(this.T,e)}createPrimitive(){return new So(this.T)}createPrimitiveTarget(e=""){return new Fx(this.T,e)}createMaterial(e=""){return new rs(this.T,e)}createTexture(e=""){return new im(this.T,e)}createAnimation(e=""){return new em(this.T,e)}createAnimationChannel(e=""){return new Il(this.T,e)}createAnimationSampler(e=""){return new Or(this.T,e)}createAccessor(e="",n=null){return n||(n=this.getRoot().listBuffers()[0]),new ue(this.T,e).setBuffer(n)}createBuffer(e=""){return new kl(this.T,e)}}Vn.v=new WeakMap;class Vx{constructor(e){this.extensionName="",this.prereadTypes=[],this.prewriteTypes=[],this.readDependencies=[],this.writeDependencies=[],this.document=void 0,this.required=!1,this.properties=new Set,this.I=void 0,this.document=e,e.getRoot().p(this),this.I=s=>{const r=s,i=r.target;i instanceof tm&&i.extensionName===this.extensionName&&(r.type==="node:create"&&this.N(i),r.type==="node:dispose"&&this.O(i))};const n=e.getGraph();n.addEventListener("node:create",this.I),n.addEventListener("node:dispose",this.I)}dispose(){this.document.getRoot().m(this);const e=this.document.getGraph();e.removeEventListener("node:create",this.I),e.removeEventListener("node:dispose",this.I);for(const n of this.properties)n.dispose()}static register(){}isRequired(){return this.required}setRequired(e){return this.required=e,this}listProperties(){return Array.from(this.properties)}N(e){return this.properties.add(e),this}O(e){return this.properties.delete(e),this}install(e,n){return this}preread(e,n){return this}prewrite(e,n){return this}}Vx.EXTENSION_NAME=void 0;class Px{constructor(e){this.jsonDoc=void 0,this.buffers=[],this.bufferViews=[],this.bufferViewBuffers=[],this.accessors=[],this.textures=[],this.textureInfos=new Map,this.materials=[],this.meshes=[],this.cameras=[],this.nodes=[],this.skins=[],this.animations=[],this.scenes=[],this.jsonDoc=e}setTextureInfo(e,n){this.textureInfos.set(e,n),n.texCoord!==void 0&&e.setTexCoord(n.texCoord),n.extras!==void 0&&e.setExtras(n.extras);const s=this.jsonDoc.json.textures[n.index];if(s.sampler===void 0)return;const r=this.jsonDoc.json.samplers[s.sampler];r.magFilter!==void 0&&e.setMagFilter(r.magFilter),r.minFilter!==void 0&&e.setMinFilter(r.minFilter),r.wrapS!==void 0&&e.setWrapS(r.wrapS),r.wrapT!==void 0&&e.setWrapT(r.wrapT)}}const $c={logger:Et.DEFAULT_INSTANCE,extensions:[],dependencies:{}};class Ox{static read(e,n=$c){const s=nt({},$c,n),{json:r}=e,i=new Vn().setLogger(s.logger);this.validate(e,s);const o=new Px(e),a=r.asset,l=i.getRoot().getAsset();a.copyright&&(l.copyright=a.copyright),a.extras&&(l.extras=a.extras),r.extras!==void 0&&i.getRoot().setExtras(nt({},r.extras));const u=r.extensionsUsed||[],c=r.extensionsRequired||[];for(const g of s.extensions)if(u.includes(g.EXTENSION_NAME)){const y=i.createExtension(g).setRequired(c.includes(g.EXTENSION_NAME));for(const b of y.readDependencies)y.install(b,s.dependencies[b])}const d=r.buffers||[];i.getRoot().listExtensionsUsed().filter(g=>g.prereadTypes.includes(se.BUFFER)).forEach(g=>g.preread(o,se.BUFFER)),o.buffers=d.map(g=>{const y=i.createBuffer(g.name);return g.extras&&y.setExtras(g.extras),g.uri&&g.uri.indexOf("__")!==0&&y.setURI(g.uri),y}),o.bufferViewBuffers=(r.bufferViews||[]).map((g,y)=>{if(!o.bufferViews[y]){const b=e.json.buffers[g.buffer];o.bufferViews[y]=Se.toView(b.uri?e.resources[b.uri]:e.resources[Hi],g.byteOffset||0,g.byteLength)}return o.buffers[g.buffer]});const f=r.accessors||[];o.accessors=f.map(g=>{const y=i.createAccessor(g.name,o.bufferViewBuffers[g.bufferView]).setType(g.type);return g.extras&&y.setExtras(g.extras),g.normalized!==void 0&&y.setNormalized(g.normalized),g.bufferView===void 0||y.setArray(si(g,o)),y});const h=r.images||[],m=r.textures||[];i.getRoot().listExtensionsUsed().filter(g=>g.prereadTypes.includes(se.TEXTURE)).forEach(g=>g.preread(o,se.TEXTURE)),o.textures=h.map(g=>{const y=i.createTexture(g.name);if(g.extras&&y.setExtras(g.extras),g.bufferView!==void 0){const b=r.bufferViews[g.bufferView],x=e.json.buffers[b.buffer],_=b.byteOffset||0,C=(x.uri?e.resources[x.uri]:e.resources[Hi]).slice(_,_+b.byteLength);y.setImage(C)}else g.uri!==void 0&&(y.setImage(e.resources[g.uri]),g.uri.indexOf("__")!==0&&y.setURI(g.uri));if(g.mimeType!==void 0)y.setMimeType(g.mimeType);else if(g.uri){const b=Os.extension(g.uri);y.setMimeType(ns.extensionToMimeType(b))}return y}),o.materials=(r.materials||[]).map(g=>{const y=i.createMaterial(g.name);g.extras&&y.setExtras(g.extras),g.alphaMode!==void 0&&y.setAlphaMode(g.alphaMode),g.alphaCutoff!==void 0&&y.setAlphaCutoff(g.alphaCutoff),g.doubleSided!==void 0&&y.setDoubleSided(g.doubleSided);const b=g.pbrMetallicRoughness||{};if(b.baseColorFactor!==void 0&&y.setBaseColorFactor(b.baseColorFactor),g.emissiveFactor!==void 0&&y.setEmissiveFactor(g.emissiveFactor),b.metallicFactor!==void 0&&y.setMetallicFactor(b.metallicFactor),b.roughnessFactor!==void 0&&y.setRoughnessFactor(b.roughnessFactor),b.baseColorTexture!==void 0){const x=b.baseColorTexture;y.setBaseColorTexture(o.textures[m[x.index].source]),o.setTextureInfo(y.getBaseColorTextureInfo(),x)}if(g.emissiveTexture!==void 0){const x=g.emissiveTexture;y.setEmissiveTexture(o.textures[m[x.index].source]),o.setTextureInfo(y.getEmissiveTextureInfo(),x)}if(g.normalTexture!==void 0){const x=g.normalTexture;y.setNormalTexture(o.textures[m[x.index].source]),o.setTextureInfo(y.getNormalTextureInfo(),x),g.normalTexture.scale!==void 0&&y.setNormalScale(g.normalTexture.scale)}if(g.occlusionTexture!==void 0){const x=g.occlusionTexture;y.setOcclusionTexture(o.textures[m[x.index].source]),o.setTextureInfo(y.getOcclusionTextureInfo(),x),g.occlusionTexture.strength!==void 0&&y.setOcclusionStrength(g.occlusionTexture.strength)}if(b.metallicRoughnessTexture!==void 0){const x=b.metallicRoughnessTexture;y.setMetallicRoughnessTexture(o.textures[m[x.index].source]),o.setTextureInfo(y.getMetallicRoughnessTextureInfo(),x)}return y});const p=r.meshes||[];i.getRoot().listExtensionsUsed().filter(g=>g.prereadTypes.includes(se.PRIMITIVE)).forEach(g=>g.preread(o,se.PRIMITIVE)),o.meshes=p.map(g=>{const y=i.createMesh(g.name);return g.extras&&y.setExtras(g.extras),g.weights!==void 0&&y.setWeights(g.weights),(g.primitives||[]).forEach(b=>{const x=i.createPrimitive();b.extras&&x.setExtras(b.extras),b.material!==void 0&&x.setMaterial(o.materials[b.material]),b.mode!==void 0&&x.setMode(b.mode);for(const[C,M]of Object.entries(b.attributes||{}))x.setAttribute(C,o.accessors[M]);b.indices!==void 0&&x.setIndices(o.accessors[b.indices]);const _=g.extras&&g.extras.targetNames||[];(b.targets||[]).forEach((C,M)=>{const R=_[M]||M.toString(),F=i.createPrimitiveTarget(R);for(const[V,T]of Object.entries(C))F.setAttribute(V,o.accessors[T]);x.addTarget(F)}),y.addPrimitive(x)}),y}),o.cameras=(r.cameras||[]).map(g=>{const y=i.createCamera(g.name).setType(g.type);if(g.extras&&y.setExtras(g.extras),g.type===hs.Type.PERSPECTIVE){const b=g.perspective;y.setYFov(b.yfov),y.setZNear(b.znear),b.zfar!==void 0&&y.setZFar(b.zfar),b.aspectRatio!==void 0&&y.setAspectRatio(b.aspectRatio)}else{const b=g.orthographic;y.setZNear(b.znear).setZFar(b.zfar).setXMag(b.xmag).setYMag(b.ymag)}return y});const v=r.nodes||[];i.getRoot().listExtensionsUsed().filter(g=>g.prereadTypes.includes(se.NODE)).forEach(g=>g.preread(o,se.NODE)),o.nodes=v.map(g=>{const y=i.createNode(g.name);if(g.extras&&y.setExtras(g.extras),g.translation!==void 0&&y.setTranslation(g.translation),g.rotation!==void 0&&y.setRotation(g.rotation),g.scale!==void 0&&y.setScale(g.scale),g.matrix!==void 0){const b=[0,0,0],x=[0,0,0,1],_=[1,1,1];ze.decompose(g.matrix,b,x,_),y.setTranslation(b),y.setRotation(x),y.setScale(_)}return g.weights!==void 0&&y.setWeights(g.weights),y}),o.skins=(r.skins||[]).map(g=>{const y=i.createSkin(g.name);g.extras&&y.setExtras(g.extras),g.inverseBindMatrices!==void 0&&y.setInverseBindMatrices(o.accessors[g.inverseBindMatrices]),g.skeleton!==void 0&&y.setSkeleton(o.nodes[g.skeleton]);for(const b of g.joints)y.addJoint(o.nodes[b]);return y}),v.map((g,y)=>{const b=o.nodes[y];(g.children||[]).forEach(x=>b.addChild(o.nodes[x])),g.mesh!==void 0&&b.setMesh(o.meshes[g.mesh]),g.camera!==void 0&&b.setCamera(o.cameras[g.camera]),g.skin!==void 0&&b.setSkin(o.skins[g.skin])}),o.animations=(r.animations||[]).map(g=>{const y=i.createAnimation(g.name);g.extras&&y.setExtras(g.extras);const b=(g.samplers||[]).map(x=>{const _=i.createAnimationSampler().setInput(o.accessors[x.input]).setOutput(o.accessors[x.output]).setInterpolation(x.interpolation||Or.Interpolation.LINEAR);return x.extras&&_.setExtras(x.extras),y.addSampler(_),_});return(g.channels||[]).forEach(x=>{const _=i.createAnimationChannel().setSampler(b[x.sampler]).setTargetPath(x.target.path);x.target.node!==void 0&&_.setTargetNode(o.nodes[x.target.node]),x.extras&&_.setExtras(x.extras),y.addChannel(_)}),y});const S=r.scenes||[];return i.getRoot().listExtensionsUsed().filter(g=>g.prereadTypes.includes(se.SCENE)).forEach(g=>g.preread(o,se.SCENE)),o.scenes=S.map(g=>{const y=i.createScene(g.name);return g.extras&&y.setExtras(g.extras),(g.nodes||[]).map(b=>o.nodes[b]).forEach(b=>y.addChild(b)),y}),r.scene!==void 0&&i.getRoot().setDefaultScene(o.scenes[r.scene]),i.getRoot().listExtensionsUsed().forEach(g=>g.read(o)),f.forEach((g,y)=>{const b=o.accessors[y],x=!!g.sparse,_=!g.bufferView&&!b.getArray();(x||_)&&b.setSparse(!0).setArray(function(C,M){const R=Ui[C.componentType],F=ue.getElementSize(C.type);let V;V=C.bufferView!==void 0?si(C,M):new R(C.count*F);const T=C.sparse;if(!T)return V;const L=T.count,A=nt({},C,T.indices,{count:L,type:"SCALAR"}),N=nt({},C,T.values,{count:L}),$=si(A,M),z=si(N,M);for(let X=0;X<A.count;X++)for(let re=0;re<F;re++)V[$[X]*F+re]=z[X*F+re];return V}(g,o))}),i}static validate(e,n){const s=e.json;if(s.asset.version!=="2.0")throw new Error(`Unsupported glTF version, "${s.asset.version}".`);if(s.extensionsRequired){for(const r of s.extensionsRequired)if(!n.extensions.find(i=>i.EXTENSION_NAME===r))throw new Error(`Missing required extension, "${r}".`)}if(s.extensionsUsed)for(const r of s.extensionsUsed)n.extensions.find(i=>i.EXTENSION_NAME===r)||n.logger.warn(`Missing optional extension, "${r}".`)}}function si(t,e){const n=e.bufferViews[t.bufferView],s=e.jsonDoc.json.bufferViews[t.bufferView],r=Ui[t.componentType],i=ue.getElementSize(t.type),o=r.BYTES_PER_ELEMENT;if(s.byteStride!==void 0&&s.byteStride!==i*o)return function(l,u){const c=u.bufferViews[l.bufferView],d=u.jsonDoc.json.bufferViews[l.bufferView],f=Ui[l.componentType],h=ue.getElementSize(l.type),m=f.BYTES_PER_ELEMENT,p=l.byteOffset||0,v=new f(l.count*h),S=new DataView(c.buffer,c.byteOffset,c.byteLength),g=d.byteStride;for(let y=0;y<l.count;y++)for(let b=0;b<h;b++){const x=p+y*g+b*m;let _;switch(l.componentType){case ue.ComponentType.FLOAT:_=S.getFloat32(x,!0);break;case ue.ComponentType.UNSIGNED_INT:_=S.getUint32(x,!0);break;case ue.ComponentType.UNSIGNED_SHORT:_=S.getUint16(x,!0);break;case ue.ComponentType.UNSIGNED_BYTE:_=S.getUint8(x);break;case ue.ComponentType.SHORT:_=S.getInt16(x,!0);break;case ue.ComponentType.BYTE:_=S.getInt8(x);break;default:throw new Error(`Unexpected componentType "${l.componentType}".`)}v[y*h+b]=_}return v}(t,e);const a=n.byteOffset+(t.byteOffset||0);return new r(n.buffer.slice(a,a+t.count*i*o))}var cr;(function(t){t[t.ARRAY_BUFFER=34962]="ARRAY_BUFFER",t[t.ELEMENT_ARRAY_BUFFER=34963]="ELEMENT_ARRAY_BUFFER"})(cr||(cr={}));class ss{constructor(e,n,s){this.C=void 0,this.jsonDoc=void 0,this.options=void 0,this.accessorIndexMap=new Map,this.animationIndexMap=new Map,this.bufferIndexMap=new Map,this.cameraIndexMap=new Map,this.skinIndexMap=new Map,this.materialIndexMap=new Map,this.meshIndexMap=new Map,this.nodeIndexMap=new Map,this.imageIndexMap=new Map,this.textureDefIndexMap=new Map,this.textureInfoDefMap=new Map,this.samplerDefIndexMap=new Map,this.sceneIndexMap=new Map,this.imageBufferViews=[],this.otherBufferViews=new Map,this.otherBufferViewsIndexMap=new Map,this.extensionData={},this.bufferURIGenerator=void 0,this.imageURIGenerator=void 0,this.logger=void 0,this.F=new Map,this.accessorUsageGroupedByParent=new Set(["ARRAY_BUFFER"]),this.accessorParents=new Map,this.C=e,this.jsonDoc=n,this.options=s;const r=e.getRoot(),i=r.listBuffers().length,o=r.listTextures().length;this.bufferURIGenerator=new Uc(i>1,()=>s.basename||"buffer"),this.imageURIGenerator=new Uc(o>1,a=>function(l,u){const c=l.getGraph().listParentEdges(u).find(d=>d.getParent()!==l.getRoot());return c?c.getName().replace(/texture$/i,""):""}(e,a)||s.basename||"texture"),this.logger=e.getLogger()}createTextureInfoDef(e,n){const s={magFilter:n.getMagFilter()||void 0,minFilter:n.getMinFilter()||void 0,wrapS:n.getWrapS(),wrapT:n.getWrapT()},r=JSON.stringify(s);this.samplerDefIndexMap.has(r)||(this.samplerDefIndexMap.set(r,this.jsonDoc.json.samplers.length),this.jsonDoc.json.samplers.push(s));const i={source:this.imageIndexMap.get(e),sampler:this.samplerDefIndexMap.get(r)},o=JSON.stringify(i);this.textureDefIndexMap.has(o)||(this.textureDefIndexMap.set(o,this.jsonDoc.json.textures.length),this.jsonDoc.json.textures.push(i));const a={index:this.textureDefIndexMap.get(o)};return n.getTexCoord()!==0&&(a.texCoord=n.getTexCoord()),Object.keys(n.getExtras()).length>0&&(a.extras=n.getExtras()),this.textureInfoDefMap.set(n,a),a}createPropertyDef(e){const n={};return e.getName()&&(n.name=e.getName()),Object.keys(e.getExtras()).length>0&&(n.extras=e.getExtras()),n}createAccessorDef(e){const n=this.createPropertyDef(e);return n.type=e.getType(),n.componentType=e.getComponentType(),n.count=e.getCount(),this.C.getGraph().listParentEdges(e).some(s=>s.getName()==="attributes"&&s.getAttributes().key==="POSITION"||s.getName()==="input")&&(n.max=e.getMax([]).map(Math.fround),n.min=e.getMin([]).map(Math.fround)),e.getNormalized()&&(n.normalized=e.getNormalized()),n}createImageData(e,n,s){if(this.options.format===ts.GLB)this.imageBufferViews.push(n),e.bufferView=this.jsonDoc.json.bufferViews.length,this.jsonDoc.json.bufferViews.push({buffer:0,byteOffset:-1,byteLength:n.byteLength});else{const r=ns.mimeTypeToExtension(s.getMimeType());e.uri=this.imageURIGenerator.createURI(s,r),this.jsonDoc.resources[e.uri]=n}}getAccessorUsage(e){const n=this.F.get(e);if(n)return n;if(e.getSparse())return bt.SPARSE;for(const s of this.C.getGraph().listParentEdges(e)){const{usage:r}=s.getAttributes();if(r)return r;s.getParent().propertyType!==se.ROOT&&this.logger.warn(`Missing attribute ".usage" on edge, "${s.getName()}".`)}return bt.OTHER}addAccessorToUsageGroup(e,n){const s=this.F.get(e);if(s&&s!==n)throw new Error(`Accessor with usage "${s}" cannot be reused as "${n}".`);return this.F.set(e,n),this}listAccessorUsageGroups(){const e={};for(const[n,s]of Array.from(this.F.entries()))e[s]=e[s]||[],e[s].push(n);return e}}ss.BufferViewTarget=cr,ss.BufferViewUsage=bt,ss.USAGE_TO_TARGET={[bt.ARRAY_BUFFER]:cr.ARRAY_BUFFER,[bt.ELEMENT_ARRAY_BUFFER]:cr.ELEMENT_ARRAY_BUFFER};class Uc{constructor(e,n){this.multiple=void 0,this.basename=void 0,this.counter={},this.multiple=e,this.basename=n}createURI(e,n){if(e.getURI())return e.getURI();if(this.multiple){const s=this.basename(e);return this.counter[s]=this.counter[s]||1,`${s}_${this.counter[s]++}.${n}`}return`${this.basename(e)}.${n}`}}const{BufferViewUsage:ri}=ss,{UNSIGNED_INT:Nx,UNSIGNED_SHORT:Bx,UNSIGNED_BYTE:Dx}=ue.ComponentType;class zx{static write(e,n){const s=e.getRoot(),r={asset:nt({generator:`glTF-Transform ${Kh}`},s.getAsset()),extras:nt({},s.getExtras())},i={json:r,resources:{}},o=new ss(e,i,n),a=n.logger||Et.DEFAULT_INSTANCE,l=new Set(n.extensions.map(v=>v.EXTENSION_NAME)),u=e.getRoot().listExtensionsUsed().filter(v=>l.has(v.extensionName)),c=e.getRoot().listExtensionsRequired().filter(v=>l.has(v.extensionName));u.length<e.getRoot().listExtensionsUsed().length&&a.warn("Some extensions were not registered for I/O, and will not be written.");for(const v of u)for(const S of v.writeDependencies)v.install(S,n.dependencies[S]);function d(v,S,g,y){const b=[];let x=0;for(const C of v){const M=o.createAccessorDef(C);M.bufferView=r.bufferViews.length;const R=C.getArray(),F=Se.pad(Se.toView(R));M.byteOffset=x,x+=F.byteLength,b.push(F),o.accessorIndexMap.set(C,r.accessors.length),r.accessors.push(M)}const _={buffer:S,byteOffset:g,byteLength:Se.concat(b).byteLength};return y&&(_.target=y),r.bufferViews.push(_),{buffers:b,byteLength:x}}function f(v,S,g){const y=v[0].getCount();let b=0;for(const M of v){const R=o.createAccessorDef(M);R.bufferView=r.bufferViews.length,R.byteOffset=b;const F=M.getElementSize(),V=M.getComponentSize();b+=Se.padNumber(F*V),o.accessorIndexMap.set(M,r.accessors.length),r.accessors.push(R)}const x=y*b,_=new ArrayBuffer(x),C=new DataView(_);for(let M=0;M<y;M++){let R=0;for(const F of v){const V=F.getElementSize(),T=F.getComponentSize(),L=F.getComponentType(),A=F.getArray();for(let N=0;N<V;N++){const $=M*b+R+N*T,z=A[M*V+N];switch(L){case ue.ComponentType.FLOAT:C.setFloat32($,z,!0);break;case ue.ComponentType.BYTE:C.setInt8($,z);break;case ue.ComponentType.SHORT:C.setInt16($,z,!0);break;case ue.ComponentType.UNSIGNED_BYTE:C.setUint8($,z);break;case ue.ComponentType.UNSIGNED_SHORT:C.setUint16($,z,!0);break;case ue.ComponentType.UNSIGNED_INT:C.setUint32($,z,!0);break;default:throw new Error("Unexpected component type: "+L)}}R+=Se.padNumber(V*T)}}return r.bufferViews.push({buffer:S,byteOffset:g,byteLength:x,byteStride:b,target:ss.BufferViewTarget.ARRAY_BUFFER}),{byteLength:x,buffers:[new Uint8Array(_)]}}function h(v,S,g){const y=[];let b=0;const x=new Map;let _=-1/0;for(const L of v){const A=o.createAccessorDef(L);r.accessors.push(A),o.accessorIndexMap.set(L,r.accessors.length-1);const N=[],$=[],z=[],X=new Array(L.getElementSize()).fill(0);for(let ce=0,Te=L.getCount();ce<Te;ce++)if(L.getElement(ce,z),!ze.eq(z,X,0)){_=Math.max(ce,_),N.push(ce);for(let ge=0;ge<z.length;ge++)$.push(z[ge])}const re=N.length,te={accessorDef:A,count:re};if(x.set(L,te),re===0)continue;if(re>L.getCount()/3){const ce=(100*N.length/L.getCount()).toFixed(1);a.warn(`Sparse accessor with many non-zero elements (${ce}%) may increase file size.`)}const Q=Ui[L.getComponentType()];te.indices=N,te.values=new Q($)}if(!Number.isFinite(_))return{buffers:y,byteLength:b};const C=_<255?Uint8Array:_<65535?Uint16Array:Uint32Array,M=_<255?Dx:_<65535?Bx:Nx,R={buffer:S,byteOffset:g+b,byteLength:0};for(const L of v){const A=x.get(L);if(A.count===0)continue;A.indicesByteOffset=R.byteLength;const N=Se.pad(Se.toView(new C(A.indices)));y.push(N),b+=N.byteLength,R.byteLength+=N.byteLength}r.bufferViews.push(R);const F=r.bufferViews.length-1,V={buffer:S,byteOffset:g+b,byteLength:0};for(const L of v){const A=x.get(L);if(A.count===0)continue;A.valuesByteOffset=V.byteLength;const N=Se.pad(Se.toView(A.values));y.push(N),b+=N.byteLength,V.byteLength+=N.byteLength}r.bufferViews.push(V);const T=r.bufferViews.length-1;for(const L of v){const A=x.get(L);A.count!==0&&(A.accessorDef.sparse={count:A.count,indices:{bufferView:F,byteOffset:A.indicesByteOffset,componentType:M},values:{bufferView:T,byteOffset:A.valuesByteOffset}})}return{buffers:y,byteLength:b}}const m=new Map;for(const v of e.getGraph().listEdges()){if(v.getParent()===s)continue;const S=v.getChild();if(S instanceof ue){const g=m.get(S)||[];g.push(v),m.set(S,g)}}if(r.accessors=[],r.bufferViews=[],r.samplers=[],r.textures=[],r.images=s.listTextures().map((v,S)=>{const g=o.createPropertyDef(v);v.getMimeType()&&(g.mimeType=v.getMimeType());const y=v.getImage();return y&&o.createImageData(g,y,v),o.imageIndexMap.set(v,S),g}),u.filter(v=>v.prewriteTypes.includes(se.ACCESSOR)).forEach(v=>v.prewrite(o,se.ACCESSOR)),s.listAccessors().forEach(v=>{const S=o.accessorUsageGroupedByParent,g=o.accessorParents;if(o.accessorIndexMap.has(v))return;const y=m.get(v)||[],b=o.getAccessorUsage(v);if(o.addAccessorToUsageGroup(v,b),S.has(b)){const x=y[0].getParent(),_=g.get(x)||new Set;_.add(v),g.set(x,_)}}),u.filter(v=>v.prewriteTypes.includes(se.BUFFER)).forEach(v=>v.prewrite(o,se.BUFFER)),(s.listAccessors().length>0||s.listTextures().length>0||o.otherBufferViews.size>0)&&s.listBuffers().length===0)throw new Error("Buffer required for Document resources, but none was found.");r.buffers=[],s.listBuffers().forEach((v,S)=>{const g=o.createPropertyDef(v),y=o.accessorUsageGroupedByParent,b=o.accessorParents,x=v.listParents().filter(V=>V instanceof ue),_=new Set(x),C=[],M=r.buffers.length;let R=0;const F=o.listAccessorUsageGroups();for(const V in F)if(y.has(V))for(const T of Array.from(b.values())){const L=Array.from(T).filter(A=>_.has(A)).filter(A=>o.getAccessorUsage(A)===V);if(L.length)if(V!==ri.ARRAY_BUFFER||n.vertexLayout===$i.INTERLEAVED){const A=V===ri.ARRAY_BUFFER?f(L,M,R):d(L,M,R);R+=A.byteLength,C.push(...A.buffers)}else for(const A of L){const N=f([A],M,R);R+=N.byteLength,C.push(...N.buffers)}}else{const T=F[V].filter(N=>_.has(N));if(!T.length)continue;const L=V===ri.ELEMENT_ARRAY_BUFFER?ss.BufferViewTarget.ELEMENT_ARRAY_BUFFER:void 0,A=V===ri.SPARSE?h(T,M,R):d(T,M,R,L);R+=A.byteLength,C.push(...A.buffers)}if(o.imageBufferViews.length&&S===0){for(let V=0;V<o.imageBufferViews.length;V++)if(r.bufferViews[r.images[V].bufferView].byteOffset=R,R+=o.imageBufferViews[V].byteLength,C.push(o.imageBufferViews[V]),R%8){const T=8-R%8;R+=T,C.push(new Uint8Array(T))}}if(o.otherBufferViews.has(v))for(const V of o.otherBufferViews.get(v))r.bufferViews.push({buffer:M,byteOffset:R,byteLength:V.byteLength}),o.otherBufferViewsIndexMap.set(V,r.bufferViews.length-1),R+=V.byteLength,C.push(V);if(R){let V;n.format===ts.GLB?V=Hi:(V=o.bufferURIGenerator.createURI(v,"bin"),g.uri=V),g.byteLength=R,i.resources[V]=Se.concat(C)}r.buffers.push(g),o.bufferIndexMap.set(v,S)}),s.listAccessors().find(v=>!v.getBuffer())&&a.warn("Skipped writing one or more Accessors: no Buffer assigned."),r.materials=s.listMaterials().map((v,S)=>{const g=o.createPropertyDef(v);if(v.getAlphaMode()!==rs.AlphaMode.OPAQUE&&(g.alphaMode=v.getAlphaMode()),v.getAlphaMode()===rs.AlphaMode.MASK&&(g.alphaCutoff=v.getAlphaCutoff()),v.getDoubleSided()&&(g.doubleSided=!0),g.pbrMetallicRoughness={},ze.eq(v.getBaseColorFactor(),[1,1,1,1])||(g.pbrMetallicRoughness.baseColorFactor=v.getBaseColorFactor()),ze.eq(v.getEmissiveFactor(),[0,0,0])||(g.emissiveFactor=v.getEmissiveFactor()),v.getRoughnessFactor()!==1&&(g.pbrMetallicRoughness.roughnessFactor=v.getRoughnessFactor()),v.getMetallicFactor()!==1&&(g.pbrMetallicRoughness.metallicFactor=v.getMetallicFactor()),v.getBaseColorTexture()){const y=v.getBaseColorTexture(),b=v.getBaseColorTextureInfo();g.pbrMetallicRoughness.baseColorTexture=o.createTextureInfoDef(y,b)}if(v.getEmissiveTexture()){const y=v.getEmissiveTexture(),b=v.getEmissiveTextureInfo();g.emissiveTexture=o.createTextureInfoDef(y,b)}if(v.getNormalTexture()){const y=v.getNormalTexture(),b=v.getNormalTextureInfo(),x=o.createTextureInfoDef(y,b);v.getNormalScale()!==1&&(x.scale=v.getNormalScale()),g.normalTexture=x}if(v.getOcclusionTexture()){const y=v.getOcclusionTexture(),b=v.getOcclusionTextureInfo(),x=o.createTextureInfoDef(y,b);v.getOcclusionStrength()!==1&&(x.strength=v.getOcclusionStrength()),g.occlusionTexture=x}if(v.getMetallicRoughnessTexture()){const y=v.getMetallicRoughnessTexture(),b=v.getMetallicRoughnessTextureInfo();g.pbrMetallicRoughness.metallicRoughnessTexture=o.createTextureInfoDef(y,b)}return o.materialIndexMap.set(v,S),g}),r.meshes=s.listMeshes().map((v,S)=>{const g=o.createPropertyDef(v);let y=null;return g.primitives=v.listPrimitives().map(b=>{const x={attributes:{}};x.mode=b.getMode();const _=b.getMaterial();_&&(x.material=o.materialIndexMap.get(_)),Object.keys(b.getExtras()).length&&(x.extras=b.getExtras());const C=b.getIndices();C&&(x.indices=o.accessorIndexMap.get(C));for(const M of b.listSemantics())x.attributes[M]=o.accessorIndexMap.get(b.getAttribute(M));for(const M of b.listTargets()){const R={};for(const F of M.listSemantics())R[F]=o.accessorIndexMap.get(M.getAttribute(F));x.targets=x.targets||[],x.targets.push(R)}return b.listTargets().length&&!y&&(y=b.listTargets().map(M=>M.getName())),x}),v.getWeights().length&&(g.weights=v.getWeights()),y&&(g.extras=g.extras||{},g.extras.targetNames=y),o.meshIndexMap.set(v,S),g}),r.cameras=s.listCameras().map((v,S)=>{const g=o.createPropertyDef(v);if(g.type=v.getType(),g.type===hs.Type.PERSPECTIVE){g.perspective={znear:v.getZNear(),zfar:v.getZFar(),yfov:v.getYFov()};const y=v.getAspectRatio();y!==null&&(g.perspective.aspectRatio=y)}else g.orthographic={znear:v.getZNear(),zfar:v.getZFar(),xmag:v.getXMag(),ymag:v.getYMag()};return o.cameraIndexMap.set(v,S),g}),r.nodes=s.listNodes().map((v,S)=>{const g=o.createPropertyDef(v);return ze.eq(v.getTranslation(),[0,0,0])||(g.translation=v.getTranslation()),ze.eq(v.getRotation(),[0,0,0,1])||(g.rotation=v.getRotation()),ze.eq(v.getScale(),[1,1,1])||(g.scale=v.getScale()),v.getWeights().length&&(g.weights=v.getWeights()),o.nodeIndexMap.set(v,S),g}),r.skins=s.listSkins().map((v,S)=>{const g=o.createPropertyDef(v),y=v.getInverseBindMatrices();y&&(g.inverseBindMatrices=o.accessorIndexMap.get(y));const b=v.getSkeleton();return b&&(g.skeleton=o.nodeIndexMap.get(b)),g.joints=v.listJoints().map(x=>o.nodeIndexMap.get(x)),o.skinIndexMap.set(v,S),g}),s.listNodes().forEach((v,S)=>{const g=r.nodes[S],y=v.getMesh();y&&(g.mesh=o.meshIndexMap.get(y));const b=v.getCamera();b&&(g.camera=o.cameraIndexMap.get(b));const x=v.getSkin();x&&(g.skin=o.skinIndexMap.get(x)),v.listChildren().length>0&&(g.children=v.listChildren().map(_=>o.nodeIndexMap.get(_)))}),r.animations=s.listAnimations().map((v,S)=>{const g=o.createPropertyDef(v),y=new Map;return g.samplers=v.listSamplers().map((b,x)=>{const _=o.createPropertyDef(b);return _.input=o.accessorIndexMap.get(b.getInput()),_.output=o.accessorIndexMap.get(b.getOutput()),_.interpolation=b.getInterpolation(),y.set(b,x),_}),g.channels=v.listChannels().map(b=>{const x=o.createPropertyDef(b);return x.sampler=y.get(b.getSampler()),x.target={node:o.nodeIndexMap.get(b.getTargetNode()),path:b.getTargetPath()},x}),o.animationIndexMap.set(v,S),g}),r.scenes=s.listScenes().map((v,S)=>{const g=o.createPropertyDef(v);return g.nodes=v.listChildren().map(y=>o.nodeIndexMap.get(y)),o.sceneIndexMap.set(v,S),g});const p=s.getDefaultScene();return p&&(r.scene=s.listScenes().indexOf(p)),r.extensionsUsed=u.map(v=>v.extensionName),r.extensionsRequired=c.map(v=>v.extensionName),u.forEach(v=>v.write(o)),function(v){const S=[];for(const g in v){const y=v[g];(Array.isArray(y)&&y.length===0||y===null||y===""||y&&typeof y=="object"&&Object.keys(y).length===0)&&S.push(g)}for(const g of S)delete v[g]}(r),i}}var Wi;(function(t){t[t.JSON=1313821514]="JSON",t[t.BIN=5130562]="BIN"})(Wi||(Wi={}));class Hx{constructor(){this.M=Et.DEFAULT_INSTANCE,this.l=new Set,this.U={},this.P=$i.INTERLEAVED,this.lastReadBytes=0,this.lastWriteBytes=0}setLogger(e){return this.M=e,this}registerExtensions(e){for(const n of e)this.l.add(n),n.register();return this}registerDependencies(e){return Object.assign(this.U,e),this}setVertexLayout(e){return this.P=e,this}async read(e){return await this.readJSON(await this.readAsJSON(e))}async readAsJSON(e){const n=await this.readURI(e,"view");this.lastReadBytes=n.byteLength;const s=jc(n)?this.L(n):{json:JSON.parse(Se.decodeText(n)),resources:{}};return await this.j(s,this.dirname(e)),this.D(s),s}async readJSON(e){return e=this._(e),this.D(e),Ox.read(e,{extensions:Array.from(this.l),dependencies:this.U,logger:this.M})}async binaryToJSON(e){const n=this.L(Se.assertView(e));this.D(n);const s=n.json;if(s.buffers&&s.buffers.some(r=>function(i,o){return o.uri!==void 0&&!(o.uri in i.resources)}(n,r)))throw new Error("Cannot resolve external buffers with binaryToJSON().");if(s.images&&s.images.some(r=>function(i,o){return o.uri!==void 0&&!(o.uri in i.resources)&&o.bufferView===void 0}(n,r)))throw new Error("Cannot resolve external images with binaryToJSON().");return n}async readBinary(e){return this.readJSON(await this.binaryToJSON(Se.assertView(e)))}async writeJSON(e,n={}){if(n.format===ts.GLB&&e.getRoot().listBuffers().length>1)throw new Error("GLB must have 0–1 buffers.");return zx.write(e,{format:n.format||ts.GLTF,basename:n.basename||"",logger:this.M,vertexLayout:this.P,dependencies:nt({},this.U),extensions:Array.from(this.l)})}async writeBinary(e){const{json:n,resources:s}=await this.writeJSON(e,{format:ts.GLB}),r=new Uint32Array([1179937895,2,12]),i=JSON.stringify(n),o=Se.pad(Se.encodeText(i),32),a=Se.toView(new Uint32Array([o.byteLength,1313821514])),l=Se.concat([a,o]);r[r.length-1]+=l.byteLength;const u=Object.values(s)[0];if(!u||!u.byteLength)return Se.concat([Se.toView(r),l]);const c=Se.pad(u,0),d=Se.toView(new Uint32Array([c.byteLength,5130562])),f=Se.concat([d,c]);return r[r.length-1]+=f.byteLength,Se.concat([Se.toView(r),l,f])}async j(e,n){var s=this;const r=[...e.json.images||[],...e.json.buffers||[]].map(async function(i){const o=i.uri;if(!o||o.match(/data:/))return Promise.resolve();e.resources[o]=await s.readURI(s.resolve(n,o),"view"),s.lastReadBytes+=e.resources[o].byteLength});await Promise.all(r)}D(e){function n(s){if(s.uri){if(s.uri in e.resources)Se.assertView(e.resources[s.uri]);else if(s.uri.match(/data:/)){const r=`__${Rx()}.${Os.extension(s.uri)}`;e.resources[r]=Se.createBufferFromDataURI(s.uri),s.uri=r}}}(e.json.images||[]).forEach(s=>{if(s.bufferView===void 0&&s.uri===void 0)throw new Error("Missing resource URI or buffer view.");n(s)}),(e.json.buffers||[]).forEach(n)}_(e){const{images:n,buffers:s}=e.json;return e={json:nt({},e.json),resources:nt({},e.resources)},n&&(e.json.images=n.map(r=>nt({},r))),s&&(e.json.buffers=s.map(r=>nt({},r))),e}L(e){if(!jc(e))throw new Error("Invalid glTF 2.0 binary.");const n=new Uint32Array(e.buffer,e.byteOffset+12,2);if(n[1]!==Wi.JSON)throw new Error("Missing required GLB JSON chunk.");const s=n[0],r=Se.decodeText(Se.toView(e,20,s)),i=JSON.parse(r),o=20+s;if(e.byteLength<=o)return{json:i,resources:{}};const a=new Uint32Array(e.buffer,e.byteOffset+o,2);if(a[1]!==Wi.BIN)throw new Error("Expected GLB BIN in second chunk.");const l=Se.toView(e,o+8,a[0]);return{json:i,resources:{[Hi]:l}}}}function jc(t){if(t.byteLength<3*Uint32Array.BYTES_PER_ELEMENT)return!1;const e=new Uint32Array(t.buffer,t.byteOffset,3);return e[0]===1179937895&&e[1]===2}class $x extends Hx{constructor(e=ur.DEFAULT_INIT){super(),this.J=void 0,this.J=e}async readURI(e,n){const s=await fetch(e,this.J);switch(n){case"view":return new Uint8Array(await s.arrayBuffer());case"text":return s.text()}}resolve(e,n){return ur.resolve(e,n)}dirname(e){return ur.dirname(e)}}function Gi(){return Gi=Object.assign?Object.assign.bind():function(t){for(var e=1;e<arguments.length;e++){var n=arguments[e];for(var s in n)Object.prototype.hasOwnProperty.call(n,s)&&(t[s]=n[s])}return t},Gi.apply(this,arguments)}function Ux(t,e){return Object.defineProperty(e,"name",{value:t}),e}var qi=typeof Float32Array<"u"?Float32Array:Array;function jx(){var t=new qi(3);return qi!=Float32Array&&(t[0]=0,t[1]=0,t[2]=0),t}Math.hypot||(Math.hypot=function(){for(var t=0,e=arguments.length;e--;)t+=arguments[e]*arguments[e];return Math.sqrt(t)});function Wx(){var t=new qi(4);return qi!=Float32Array&&(t[0]=0,t[1]=0,t[2]=0,t[3]=0),t}jx();Wx();se.ACCESSOR,se.MESH,se.TEXTURE,se.MATERIAL,se.SKIN;se.NODE,se.SKIN,se.MESH,se.CAMERA,se.PRIMITIVE,se.PRIMITIVE_TARGET,se.ANIMATION,se.MATERIAL,se.TEXTURE,se.ACCESSOR,se.BUFFER;Il.TargetPath;const Gx={pattern:/.*/,quantizationVolume:"mesh",quantizePosition:14,quantizeNormal:10,quantizeTexcoord:12,quantizeColor:8,quantizeWeight:8,quantizeGeneric:12,normalizeWeights:!0};Gi({level:"high"},Gx);var Wc;function Gs(t,e,n){for(let s=0,r=n.length;s<r;s++)n[s]=t[e*r+s];return n}function Gc(t,e,n){for(let s=0,r=n.length;s<r;s++)t[e*r+s]=n[s]}function ii(t,e,n=0){if(t.length!==e.length)return!1;for(let s=0;s<t.length;s++)if(Math.abs(t[s]-e[s])>n)return!1;return!0}function qx(t,e,n){return t*(1-n)+e*n}function Yx(t,e,n,s){for(let r=0;r<e.length;r++)t[r]=qx(e[r],n[r],s);return t}function Xx(t,e,n,s){let r,i,o,a,l,u=e[0],c=e[1],d=e[2],f=e[3],h=n[0],m=n[1],p=n[2],v=n[3];return i=u*h+c*m+d*p+f*v,i<0&&(i=-i,h=-h,m=-m,p=-p,v=-v),1-i>1e-6?(r=Math.acos(i),o=Math.sin(r),a=Math.sin((1-s)*r)/o,l=Math.sin(s*r)/o):(a=1-s,l=s),t[0]=a*u+l*h,t[1]=a*c+l*m,t[2]=a*d+l*p,t[3]=a*f+l*v,t}function qc(t,e){const n=function(s,r){return s[0]*r[0]+s[1]*r[1]+s[2]*r[2]+s[3]*r[3]}(t,e);return Math.acos(2*n*n-1)}(function(t){t[t.STEP=0]="STEP",t[t.LERP=1]="LERP",t[t.SLERP=2]="SLERP"})(Wc||(Wc={}));Promise.resolve();var Yi;(function(t){t.LANCZOS3="lanczos3",t.LANCZOS2="lanczos2"})(Yi||(Yi={}));Yi.LANCZOS3;Yi.LANCZOS3;const Yc="unpartition",Xc={};function Zx(t=Xc){return Gi({},Xc,t),Ux(Yc,async e=>{const n=e.getLogger(),s=e.getRoot().listBuffers()[0];e.getRoot().listAccessors().forEach(r=>r.setBuffer(s)),e.getRoot().listBuffers().forEach((r,i)=>i>0?r.dispose():null),n.debug(`${Yc}: Complete.`)})}let pi=new $x,gt="__yacv_name",yi="__helpers";async function Kx(t,e,n,s=()=>{}){let i=await(await fetch(t)).arrayBuffer();s();let o=null,a={};for(;o==null;)try{o=await pi.readBinary(new Uint8Array(i))}catch(l){if(l instanceof Error&&l.message.toLowerCase().includes("khr_draco_mesh_compression")){if(a.draco)throw l;a.draco=!0;let{KHRDracoMeshCompression:u}=await Ut(()=>import("./extensions.modern-Ccq9gWNQ.js"),[],import.meta.url),c=await Ut(()=>import("./draco_decoder-Ce3rGr-Y.js").then(f=>f.d),__vite__mapDeps([3,4]),import.meta.url),d=await Ut(()=>import("./draco_encoder-D4x7gQUW.js").then(f=>f.d),__vite__mapDeps([5,4]),import.meta.url);pi.registerExtensions([u]).registerDependencies({"draco3d.decoder":await c.default({}),"draco3d.encoder":await d.default({})})}else if(l instanceof Error&&l.message.toLowerCase().includes("ext_texture_webp")){if(a.webp)throw l;a.webp=!0;let{EXTTextureWebP:u}=await Ut(()=>import("./extensions.modern-Ccq9gWNQ.js"),[],import.meta.url);pi.registerExtensions([u])}else throw l}return await n.transform(om(e)),await o.transform(ew(e)),n.merge(o)}async function Jx(t){return await t.transform(tw(),Zx())}async function Zc(t){return pi.writeBinary(t)}async function Qx(t,e){return await e.transform(om(t))}function ew(t){return e=>{for(let n of e.getGraph().listEdges().map(s=>s.getChild()))n.getExtras()||n.setExtras({}),n.getExtras()[gt]=t;return e}}function om(t){return e=>{var n;for(let s of e.getGraph().listEdges().map(r=>r.getChild()))s.getExtras()==null||s instanceof Fl||s instanceof kl||(((n=s.getExtras()[gt])==null?void 0:n.toString())??"")==t&&s.dispose();return e}}function tw(){return t=>{let e=t.getRoot(),n=e.getDefaultScene()??e.listScenes()[0];for(let s of e.listScenes())if(s!==n){for(let r of s.listChildren())n.addChild(r);s.dispose()}}}class Ll{constructor(e=new J(1/0,1/0,1/0),n=new J(-1/0,-1/0,-1/0)){this.isBox3=!0,this.min=e,this.max=n}set(e,n){return this.min.copy(e),this.max.copy(n),this}setFromArray(e){this.makeEmpty();for(let n=0,s=e.length;n<s;n+=3)this.expandByPoint(Pt.fromArray(e,n));return this}setFromBufferAttribute(e){this.makeEmpty();for(let n=0,s=e.count;n<s;n++)this.expandByPoint(Pt.fromBufferAttribute(e,n));return this}setFromPoints(e){this.makeEmpty();for(let n=0,s=e.length;n<s;n++)this.expandByPoint(e[n]);return this}setFromCenterAndSize(e,n){const s=Pt.copy(n).multiplyScalar(.5);return this.min.copy(e).sub(s),this.max.copy(e).add(s),this}setFromObject(e,n=!1){return this.makeEmpty(),this.expandByObject(e,n)}clone(){return new this.constructor().copy(this)}copy(e){return this.min.copy(e.min),this.max.copy(e.max),this}makeEmpty(){return this.min.x=this.min.y=this.min.z=1/0,this.max.x=this.max.y=this.max.z=-1/0,this}isEmpty(){return this.max.x<this.min.x||this.max.y<this.min.y||this.max.z<this.min.z}getCenter(e){return this.isEmpty()?e.set(0,0,0):e.addVectors(this.min,this.max).multiplyScalar(.5)}getSize(e){return this.isEmpty()?e.set(0,0,0):e.subVectors(this.max,this.min)}expandByPoint(e){return this.min.min(e),this.max.max(e),this}expandByVector(e){return this.min.sub(e),this.max.add(e),this}expandByScalar(e){return this.min.addScalar(-e),this.max.addScalar(e),this}expandByObject(e,n=!1){e.updateWorldMatrix(!1,!1);const s=e.geometry;if(s!==void 0){const i=s.getAttribute("position");if(n===!0&&i!==void 0&&e.isInstancedMesh!==!0)for(let o=0,a=i.count;o<a;o++)e.isMesh===!0?e.getVertexPosition(o,Pt):Pt.fromBufferAttribute(i,o),Pt.applyMatrix4(e.matrixWorld),this.expandByPoint(Pt);else e.boundingBox!==void 0?(e.boundingBox===null&&e.computeBoundingBox(),oi.copy(e.boundingBox)):(s.boundingBox===null&&s.computeBoundingBox(),oi.copy(s.boundingBox)),oi.applyMatrix4(e.matrixWorld),this.union(oi)}const r=e.children;for(let i=0,o=r.length;i<o;i++)this.expandByObject(r[i],n);return this}containsPoint(e){return!(e.x<this.min.x||e.x>this.max.x||e.y<this.min.y||e.y>this.max.y||e.z<this.min.z||e.z>this.max.z)}containsBox(e){return this.min.x<=e.min.x&&e.max.x<=this.max.x&&this.min.y<=e.min.y&&e.max.y<=this.max.y&&this.min.z<=e.min.z&&e.max.z<=this.max.z}getParameter(e,n){return n.set((e.x-this.min.x)/(this.max.x-this.min.x),(e.y-this.min.y)/(this.max.y-this.min.y),(e.z-this.min.z)/(this.max.z-this.min.z))}intersectsBox(e){return!(e.max.x<this.min.x||e.min.x>this.max.x||e.max.y<this.min.y||e.min.y>this.max.y||e.max.z<this.min.z||e.min.z>this.max.z)}intersectsSphere(e){return this.clampPoint(e.center,Pt),Pt.distanceToSquared(e.center)<=e.radius*e.radius}intersectsPlane(e){let n,s;return e.normal.x>0?(n=e.normal.x*this.min.x,s=e.normal.x*this.max.x):(n=e.normal.x*this.max.x,s=e.normal.x*this.min.x),e.normal.y>0?(n+=e.normal.y*this.min.y,s+=e.normal.y*this.max.y):(n+=e.normal.y*this.max.y,s+=e.normal.y*this.min.y),e.normal.z>0?(n+=e.normal.z*this.min.z,s+=e.normal.z*this.max.z):(n+=e.normal.z*this.max.z,s+=e.normal.z*this.min.z),n<=-e.constant&&s>=-e.constant}intersectsTriangle(e){if(this.isEmpty())return!1;this.getCenter(qs),ai.subVectors(this.max,qs),xs.subVectors(e.a,qs),ws.subVectors(e.b,qs),_s.subVectors(e.c,qs),xn.subVectors(ws,xs),wn.subVectors(_s,ws),Wn.subVectors(xs,_s);let n=[0,-xn.z,xn.y,0,-wn.z,wn.y,0,-Wn.z,Wn.y,xn.z,0,-xn.x,wn.z,0,-wn.x,Wn.z,0,-Wn.x,-xn.y,xn.x,0,-wn.y,wn.x,0,-Wn.y,Wn.x,0];return!Qo(n,xs,ws,_s,ai)||(n=[1,0,0,0,1,0,0,0,1],!Qo(n,xs,ws,_s,ai))?!1:(li.crossVectors(xn,wn),n=[li.x,li.y,li.z],Qo(n,xs,ws,_s,ai))}clampPoint(e,n){return n.copy(e).clamp(this.min,this.max)}distanceToPoint(e){return this.clampPoint(e,Pt).distanceTo(e)}getBoundingSphere(e){return this.isEmpty()?e.makeEmpty():(this.getCenter(e.center),e.radius=this.getSize(Pt).length()*.5),e}intersect(e){return this.min.max(e.min),this.max.min(e.max),this.isEmpty()&&this.makeEmpty(),this}union(e){return this.min.min(e.min),this.max.max(e.max),this}applyMatrix4(e){return this.isEmpty()?this:(nn[0].set(this.min.x,this.min.y,this.min.z).applyMatrix4(e),nn[1].set(this.min.x,this.min.y,this.max.z).applyMatrix4(e),nn[2].set(this.min.x,this.max.y,this.min.z).applyMatrix4(e),nn[3].set(this.min.x,this.max.y,this.max.z).applyMatrix4(e),nn[4].set(this.max.x,this.min.y,this.min.z).applyMatrix4(e),nn[5].set(this.max.x,this.min.y,this.max.z).applyMatrix4(e),nn[6].set(this.max.x,this.max.y,this.min.z).applyMatrix4(e),nn[7].set(this.max.x,this.max.y,this.max.z).applyMatrix4(e),this.setFromPoints(nn),this)}translate(e){return this.min.add(e),this.max.add(e),this}equals(e){return e.min.equals(this.min)&&e.max.equals(this.max)}}const nn=[new J,new J,new J,new J,new J,new J,new J,new J],Pt=new J,oi=new Ll,xs=new J,ws=new J,_s=new J,xn=new J,wn=new J,Wn=new J,qs=new J,ai=new J,li=new J,Gn=new J;function Qo(t,e,n,s,r){for(let i=0,o=t.length-3;i<=o;i+=3){Gn.fromArray(t,i);const a=r.x*Math.abs(Gn.x)+r.y*Math.abs(Gn.y)+r.z*Math.abs(Gn.z),l=e.dot(Gn),u=n.dot(Gn),c=s.dot(Gn);if(Math.max(-Math.max(l,u,c),Math.min(l,u,c))>a)return!1}return!0}const Kc=new fs().set(.8224621,.177538,0,.0331941,.9668058,0,.0170827,.0723974,.9105199),Jc=new fs().set(1.2249401,-.2249404,0,-.0420569,1.0420571,0,-.0196376,-.0786361,1.0982735),ui={[Ml]:{transfer:La,primaries:Ic,toReference:t=>t,fromReference:t=>t},[Cn]:{transfer:Rc,primaries:Ic,toReference:t=>t.convertSRGBToLinear(),fromReference:t=>t.convertLinearToSRGB()},[Zh]:{transfer:La,primaries:kc,toReference:t=>t.applyMatrix3(Jc),fromReference:t=>t.applyMatrix3(Kc)},[Qb]:{transfer:Rc,primaries:kc,toReference:t=>t.convertSRGBToLinear().applyMatrix3(Jc),fromReference:t=>t.applyMatrix3(Kc).convertLinearToSRGB()}},nw=new Set([Ml,Zh]),Ot={enabled:!0,_workingColorSpace:Ml,get workingColorSpace(){return this._workingColorSpace},set workingColorSpace(t){if(!nw.has(t))throw new Error(`Unsupported working color space, "${t}".`);this._workingColorSpace=t},convert:function(t,e,n){if(this.enabled===!1||e===n||!e||!n)return t;const s=ui[e].toReference,r=ui[n].fromReference;return r(s(t))},fromWorkingColorSpace:function(t,e){return this.convert(t,this._workingColorSpace,e)},toWorkingColorSpace:function(t,e){return this.convert(t,e,this._workingColorSpace)},getPrimaries:function(t){return ui[t].primaries},getTransfer:function(t){return t===Jb?La:ui[t].transfer}};function ea(t){return t<.04045?t*.0773993808:Math.pow(t*.9478672986+.0521327014,2.4)}function ta(t){return t<.0031308?t*12.92:1.055*Math.pow(t,.41666)-.055}const am={aliceblue:15792383,antiquewhite:16444375,aqua:65535,aquamarine:8388564,azure:15794175,beige:16119260,bisque:16770244,black:0,blanchedalmond:16772045,blue:255,blueviolet:9055202,brown:10824234,burlywood:14596231,cadetblue:6266528,chartreuse:8388352,chocolate:13789470,coral:16744272,cornflowerblue:6591981,cornsilk:16775388,crimson:14423100,cyan:65535,darkblue:139,darkcyan:35723,darkgoldenrod:12092939,darkgray:11119017,darkgreen:25600,darkgrey:11119017,darkkhaki:12433259,darkmagenta:9109643,darkolivegreen:5597999,darkorange:16747520,darkorchid:10040012,darkred:9109504,darksalmon:15308410,darkseagreen:9419919,darkslateblue:4734347,darkslategray:3100495,darkslategrey:3100495,darkturquoise:52945,darkviolet:9699539,deeppink:16716947,deepskyblue:49151,dimgray:6908265,dimgrey:6908265,dodgerblue:2003199,firebrick:11674146,floralwhite:16775920,forestgreen:2263842,fuchsia:16711935,gainsboro:14474460,ghostwhite:16316671,gold:16766720,goldenrod:14329120,gray:8421504,green:32768,greenyellow:11403055,grey:8421504,honeydew:15794160,hotpink:16738740,indianred:13458524,indigo:4915330,ivory:16777200,khaki:15787660,lavender:15132410,lavenderblush:16773365,lawngreen:8190976,lemonchiffon:16775885,lightblue:11393254,lightcoral:15761536,lightcyan:14745599,lightgoldenrodyellow:16448210,lightgray:13882323,lightgreen:9498256,lightgrey:13882323,lightpink:16758465,lightsalmon:16752762,lightseagreen:2142890,lightskyblue:8900346,lightslategray:7833753,lightslategrey:7833753,lightsteelblue:11584734,lightyellow:16777184,lime:65280,limegreen:3329330,linen:16445670,magenta:16711935,maroon:8388608,mediumaquamarine:6737322,mediumblue:205,mediumorchid:12211667,mediumpurple:9662683,mediumseagreen:3978097,mediumslateblue:8087790,mediumspringgreen:64154,mediumturquoise:4772300,mediumvioletred:13047173,midnightblue:1644912,mintcream:16121850,mistyrose:16770273,moccasin:16770229,navajowhite:16768685,navy:128,oldlace:16643558,olive:8421376,olivedrab:7048739,orange:16753920,orangered:16729344,orchid:14315734,palegoldenrod:15657130,palegreen:10025880,paleturquoise:11529966,palevioletred:14381203,papayawhip:16773077,peachpuff:16767673,peru:13468991,pink:16761035,plum:14524637,powderblue:11591910,purple:8388736,rebeccapurple:6697881,red:16711680,rosybrown:12357519,royalblue:4286945,saddlebrown:9127187,salmon:16416882,sandybrown:16032864,seagreen:3050327,seashell:16774638,sienna:10506797,silver:12632256,skyblue:8900331,slateblue:6970061,slategray:7372944,slategrey:7372944,snow:16775930,springgreen:65407,steelblue:4620980,tan:13808780,teal:32896,thistle:14204888,tomato:16737095,turquoise:4251856,violet:15631086,wheat:16113331,white:16777215,whitesmoke:16119285,yellow:16776960,yellowgreen:10145074},_n={h:0,s:0,l:0},ci={h:0,s:0,l:0};function na(t,e,n){return n<0&&(n+=1),n>1&&(n-=1),n<1/6?t+(e-t)*6*n:n<1/2?e:n<2/3?t+(e-t)*6*(2/3-n):t}class Vl{constructor(e,n,s){return this.isColor=!0,this.r=1,this.g=1,this.b=1,this.set(e,n,s)}set(e,n,s){if(n===void 0&&s===void 0){const r=e;r&&r.isColor?this.copy(r):typeof r=="number"?this.setHex(r):typeof r=="string"&&this.setStyle(r)}else this.setRGB(e,n,s);return this}setScalar(e){return this.r=e,this.g=e,this.b=e,this}setHex(e,n=Cn){return e=Math.floor(e),this.r=(e>>16&255)/255,this.g=(e>>8&255)/255,this.b=(e&255)/255,Ot.toWorkingColorSpace(this,n),this}setRGB(e,n,s,r=Ot.workingColorSpace){return this.r=e,this.g=n,this.b=s,Ot.toWorkingColorSpace(this,r),this}setHSL(e,n,s,r=Ot.workingColorSpace){if(e=Xb(e,1),n=at(n,0,1),s=at(s,0,1),n===0)this.r=this.g=this.b=s;else{const i=s<=.5?s*(1+n):s+n-s*n,o=2*s-i;this.r=na(o,i,e+1/3),this.g=na(o,i,e),this.b=na(o,i,e-1/3)}return Ot.toWorkingColorSpace(this,r),this}setStyle(e,n=Cn){function s(i){i!==void 0&&parseFloat(i)<1&&console.warn("THREE.Color: Alpha component of "+e+" will be ignored.")}let r;if(r=/^(\w+)\(([^\)]*)\)/.exec(e)){let i;const o=r[1],a=r[2];switch(o){case"rgb":case"rgba":if(i=/^\s*(\d+)\s*,\s*(\d+)\s*,\s*(\d+)\s*(?:,\s*(\d*\.?\d+)\s*)?$/.exec(a))return s(i[4]),this.setRGB(Math.min(255,parseInt(i[1],10))/255,Math.min(255,parseInt(i[2],10))/255,Math.min(255,parseInt(i[3],10))/255,n);if(i=/^\s*(\d+)\%\s*,\s*(\d+)\%\s*,\s*(\d+)\%\s*(?:,\s*(\d*\.?\d+)\s*)?$/.exec(a))return s(i[4]),this.setRGB(Math.min(100,parseInt(i[1],10))/100,Math.min(100,parseInt(i[2],10))/100,Math.min(100,parseInt(i[3],10))/100,n);break;case"hsl":case"hsla":if(i=/^\s*(\d*\.?\d+)\s*,\s*(\d*\.?\d+)\%\s*,\s*(\d*\.?\d+)\%\s*(?:,\s*(\d*\.?\d+)\s*)?$/.exec(a))return s(i[4]),this.setHSL(parseFloat(i[1])/360,parseFloat(i[2])/100,parseFloat(i[3])/100,n);break;default:console.warn("THREE.Color: Unknown color model "+e)}}else if(r=/^\#([A-Fa-f\d]+)$/.exec(e)){const i=r[1],o=i.length;if(o===3)return this.setRGB(parseInt(i.charAt(0),16)/15,parseInt(i.charAt(1),16)/15,parseInt(i.charAt(2),16)/15,n);if(o===6)return this.setHex(parseInt(i,16),n);console.warn("THREE.Color: Invalid hex color "+e)}else if(e&&e.length>0)return this.setColorName(e,n);return this}setColorName(e,n=Cn){const s=am[e.toLowerCase()];return s!==void 0?this.setHex(s,n):console.warn("THREE.Color: Unknown color "+e),this}clone(){return new this.constructor(this.r,this.g,this.b)}copy(e){return this.r=e.r,this.g=e.g,this.b=e.b,this}copySRGBToLinear(e){return this.r=ea(e.r),this.g=ea(e.g),this.b=ea(e.b),this}copyLinearToSRGB(e){return this.r=ta(e.r),this.g=ta(e.g),this.b=ta(e.b),this}convertSRGBToLinear(){return this.copySRGBToLinear(this),this}convertLinearToSRGB(){return this.copyLinearToSRGB(this),this}getHex(e=Cn){return Ot.fromWorkingColorSpace(Xe.copy(this),e),Math.round(at(Xe.r*255,0,255))*65536+Math.round(at(Xe.g*255,0,255))*256+Math.round(at(Xe.b*255,0,255))}getHexString(e=Cn){return("000000"+this.getHex(e).toString(16)).slice(-6)}getHSL(e,n=Ot.workingColorSpace){Ot.fromWorkingColorSpace(Xe.copy(this),n);const s=Xe.r,r=Xe.g,i=Xe.b,o=Math.max(s,r,i),a=Math.min(s,r,i);let l,u;const c=(a+o)/2;if(a===o)l=0,u=0;else{const d=o-a;switch(u=c<=.5?d/(o+a):d/(2-o-a),o){case s:l=(r-i)/d+(r<i?6:0);break;case r:l=(i-s)/d+2;break;case i:l=(s-r)/d+4;break}l/=6}return e.h=l,e.s=u,e.l=c,e}getRGB(e,n=Ot.workingColorSpace){return Ot.fromWorkingColorSpace(Xe.copy(this),n),e.r=Xe.r,e.g=Xe.g,e.b=Xe.b,e}getStyle(e=Cn){Ot.fromWorkingColorSpace(Xe.copy(this),e);const n=Xe.r,s=Xe.g,r=Xe.b;return e!==Cn?`color(${e} ${n.toFixed(3)} ${s.toFixed(3)} ${r.toFixed(3)})`:`rgb(${Math.round(n*255)},${Math.round(s*255)},${Math.round(r*255)})`}offsetHSL(e,n,s){return this.getHSL(_n),this.setHSL(_n.h+e,_n.s+n,_n.l+s)}add(e){return this.r+=e.r,this.g+=e.g,this.b+=e.b,this}addColors(e,n){return this.r=e.r+n.r,this.g=e.g+n.g,this.b=e.b+n.b,this}addScalar(e){return this.r+=e,this.g+=e,this.b+=e,this}sub(e){return this.r=Math.max(0,this.r-e.r),this.g=Math.max(0,this.g-e.g),this.b=Math.max(0,this.b-e.b),this}multiply(e){return this.r*=e.r,this.g*=e.g,this.b*=e.b,this}multiplyScalar(e){return this.r*=e,this.g*=e,this.b*=e,this}lerp(e,n){return this.r+=(e.r-this.r)*n,this.g+=(e.g-this.g)*n,this.b+=(e.b-this.b)*n,this}lerpColors(e,n,s){return this.r=e.r+(n.r-e.r)*s,this.g=e.g+(n.g-e.g)*s,this.b=e.b+(n.b-e.b)*s,this}lerpHSL(e,n){this.getHSL(_n),e.getHSL(ci);const s=Yo(_n.h,ci.h,n),r=Yo(_n.s,ci.s,n),i=Yo(_n.l,ci.l,n);return this.setHSL(s,r,i),this}setFromVector3(e){return this.r=e.x,this.g=e.y,this.b=e.z,this}applyMatrix3(e){const n=this.r,s=this.g,r=this.b,i=e.elements;return this.r=i[0]*n+i[3]*s+i[6]*r,this.g=i[1]*n+i[4]*s+i[7]*r,this.b=i[2]*n+i[5]*s+i[8]*r,this}equals(e){return e.r===this.r&&e.g===this.g&&e.b===this.b}fromArray(e,n=0){return this.r=e[n],this.g=e[n+1],this.b=e[n+2],this}toArray(e=[],n=0){return e[n]=this.r,e[n+1]=this.g,e[n+2]=this.b,e}fromBufferAttribute(e,n){return this.r=e.getX(n),this.g=e.getY(n),this.b=e.getZ(n),this}toJSON(){return this.getHex()}*[Symbol.iterator](){yield this.r,yield this.g,yield this.b}}const Xe=new Vl;Vl.NAMES=am;const sa=new J,sw=new J,rw=new fs;class ra{constructor(e=new J(1,0,0),n=0){this.isPlane=!0,this.normal=e,this.constant=n}set(e,n){return this.normal.copy(e),this.constant=n,this}setComponents(e,n,s,r){return this.normal.set(e,n,s),this.constant=r,this}setFromNormalAndCoplanarPoint(e,n){return this.normal.copy(e),this.constant=-n.dot(this.normal),this}setFromCoplanarPoints(e,n,s){const r=sa.subVectors(s,n).cross(sw.subVectors(e,n)).normalize();return this.setFromNormalAndCoplanarPoint(r,e),this}copy(e){return this.normal.copy(e.normal),this.constant=e.constant,this}normalize(){const e=1/this.normal.length();return this.normal.multiplyScalar(e),this.constant*=e,this}negate(){return this.constant*=-1,this.normal.negate(),this}distanceToPoint(e){return this.normal.dot(e)+this.constant}distanceToSphere(e){return this.distanceToPoint(e.center)-e.radius}projectPoint(e,n){return n.copy(e).addScaledVector(this.normal,-this.distanceToPoint(e))}intersectLine(e,n){const s=e.delta(sa),r=this.normal.dot(s);if(r===0)return this.distanceToPoint(e.start)===0?n.copy(e.start):null;const i=-(e.start.dot(this.normal)+this.constant)/r;return i<0||i>1?null:n.copy(e.start).addScaledVector(s,i)}intersectsLine(e){const n=this.distanceToPoint(e.start),s=this.distanceToPoint(e.end);return n<0&&s>0||s<0&&n>0}intersectsBox(e){return e.intersectsPlane(this)}intersectsSphere(e){return e.intersectsPlane(this)}coplanarPoint(e){return e.copy(this.normal).multiplyScalar(-this.constant)}applyMatrix4(e,n){const s=n||rw.getNormalMatrix(e),r=this.coplanarPoint(sa).applyMatrix4(e),i=this.normal.applyMatrix3(s).normalize();return this.constant=-r.dot(i),this}translate(e){return this.constant-=e.dot(this.normal),this}equals(e){return e.normal.equals(this.normal)&&e.constant===this.constant}clone(){return new this.constructor().copy(this)}}class Co{constructor(e=0,n=0){Co.prototype.isVector2=!0,this.x=e,this.y=n}get width(){return this.x}set width(e){this.x=e}get height(){return this.y}set height(e){this.y=e}set(e,n){return this.x=e,this.y=n,this}setScalar(e){return this.x=e,this.y=e,this}setX(e){return this.x=e,this}setY(e){return this.y=e,this}setComponent(e,n){switch(e){case 0:this.x=n;break;case 1:this.y=n;break;default:throw new Error("index is out of range: "+e)}return this}getComponent(e){switch(e){case 0:return this.x;case 1:return this.y;default:throw new Error("index is out of range: "+e)}}clone(){return new this.constructor(this.x,this.y)}copy(e){return this.x=e.x,this.y=e.y,this}add(e){return this.x+=e.x,this.y+=e.y,this}addScalar(e){return this.x+=e,this.y+=e,this}addVectors(e,n){return this.x=e.x+n.x,this.y=e.y+n.y,this}addScaledVector(e,n){return this.x+=e.x*n,this.y+=e.y*n,this}sub(e){return this.x-=e.x,this.y-=e.y,this}subScalar(e){return this.x-=e,this.y-=e,this}subVectors(e,n){return this.x=e.x-n.x,this.y=e.y-n.y,this}multiply(e){return this.x*=e.x,this.y*=e.y,this}multiplyScalar(e){return this.x*=e,this.y*=e,this}divide(e){return this.x/=e.x,this.y/=e.y,this}divideScalar(e){return this.multiplyScalar(1/e)}applyMatrix3(e){const n=this.x,s=this.y,r=e.elements;return this.x=r[0]*n+r[3]*s+r[6],this.y=r[1]*n+r[4]*s+r[7],this}min(e){return this.x=Math.min(this.x,e.x),this.y=Math.min(this.y,e.y),this}max(e){return this.x=Math.max(this.x,e.x),this.y=Math.max(this.y,e.y),this}clamp(e,n){return this.x=Math.max(e.x,Math.min(n.x,this.x)),this.y=Math.max(e.y,Math.min(n.y,this.y)),this}clampScalar(e,n){return this.x=Math.max(e,Math.min(n,this.x)),this.y=Math.max(e,Math.min(n,this.y)),this}clampLength(e,n){const s=this.length();return this.divideScalar(s||1).multiplyScalar(Math.max(e,Math.min(n,s)))}floor(){return this.x=Math.floor(this.x),this.y=Math.floor(this.y),this}ceil(){return this.x=Math.ceil(this.x),this.y=Math.ceil(this.y),this}round(){return this.x=Math.round(this.x),this.y=Math.round(this.y),this}roundToZero(){return this.x=Math.trunc(this.x),this.y=Math.trunc(this.y),this}negate(){return this.x=-this.x,this.y=-this.y,this}dot(e){return this.x*e.x+this.y*e.y}cross(e){return this.x*e.y-this.y*e.x}lengthSq(){return this.x*this.x+this.y*this.y}length(){return Math.sqrt(this.x*this.x+this.y*this.y)}manhattanLength(){return Math.abs(this.x)+Math.abs(this.y)}normalize(){return this.divideScalar(this.length()||1)}angle(){return Math.atan2(-this.y,-this.x)+Math.PI}angleTo(e){const n=Math.sqrt(this.lengthSq()*e.lengthSq());if(n===0)return Math.PI/2;const s=this.dot(e)/n;return Math.acos(at(s,-1,1))}distanceTo(e){return Math.sqrt(this.distanceToSquared(e))}distanceToSquared(e){const n=this.x-e.x,s=this.y-e.y;return n*n+s*s}manhattanDistanceTo(e){return Math.abs(this.x-e.x)+Math.abs(this.y-e.y)}setLength(e){return this.normalize().multiplyScalar(e)}lerp(e,n){return this.x+=(e.x-this.x)*n,this.y+=(e.y-this.y)*n,this}lerpVectors(e,n,s){return this.x=e.x+(n.x-e.x)*s,this.y=e.y+(n.y-e.y)*s,this}equals(e){return e.x===this.x&&e.y===this.y}fromArray(e,n=0){return this.x=e[n],this.y=e[n+1],this}toArray(e=[],n=0){return e[n]=this.x,e[n+1]=this.y,e}fromBufferAttribute(e,n){return this.x=e.getX(n),this.y=e.getY(n),this}rotateAround(e,n){const s=Math.cos(n),r=Math.sin(n),i=this.x-e.x,o=this.y-e.y;return this.x=i*s-o*r+e.x,this.y=i*r+o*s+e.y,this}random(){return this.x=Math.random(),this.y=Math.random(),this}*[Symbol.iterator](){yield this.x,yield this.y}}const iw=Ut(()=>import("./LineSegments2-Iva-HSlT.js"),__vite__mapDeps([6,1,7,8]),import.meta.url),ow=Ut(()=>import("./LineMaterial-BqKi9HCI.js"),__vite__mapDeps([8,1]),import.meta.url),aw=Ut(()=>import("./LineSegmentsGeometry-dwUEb1B-.js"),__vite__mapDeps([7,1]),import.meta.url);async function lw(t,e=.1){const n=(await iw).LineSegments2,s=(await ow).LineMaterial;return new n(await uw(t),new s({color:4294967295,vertexColors:!0,linewidth:e,worldUnits:!0,resolution:new Co(1,1)}))}async function uw(t){const e=(await aw).LineSegmentsGeometry,n=new e,s=t.getAttribute("position"),r=t.index,i=[];for(let l=0;l!=r.count;++l){const u=r.getX(l),c=s.getX(u),d=s.getY(u),f=s.getZ(u);i.push(c,d,f)}n.setPositions(i);const o=[],a=t.getAttribute("color");if(a){for(let l=0;l!=r.count;++l){const u=r.getX(l),c=a.getX(u),d=a.getY(u),f=a.getZ(u);o.push(c,d,f)}n.setColors(o)}return n.userData=t.userData,n}const un={preload:["dev+http://127.0.0.1:32323/"],loadHelpers:!0,edgeWidth:0,displayLoadingEveryMs:1e3,monitorEveryMs:100,monitorOpenTimeoutMs:1e3,autoplay:!0,arModes:"webxr scene-viewer quick-look",exposure:1,shadowIntensity:0,background:""},Qc=[];function lm(t,e){let n=t.endsWith(".0");n&&(t=t.slice(0,-2));let s=un[t];if(s===void 0)throw new Error(`Unknown setting: ${t}`);if(Array.isArray(s))if(n)s=s[0];else{let r=[];return Qc.includes(t)?r=s:Qc.push(t),r.push(lm(t+".0",e)),r}switch(typeof s){case"boolean":return e==="true";case"number":return Number(e);case"string":return e;default:throw new Error(`Unknown setting type: ${typeof s} -- ${s}`)}}const cw=new URL(window.location.href);cw.searchParams.forEach((t,e)=>{e in un&&(un[e]=lm(e,t))});const dw={class:"model-name"},fw=fn({__name:"Model",props:Zl({meshes:{},viewer:{}},{enabledFeatures:{default:[0,1,2]},enabledFeaturesModifiers:{},opacity:{default:1},opacityModifiers:{}}),emits:Zl(["remove"],["update:enabledFeatures","update:opacity"]),setup(t,{emit:e}){const n=t,s=e;let r=n.meshes[0].getExtras()[gt],i=ee(-1),o=ee(-1),a=ee(-1);const l=ee(1),u=ee(!1),c=ee(1),d=ee(!1),f=ee(1),h=ee(!1),m=ee(un.edgeWidth),p=cu(t,"enabledFeatures"),v=cu(t,"opacity"),S=ee(!1);function g(R){var T;let F=(T=n.viewer)==null?void 0:T.scene,V=F==null?void 0:F._model;!F||!V||(V.traverse(L=>{if(L.userData[gt]===r){let A=L.type=="Mesh"||L.type=="SkinnedMesh",N=L.type=="Line"||L.type=="LineSegments"||L.type=="LineSegments2",$=L.type=="Points";if(A||N||$){let z=R.includes(A?0:N?1:$?2:-1);L.visible!==z&&(L.visible=z,L.userData.backChild&&(L.userData.backChild.visible=z))}}}),F.queueRender())}le(p,g,{deep:!0});function y(R){var T;let F=(T=n.viewer)==null?void 0:T.scene,V=F==null?void 0:F._model;!F||!V||(V.traverse(L=>{L.userData[gt]===r&&L.material&&L.material.opacity!==R&&(L.material.transparent=R<1,L.material.opacity=R,L.material.needsUpdate=!0)}),F.queueRender())}le(v,y);function b(R){var T;let F=(T=n.viewer)==null?void 0:T.scene,V=F==null?void 0:F._model;!F||!V||(V.traverse(L=>{L.userData[gt]===r&&L.material&&L.material.wireframe!==R&&(L.material.wireframe=R,L.material.needsUpdate=!0)}),F.queueRender())}le(S,b);function x(){var N,$;let R=(N=n.viewer)==null?void 0:N.scene,F=R==null?void 0:R._model;if(!R||!F)return;let V=l.value<1&&!u.value||l.value>0&&u.value,T=c.value<1&&!d.value||c.value>0&&d.value,L=f.value<1&&!h.value||f.value>0&&h.value,A;($=n.viewer)!=null&&$.renderer&&(V||T||L)&&(n.viewer.renderer.threeRenderer.localClippingEnabled=!0,A=new Ll,F.traverse(z=>{z.userData[gt]===r&&A.expandByObject(z)})),F.traverse(z=>{if(z.userData[gt]===r&&z.material)if((A==null?void 0:A.isEmpty())==!1){let X=A.min.x+l.value*(A.max.x-A.min.x),re=A.min.y+c.value*(A.max.y-A.min.y),te=A.min.z+(1-f.value)*(A.max.z-A.min.z),Q=[new ra(new J(-1,0,0),X),new ra(new J(0,-1,0),re),new ra(new J(0,0,1),-te)];u.value&&Q[0].negate(),d.value&&Q[1].negate(),h.value&&Q[2].negate(),L||Q.pop(),T||Q.splice(1,1),V||Q.shift(),z.material.clippingPlanes=Q,z.userData.backChild&&z.userData.backChild.material&&(z.userData.backChild.material.clippingPlanes=Q)}else z.material.clippingPlanes=[],z.userData.backChild&&z.userData.backChild.material&&(z.userData.backChild.material.clippingPlanes=[])}),R.queueRender()}le(l,x),le(c,x),le(f,x),le(u,x),le(d,x),le(h,x),n.viewer.onElemReady(R=>R.addEventListener("camera-change",x));let _=[];function C(R){var L;let F=(L=n.viewer)==null?void 0:L.scene,V=F==null?void 0:F._model;if(!F||!V)return;_.forEach(A=>A()),_=[];let T=[];V.traverse(A=>{A.userData[gt]===r&&((A.type=="Line"||A.type=="LineSegments")&&R>0&&T.push(A),A.type=="Points"&&(A.material.size=R>0?R*50:5,A.material.needsUpdate=!0))}),T.forEach(async A=>{let N=await lw(A.geometry,R),$=z=>{N.material.resolution.set(z.clientWidth,z.clientHeight),N.material.needsUpdate=!0};n.viewer.onElemReady(z=>{z.addEventListener("resize",()=>$(z)),$(z)}),N.position.copy(A.position),N.computeLineDistances(),N.userData=Object.assign({},A.userData),A.parent.add(N),A.children.forEach(z=>N.add(z)),A.visible=!1,A.userData.niceLine=N,N.userData.noHit=!0,_.push(()=>{N.parent.remove(N),A.visible=!0,n.viewer.onElemReady(z=>{z.removeEventListener("resize",()=>$(z))})})}),F.queueRender()}le(m,C);function M(){var L;let R=(L=n.viewer)==null?void 0:L.scene,F=R==null?void 0:R._model;if(!R||!F)return;const V=i.value===-1;i.value=n.meshes.flatMap(A=>A.listPrimitives().filter(N=>N.getMode()===WebGL2RenderingContext.TRIANGLES)).map(A=>{var N,$;return(($=(N=A.getExtras())==null?void 0:N.face_triangles_end)==null?void 0:$.length)??1}).reduce((A,N)=>A+N,0),o.value=n.meshes.flatMap(A=>A.listPrimitives().filter(N=>N.getMode()in[WebGL2RenderingContext.LINE_STRIP,WebGL2RenderingContext.LINES])).map(A=>{var N,$;return(($=(N=A.getExtras())==null?void 0:N.edge_points_end)==null?void 0:$.length)??0}).reduce((A,N)=>A+N,0),a.value=n.meshes.flatMap(A=>A.listPrimitives().filter(N=>N.getMode()===WebGL2RenderingContext.POINTS)).map(A=>{var N;return((N=A.getAttribute("POSITION"))==null?void 0:N.getCount())??0}).reduce((A,N)=>A+N,0),V&&(i.value===0?p.value=p.value.filter(A=>A!==0):p.value.includes(0)||p.value.push(0),o.value===0?p.value=p.value.filter(A=>A!==1):p.value.includes(1)||p.value.push(1),a.value===0?p.value=p.value.filter(A=>A!==2):p.value.includes(2)||p.value.push(2));let T=[];F.traverse(A=>{var N;if(A.updateMatrixWorld(),A.userData[gt]===r&&(A.type=="Mesh"||A.type=="SkinnedMesh")&&((N=A.geometry)==null||N.computeBoundsTree({indirect:!0}),A.material.side=Zb,r!==yi)){let $=A.clone();$.material=A.material.clone(),$.material.side=Kb,$.material.color=new Vl(.25,.25,.25),$.userData.noHit=!0,A.userData.backChild=$,T.push($)}}),T.forEach(A=>F.add(A)),g(p.value),y(v.value),b(S.value),x(),C(m.value),R.queueRender()}return n.viewer.onElemReady(R=>R.addEventListener("load",M)),(R,F)=>(st(),Yt(P(cb),{value:P(r)},{default:q(()=>[w(P(qh),{"expand-icon":"hide-this-icon","collapse-icon":"hide-this-icon"},{default:q(()=>[w(P(Cy),{modelValue:p.value,"onUpdate:modelValue":F[0]||(F[0]=V=>p.value=V),multiple:"",onClick:F[1]||(F[1]=Cu(()=>{},["stop"])),color:"surface-light"},{default:q(()=>[w(P(ot),{icon:""},{default:q(()=>[w(P(We),{activator:"parent"},{default:q(()=>[Ae("Toggle Faces ("+rn(P(i))+")",1)]),_:1}),w(P(Be),{type:"mdi",path:P(Ob),rotate:90},null,8,["path"])]),_:1}),w(P(ot),{icon:""},{default:q(()=>[w(P(We),{activator:"parent"},{default:q(()=>[Ae("Toggle Edges ("+rn(P(o))+")",1)]),_:1}),w(P(Be),{type:"mdi",path:P(Nb),rotate:90},null,8,["path"])]),_:1}),w(P(ot),{icon:""},{default:q(()=>[w(P(We),{activator:"parent"},{default:q(()=>[Ae("Toggle Vertices ("+rn(P(a))+")",1)]),_:1}),w(P(Be),{type:"mdi",path:P(Db),rotate:90},null,8,["path"])]),_:1})]),_:1},8,["modelValue"]),Wt("div",dw,rn(P(r)),1),w(P(ka)),w(P(ot),{icon:"",onClick:F[2]||(F[2]=Cu(V=>s("remove"),["stop"]))},{default:q(()=>[w(P(We),{activator:"parent"},{default:q(()=>[Ae("Remove")]),_:1}),w(P(Be),{type:"mdi",path:P(Ib)},null,8,["path"])]),_:1})]),_:1}),w(P(Wh),null,{default:q(()=>[w(P(js),{modelValue:v.value,"onUpdate:modelValue":F[4]||(F[4]=V=>v.value=V),"hide-details":"",min:"0",max:"1",step:.1},{prepend:q(()=>[w(P(We),{activator:"parent"},{default:q(()=>[Ae("Change opacity")]),_:1}),w(P(Be),{type:"mdi",path:P(Ab)},null,8,["path"])]),append:q(()=>[w(P(We),{activator:"parent"},{default:q(()=>[Ae("Wireframe")]),_:1}),w(P(Xr),{trueIcon:"mdi-triangle-outline",falseIcon:"mdi-triangle",modelValue:S.value,"onUpdate:modelValue":F[3]||(F[3]=V=>S.value=V)},null,8,["modelValue"])]),_:1},8,["modelValue"]),P(o)>0||P(a)>0?(st(),Yt(P(js),{key:0,modelValue:m.value,"onUpdate:modelValue":F[5]||(F[5]=V=>m.value=V),"hide-details":"",min:"0",max:"1"},{prepend:q(()=>[w(P(We),{activator:"parent"},{default:q(()=>[Ae("Edge and vertex sizes")]),_:1}),w(P(Be),{type:"mdi",path:P(Bb)},null,8,["path"])]),_:1},8,["modelValue"])):Ei("",!0),w(P(gi)),w(P(js),{modelValue:l.value,"onUpdate:modelValue":F[7]||(F[7]=V=>l.value=V),"hide-details":"",min:"0",max:"1"},{prepend:q(()=>[w(P(We),{activator:"parent"},{default:q(()=>[Ae("Clip plane X")]),_:1}),w(P(Be),{type:"mdi",path:P(Go),rotate:120},null,8,["path"]),Ae(" X ")]),append:q(()=>[w(P(We),{activator:"parent"},{default:q(()=>[Ae("Swap clip plane X")]),_:1}),w(P(Xr),{trueIcon:"mdi-checkbox-marked-outline",falseIcon:"mdi-checkbox-blank-outline",modelValue:u.value,"onUpdate:modelValue":F[6]||(F[6]=V=>u.value=V)},{label:q(()=>[w(P(Be),{type:"mdi",path:P(qo)},null,8,["path"])]),_:1},8,["modelValue"])]),_:1},8,["modelValue"]),w(P(js),{modelValue:f.value,"onUpdate:modelValue":F[9]||(F[9]=V=>f.value=V),"hide-details":"",min:"0",max:"1"},{prepend:q(()=>[w(P(We),{activator:"parent"},{default:q(()=>[Ae("Clip plane Y")]),_:1}),w(P(Be),{type:"mdi",path:P(Go),rotate:-120},null,8,["path"]),Ae(" Y ")]),append:q(()=>[w(P(We),{activator:"parent"},{default:q(()=>[Ae("Swap clip plane Y")]),_:1}),w(P(Xr),{trueIcon:"mdi-checkbox-marked-outline",falseIcon:"mdi-checkbox-blank-outline",modelValue:h.value,"onUpdate:modelValue":F[8]||(F[8]=V=>h.value=V)},{label:q(()=>[w(P(Be),{type:"mdi",path:P(qo)},null,8,["path"])]),_:1},8,["modelValue"])]),_:1},8,["modelValue"]),w(P(js),{modelValue:c.value,"onUpdate:modelValue":F[11]||(F[11]=V=>c.value=V),"hide-details":"",min:"0",max:"1"},{prepend:q(()=>[w(P(We),{activator:"parent"},{default:q(()=>[Ae("Clip plane Z")]),_:1}),w(P(Be),{type:"mdi",path:P(Go)},null,8,["path"]),Ae(" Z ")]),append:q(()=>[w(P(We),{activator:"parent"},{default:q(()=>[Ae("Swap clip plane Z")]),_:1}),w(P(Xr),{trueIcon:"mdi-checkbox-marked-outline",falseIcon:"mdi-checkbox-blank-outline",modelValue:d.value,"onUpdate:modelValue":F[10]||(F[10]=V=>d.value=V)},{label:q(()=>[w(P(Be),{type:"mdi",path:P(qo)},null,8,["path"])]),_:1},8,["modelValue"])]),_:1},8,["modelValue"])]),_:1})]),_:1},8,["value"]))}}),hw=El(fw,[["__scopeId","data-v-dc6a0c8e"]]),mw=fn({__name:"Models",props:{viewer:{}},emits:["remove"],setup(t,{expose:e,emit:n}){const s=t,r=n;let{sceneDocument:i}=Ne("sceneDocument"),o=ee([]);function a(d){return d.getRoot().listMeshes().reduce((f,h)=>{var v;let m=((v=h.getExtras()[gt])==null?void 0:v.toString())??"Unnamed",p=f.find(S=>l(S[0])===m);return p?p.push(h):f.push([h]),f},[])}function l(d){var f;return((f=d.getExtras()[gt])==null?void 0:f.toString())??"Unnamed"}function u(d){r("remove",l(d))}function c(d){o.value.includes(d)||o.value.push(d)}return e({findModel:c}),(d,f)=>(st(!0),Ar(Ie,null,Vg(a(P(i)),h=>(st(),Yt(P(ab),{key:l(h[0]),modelValue:P(o),"onUpdate:modelValue":f[0]||(f[0]=m=>Me(o)?o.value=m:o=m),multiple:""},{default:q(()=>[w(hw,{meshes:h,viewer:s.viewer,onRemove:m=>u(h[0])},null,8,["meshes","viewer","onRemove"])]),_:2},1032,["modelValue"]))),128))}}),gw=250;class um{constructor(e,n,s,r){Dt(this,"name");Dt(this,"url");Dt(this,"hash");Dt(this,"isRemove");this.name=e,this.url=n,this.hash=s,this.isRemove=r}}class cm extends Event{constructor(n,s){super("update");Dt(this,"models");Dt(this,"disconnect");this.models=n,this.disconnect=s}}class vw extends EventTarget{constructor(){super(...arguments);Dt(this,"knownObjectHashes",{});Dt(this,"bufferedUpdates",[]);Dt(this,"batchTimeout",null)}async load(n){if(n.startsWith("dev+")){let s=new URL(n.slice(4));s.searchParams.set("api_updates","true"),await this.monitorDevServer(s)}else{let s=n.split("/").pop();s=(s==null?void 0:s.split(".")[0])||`unknown-${Math.random()}`;let i=(await fetch(n,{method:"HEAD"})).headers.get("etag");this.foundModel(s,i,n,!1)}}async monitorDevServer(n,s=()=>!1){for(;!s();){try{const r=new AbortController;let i=await fetch(n.toString(),{signal:r.signal});if(i.status===200){let o=pw(i.body.getReader());for await(let a of o){if(s())break;if(!a||!a.startsWith("data:"))continue;let l=JSON.parse(a.slice(5)),u=new URL(n);u.searchParams.delete("api_updates"),u.searchParams.set("api_object",l.name),this.foundModel(l.name,l.hash,u.toString(),l.is_remove,async()=>{r.abort()})}}else await new Promise(o=>setTimeout(o,10*un.monitorEveryMs));r.abort()}catch{}await new Promise(r=>setTimeout(r,un.monitorEveryMs))}}foundModel(n,s,r,i,o=()=>{}){this.bufferedUpdates=this.bufferedUpdates.filter(l=>l.name!==n);let a=new um(n,r,s,i);this.bufferedUpdates.push(a),this.dispatchEvent(new CustomEvent("update-early",{detail:this.bufferedUpdates})),this.batchTimeout!==null&&clearTimeout(this.batchTimeout),this.batchTimeout=setTimeout(()=>{for(let l of this.bufferedUpdates)if(l.isRemove==!1&&l.hash&&l.hash===this.knownObjectHashes[l.name]){let u=!1;this.bufferedUpdates=this.bufferedUpdates.filter(c=>c===l&&!u?(u=!0,!1):!0)}else l.isRemove==!0?l.name in this.knownObjectHashes&&delete this.knownObjectHashes[l.name]:l.isRemove==!1&&(this.knownObjectHashes[l.name]=l.hash);this.dispatchEvent(new cm(this.bufferedUpdates,o)),this.bufferedUpdates=[]},gw)}}async function*pw(t){let e=new TextDecoder,n=new Uint8Array;for(;;){let{value:s,done:r}=await t.read();if(r||!s)break;n=new Uint8Array([...n,...s]);let o=e.decode(n).split(`
`);for(let a=0;a<o.length-1;a++)yield o[a];n=new Uint8Array([...n.slice(-1)])}}const Ss={x:[[247,60,60],[148,36,36]],z:[[108,203,38],[65,122,23]],y:[[23,140,240],[14,84,144]]};function er(t,e,n,s,r,i="__helper",o=WebGL2RenderingContext.LINES){const a=t.getRoot().listBuffers()[0]??t.createBuffer(i+"Buffer"),l=t.getRoot().getDefaultScene()??t.getRoot().listScenes()[0]??t.createScene(i+"Scene"),u=t.createAccessor(i+"Position").setArray(new Float32Array(e)).setType("VEC3").setBuffer(a),c=t.createAccessor(i+"Indices").setArray(new Uint32Array(n)).setType("SCALAR").setBuffer(a);let d=null;s&&(d=t.createAccessor(i+"Color").setArray(new Float32Array(s)).setType("VEC4").setBuffer(a));const f=t.createMaterial(i+"Material").setAlphaMode("OPAQUE"),h=t.createPrimitive().setIndices(c).setAttribute("POSITION",u).setMode(o).setMaterial(f);s&&h.setAttribute("COLOR_0",d),o==WebGL2RenderingContext.TRIANGLES?h.setExtras({face_triangles_end:[n.length/6,n.length*2/6,n.length*3/6,n.length*4/6,n.length*5/6,n.length]}):o==WebGL2RenderingContext.LINES&&h.setExtras({edge_points_end:[n.length/3,n.length*2/3,n.length]});const m=t.createMesh(i+"Mesh").addPrimitive(h),p=t.createNode(i+"Node").setMesh(m).setMatrix(r.elements);l.addChild(p)}function yw(t,e,n){let s=[0,1,2,3,4,5],r=[0,0,0,e.x,0,0,0,0,0,0,e.y,0,0,0,0,0,0,-e.z],i=[...Ss.x[0],255,...Ss.x[1],255,...Ss.y[0],255,...Ss.y[1],255,...Ss.z[0],255,...Ss.z[1],255].map(o=>o/255);er(t,r,s,i,new He,"__helper_axes"),er(t,[0,0,0],[0],[1,1,1,1],new He,"__helper_axes",WebGL2RenderingContext.POINTS),new He!=n&&(er(t,r,s,i,n,"__helper_axes_center"),er(t,[0,0,0],[0],[1,1,1,1],n,"__helper_axes_center",WebGL2RenderingContext.POINTS))}async function bw(t,e,n,s=10){let r=[],i=[];for(let a of[new J(1,0,0),new J(0,1,0),new J(0,0,-1)])for(let l of[1,-1]){let u=a.clone().multiply(e.clone().multiplyScalar(.5*l)),c=new He().makeTranslation(u.x,u.y,u.z),d=new He().lookAt(new J,u,new J(0,1,0)),f=new Co;a.x&&f.set(e.z,e.y),a.y&&f.set(e.x,e.z),a.z&&f.set(e.x,e.y);let h=new He().multiply(c).multiply(d),[m,p]=xw(f,s),v=r.length/3;for(let S of p)i.push(S+v);for(let S=0;S<m.length;S+=3){let g=new J(m[S],m[S+1],m[S+2]);g.applyMatrix4(h),r.push(g.x,g.y,g.z)}}let o=new Array(r.length/3*4).fill(1);er(t,r,i,o,n,"__helper_grid",WebGL2RenderingContext.TRIANGLES)}function xw(t,e=10,n=.002){const s=[],r=[];for(let i=0;i<=e;i++){const o=-t.x/2+t.x*i/e,a=-t.y/2+t.y*i/e;s.push(o-n*t.x/2,-t.y/2,0),s.push(o+n*t.x/2,-t.y/2,0),s.push(o+n*t.x/2,t.y/2,0),s.push(o-n*t.x/2,t.y/2,0);const l=i*4;r.push(l,l+1,l+2),r.push(l,l+2,l+3),s.push(-t.x/2,a-n*t.y/2,0),s.push(t.x/2,a-n*t.y/2,0),s.push(t.x/2,a+n*t.y/2,0),s.push(-t.x/2,a+n*t.y/2,0);const u=(e+1+i)*4;r.push(u,u+1,u+2),r.push(u,u+2,u+3)}return[s,r]}class Sr{static async loadModel(e,n,s,r,i=!0,o=!0){let a=performance.now(),l;try{n=await Kx(r,s,n,()=>l=performance.now()),console.log("Model",s,"loaded in",performance.now()-l,"ms after",l-a,"ms of transferring data (maybe building the object on the server)")}finally{if(i&&(await this.reloadHelpers(e,n,o),o=!1),o){let u=performance.now();n=await this.showCurrentDoc(e,n),console.log("Scene displayed in",performance.now()-u,"ms")}}return n}static async reloadHelpers(e,n,s){let r=Sr.getBoundingBox(n);if(!r)return n;let i=new Vn,o=new He().makeTranslation(r.getCenter(new J));yw(i,r.getSize(new J).multiplyScalar(.5),o),bw(i,r.getSize(new J),o);let a=URL.createObjectURL(new Blob([await Zc(i)]));return await Sr.loadModel(e,n,yi,a,!1,s)}static getBoundingBox(e){if(e.getRoot().listNodes().length===0)return null;let n=[1e6,1e6,1e6],s=[-1e6,-1e6,-1e6];e.getRoot().listNodes().forEach(o=>{var l;if((o.getExtras()[gt]??yi)===yi)return;let a=new He(...o.getWorldMatrix());for(let u of((l=o.getMesh())==null?void 0:l.listPrimitives())??[]){let c=u.getAttribute("POSITION");if(!c)continue;let d=new J(...c.getMin([0,0,0])).applyMatrix4(a),f=new J(...c.getMax([0,0,0])).applyMatrix4(a);n=n.map((h,m)=>Math.min(h,d.getComponent(m))),s=s.map((h,m)=>Math.max(h,f.getComponent(m)))}});let r=new J().fromArray(s).sub(new J().fromArray(n)),i=new J().fromArray(n).add(r.clone().multiplyScalar(.5));return new Ll().setFromCenterAndSize(i,r)}static async removeModel(e,n,s,r=!0,i=!0){let o=performance.now();return n=await Qx(s,n),console.log("Model",s,"removed in",performance.now()-o,"ms"),r&&(n=await this.reloadHelpers(e,n,i)),n}static async showCurrentDoc(e,n){n=await Jx(n);let s=await Zc(n),r=new Blob([s],{type:"model/gltf-binary"});return console.debug("Showing current doc",n,"with",s.length,"total bytes"),e.value=URL.createObjectURL(r),n}}const ww=fn({__name:"App",setup(t){const e=ua({loader:()=>Ut(()=>import("./ModelViewerWrapper-CUAxQuq7.js"),__vite__mapDeps([9,1,2]),import.meta.url),loadingComponent:Yh,delay:0});let n=ee(window.innerWidth>1200);const s=ee(""),r=ee(null),i=be(new Vn);ut("sceneDocument",{sceneDocument:i});const o=ee(null),a=ee(null),l=ee(!1);ut("disableTap",{disableTap:l,setDisableTap:m=>l.value=m});async function c(m){var g;r.value&&m.models.length>0&&r.value.onProgress(.1),console.log("Received model update request",m.models);let p=m.models.findIndex(y=>y.isRemove==null),v=null;p!==-1&&(console.log("Will shut down the connection after this load, as requested by the server"),v=m.models.splice(p,1)[0]);let S=i.value;for(let y in m.models){let b=parseInt(y)===m.models.length-1,x=m.models[y];(g=a.value)==null||g.removeObjectSelections(x.name);try{x.isRemove?S=await Sr.removeModel(s,S,x.name,b&&un.loadHelpers,b):S=await Sr.loadModel(s,S,x.name,x.url,b&&un.loadHelpers,b)}catch(_){console.error("Error loading model",x,_)}}v!==null&&(console.log("Shutting down the connection as requested by the server"),m.disconnect()),i.value=S,ng(i)}async function d(m){await c(new cm([new um(m,"",null,!0)],()=>{}))}let f=new vw;f.addEventListener("update-early",m=>{var p;return(p=r.value)==null?void 0:p.onProgress(m.detail.length*.01)}),f.addEventListener("update",m=>c(m));for(let m of un.preload)f.load(m);async function h(){const m=prompt(`For an improved experience in viewing CAD/GLTF models with automatic updates, it's recommended to use the official yacv_server Python package. This ensures seamless serving of models and automatic updates.

Otherwise, enter the URL of the model to load:`);m&&await f.load(m)}return(m,p)=>(st(),Yt(P(fb),{"full-height":""},{default:q(()=>[w(P(mb),{id:"main"},{default:q(()=>[w(P(e),{ref_key:"viewer",ref:r,src:s.value},null,8,["src"])]),_:1}),w(Tc,{"opened-init":P(n),side:"left",width:300},{toolbar:q(()=>[w(P(Ni),null,{default:q(()=>[Ae("Models")]),_:1})]),"toolbar-items":q(()=>[w(P(ot),{icon:"",onClick:h},{default:q(()=>[w(P(Be),{type:"mdi",path:P(Vb)},null,8,["path"])]),_:1})]),default:q(()=>[w(mw,{ref_key:"models",ref:o,viewer:r.value,onRemove:d},null,8,["viewer"])]),_:1},8,["opened-init"]),w(Tc,{"opened-init":P(n),side:"right",width:48*3+1},{toolbar:q(()=>[w(P(Ni),null,{default:q(()=>[Ae("Tools")]),_:1})]),default:q(()=>[w(px,{ref_key:"tools",ref:a,viewer:r.value,onFindModel:p[0]||(p[0]=v=>{var S;return(S=o.value)==null?void 0:S.findModel(v)})},null,8,["viewer"])]),_:1},8,["opened-init"])]),_:1}))}});function dm(){let t=arguments.length>0&&arguments[0]!==void 0?arguments[0]:{};const{blueprint:e,...n}=t,s=pt(e,n),{aliases:r={},components:i={},directives:o={}}=s,a=Np(s.defaults),l=o0(s.display,s.ssr),u=oy(s.theme),c=Ay(s.icons),d=ty(s.locale),f=H1(s.date,d),h=W1(s.goTo,d);return{install:p=>{for(const v in o)p.directive(v,o[v]);for(const v in i)p.component(v,i[v]);for(const v in r)p.component(v,kr({...r[v],name:v,aliasName:r[v].name}));if(u.install(p),p.provide(Vs,a),p.provide(Ma,l),p.provide(Oi,u),p.provide(Sa,c),p.provide(Pi,d),p.provide(z1,f.options),p.provide(xc,f.instance),p.provide(U1,h),Oe&&s.ssr)if(p.$nuxt)p.$nuxt.hook("app:suspense:resolve",()=>{l.update()});else{const{mount:v}=p;p.mount=function(){const S=v(...arguments);return It(()=>l.update()),p.mount=v,S}}gn.reset(),p.mixin({computed:{$vuetify(){return tt({defaults:Cs.call(this,Vs),display:Cs.call(this,Ma),theme:Cs.call(this,Oi),icons:Cs.call(this,Sa),locale:Cs.call(this,Pi),date:Cs.call(this,xc)})}}})},defaults:a,display:l,theme:u,icons:c,locale:d,date:f,goTo:h}}const _w="3.5.13";dm.version=_w;function Cs(t){var s,r;const e=this.$,n=((s=e.parent)==null?void 0:s.provides)??((r=e.vnode.appContext)==null?void 0:r.provides);if(n&&t in n)return n[t]}function Sw(t,e){const n=e.modifiers||{},s=e.value,{once:r,immediate:i,...o}=n,a=!Object.keys(o).length,{handler:l,options:u}=typeof s=="object"?s:{handler:s,options:{attributes:(o==null?void 0:o.attr)??a,characterData:(o==null?void 0:o.char)??a,childList:(o==null?void 0:o.child)??a,subtree:(o==null?void 0:o.sub)??a}},c=new MutationObserver(function(){let d=arguments.length>0&&arguments[0]!==void 0?arguments[0]:[],f=arguments.length>1?arguments[1]:void 0;l==null||l(d,f),r&&fm(t,e)});i&&(l==null||l([],c)),t._mutate=Object(t._mutate),t._mutate[e.instance.$.uid]={observer:c},c.observe(t,u)}function fm(t,e){var n;(n=t._mutate)!=null&&n[e.instance.$.uid]&&(t._mutate[e.instance.$.uid].observer.disconnect(),delete t._mutate[e.instance.$.uid])}const Cw={mounted:Sw,unmounted:fm};function Ew(t,e){var r,i;const n=e.value,s={passive:!((r=e.modifiers)!=null&&r.active)};window.addEventListener("resize",n,s),t._onResize=Object(t._onResize),t._onResize[e.instance.$.uid]={handler:n,options:s},(i=e.modifiers)!=null&&i.quiet||n()}function Mw(t,e){var r;if(!((r=t._onResize)!=null&&r[e.instance.$.uid]))return;const{handler:n,options:s}=t._onResize[e.instance.$.uid];window.removeEventListener("resize",n,s),delete t._onResize[e.instance.$.uid]}const Tw={mounted:Ew,unmounted:Mw};function hm(t,e){const{self:n=!1}=e.modifiers??{},s=e.value,r=typeof s=="object"&&s.options||{passive:!0},i=typeof s=="function"||"handleEvent"in s?s:s.handler,o=n?t:e.arg?document.querySelector(e.arg):window;o&&(o.addEventListener("scroll",i,r),t._onScroll=Object(t._onScroll),t._onScroll[e.instance.$.uid]={handler:i,options:r,target:n?void 0:o})}function mm(t,e){var i;if(!((i=t._onScroll)!=null&&i[e.instance.$.uid]))return;const{handler:n,options:s,target:r=t}=t._onScroll[e.instance.$.uid];r.removeEventListener("scroll",n,s),delete t._onScroll[e.instance.$.uid]}function Aw(t,e){e.value!==e.oldValue&&(mm(t,e),hm(t,e))}const Rw={mounted:hm,unmounted:mm,updated:Aw},Iw=Object.freeze(Object.defineProperty({__proto__:null,ClickOutside:Rh,Intersect:Hf,Mutate:Cw,Resize:Tw,Ripple:Ds,Scroll:Rw,Touch:X0},Symbol.toStringTag,{value:"Module"}));console.log("Starting yet-another-cad-viewer v0.8.7 (345636e4785144669d192932b9a60939ae194e19)...");const kw=dm({directives:Iw,theme:{defaultTheme:window.matchMedia("(prefers-color-scheme: dark)").matches?"dark":"light"}}),gm=qv(ww);gm.use(kw);gm.mount("body");export{Gf as $,Wf as A,me as B,Jt as C,_h as D,Yf as E,Ie as F,be as G,k as H,Ke as I,kf as J,Oe as K,ye as L,ln as M,bf as N,yr as O,co as P,dt as Q,Vr as R,Ft as S,mn as T,fe as U,Bw as V,ki as W,Fr as X,ds as Y,us as Z,Ut as _,st as a,mo as a$,Qt as a0,ah as a1,po as a2,Ds as a3,ny as a4,Lr as a5,pl as a6,Bs as a7,cs as a8,en as a9,Es as aA,vh as aB,th as aC,xh as aD,nh as aE,e0 as aF,Ky as aG,At as aH,On as aI,Sf as aJ,oe as aK,Zs as aL,_a as aM,Py as aN,vf as aO,rp as aP,wh as aQ,Hf as aR,op as aS,lc as aT,Pn as aU,Pw as aV,Nn as aW,Rt as aX,Vw as aY,ho as aZ,qe as a_,yo as aa,ct as ab,qf as ac,oh as ad,qt as ae,ls as af,vl as ag,Dw as ah,Tt as ai,uo as aj,oc as ak,ul as al,wl as am,cy as an,xo as ao,gn as ap,w0 as aq,ut as ar,Ow as as,zi as at,Ih as au,It as av,up as aw,cp as ax,gl as ay,fy as az,Wt as b,an as b0,$p as b1,Fn as b2,n0 as b3,Xr as b4,dp as b5,xf as b6,gt as b7,Zl as b8,cu as b9,ns as bA,So as bB,Gw as bC,Qr as bD,ze as bE,Rn as bF,Lx as bG,Or as bH,Il as bI,Pa as bJ,Wb as bK,jw as bL,We as ba,Be as bb,Hw as bc,ot as bd,Me as be,$w as bf,zw as bg,Uw as bh,Sr as bi,Ss as bj,pe as bk,kt as bl,kr as bm,Uf as bn,Bf as bo,fo as bp,Nw as bq,gi as br,Lw as bs,tm as bt,se as bu,Vx as bv,ue as bw,Se as bx,Hi as by,ss as bz,Ar as c,fn as d,Ys as e,Ae as f,Vg as g,Yt as h,Ne as i,q as j,w as k,Yo as l,Pe as m,Ei as n,Zt as o,Qi as p,El as q,ee as r,un as s,rn as t,P as u,Y as v,le as w,we as x,a0 as y,Je as z};
//# sourceMappingURL=index-CKOVumwO.js.map
